@@ -7,6 +7,7 @@
 #include "proxy.h"
 #include "servercapabilities.h"
 #include "sizeformatting_base.h"
+#include "socket_errors.h"
 
 #include <libfilezilla/event_loop.hpp>
 #include <libfilezilla/iputils.hpp>
@@ -28,15 +29,11 @@
 	#endif
 #endif
 
-struct obtain_lock_event_type;
-typedef fz::simple_event<obtain_lock_event_type> CObtainLockEvent;
-
-std::list<CControlSocket::t_lockInfo> CControlSocket::m_lockInfoList;
-
 CControlSocket::CControlSocket(CFileZillaEnginePrivate & engine)
 	: CLogging(engine)
 	, event_handler(engine.event_loop_)
 	, engine_(engine)
+	, opLockManager_(engine.opLockManager_)
 {
 }
 
@@ -132,12 +129,10 @@ int CControlSocket::ResetOperation(int nErrorCode)
 
 	std::unique_ptr<COpData> oldOperation;
 	if (!operations_.empty()) {
-		if (operations_.back()->holdsLock_) {
-			ReleaseLock();
-		}
 		oldOperation = std::move(operations_.back());
 		operations_.pop_back();
 
+		LogMessage(MessageType::Debug_Verbose, L"%s::Reset(%d) in state %d", oldOperation->name_, nErrorCode, oldOperation->opState);
 		nErrorCode = oldOperation->Reset(nErrorCode);
 	}
 	if (!operations_.empty()) {
@@ -232,7 +227,7 @@ void CControlSocket::UpdateCache(COpData const &, CServerPath const& serverPath,
 {
 	bool updated = engine_.GetDirectoryCache().UpdateFile(currentServer_, serverPath, remoteFile, true, CDirectoryCache::file, fileSize);
 	if (updated) {
-		SendDirectoryListingNotification(serverPath, false, false);
+		SendDirectoryListingNotification(serverPath, false);
 	}
 }
 
@@ -434,10 +429,11 @@ int CControlSocket::CheckOverwriteFile()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-CFileTransferOpData::CFileTransferOpData(bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path)
-	: COpData(Command::transfer)
+CFileTransferOpData::CFileTransferOpData(wchar_t const* name, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings)
+	: COpData(Command::transfer, name)
 	, localFile_(local_file), remoteFile_(remote_file), remotePath_(remote_path)
 	, download_(is_download)
+	, transferSettings_(settings)
 {
 }
 
@@ -511,7 +507,7 @@ void CControlSocket::OnTimer(fz::timer_id)
 	if (timeout > 0) {
 		fz::duration elapsed = fz::monotonic_clock::now() - m_lastActivity;
 
-		if ((operations_.empty() || !operations_.back()->waitForAsyncRequest) && !IsWaitingForLock()) {
+		if ((operations_.empty() || !operations_.back()->waitForAsyncRequest) && !opLockManager_.Waiting(this)) {
 			if (elapsed > fz::duration::from_seconds(timeout)) {
 				LogMessage(MessageType::Error, fztranslate("Connection timed out after %d second of inactivity", "Connection timed out after %d seconds of inactivity", timeout), timeout);
 				DoClose(FZ_REPLY_TIMEOUT);
@@ -574,6 +570,7 @@ int CControlSocket::SendNextCommand()
 			return FZ_REPLY_WOULDBLOCK;
 		}
 
+		LogMessage(data.sendLogLevel_, L"%s::Send() in state %d", data.name_, data.opState);
 		int res = data.Send();
 		if (res != FZ_REPLY_CONTINUE) {
 			if (res == FZ_REPLY_OK) {
@@ -600,14 +597,14 @@ int CControlSocket::SendNextCommand()
 
 int CControlSocket::ParseSubcommandResult(int prevResult, COpData const& opData)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CControlSocket::ParseSubcommandResult(%d)", prevResult);
 	if (operations_.empty()) {
-		LogMessage(MessageType::Debug_Warning, L"ParseSubcommandResult called without active operation");
+		LogMessage(MessageType::Debug_Warning, L"CControlSocket::ParseSubcommandResult(%d) called without active operation", prevResult);
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	auto & data = *operations_.back();
+	LogMessage(MessageType::Debug_Verbose, L"%s::SubcommandResult(%d) in state %d", data.name_, prevResult, data.opState);
 	int res = data.SubcommandResult(prevResult, opData);
 	if (res == FZ_REPLY_WOULDBLOCK) {
 		return FZ_REPLY_WOULDBLOCK;
@@ -620,211 +617,16 @@ int CControlSocket::ParseSubcommandResult(int prevResult, COpData const& opData)
 	}
 }
 
-const std::list<CControlSocket::t_lockInfo>::iterator CControlSocket::GetLockStatus()
+OpLock CControlSocket::Lock(locking_reason reason, CServerPath const& path, bool inclusive)
 {
-	std::list<t_lockInfo>::iterator iter;
-	for (iter = m_lockInfoList.begin(); iter != m_lockInfoList.end(); ++iter) {
-		if (iter->pControlSocket == this) {
-			break;
-		}
-	}
-
-	return iter;
-}
-
-bool CControlSocket::TryLock(locking_reason reason, CServerPath const& directory)
-{
-	assert(currentServer_);
-	assert(!operations_.empty());
-
-	std::list<t_lockInfo>::iterator own = GetLockStatus();
-	if (own == m_lockInfoList.end()) {
-		t_lockInfo info;
-		info.directory = directory;
-		info.pControlSocket = this;
-		info.waiting = true;
-		info.reason = reason;
-		info.lockcount = 0;
-		m_lockInfoList.push_back(info);
-		own = --m_lockInfoList.end();
-	}
-	else {
-		if (own->lockcount) {
-			if (!operations_.back()->holdsLock_) {
-				operations_.back()->holdsLock_ = true;
-				own->lockcount++;
-			}
-			return true;
-		}
-		assert(own->waiting);
-		assert(own->reason == reason);
-	}
-
-	// Needs to be set in any case so that ResetOperation
-	// unlocks or cancels the lock wait
-	operations_.back()->holdsLock_ = true;
-
-	// Try to find other instance holding the lock
-	for (auto iter = m_lockInfoList.cbegin(); iter != own; ++iter) {
-		if (currentServer_ != iter->pControlSocket->currentServer_) {
-			continue;
-		}
-		if (directory != iter->directory) {
-			continue;
-		}
-		if (reason != iter->reason) {
-			continue;
-		}
-
-		// Some other instance is holding the lock
-		return false;
-	}
-
-	own->lockcount++;
-	own->waiting = false;
-	return true;
-}
-
-bool CControlSocket::IsLocked(locking_reason reason, CServerPath const& directory)
-{
-	assert(currentServer_);
-
-	std::list<t_lockInfo>::iterator own = GetLockStatus();
-	if (own != m_lockInfoList.end()) {
-		return true;
-	}
-
-	// Try to find other instance holding the lock
-	for (auto iter = m_lockInfoList.cbegin(); iter != own; ++iter) {
-		if (currentServer_ != iter->pControlSocket->currentServer_) {
-			continue;
-		}
-		if (directory != iter->directory) {
-			continue;
-		}
-		if (reason != iter->reason) {
-			continue;
-		}
-
-		// Some instance is holding the lock
-		return true;
-	}
-
-	return false;
-}
-
-void CControlSocket::ReleaseLock()
-{
-	if (operations_.empty() || !operations_.back()->holdsLock_) {
-		return;
-	}
-	operations_.back()->holdsLock_ = false;
-
-	std::list<t_lockInfo>::iterator iter = GetLockStatus();
-	if (iter == m_lockInfoList.end()) {
-		return;
-	}
-
-	assert(!iter->waiting || iter->lockcount == 0);
-	if (!iter->waiting) {
-		iter->lockcount--;
-		assert(iter->lockcount >= 0);
-		if (iter->lockcount) {
-			return;
-		}
-	}
-
-	CServerPath directory = iter->directory;
-	locking_reason reason = iter->reason;
-
-	m_lockInfoList.erase(iter);
-
-	// Find other instance waiting for the lock
-	if (!currentServer_) {
-		LogMessage(MessageType::Debug_Warning, L"ReleaseLock called with !currentServer_");
-		return;
-	}
-	for (auto & lockInfo : m_lockInfoList) {
-		if (!lockInfo.pControlSocket->currentServer_) {
-			LogMessage(MessageType::Debug_Warning, L"ReleaseLock found other instance with !currentServer_");
-			continue;
-		}
-
-		if (currentServer_ != lockInfo.pControlSocket->currentServer_) {
-			continue;
-		}
-
-		if (lockInfo.directory != directory) {
-			continue;
-		}
-
-		if (lockInfo.reason != reason) {
-			continue;
-		}
-
-		// Send notification
-		lockInfo.pControlSocket->send_event<CObtainLockEvent>();
-		break;
-	}
-}
-
-locking_reason CControlSocket::ObtainLockFromEvent()
-{
-	if (operations_.empty()) {
-		return locking_reason::unknown;
-	}
-
-	std::list<t_lockInfo>::iterator own = GetLockStatus();
-	if (own == m_lockInfoList.end()) {
-		return locking_reason::unknown;
-	}
-
-	if (!own->waiting) {
-		return locking_reason::unknown;
-	}
-
-	for (auto iter = m_lockInfoList.cbegin(); iter != own; ++iter) {
-		if (currentServer_ != iter->pControlSocket->currentServer_) {
-			continue;
-		}
-
-		if (iter->directory != own->directory) {
-			continue;
-		}
-
-		if (iter->reason != own->reason) {
-			continue;
-		}
-
-		// Another instance comes before us
-		return locking_reason::unknown;
-	}
-
-	own->waiting = false;
-	own->lockcount++;
-
-	return own->reason;
+	return opLockManager_.Lock(this, reason, path, inclusive);
 }
 
 void CControlSocket::OnObtainLock()
 {
-	if (ObtainLockFromEvent() == locking_reason::unknown) {
-		return;
+	if (opLockManager_.ObtainWaiting(this)) {
+		SendNextCommand();
 	}
-
-	SendNextCommand();
-
-	ReleaseLock();
-}
-
-bool CControlSocket::IsWaitingForLock()
-{
-	std::list<t_lockInfo>::iterator own = GetLockStatus();
-	if (own == m_lockInfoList.end()) {
-		return false;
-	}
-
-	return own->waiting == true;
 }
 
 void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
@@ -916,7 +718,7 @@ int CRealControlSocket::Send(unsigned char const* buffer, unsigned int len)
 		int written = m_pBackend->Write(buffer, len, error);
 		if (written < 0) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket::error_description(error));
+				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket_error_description(error));
 				LogMessage(MessageType::Error, _("Disconnected from server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
@@ -955,14 +757,14 @@ void CRealControlSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_even
 	{
 	case fz::socket_event_flag::connection_next:
 		if (error) {
-			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), fz::socket::error_description(error));
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), fz::socket_error_description(error));
 		}
 		SetAlive();
 		break;
 	case fz::socket_event_flag::connection:
 		if (error) {
-			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), fz::socket::error_description(error));
-			OnClose(error);
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), fz::socket_error_description(error));
+			OnSocketError(error);
 		}
 		else {
 			if (m_pProxyBackend && !m_pProxyBackend->Detached()) {
@@ -973,13 +775,20 @@ void CRealControlSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_even
 		}
 		break;
 	case fz::socket_event_flag::read:
-		OnReceive();
+		if (error) {
+			OnSocketError(error);
+		}
+		else {
+			OnReceive();
+		}
 		break;
 	case fz::socket_event_flag::write:
-		OnSend();
-		break;
-	case fz::socket_event_flag::close:
-		OnClose(error);
+		if (error) {
+			OnSocketError(error);
+		}
+		else {
+			OnSend();
+		}
 		break;
 	default:
 		LogMessage(MessageType::Debug_Warning, L"Unhandled socket event %d", t);
@@ -1011,7 +820,7 @@ int CRealControlSocket::OnSend()
 		int written = m_pBackend->Write(sendBuffer_.get(), sendBuffer_.size(), error);
 		if (written < 0) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket::error_description(error));
+				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket_error_description(error));
 				if (GetCurrentCommandId() != Command::connect) {
 					LogMessage(MessageType::Error, _("Disconnected from server"));
 				}
@@ -1031,19 +840,14 @@ int CRealControlSocket::OnSend()
 	return FZ_REPLY_CONTINUE;
 }
 
-void CRealControlSocket::OnClose(int error)
+void CRealControlSocket::OnSocketError(int error)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CRealControlSocket::OnClose(%d)", error);
+	LogMessage(MessageType::Debug_Verbose, L"CRealControlSocket::OnSocketError(%d)", error);
 
 	auto cmd = GetCurrentCommandId();
 	if (cmd != Command::connect) {
 		auto messageType = (cmd == Command::none) ? MessageType::Status : MessageType::Error;
-		if (!error) {
-			LogMessage(messageType, _("Connection closed by server"));
-		}
-		else {
-			LogMessage(messageType, _("Disconnected from server: %s"), fz::socket::error_description(error));
-		}
+		LogMessage(messageType, _("Disconnected from server: %s"), fz::socket_error_description(error));
 	}
 	DoClose();
 }
@@ -1075,7 +879,7 @@ int CRealControlSocket::DoConnect(std::wstring const& host, unsigned int port)
 											engine_.GetOptions().GetOption(OPTION_PROXY_PASS));
 
 		if (res != EINPROGRESS) {
-			LogMessage(MessageType::Error, _("Could not start proxy handshake: %s"), fz::socket::error_description(res));
+			LogMessage(MessageType::Error, _("Could not start proxy handshake: %s"), fz::socket_error_description(res));
 			return FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
 		}
 	}
@@ -1094,7 +898,7 @@ int CRealControlSocket::DoConnect(std::wstring const& host, unsigned int port)
 
 	// Treat success same as EINPROGRESS, we wait for connect notification in any case
 	if (res && res != EINPROGRESS) {
-		LogMessage(MessageType::Error, _("Could not connect to server: %s"), fz::socket::error_description(res));
+		LogMessage(MessageType::Error, _("Could not connect to server: %s"), fz::socket_error_description(res));
 		return FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR; 
 	}
 
@@ -1358,11 +1162,11 @@ void CControlSocket::SetActive(CFileZillaEngine::_direction direction)
 	engine_.SetActive(direction);
 }
 
-void CControlSocket::SendDirectoryListingNotification(CServerPath const& path, bool onList, bool failed)
+void CControlSocket::SendDirectoryListingNotification(CServerPath const& path, bool failed)
 {
 	if (!currentServer_) {
 		return;
 	}
 
-	engine_.AddNotification(new CDirectoryListingNotification(path, !onList, failed));
+	engine_.AddNotification(new CDirectoryListingNotification(path, operations_.size() == 1 && operations_.back()->opId == Command::list, failed));
 }
