@@ -6,11 +6,14 @@
 
 #include <libfilezilla/buffer.hpp>
 
+#include "oplock_manager.h"
+
 class COpData
 {
 public:
-	explicit COpData(Command op_Id)
+	explicit COpData(Command op_Id, wchar_t const* name)
 		: opId(op_Id)
+		, name_(name)
 	{}
 
 	virtual ~COpData() = default;
@@ -36,7 +39,11 @@ public:
 	Command const opId;
 
 	bool waitForAsyncRequest{};
-	bool holdsLock_{};
+	OpLock opLock_;
+
+	wchar_t const* const name_;
+
+	MessageType sendLogLevel_{MessageType::Debug_Verbose};
 };
 
 template<typename T>
@@ -68,36 +75,17 @@ class CNotSupportedOpData : public COpData
 {
 public:
 	CNotSupportedOpData()
-		: COpData(Command::none)
+		: COpData(Command::none, L"CNotSupportedOpData")
 	{}
 
 	virtual int Send() { return FZ_REPLY_NOTSUPPORTED; }
 	virtual int ParseResponse() { return FZ_REPLY_INTERNALERROR; }
 };
 
-class CConnectOpData : public COpData
-{
-public:
-	CConnectOpData(CServer const& server)
-		: COpData(Command::connect)
-		, server_(server)
-	{
-	}
-
-	// What to connect the socket to,
-	// can be different from server_ if using
-	// a proxy
-	std::wstring host_;
-	unsigned int port_{};
-
-	// Target server
-	CServer server_;
-};
-
 class CFileTransferOpData : public COpData
 {
 public:
-	CFileTransferOpData(bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path);
+	CFileTransferOpData(wchar_t const* name, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings);
 
 	// Transfer data
 	std::wstring localFile_, remoteFile_;
@@ -121,8 +109,8 @@ public:
 class CMkdirOpData : public COpData
 {
 public:
-	CMkdirOpData()
-		: COpData(Command::mkdir)
+	CMkdirOpData(wchar_t const* name)
+		: COpData(Command::mkdir, name)
 	{
 	}
 
@@ -135,8 +123,8 @@ public:
 class CChangeDirOpData : public COpData
 {
 public:
-	CChangeDirOpData()
-		: COpData(Command::cwd)
+	CChangeDirOpData(wchar_t const* name)
+		: COpData(Command::cwd, name)
 	{
 	}
 
@@ -162,15 +150,6 @@ enum class TransferEndReason
 	failed_resumetest
 };
 
-enum class locking_reason
-{
-	unknown = -1,
-	list,
-	mkdir,
-
-	private1
-};
-
 class CBackend;
 class CTransferStatus;
 class CControlSocket: public CLogging, public fz::event_handler
@@ -237,7 +216,7 @@ public:
 	int SendNextCommand();
 
 protected:
-	void SendDirectoryListingNotification(CServerPath const& path, bool onList, bool failed);
+	void SendDirectoryListingNotification(CServerPath const& path, bool failed);
 
 	fz::duration GetTimezoneOffset() const;
 
@@ -262,6 +241,8 @@ protected:
 
 	void Push(std::unique_ptr<COpData> && pNewOpData);
 
+	OpLock Lock(locking_reason reason, CServerPath const& path, bool inclusive = false);
+
 	std::vector<std::unique_ptr<COpData>> operations_;
 	CFileZillaEnginePrivate & engine_;
 	CServer currentServer_;
@@ -274,45 +255,7 @@ protected:
 	fz::timer_id m_timer{};
 	fz::monotonic_clock m_lastActivity;
 
-	// -------------------------
-	// Begin cache locking stuff
-	// -------------------------
-
-	// Tries to obtain lock. Returns true on success.
-	// On failure, caller has to pass control.
-	// SendNextCommand will be called once the lock gets available
-	// and engine could obtain it.
-	// Lock is recursive. Lock counter increases on suboperations.
-	bool TryLock(locking_reason reason, CServerPath const& directory);
-	bool IsLocked(locking_reason reason, CServerPath const& directory);
-
-	// Unlocks the cache. Can be called if not holding the lock
-	// Doesn't need reason as one engine can at most hold one lock
-	void ReleaseLock();
-
-	// Called from the obtain_lock_event_type event.
-	// Returns reason != unknown iff engine is the first waiting engine
-	// and obtains the lock.
-	// On failure, the engine was not waiting for a lock.
-	locking_reason ObtainLockFromEvent();
-
-	bool IsWaitingForLock();
-
-	struct t_lockInfo
-	{
-		CControlSocket* pControlSocket;
-		CServerPath directory;
-		locking_reason reason;
-		bool waiting;
-		int lockcount;
-	};
-	static std::list<t_lockInfo> m_lockInfoList;
-
-	const std::list<t_lockInfo>::iterator GetLockStatus();
-
-	// -----------------------
-	// End cache locking stuff
-	// -----------------------
+	OpLockManager & opLockManager_;
 
 	bool m_invalidateCurrentPath{};
 
@@ -344,7 +287,7 @@ protected:
 	virtual void OnConnect();
 	virtual void OnReceive();
 	virtual int OnSend();
-	virtual void OnClose(int error);
+	virtual void OnSocketError(int error);
 
 	int Send(unsigned char const* buffer, unsigned int len);
 	int Send(char const* buffer, unsigned int len) {
