@@ -9,7 +9,7 @@ CConnectCommand::CConnectCommand(CServer const& server, Credentials const& crede
 
 bool CConnectCommand::valid() const
 {
-	if (!server_ || server_.GetUser().empty()) {
+	if (!server_) {
 		return false;
 	}
 	return true;
@@ -37,16 +37,19 @@ std::wstring CListCommand::GetSubDir() const
 
 bool CListCommand::valid() const
 {
-	if (GetPath().empty() && !GetSubDir().empty())
+	if (GetPath().empty() && !GetSubDir().empty()) {
 		return false;
+	}
 
-	if (GetFlags() & LIST_FLAG_LINK && GetSubDir().empty())
+	if (GetFlags() & LIST_FLAG_LINK && GetSubDir().empty()) {
 		return false;
+	}
 
 	bool const refresh = (m_flags & LIST_FLAG_REFRESH) != 0;
 	bool const avoid = (m_flags & LIST_FLAG_AVOID) != 0;
-	if (refresh && avoid)
+	if (refresh && avoid) {
 		return false;
+	}
 
 	return true;
 }
@@ -137,4 +140,4 @@ CChmodCommand::CChmodCommand(CServerPath const& path, std::wstring const& file,
 bool CChmodCommand::valid() const
 {
 	return !GetPath().empty() && !GetFile().empty() && !GetPermission().empty();
-}
\ No newline at end of file
+}
