@@ -46,6 +46,15 @@ ObjectCache objcache;
 class CToken final
 {
 protected:
+	enum flags : unsigned char {
+		numeric_left = 0x01,
+		non_numeric_left = 0x02,
+		numeric_right = 0x04,
+		non_numeric_right = 0x08,
+		numeric  = 0x10,
+		non_numeric = 0x20
+	};
+
 	enum TokenInformation
 	{
 		Unknown,
@@ -72,11 +81,14 @@ public:
 		return m_pToken;
 	}
 
-	unsigned int GetLength() const
-	{
+	unsigned size() const {
 		return m_len;
 	}
 
+	explicit operator bool() const { return m_len != 0; }
+
+	wchar_t operator[](unsigned int i) const { return m_pToken[i]; }
+
 	std::wstring GetString() const
 	{
 		if (!m_pToken || !m_len) {
@@ -93,16 +105,16 @@ public:
 		{
 		case decimal:
 		default:
-			if (m_numeric == Unknown) {
-				m_numeric = Yes;
+			if (!(flags_ & (numeric | non_numeric))) {
+				flags_ |= numeric;
 				for (unsigned int i = 0; i < m_len; ++i) {
 					if (m_pToken[i] < '0' || m_pToken[i] > '9') {
-						m_numeric = No;
+						flags_ ^= numeric | non_numeric;
 						break;
 					}
 				}
 			}
-			return m_numeric == Yes;
+			return flags_ & numeric;
 		case hex:
 			for (unsigned int i = 0; i < m_len; ++i) {
 				auto const c = m_pToken[i];
@@ -126,34 +138,28 @@ public:
 
 	bool IsLeftNumeric()
 	{
-		if (m_leftNumeric == Unknown) {
-			if (m_len < 2) {
-				m_leftNumeric = No;
-			}
-			else if (m_pToken[0] < '0' || m_pToken[0] > '9') {
-				m_leftNumeric = No;
+		if (!(flags_ & (numeric_left | non_numeric_left))) {
+			if (m_len < 2 || m_pToken[0] < '0' || m_pToken[0] > '9') {
+				flags_ |= non_numeric_left;
 			}
 			else {
-				m_leftNumeric = Yes;
+				flags_ |= numeric_left;
 			}
 		}
-		return m_leftNumeric == Yes;
+		return flags_ & numeric_left;
 	}
 
 	bool IsRightNumeric()
 	{
-		if (m_rightNumeric == Unknown) {
-			if (m_len < 2) {
-				m_rightNumeric = No;
-			}
-			else if (m_pToken[m_len - 1] < '0' || m_pToken[m_len - 1] > '9') {
-				m_rightNumeric = No;
+		if (!(flags_ & (numeric_right | non_numeric_right))) {
+			if (m_len < 2 || m_pToken[m_len - 1] < '0' || m_pToken[m_len - 1] > '9') {
+				flags_ |= non_numeric_right;
 			}
 			else {
-				m_rightNumeric = Yes;
+				flags_ |= numeric_right;
 			}
 		}
-		return m_rightNumeric == Yes;
+		return flags_ & numeric_right;
 	}
 
 	int Find(const wchar_t* chr, int start = 0) const
@@ -270,29 +276,20 @@ public:
 		}
 	}
 
-	wchar_t operator[](unsigned int n) const
-	{
-		if (n >= m_len) {
-			return 0;
-		}
-
-		return m_pToken[n];
-	}
-
 protected:
+	int64_t m_number{-1};
+
 	wchar_t const* m_pToken{};
+
 	unsigned int m_len{};
 
-	TokenInformation m_numeric{Unknown};
-	TokenInformation m_leftNumeric{Unknown};
-	TokenInformation m_rightNumeric{Unknown};
-	int64_t m_number{-1};
+	unsigned char flags_{};
 };
 
 class CLine final
 {
 public:
-	CLine(std::wstring && line, int trailing_whitespace = -1)
+	CLine(std::wstring && line, size_t trailing_whitespace = std::string::npos)
 		: trailing_whitespace_(trailing_whitespace)
 		, line_(line)
 	{
@@ -305,104 +302,105 @@ public:
 
 	~CLine()
 	{
-		std::vector<CToken *>::iterator iter;
-		for (iter = m_Tokens.begin(); iter != m_Tokens.end(); ++iter)
-			delete *iter;
-		for (iter = m_LineEndTokens.begin(); iter != m_LineEndTokens.end(); ++iter)
-			delete *iter;
 	}
 
-	bool GetToken(unsigned int n, CToken &token, bool toEnd = false, bool include_whitespace = false)
+	CToken GetToken(unsigned int n)
 	{
-		if (!toEnd) {
-			if (m_Tokens.size() > n) {
-				token = *(m_Tokens[n]);
-				return true;
-			}
+		if (m_Tokens.size() > n) {
+			return m_Tokens[n];
+		}
 
-			size_t start = m_parsePos;
-			while (m_parsePos < line_.size()) {
-				if (line_[m_parsePos] == ' ' || line_[m_parsePos] == '\t') {
-					CToken *pToken = new CToken(line_.c_str() + start, m_parsePos - start);
-					m_Tokens.push_back(pToken);
+		size_t start = m_parsePos;
+		while (m_parsePos < line_.size()) {
+			if (line_[m_parsePos] == ' ' || line_[m_parsePos] == '\t') {
+				m_Tokens.emplace_back(line_.c_str() + start, m_parsePos - start);
 
-					while (m_parsePos < line_.size() && (line_[m_parsePos] == ' ' || line_[m_parsePos] == '\t'))
-						++m_parsePos;
-
-					if (m_Tokens.size() > n) {
-						token = *(m_Tokens[n]);
-						return true;
-					}
-
-					start = m_parsePos;
+				while (m_parsePos < line_.size() && (line_[m_parsePos] == ' ' || line_[m_parsePos] == '\t')) {
+					++m_parsePos;
 				}
-				++m_parsePos;
+
+				if (m_Tokens.size() > n) {
+					return m_Tokens[n];
+				}
+
+				start = m_parsePos;
 			}
-			if (m_parsePos != start) {
-				CToken *pToken = new CToken(line_.c_str() + start, m_parsePos - start);
-				m_Tokens.push_back(pToken);
+			++m_parsePos;
+		}
+		if (m_parsePos != start) {
+			m_Tokens.emplace_back(line_.c_str() + start, m_parsePos - start);
+		}
+
+		if (m_Tokens.size() > n) {
+			return m_Tokens[n];
+		}
+
+		return CToken();
+	}
+
+	CToken GetEndToken(unsigned int n, bool include_whitespace = false)
+	{
+		if (include_whitespace) {
+			int prev = n;
+			if (prev) {
+				--prev;
 			}
 
-			if (m_Tokens.size() > n) {
-				token = *(m_Tokens[n]);
-				return true;
+			CToken ref = GetToken(prev);
+			if (!ref) {
+				return ref;
+			}
+			wchar_t const* p = ref.GetToken() + ref.size() + 1;
+
+			if (static_cast<size_t>(p - line_.c_str()) >= line_.size()) {
+				return CToken();
 			}
 
-			return false;
+			auto newLen = line_.size() - (p - line_.c_str());
+			return CToken(p, newLen);
+		}
+
+		if (m_LineEndTokens.size() > n) {
+			return m_LineEndTokens[n];
+		}
+
+		if (m_Tokens.size() <= n) {
+			if (!GetToken(n)) {
+				return CToken();
+			}
+		}
+
+		if (trailing_whitespace_ == std::string::npos) {
+			trailing_whitespace_ = 0;
+			size_t i = line_.size() - 1;
+			while (i < line_.size() && (line_[i] == ' ' || line_[i] == '\t')) {
+				--i;
+				++trailing_whitespace_;
+			}
+		}
+
+		for (unsigned int i = static_cast<unsigned int>(m_LineEndTokens.size()); i <= n; ++i) {
+			CToken const& refToken = m_Tokens[i];
+			const wchar_t* p = refToken.GetToken();
+			if ((p - line_.c_str()) + trailing_whitespace_ >= line_.size()) {
+				return CToken();
+			}
+			auto newLen = line_.size() - (p - line_.c_str()) - trailing_whitespace_;
+			m_LineEndTokens.emplace_back(p, newLen);
+		}
+		return m_LineEndTokens[n];
+	}
+
+	bool GetToken(unsigned int n, CToken & token, bool to_end = false, bool include_whitespace = false)
+	{
+		if (to_end) {
+			token = GetEndToken(n, include_whitespace);
 		}
 		else {
-			if (include_whitespace) {
-				int prev = n;
-				if (prev)
-					--prev;
-
-				CToken ref;
-				if (!GetToken(prev, ref))
-					return false;
-				wchar_t const* p = ref.GetToken() + ref.GetLength() + 1;
-
-				auto const newLen = line_.size() - (p - line_.c_str());
-				if (newLen <= 0) {
-					return false;
-				}
-				token = CToken(p, newLen);
-				return true;
-			}
-
-			if (m_LineEndTokens.size() > n) {
-				token = *(m_LineEndTokens[n]);
-				return true;
-			}
-
-			if (m_Tokens.size() <= n) {
-				if (!GetToken(n, token)) {
-					return false;
-				}
-			}
-
-			if (trailing_whitespace_ == -1) {
-				trailing_whitespace_ = 0;
-				size_t i = line_.size() - 1;
-				while (i < line_.size() && (line_[i] == ' ' || line_[i] == '\t')) {
-					--i;
-					++trailing_whitespace_;
-				}
-			}
-
-			for (unsigned int i = static_cast<unsigned int>(m_LineEndTokens.size()); i <= n; ++i) {
-				const CToken *refToken = m_Tokens[i];
-				const wchar_t* p = refToken->GetToken();
-				auto const newLen = line_.size() - (p - line_.c_str()) - trailing_whitespace_;
-				if (newLen <= 0) {
-					return false;
-				}
-				CToken *pToken = new CToken(p, newLen);
-				m_LineEndTokens.push_back(pToken);
-			}
-			token = *(m_LineEndTokens[n]);
-			return true;
+			token = GetToken(n);
 		}
-	};
+		return token.operator bool();
+	}
 
 	CLine *Concat(CLine const* pLine) const
 	{
@@ -415,10 +413,10 @@ public:
 	}
 
 protected:
-	std::vector<CToken *> m_Tokens;
-	std::vector<CToken *> m_LineEndTokens;
+	std::vector<CToken> m_Tokens;
+	std::vector<CToken> m_LineEndTokens;
 	size_t m_parsePos{};
-	int trailing_whitespace_;
+	size_t trailing_whitespace_;
 	std::wstring const line_;
 };
 
@@ -617,14 +615,18 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 			// January could be 1 or 0, depends how the server counts
 			combo[fz::sprintf(L"%s%02d", iter->first, iter->second)] = iter->second;
 			combo[fz::sprintf(L"%s%02d", iter->first, iter->second - 1)] = iter->second;
-			if (iter->second < 10)
+			if (iter->second < 10) {
 				combo[fz::sprintf(L"%s%d", iter->first, iter->second)] = iter->second;
-			else
+			}
+			else {
 				combo[fz::sprintf(L"%s%d", iter->first, iter->second % 10)] = iter->second;
-			if (iter->second <= 10)
+			}
+			if (iter->second <= 10) {
 				combo[fz::sprintf(L"%s%d", iter->first, iter->second - 1)] = iter->second;
-			else
+			}
+			else {
 				combo[fz::sprintf(L"%s%d", iter->first, (iter->second - 1) % 10)] = iter->second;
+			}
 		}
 		m_MonthNamesMap.insert(combo.begin(), combo.end());
 
@@ -712,18 +714,19 @@ CDirectoryListing CDirectoryListingParser::Parse(const CServerPath &path)
 	}
 
 	if (!m_fileList.empty()) {
-		assert(m_entryList.empty());
+		assert(entries_.empty());
 
+		entries_.reserve(m_fileList.size());
 		for (auto const& file : m_fileList) {
 			CDirentry entry;
 			entry.name = file;
 			entry.flags = 0;
 			entry.size = -1;
-			m_entryList.emplace_back(std::move(entry));
+			entries_.emplace_back(std::move(entry));
 		}
 	}
 
-	listing.Assign(m_entryList);
+	listing.Assign(std::move(entries_));
 
 	return listing;
 }
@@ -738,67 +741,85 @@ bool CDirectoryListingParser::ParseLine(CLine &line, ServerType const serverType
 
 	if (serverType == ZVM) {
 		res = ParseAsZVM(line, entry);
-		if (res)
+		if (res) {
 			goto done;
+		}
 	}
 	else if (serverType == HPNONSTOP) {
 		res = ParseAsHPNonstop(line, entry);
-		if (res)
+		if (res) {
 			goto done;
+		}
 	}
 
 	ires = ParseAsMlsd(line, entry);
-	if (ires == 1)
+	if (ires == 1) {
 		goto done;
-	else if (ires == 2)
+	}
+	else if (ires == 2) {
 		goto skip;
+	}
 	res = ParseAsUnix(line, entry, true); // Common 'ls -l'
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseAsDos(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseAsEplf(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseAsVms(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseOther(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseAsIbm(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseAsWfFtp(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseAsIBM_MVS(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseAsIBM_MVS_PDS(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 	res = ParseAsOS9(line, entry);
-	if (res)
+	if (res) {
 		goto done;
+	}
 #ifndef LISTDEBUG_MVS
 	if (serverType == MVS)
 #endif //LISTDEBUG_MVS
 	{
 		res = ParseAsIBM_MVS_Migrated(line, entry);
-		if (res)
+		if (res) {
 			goto done;
+		}
 		res = ParseAsIBM_MVS_PDS2(line, entry);
-		if (res)
+		if (res) {
 			goto done;
+		}
 		res = ParseAsIBM_MVS_Tape(line, entry);
-		if (res)
+		if (res) {
 			goto done;
+		}
 	}
 	res = ParseAsUnix(line, entry, false); // 'ls -l' but without the date/time
-	if (res)
+	if (res) {
 		goto done;
+	}
 
 	// Some servers just send a list of filenames. If a line could not be parsed,
 	// check if it's a filename. If that's the case, store it for later, else clear
@@ -807,20 +828,22 @@ bool CDirectoryListingParser::ParseLine(CLine &line, ServerType const serverType
 	// contained a space, assume it's a raw filelisting.
 
 	if (!concatenated) {
-		CToken token;
-		if (!line.GetToken(0, token, true) || token.Find(' ') != -1) {
+		CToken token = line.GetEndToken(0);
+		if (!token || token.Find(' ') != -1) {
 			m_maybeMultilineVms = false;
 			m_fileList.clear();
 			m_fileListOnly = false;
 		}
 		else {
 			m_maybeMultilineVms = token.Find(';') != -1;
-			if (m_fileListOnly)
+			if (m_fileListOnly) {
 				m_fileList.emplace_back(token.GetString());
+			}
 		}
 	}
-	else
+	else {
 		m_maybeMultilineVms = false;
+	}
 
 	if (!override || override->name.empty()) {
 		return false;
@@ -842,8 +865,9 @@ done:
 	m_fileListOnly = false;
 
 	// Don't add . or ..
-	if (entry.name == L"." || entry.name == L"..")
+	if (entry.name == L"." || entry.name == L"..") {
 		return true;
+	}
 
 	if (serverType == VMS && entry.is_dir()) {
 		// Trim version information from directories
@@ -859,7 +883,7 @@ done:
 		}
 	}
 
-	m_entryList.emplace_back(std::move(refEntry));
+	entries_.emplace_back(std::move(refEntry));
 
 skip:
 	m_maybeMultilineVms = false;
@@ -872,11 +896,12 @@ skip:
 bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool expect_date)
 {
 	int index = 0;
-	CToken token;
-	if (!line.GetToken(index, token))
+	CToken permissionToken = line.GetToken(index);
+	if (!permissionToken) {
 		return false;
+	}
 
-	wchar_t chr = token[0];
+	wchar_t chr = permissionToken[0];
 	if (chr != 'b' &&
 		chr != 'c' &&
 		chr != 'd' &&
@@ -884,35 +909,44 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 		chr != 'p' &&
 		chr != 's' &&
 		chr != '-')
+	{
 		return false;
+	}
 
-	std::wstring permissions = token.GetString();
+	std::wstring permissions = permissionToken.GetString();
 
 	entry.flags = 0;
 
-	if (chr == 'd' || chr == 'l')
+	if (chr == 'd' || chr == 'l') {
 		entry.flags |= CDirentry::flag_dir;
+	}
 
-	if (chr == 'l')
+	if (chr == 'l') {
 		entry.flags |= CDirentry::flag_link;
+	}
 
 	// Check for netware servers, which split the permissions into two parts
 	bool netware = false;
-	if (token.GetLength() == 1) {
-		if (!line.GetToken(++index, token))
+	if (permissionToken.size() == 1) {
+		CToken cont_perm = line.GetToken(++index);
+		if (!cont_perm) {
 			return false;
-		permissions += L" " + token.GetString();
+		}
+		permissions += L" " + cont_perm.GetString();
 		netware = true;
 	}
 
 	int numOwnerGroup = 3;
 	if (!netware) {
 		// Filter out link count, we don't need it
-		if (!line.GetToken(++index, token))
+		CToken linkCount = line.GetToken(++index);
+		if (!linkCount) {
 			return false;
+		}
 
-		if (!token.IsNumeric())
+		if (!linkCount.IsNumeric()) {
 			--index;
+		}
 	}
 
 	// Repeat until numOwnerGroup is 0 since not all servers send every possible field
@@ -923,33 +957,41 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 
 		std::wstring ownerGroup;
 		for (int i = 0; i < numOwnerGroup; ++i) {
-			if (!line.GetToken(++index, token))
+			CToken ownerGroupToken = line.GetToken(++index);
+			if (!ownerGroupToken) {
 				return false;
-			if (i)
+			}
+			if (i) {
 				ownerGroup += L" ";
-			ownerGroup += token.GetString();
+			}
+			ownerGroup += ownerGroupToken.GetString();
 		}
 
-		if (!line.GetToken(++index, token))
+
+		CToken sizeToken = line.GetToken(++index);
+		if (!sizeToken) {
 			return false;
+		}
 
 		// Check for concatenated groupname and size fields
-		if (!ParseComplexFileSize(token, entry.size)) {
-			if (!token.IsRightNumeric())
+		if (!ParseComplexFileSize(sizeToken, entry.size)) {
+			if (!sizeToken.IsRightNumeric()) {
 				continue;
-			entry.size = token.GetNumber();
-		}
+			}
+			entry.size = sizeToken.GetNumber();
 
-		// Append missing group to ownerGroup
-		if (!token.IsNumeric() && token.IsRightNumeric()) {
-			if (!ownerGroup.empty())
+			// Append missing group to ownerGroup
+			if (!ownerGroup.empty()) {
 				ownerGroup += L" ";
+			}
 
-			std::wstring const group = token.GetString();
+			std::wstring const group = sizeToken.GetString();
 			int i;
 			for (i = group.size() - 1;
 				 i >= 0 && group[i] >= '0' && group[i] <= '9';
-				 --i) {}
+				 --i)
+			{
+			}
 
 			ownerGroup += group.substr(0, i + 1);
 		}
@@ -961,17 +1003,21 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 		}
 
 		// Get the filename
-		if (!line.GetToken(++index, token, 1))
+		CToken nameToken = line.GetEndToken(++index);
+		if (!nameToken) {
 			continue;
+		}
 
-		entry.name = token.GetString();
+		entry.name = nameToken.GetString();
 
 		// Filter out special chars at the end of the filenames
-		chr = token[token.GetLength() - 1];
+		chr = nameToken[nameToken.size() - 1];
 		if (chr == '/' ||
 			chr == '|' ||
 			chr == '*')
+		{
 			entry.name.pop_back();
+		}
 
 		if (entry.is_link()) {
 			size_t pos;
@@ -997,11 +1043,9 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 	bool mayHaveTime = true;
 	bool bHasYearAndTime = false;
 
-	CToken token;
-
 	// Get the month date field
-	CToken dateMonth;
-	if (!line.GetToken(++index, token)) {
+	CToken token = line.GetToken(++index);
+	if (!token) {
 		return false;
 	}
 
@@ -1011,6 +1055,8 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 	long hour = -1;
 	long minute = -1;
 
+	CToken dateMonth;
+
 	// Some servers use the following date formats:
 	// 26-05 2002, 2002-10-14, 01-jun-99 or 2004.07.15
 	// slashes instead of dashes are also possible
@@ -1020,7 +1066,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 		if (pos2 == -1) {
 			if (token[pos] != '.') {
 				// something like 26-05 2002
-				day = token.GetNumber(pos + 1, token.GetLength() - pos - 1);
+				day = token.GetNumber(pos + 1, token.size() - pos - 1);
 				if (day < 1 || day > 31) {
 					return false;
 				}
@@ -1055,7 +1101,8 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 			// 2) 2005 13 3
 			// assume first one.
 			year = token.GetNumber();
-			if (!line.GetToken(++index, dateMonth)) {
+			dateMonth = line.GetToken(++index);
+			if (!dateMonth) {
 				return false;
 			}
 			mayHaveTime = false;
@@ -1065,15 +1112,18 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 		}
 	}
 	else {
-		if (token.IsLeftNumeric() && (unsigned int)token[token.GetLength() - 1] > 127 &&
+		if (token.IsLeftNumeric() && (unsigned int)token[token.size() - 1] > 127 &&
 			token.GetNumber() > 1000)
 		{
-			if (token.GetNumber() > 10000)
+			if (token.GetNumber() > 10000) {
 				return false;
+			}
 
 			// Asian date format: 2005xxx 5xx 20xxx with some non-ascii characters following
 			year = token.GetNumber();
-			if (!line.GetToken(++index, dateMonth)) {
+
+			dateMonth = line.GetToken(++index);
+			if (!dateMonth) {
 				return false;
 			}
 			mayHaveTime = false;
@@ -1085,29 +1135,32 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 
 	if (day < 1) {
 		// Get day field
-		if (!line.GetToken(++index, token)) {
+		CToken dayToken = line.GetToken(++index);
+		if (!dayToken) {
 			return false;
 		}
 
 		int dateDay;
 
 		// Check for non-numeric day
-		if (!token.IsNumeric() && !token.IsLeftNumeric()) {
+		if (!dayToken.IsNumeric() && !dayToken.IsLeftNumeric()) {
 			int offset = 0;
-			if (dateMonth.GetString().back() == '.')
+			if (dateMonth.GetString().back() == '.') {
 				++offset;
-			if (!dateMonth.IsNumeric(0, dateMonth.GetLength() - offset))
+			}
+			if (!dateMonth.IsNumeric(0, dateMonth.size() - offset)) {
 				return false;
-			dateDay = dateMonth.GetNumber(0, dateMonth.GetLength() - offset);
-			dateMonth = token;
+			}
+			dateDay = dateMonth.GetNumber(0, dateMonth.size() - offset);
+			dateMonth = dayToken;
 		}
-		else if( token.GetLength() == 5 && token[2] == ':' && token.IsRightNumeric() ) {
+		else if (dayToken.size() == 5 && dayToken[2] == ':' && dayToken.IsRightNumeric() ) {
 			// This is a time. We consumed too much already.
 			return false;
 		}
 		else {
-			dateDay = token.GetNumber();
-			if (token[token.GetLength() - 1] == ',') {
+			dateDay = dayToken.GetNumber();
+			if (dayToken[dayToken.size() - 1] == ',') {
 				bHasYearAndTime = true;
 			}
 		}
@@ -1141,18 +1194,19 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 	}
 
 	// Get time/year field
-	if (!line.GetToken(++index, token)) {
+	CToken timeOrYearToken = line.GetToken(++index);
+	if (!timeOrYearToken) {
 		return false;
 	}
 
-	pos = token.Find(L":.-");
+	pos = timeOrYearToken.Find(L":.-");
 	if (pos != -1 && mayHaveTime) {
 		// token is a time
-		if (!pos || static_cast<size_t>(pos) == (token.GetLength() - 1)) {
+		if (!pos || static_cast<size_t>(pos) == (timeOrYearToken.size() - 1)) {
 			return false;
 		}
 
-		std::wstring str = token.GetString();
+		std::wstring str = timeOrYearToken.GetString();
 		hour = fz::to_integral<int>(str.substr(0, pos), -1);
 		minute = fz::to_integral<int>(str.substr(pos + 1), -1);
 
@@ -1187,11 +1241,11 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 	}
 	else if (year <= 0) {
 		// token is a year
-		if (!token.IsNumeric() && !token.IsLeftNumeric()) {
+		if (!timeOrYearToken.IsNumeric() && !timeOrYearToken.IsLeftNumeric()) {
 			return false;
 		}
 
-		year = token.GetNumber();
+		year = timeOrYearToken.GetNumber();
 
 		if (year > 3000) {
 			return false;
@@ -1201,18 +1255,19 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 		}
 
 		if (bHasYearAndTime) {
-			if (!line.GetToken(++index, token)) {
+			CToken timeToken = line.GetToken(++index);
+			if (!timeToken) {
 				return false;
 			}
 
-			if (token.Find(':') == 2 && token.GetLength() == 5 && token.IsLeftNumeric() && token.IsRightNumeric()) {
-				pos = token.Find(':');
+			if (timeToken.Find(':') == 2 && timeToken.size() == 5 && timeToken.IsLeftNumeric() && timeToken.IsRightNumeric()) {
+				pos = timeToken.Find(':');
 				// token is a time
-				if (!pos || static_cast<size_t>(pos) == (token.GetLength() - 1)) {
+				if (!pos || static_cast<size_t>(pos) == (timeToken.size() - 1)) {
 					return false;
 				}
 
-				std::wstring str = token.GetString();
+				std::wstring str = timeToken.GetString();
 
 				hour = fz::to_integral<int>(str.substr(0, pos), -1);
 				minute = fz::to_integral<int>(str.substr(pos + 1), -1);
@@ -1245,8 +1300,9 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 
 bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bool saneFieldOrder)
 {
-	if (token.GetLength() < 1)
+	if (token.size() < 1) {
 		return false;
+	}
 
 	bool gotYear = false;
 	bool gotMonth = false;
@@ -1258,52 +1314,60 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 	int day = 0;
 
 	int pos = token.Find(L"-./");
-	if (pos < 1)
+	if (pos < 1) {
 		return false;
+	}
 
 	if (!token.IsNumeric(0, pos)) {
 		// Seems to be monthname-dd-yy
 
 		// Check month name
 		std::wstring const dateMonth = token.GetString().substr(0, pos);
-		if (!GetMonthFromName(dateMonth, month))
+		if (!GetMonthFromName(dateMonth, month)) {
 			return false;
+		}
 		gotMonth = true;
 		gotMonthName = true;
 	}
 	else if (pos == 4) {
 		// Seems to be yyyy-mm-dd
 		year = token.GetNumber(0, pos);
-		if (year < 1900 || year > 3000)
+		if (year < 1900 || year > 3000) {
 			return false;
+		}
 		gotYear = true;
 	}
 	else if (pos <= 2) {
 		int64_t value = token.GetNumber(0, pos);
 		if (token[pos] == '.') {
 			// Maybe dd.mm.yyyy
-			if (value < 1 || value > 31)
+			if (value < 1 || value > 31) {
 				return false;
+			}
 			day = value;
 			gotDay = true;
 		}
 		else {
 			if (saneFieldOrder) {
 				year = value;
-				if (year < 50)
+				if (year < 50) {
 					year += 2000;
-				else
+				}
+				else {
 					year += 1900;
+				}
 				gotYear = true;
 			}
 			else {
 				// Detect mm-dd-yyyy or mm/dd/yyyy and
 				// dd-mm-yyyy or dd/mm/yyyy
-				if (value < 1)
+				if (value < 1) {
 					return false;
+				}
 				if (value > 12) {
-					if (value > 31)
+					if (value > 31) {
 						return false;
+					}
 
 					day = value;
 					gotDay = true;
@@ -1315,23 +1379,28 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 			}
 		}
 	}
-	else
+	else {
 		return false;
+	}
 
 	int pos2 = token.Find(L"-./", pos + 1);
-	if (pos2 == -1 || (pos2 - pos) == 1)
+	if (pos2 == -1 || (pos2 - pos) == 1) {
 		return false;
-	if (static_cast<size_t>(pos2) == (token.GetLength() - 1))
+	}
+	if (static_cast<size_t>(pos2) == (token.size() - 1)) {
 		return false;
+	}
 
 	// If we already got the month and the second field is not numeric,
 	// change old month into day and use new token as month
 	if (!token.IsNumeric(pos + 1, pos2 - pos - 1) && gotMonth) {
-		if (gotMonthName)
+		if (gotMonthName) {
 			return false;
+		}
 
-		if (gotDay)
+		if (gotDay) {
 			return false;
+		}
 
 		gotDay = true;
 		gotMonth = false;
@@ -1342,35 +1411,41 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 		// Month field in yyyy-mm-dd or dd-mm-yyyy
 		// Check month name
 		std::wstring dateMonth = token.GetString().substr(pos + 1, pos2 - pos - 1);
-		if (!GetMonthFromName(dateMonth, month))
+		if (!GetMonthFromName(dateMonth, month)) {
 			return false;
+		}
 		gotMonth = true;
 	}
 	else {
 		int64_t value = token.GetNumber(pos + 1, pos2 - pos - 1);
 		// Day field in mm-dd-yyyy
-		if (value < 1 || value > 31)
+		if (value < 1 || value > 31) {
 			return false;
+		}
 		day = value;
 		gotDay = true;
 	}
 
-	int64_t value = token.GetNumber(pos2 + 1, token.GetLength() - pos2 - 1);
+	int64_t value = token.GetNumber(pos2 + 1, token.size() - pos2 - 1);
 	if (gotYear) {
 		// Day field in yyy-mm-dd
-		if (value <= 0 || value > 31)
+		if (value <= 0 || value > 31) {
 			return false;
+		}
 		day = value;
 		gotDay = true;
 	}
 	else {
-		if (value < 0 || value > 9999)
+		if (value < 0 || value > 9999) {
 			return false;
+		}
 
-		if (value < 50)
+		if (value < 50) {
 			value += 2000;
-		else if (value < 1000)
+		}
+		else if (value < 1000) {
 			value += 1900;
+		}
 		year = value;
 
 		gotYear = true;
@@ -1424,7 +1499,7 @@ bool CDirectoryListingParser::ParseAsDos(CLine &line, CDirentry &entry)
 	else if (token.IsNumeric() || token.IsLeftNumeric()) {
 		// Convert size, filter out separators
 		int64_t size = 0;
-		int len = token.GetLength();
+		int len = token.size();
 		for (int i = 0; i < len; ++i) {
 			auto const chr = token[i];
 			if (chr == ',' || chr == '.') {
@@ -1463,7 +1538,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 		return false;
 
 	int pos = token.Find(':');
-	if (pos < 1 || static_cast<unsigned int>(pos) >= (token.GetLength() - 1))
+	if (pos < 1 || static_cast<unsigned int>(pos) >= (token.size() - 1))
 		return false;
 
 	int64_t hour = token.GetNumber(0, pos);
@@ -1495,7 +1570,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 
 	// Convert to 24h format
 	if (!token.IsRightNumeric()) {
-		if (token[token.GetLength() - 2] == 'P') {
+		if (token[token.size() - 2] == 'P') {
 			if (hour < 12)
 				hour += 12;
 		}
@@ -1517,7 +1592,7 @@ bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
 		return false;
 
 	int pos = token.Find('\t');
-	if (pos == -1 || static_cast<size_t>(pos) == (token.GetLength() - 1))
+	if (pos == -1 || static_cast<size_t>(pos) == (token.size() - 1))
 		return false;
 
 	entry.name = token.GetString().substr(pos + 1);
@@ -1626,7 +1701,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 	// This field can either be the filesize, a username (at least that's what I think) enclosed in [] or a date.
 	if (!token.IsNumeric() && !token.IsLeftNumeric()) {
 		// Must be username
-		const int len = token.GetLength();
+		const int len = token.size();
 		if (len < 3 || token[0] != '[' || token[len - 1] != ']')
 			return false;
 		ownerGroup = token.GetString().substr(1, len - 2);
@@ -1677,7 +1752,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 		return true;
 
 	if (!ParseTime(token, entry)) {
-		int len = token.GetLength();
+		int len = token.size();
 		if (token[0] == '[' && token[len - 1] != ']')
 			return false;
 		if (token[0] == '(' && token[len - 1] != ')')
@@ -1712,7 +1787,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 
 	// Owner / group and permissions
 	while (line.GetToken(++index, token)) {
-		const int len = token.GetLength();
+		const int len = token.size();
 		if (len > 2 && token[0] == '(' && token[len - 1] == ')') {
 			if (!permissions.empty())
 				permissions += L" ";
@@ -1777,7 +1852,7 @@ bool CDirectoryListingParser::ParseAsIbm(CLine &line, CDirentry &entry)
 		return false;
 
 	entry.name = token.GetString();
-	if (token[token.GetLength() - 1] == '/') {
+	if (token[token.size() - 1] == '/') {
 		entry.name.pop_back();
 		entry.flags |= CDirentry::flag_dir;
 	}
@@ -1815,7 +1890,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 	// If token is a number, than it's the numerical Unix style format,
 	// else it's the VShell, OS/2 or nortel.VxWorks format
 	if (token.IsNumeric()) {
-		if (firstToken.GetLength() >= 2 && firstToken[1] == '4') {
+		if (firstToken.size() >= 2 && firstToken[1] == '4') {
 			entry.flags |= CDirentry::flag_dir;
 		}
 
@@ -1971,7 +2046,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 			}
 
 			entry.name = token.GetString();
-			auto const chr = token[token.GetLength() - 1];
+			auto const chr = token[token.size() - 1];
 			if (chr == '/' || chr == '\\') {
 				entry.flags |= CDirentry::flag_dir;
 				entry.name.pop_back();
@@ -2243,7 +2318,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine &line, CDirentry &entry)
 	if (!token.IsNumeric())
 		return false;
 
-	int prevLen = token.GetLength();
+	int prevLen = token.size();
 
 	// used
 	if (!line.GetToken(index++, token))
@@ -2394,8 +2469,9 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS2(CLine &line, CDirentry &entry)
 {
 	int index = 0;
 	CToken token;
-	if (!line.GetToken(index, token))
+	if (!line.GetToken(index, token)) {
 		return false;
+	}
 
 	entry.name = token.GetString();
 
@@ -2404,53 +2480,64 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS2(CLine &line, CDirentry &entry)
 	entry.permissions = entry.ownerGroup;
 	entry.size = -1;
 
-	if (!line.GetToken(++index, token))
+	if (!line.GetToken(++index, token)) {
 		return true;
+	}
 
 	entry.size = token.GetNumber(CToken::hex);
-	if (entry.size == -1)
+	if (entry.size == -1) {
 		return false;
+	}
 
 	// Unused hexadecimal token
-	if (!line.GetToken(++index, token))
+	if (!line.GetToken(++index, token)) {
 		return false;
-	if (!token.IsNumeric(CToken::hex))
+	}
+	if (!token.IsNumeric(CToken::hex)) {
 		return false;
+	}
 
 	// Unused numeric token
-	if (!line.GetToken(++index, token))
+	if (!line.GetToken(++index, token)) {
 		return false;
-	if (!token.IsNumeric())
+	}
+	if (!token.IsNumeric()) {
 		return false;
+	}
 
 	int start = ++index;
 	while (line.GetToken(index, token)) {
 		++index;
 	}
-	if ((index - start < 2))
+	if ((index - start < 2)) {
 		return false;
+	}
 	--index;
 
 	if (!line.GetToken(index, token)) {
 		return false;
 	}
-	if (!token.IsNumeric() && (token.GetString() != L"ANY"))
+	if (!token.IsNumeric() && (token.GetString() != L"ANY")) {
 		return false;
+	}
 
 	if (!line.GetToken(index - 1, token)) {
 		return false;
 	}
-	if (!token.IsNumeric() && (token.GetString() != L"ANY"))
+	if (!token.IsNumeric() && (token.GetString() != L"ANY")) {
 		return false;
+	}
 
 	for (int i = start; i < index - 1; ++i) {
 		if (!line.GetToken(i, token)) {
 			return false;
 		}
-		int len = token.GetLength();
-		for (int j = 0; j < len; ++j)
-			if (token[j] < 'A' || token[j] > 'Z')
+		int len = token.size();
+		for (int j = 0; j < len; ++j) {
+			if (token[j] < 'A' || token[j] > 'Z') {
 				return false;
+			}
+		}
 	}
 
 	return true;
@@ -2462,20 +2549,24 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry)
 	CToken token;
 
 	// volume
-	if (!line.GetToken(index++, token))
+	if (!line.GetToken(index++, token)) {
 		return false;
+	}
 
 	// unit
-	if (!line.GetToken(index++, token))
+	if (!line.GetToken(index++, token)) {
 		return false;
+	}
 
 	std::wstring s = fz::str_tolower_ascii(token.GetString());
-	if (s != L"tape")
+	if (s != L"tape") {
 		return false;
+	}
 
 	// dsname
-	if (!line.GetToken(index++, token))
+	if (!line.GetToken(index++, token)) {
 		return false;
+	}
 
 	entry.name = token.GetString();
 	entry.flags = 0;
@@ -2483,8 +2574,9 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry)
 	entry.permissions = objcache.get(std::wstring());
 	entry.size = -1;
 
-	if (line.GetToken(index++, token))
+	if (line.GetToken(index++, token)) {
 		return false;
+	}
 
 	return true;
 }
@@ -2500,7 +2592,7 @@ bool CDirectoryListingParser::ParseComplexFileSize(CToken& token, int64_t& size,
 		return true;
 	}
 
-	int len = token.GetLength();
+	int len = token.size();
 
 	auto last = token[len - 1];
 	if (last == 'B' || last == 'b') {
@@ -2588,9 +2680,8 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 
 	// Parsing is done strict, abort on slightest error.
 
-	CToken token;
-
-	if (!line.GetToken(0, token)) {
+	CToken token = line.GetToken(0);
+	if (!token) {
 		return 0;
 	}
 
@@ -2736,11 +2827,12 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 		ownerGroup += L" " + gid;
 	}
 
-	if (!line.GetToken(1, token, true, true)) {
+	CToken nameToken = line.GetEndToken(1, true);
+	if (!nameToken) {
 		return 0;
 	}
 
-	entry.name = token.GetString();
+	entry.name = nameToken.GetString();
 	entry.ownerGroup = objcache.get(ownerGroup);
 	entry.permissions = objcache.get(permissions);
 
@@ -2758,13 +2850,13 @@ bool CDirectoryListingParser::ParseAsOS9(CLine &line, CDirentry &entry)
 
 	// Make sure it's number.number
 	int pos = ownerGroupToken.Find('.');
-	if (pos == -1 || !pos || pos == ((int)ownerGroupToken.GetLength() - 1))
+	if (pos == -1 || !pos || pos == ((int)ownerGroupToken.size() - 1))
 		return false;
 
 	if (!ownerGroupToken.IsNumeric(0, pos))
 		return false;
 
-	if (!ownerGroupToken.IsNumeric(pos + 1, ownerGroupToken.GetLength() - pos - 1))
+	if (!ownerGroupToken.IsNumeric(pos + 1, ownerGroupToken.size() - pos - 1))
 		return false;
 
 	entry.flags = 0;
@@ -2823,7 +2915,7 @@ void CDirectoryListingParser::Reset()
 	delete m_prevLine;
 	m_prevLine = nullptr;
 
-	m_entryList.clear();
+	entries_.clear();
 	m_fileList.clear();
 	m_currentOffset = 0;
 	m_fileListOnly = true;
@@ -2955,7 +3047,7 @@ bool CDirectoryListingParser::ParseAsHPNonstop(CLine &line, CDirentry &entry)
 		return false;
 	std::wstring ownerGroup = token.GetString();
 
-	if (token[token.GetLength() - 1] == ',') {
+	if (token[token.size() - 1] == ',') {
 		// Owner, part 2
 		if (!line.GetToken(++index, token))
 			return false;
@@ -2989,7 +3081,7 @@ bool CDirectoryListingParser::GetMonthFromName(const std::wstring& name, int &mo
 	return true;
 }
 
-char ebcdic_table[256] = {
+char const ebcdic_table[256] = {
 	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 0
 	' ',  ' ',  ' ',  ' ',  ' ',  '\n', ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '\n', // 1
 	' ',  ' ',  ' ',  ' ',  ' ',  '\n', ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 2
