@@ -3,8 +3,10 @@
 
 #include "directorycache.h"
 #include "logging_private.h"
+#include "oplock_manager.h"
 #include "pathcache.h"
 #include "ratelimiter.h"
+#include "tls_system_trust_store.h"
 
 #include <libfilezilla/event_loop.hpp>
 #include <libfilezilla/thread_pool.hpp>
@@ -48,6 +50,7 @@ public:
 	Impl(COptionsBase& options)
 		: limiter_(loop_, options)
 		, optionChangeHandler_(options, loop_)
+		, tlsSystemTrustStore_(pool_)
 	{
 		CLogging::UpdateLogLevel(options);
 
@@ -65,6 +68,8 @@ public:
 	CDirectoryCache directory_cache_;
 	CPathCache path_cache_;
 	CLoggingOptionsChanged optionChangeHandler_;
+	OpLockManager opLockManager_;
+	TlsSystemTrustStore tlsSystemTrustStore_;
 };
 
 CFileZillaEngineContext::CFileZillaEngineContext(COptionsBase & options, CustomEncodingConverterBase const& customEncodingConverter)
@@ -102,3 +107,13 @@ CPathCache& CFileZillaEngineContext::GetPathCache()
 {
 	return impl_->path_cache_;
 }
+
+OpLockManager& CFileZillaEngineContext::GetOpLockManager()
+{
+	return impl_->opLockManager_;
+}
+
+TlsSystemTrustStore& CFileZillaEngineContext::GetTlsSystemTrustStore()
+{
+	return impl_->tlsSystemTrustStore_;
+}
