@@ -8,16 +8,14 @@
 #include <libfilezilla/file.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
-CFtpFileTransferOpData::CFtpFileTransferOpData(CFtpControlSocket& controlSocket, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path)
-	: CFileTransferOpData(is_download, local_file, remote_file, remote_path)
+CFtpFileTransferOpData::CFtpFileTransferOpData(CFtpControlSocket& controlSocket, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings)
+	: CFileTransferOpData(L"CFtpFileTransferOpData", is_download, local_file, remote_file, remote_path, settings)
 	, CFtpOpData(controlSocket)
 {
 }
 
 int CFtpFileTransferOpData::Send()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CFtpFileTransferOpData::Send() in state %d", opState);
-
 	std::wstring cmd;
 	switch (opState)
 	{
@@ -128,7 +126,10 @@ int CFtpFileTransferOpData::Send()
 									LogMessage(MessageType::Debug_Warning, L"Could not preallocate the file");
 								}
 							}
-							pFile->seek(oldPos, fz::file::begin);
+							if (pFile->seek(oldPos, fz::file::begin) != oldPos) {
+								LogMessage(MessageType::Error, _("Could not seek to offset %d within file"), oldPos);
+								return FZ_REPLY_ERROR;
+							}
 						}
 					}
 				}
@@ -167,10 +168,8 @@ int CFtpFileTransferOpData::Send()
 							return FZ_REPLY_OK;
 						}
 
-						// Assume native 64 bit type exists
 						if (pFile->seek(startOffset, fz::file::begin) == -1) {
-							std::wstring const s = std::to_wstring(startOffset);
-							LogMessage(MessageType::Error, _("Could not seek to offset %s within file"), s);
+							LogMessage(MessageType::Error, _("Could not seek to offset %d within file"), startOffset);
 							return FZ_REPLY_ERROR;
 						}
 					}
@@ -300,8 +299,6 @@ int CFtpFileTransferOpData::TestResumeCapability()
 
 int CFtpFileTransferOpData::ParseResponse()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CFtpFileTransferOpData::ParseResponse() in state %d", opState);
-
 	int code = controlSocket_.GetReplyCode();
 	auto const& response = controlSocket_.m_Response;
 
@@ -379,8 +376,6 @@ int CFtpFileTransferOpData::ParseResponse()
 
 int CFtpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CFtpFileTransferOpData::SubcommandResult() in state %d", opState);
-
 	if (opState == filetransfer_waitcwd) {
 		if (prevResult == FZ_REPLY_OK) {
 			CDirentry entry;
