@@ -20,6 +20,7 @@
 #include "rename.h"
 #include "rmd.h"
 #include "servercapabilities.h"
+#include "socket_errors.h"
 #include "tlssocket.h"
 #include "transfersocket.h"
 
@@ -61,7 +62,7 @@ void CFtpControlSocket::OnReceive()
 
 		if (read < 0) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Error, _("Could not read from socket: %s"), fz::socket::error_description(error));
+				LogMessage(MessageType::Error, _("Could not read from socket: %s"), fz::socket_error_description(error));
 				if (GetCurrentCommandId() != Command::connect) {
 					LogMessage(MessageType::Error, _("Disconnected from server"));
 				}
@@ -259,7 +260,9 @@ void CFtpControlSocket::ParseResponse()
 		return;
 	}
 
-	int res = operations_.back()->ParseResponse();
+	auto & data = *operations_.back();
+	LogMessage(MessageType::Debug_Verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
+	int res = data.ParseResponse();
 	if (res == FZ_REPLY_OK) {
 		ResetOperation(FZ_REPLY_OK);
 	}
@@ -338,7 +341,7 @@ void CFtpControlSocket::List(CServerPath const& path, std::wstring const& subDir
 		LogMessage(MessageType::Status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
 	}
 
-	Push(std::make_unique<CFtpListOpData>(*this, path, subDir, flags, operations_.empty()));
+	Push(std::make_unique<CFtpListOpData>(*this, path, subDir, flags));
 }
 
 int CFtpControlSocket::ResetOperation(int nErrorCode)
@@ -441,8 +444,7 @@ void CFtpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 {
 	LogMessage(MessageType::Debug_Verbose, L"CFtpControlSocket::FileTransfer()");
 
-	auto pData = std::make_unique<CFtpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath);
-	pData->transferSettings_ = transferSettings;
+	auto pData = std::make_unique<CFtpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, transferSettings);
 	pData->binary = transferSettings.binary;
 	Push(std::move(pData));
 }
@@ -554,6 +556,18 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 			}
 		}
 		break;
+	case reqId_insecure_ftp:
+		{
+			auto & notification = static_cast<CInsecureFTPNotification&>(*pNotification);
+			if (!notification.allow_) {
+				ResetOperation(FZ_REPLY_CANCELED);
+				return false;
+			}
+			else {
+				SendNextCommand();
+				return true;
+			}
+		}
 	default:
 		LogMessage(MessageType::Debug_Warning, L"Unknown request %d", pNotification->GetRequestID());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
