@@ -4,10 +4,11 @@
 #include "ftp/ftpcontrolsocket.h"
 #include "iothread.h"
 #include "optionsbase.h"
-#include "tlssocket.h"
-#include "transfersocket.h"
 #include "proxy.h"
 #include "servercapabilities.h"
+#include "socket_errors.h"
+#include "tlssocket.h"
+#include "transfersocket.h"
 
 #include <libfilezilla/util.hpp>
 
@@ -67,7 +68,7 @@ std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
 	if (port == -1)	{
 		ResetSocket();
 
-		controlSocket_.LogMessage(MessageType::Debug_Warning, L"GetLocalPort failed: %s", fz::socket::error_description(error));
+		controlSocket_.LogMessage(MessageType::Debug_Warning, L"GetLocalPort failed: %s", fz::socket_error_description(error));
 		return std::wstring();
 	}
 
@@ -94,13 +95,17 @@ std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
 
 void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_flag t, int error)
 {
+	if (m_transferEndReason != TransferEndReason::none) {
+		return;
+	}
+
 	if (m_pProxyBackend) {
 		switch (t)
 		{
 		case fz::socket_event_flag::connection:
 			{
 				if (error) {
-					controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), fz::socket::error_description(error));
+					controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), fz::socket_error_description(error));
 					TransferEnd(TransferEndReason::failure);
 				}
 				else {
@@ -110,12 +115,6 @@ void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_f
 				}
 			}
 			return;
-		case fz::socket_event_flag::close:
-			{
-				controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), fz::socket::error_description(error));
-				TransferEnd(TransferEndReason::failure);
-			}
-			return;
 		default:
 			// Uninteresting
 			break;
@@ -137,22 +136,28 @@ void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_f
 	{
 	case fz::socket_event_flag::connection:
 		if (error) {
-			if (m_transferEndReason == TransferEndReason::none) {
-				controlSocket_.LogMessage(MessageType::Error, _("The data connection could not be established: %s"), fz::socket::error_description(error));
-				TransferEnd(TransferEndReason::transfer_failure);
-			}
+			controlSocket_.LogMessage(MessageType::Error, _("The data connection could not be established: %s"), fz::socket_error_description(error));
+			TransferEnd(TransferEndReason::transfer_failure);
 		}
-		else
+		else {
 			OnConnect();
+		}
 		break;
 	case fz::socket_event_flag::read:
-		OnReceive();
+		if (error) {
+			OnSocketError(error);
+		}
+		else {
+			OnReceive();
+		}
 		break;
 	case fz::socket_event_flag::write:
-		OnSend();
-		break;
-	case fz::socket_event_flag::close:
-		OnClose(error);
+		if (error) {
+			OnSocketError(error);
+		}
+		else {
+			OnSend();
+		}
 		break;
 	default:
 		// Uninteresting
@@ -176,7 +181,7 @@ void CTransferSocket::OnAccept(int error)
 			controlSocket_.LogMessage(MessageType::Debug_Verbose, L"No pending connection");
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Status, _("Could not accept connection: %s"), fz::socket::error_description(error));
+			controlSocket_.LogMessage(MessageType::Status, _("Could not accept connection: %s"), fz::socket_error_description(error));
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 		return;
@@ -239,12 +244,9 @@ void CTransferSocket::OnReceive()
 			if (numread < 0) {
 				delete [] pBuffer;
 				if (error != EAGAIN) {
-					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket::error_description(error));
+					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
-				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
-					TransferEnd(TransferEndReason::successful);
-				}
 				return;
 			}
 
@@ -298,12 +300,9 @@ void CTransferSocket::OnReceive()
 
 		if (numread < 0) {
 			if (error != EAGAIN) {
-				controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket::error_description(error));
+				controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
 				TransferEnd(TransferEndReason::transfer_failure);
 			}
-			else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
-				FinalizeWrite();
-			}
 		}
 		else if (!numread) {
 			FinalizeWrite();
@@ -319,18 +318,9 @@ void CTransferSocket::OnReceive()
 			int numread = m_pBackend->Read(buffer, 2, error);
 			if (numread < 0) {
 				if (error != EAGAIN) {
-					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket::error_description(error));
+					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
-				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
-					if (m_transferBufferLen == 1) {
-						TransferEnd(TransferEndReason::successful);
-					}
-					else {
-						controlSocket_.LogMessage(MessageType::Debug_Warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
-						TransferEnd(TransferEndReason::failed_resumetest);
-					}
-				}
 				return;
 			}
 
@@ -409,7 +399,7 @@ void CTransferSocket::OnSend()
 			}
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Error, L"Could not write to transfer socket: %s", fz::socket::error_description(error));
+			controlSocket_.LogMessage(MessageType::Error, L"Could not write to transfer socket: %s", fz::socket_error_description(error));
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 	}
@@ -418,77 +408,16 @@ void CTransferSocket::OnSend()
 	}
 }
 
-void CTransferSocket::OnClose(int error)
+void CTransferSocket::OnSocketError(int error)
 {
-	controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::OnClose(%d)", error);
-	m_onCloseCalled = true;
+	controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::OnSocketError(%d)", error);
 
 	if (m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 
-	if (!m_pBackend) {
-		if (!InitBackend()) {
-			TransferEnd(TransferEndReason::transfer_failure);
-			return;
-		}
-	}
-
-	if (m_transferMode == TransferMode::upload) {
-		if (m_shutdown && m_pTlsSocket) {
-			if (m_pTlsSocket->Shutdown() != 0) {
-				TransferEnd(TransferEndReason::transfer_failure);
-			}
-			else {
-				TransferEnd(TransferEndReason::successful);
-			}
-		}
-		else {
-			TransferEnd(TransferEndReason::transfer_failure);
-		}
-		return;
-	}
-
-	if (error) {
-		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket::error_description(error));
-		TransferEnd(TransferEndReason::transfer_failure);
-		return;
-	}
-
-	char buffer[100];
-	int numread = m_pBackend->Peek(&buffer, 100, error);
-	if (numread > 0) {
-#ifndef FZ_WINDOWS
-		controlSocket_.LogMessage(MessageType::Debug_Warning, L"Peek isn't supposed to return data after close notification");
-#endif
-
-		// MSDN says this:
-		//   FD_CLOSE being posted after all data is read from a socket.
-		//   An application should check for remaining data upon receipt
-		//   of FD_CLOSE to avoid any possibility of losing data.
-		// First half is actually plain wrong.
-		OnReceive();
-
-		return;
-	}
-	else if (numread < 0 && error != EAGAIN) {
-		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket::error_description(error));
-		TransferEnd(TransferEndReason::transfer_failure);
-		return;
-	}
-
-	if (m_transferMode == TransferMode::resumetest) {
-		if (m_transferBufferLen != 1) {
-			TransferEnd(TransferEndReason::failed_resumetest);
-			return;
-		}
-	}
-	if (m_transferMode == TransferMode::download) {
-		FinalizeWrite();
-	}
-	else {
-		TransferEnd(TransferEndReason::successful);
-	}
+	controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket_error_description(error));
+	TransferEnd(TransferEndReason::transfer_failure);
 }
 
 bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
@@ -573,7 +502,7 @@ void CTransferSocket::SetActive()
 		return;
 	}
 
-	if (socket_->get_state() == fz::socket::connected || socket_->get_state() == fz::socket::closing) {
+	if (socket_->get_state() == fz::socket::connected) {
 		TriggerPostponedEvents();
 	}
 }
@@ -592,12 +521,12 @@ void CTransferSocket::TransferEnd(TransferEndReason reason)
 	controlSocket_.send_event<TransferEndEvent>();
 }
 
-std::unique_ptr<fz::socket> CTransferSocket::CreateSocketServer(int port)
+std::unique_ptr<fz::listen_socket> CTransferSocket::CreateSocketServer(int port)
 {
-	auto socket = std::make_unique<fz::socket>(engine_.GetThreadPool(), this);
+	auto socket = std::make_unique<fz::listen_socket>(engine_.GetThreadPool(), this);
 	int res = socket->listen(controlSocket_.socket_->address_family(), port);
 	if (res) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Could not listen on port %d: %s", port, fz::socket::error_description(res));
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Could not listen on port %d: %s", port, fz::socket_error_description(res));
 		socket.reset();
 	}
 	else {
@@ -607,7 +536,7 @@ std::unique_ptr<fz::socket> CTransferSocket::CreateSocketServer(int port)
 	return socket;
 }
 
-std::unique_ptr<fz::socket> CTransferSocket::CreateSocketServer()
+std::unique_ptr<fz::listen_socket> CTransferSocket::CreateSocketServer()
 {
 	if (!engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS)) {
 		// Ask the systen for a port
@@ -637,7 +566,7 @@ std::unique_ptr<fz::socket> CTransferSocket::CreateSocketServer()
 		assert(start >= low && start <= high);
 	}
 
-	std::unique_ptr<fz::socket> server;
+	std::unique_ptr<fz::listen_socket> server;
 
 	int count = high - low + 1;
 	while (count--) {
@@ -693,8 +622,6 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 		}
 		else if (res == IO_Success) {
 			if (m_pTlsSocket) {
-				m_shutdown = true;
-
 				int error = m_pTlsSocket->Shutdown();
 				if (error != 0) {
 					if (error != EAGAIN) {
@@ -798,9 +725,6 @@ void CTransferSocket::TriggerPostponedEvents()
 			return;
 		}
 	}
-	if (m_onCloseCalled) {
-		OnClose(0);
-	}
 }
 
 bool CTransferSocket::InitBackend()
@@ -826,7 +750,7 @@ bool CTransferSocket::InitBackend()
 	return true;
 }
 
-void CTransferSocket::SetSocketBufferSizes(fz::socket& socket)
+void CTransferSocket::SetSocketBufferSizes(fz::socket_base& socket)
 {
 	const int size_read = engine_.GetOptions().GetOptionVal(OPTION_SOCKET_BUFFERSIZE_RECV);
 #if FZ_WINDOWS
