@@ -4,30 +4,17 @@
 #include <libfilezilla/format.hpp>
 #include <libfilezilla/time.hpp>
 
-#include <wx/utils.h>
-
-#include <gnutls/gnutls.h>
-#include <sqlite3.h>
 #include <random>
 #include <cstdint>
 #include <cwctype>
 
+#include <string.h>
+
 std::wstring GetDependencyVersion(lib_dependency d)
 {
 	switch (d) {
-	case lib_dependency::wxwidgets:
-		return wxVERSION_NUM_DOT_STRING_T;
 	case lib_dependency::gnutls:
-		{
-			const char* v = gnutls_check_version(0);
-			if (!v || !*v) {
-				return L"unknown";
-			}
-
-			return fz::to_wstring(v);
-		}
-	case lib_dependency::sqlite:
-		return fz::to_wstring_from_utf8(sqlite3_libversion());
+		return CTlsSocket::GetGnutlsVersion();
 	default:
 		return std::wstring();
 	}
@@ -36,12 +23,8 @@ std::wstring GetDependencyVersion(lib_dependency d)
 std::wstring GetDependencyName(lib_dependency d)
 {
 	switch (d) {
-	case lib_dependency::wxwidgets:
-		return L"wxWidgets";
 	case lib_dependency::gnutls:
 		return L"GnuTLS";
-	case lib_dependency::sqlite:
-		return L"SQLite";
 	default:
 		return std::wstring();
 	}
@@ -52,43 +35,6 @@ std::string ListTlsCiphers(std::string const& priority)
 	return CTlsSocket::ListTlsCiphers(priority);
 }
 
-#ifdef FZ_WINDOWS
-bool IsAtLeast(int major, int minor = 0)
-{
-	OSVERSIONINFOEX vi = { 0 };
-	vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
-	vi.dwMajorVersion = major;
-	vi.dwMinorVersion = minor;
-	vi.dwPlatformId = VER_PLATFORM_WIN32_NT;
-
-	DWORDLONG mask = 0;
-	VER_SET_CONDITION(mask, VER_MAJORVERSION, VER_GREATER_EQUAL);
-	VER_SET_CONDITION(mask, VER_MINORVERSION, VER_GREATER_EQUAL);
-	VER_SET_CONDITION(mask, VER_PLATFORMID, VER_EQUAL);
-	return VerifyVersionInfo(&vi, VER_MAJORVERSION | VER_MINORVERSION | VER_PLATFORMID, mask) != 0;
-}
-#endif
-
-bool GetRealOsVersion(int& major, int& minor)
-{
-#ifndef FZ_WINDOWS
-	return wxGetOsVersion(&major, &minor) != wxOS_UNKNOWN;
-#else
-	major = 4;
-	minor = 0;
-	while (IsAtLeast(++major, minor))
-	{
-	}
-	--major;
-	while (IsAtLeast(major, ++minor))
-	{
-	}
-	--minor;
-
-	return true;
-#endif
-}
-
 #if FZ_WINDOWS
 DWORD GetSystemErrorCode()
 {
@@ -98,7 +44,7 @@ DWORD GetSystemErrorCode()
 std::wstring GetSystemErrorDescription(DWORD err)
 {
 	wchar_t* buf{};
-	if (!FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 0, err, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), reinterpret_cast<wchar_t*>(&buf), 0, 0) || !buf) {
+	if (!FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, nullptr, err, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), reinterpret_cast<wchar_t*>(&buf), 0, nullptr) || !buf) {
 		return fz::sprintf(_("Unknown error %u"), err);
 	}
 	std::wstring ret = buf;
@@ -188,6 +134,7 @@ std::wstring translate(char const * const singular, char const * const plural, i
 std::wstring str_tolower(std::wstring const& source)
 {
 	std::wstring ret;
+	ret.reserve(source.size());
 	for (auto const& c : source) {
 		ret.push_back(std::towlower(c));
 	}
