@@ -1,7 +1,7 @@
 #include <filezilla.h>
 
-CDirectoryListingNotification::CDirectoryListingNotification(const CServerPath& path, const bool modified /*=false*/, const bool failed /*=false*/)
-	: m_modified(modified), m_failed(failed), m_path(path)
+CDirectoryListingNotification::CDirectoryListingNotification(CServerPath const& path, bool const primary, bool const failed)
+	: primary_(primary), m_failed(failed), m_path(path)
 {
 }
 
@@ -60,7 +60,7 @@ int CHostKeyNotification::GetPort() const
 	return m_port;
 }
 
-CDataNotification::CDataNotification(char* pData, int len)
+CDataNotification::CDataNotification(char* pData, size_t len)
 	: m_pData(pData), m_len(len)
 {
 }
@@ -70,7 +70,7 @@ CDataNotification::~CDataNotification()
 	delete [] m_pData;
 }
 
-char* CDataNotification::Detach(int& len)
+char* CDataNotification::Detach(size_t& len)
 {
 	len = m_len;
 	char* pData = m_pData;
@@ -88,7 +88,7 @@ CCertificate::CCertificate(
 		std::wstring const& fingerprint_sha1,
 		std::wstring const& issuer,
 		std::wstring const& subject,
-		std::vector<std::wstring> const& altSubjectNames)
+		std::vector<SubjectName> const& altSubjectNames)
 	: m_activationTime(activationTime)
 	, m_expirationTime(expirationTime)
 	, m_rawData(rawData)
@@ -114,7 +114,7 @@ CCertificate::CCertificate(
 	std::wstring const& fingerprint_sha1,
 	std::wstring const& issuer,
 	std::wstring const& subject,
-	std::vector<std::wstring> && altSubjectNames)
+	std::vector<SubjectName> && altSubjectNames)
 	: m_activationTime(activationTime)
 	, m_expirationTime(expirationTime)
 	, m_rawData(rawData)
@@ -136,7 +136,9 @@ CCertificateNotification::CCertificateNotification(std::wstring const& host, uns
 		std::wstring const& sessionCipher,
 		std::wstring const& sessionMac,
 		int algorithmWarnings,
-		std::vector<CCertificate> && certificates)
+		std::vector<CCertificate> && certificates,
+		bool systemTrust,
+		bool hostnameMismatch)
 	: m_host(host)
 	, m_port(port)
 	, m_protocol(protocol)
@@ -145,5 +147,13 @@ CCertificateNotification::CCertificateNotification(std::wstring const& host, uns
 	, m_sessionMac(sessionMac)
 	, m_algorithmWarnings(algorithmWarnings)
 	, m_certificates(certificates)
+	, systemTrust_(systemTrust)
+	, hostnameMismatch_(hostnameMismatch)
+{
+}
+
+
+CInsecureFTPNotification::CInsecureFTPNotification(CServer const& server)
+	: server_(server)
 {
 }
