@@ -13,20 +13,29 @@ struct t_protocolInfo
 	unsigned int defaultPort;
 	bool const translateable;
 	char const* const name;
-	bool supportsPostlogin;
+	std::wstring const alternative_prefix;
 };
 
 static const t_protocolInfo protocolInfos[] = {
-	{ FTP,          L"ftp",   false, 21,  true,  fztranslate_mark("FTP - File Transfer Protocol with optional encryption"), true  },
-	{ SFTP,         L"sftp",  true,  22,  false, "SFTP - SSH File Transfer Protocol",                                       false },
-	{ HTTP,         L"http",  true,  80,  false, "HTTP - Hypertext Transfer Protocol",                                      false },
-	{ HTTPS,        L"https", true, 443,  true,  fztranslate_mark("HTTPS - HTTP over TLS"),                                 false },
-	{ FTPS,         L"ftps",  true, 990,  true,  fztranslate_mark("FTPS - FTP over implicit TLS"),                          true  },
-	{ FTPES,        L"ftpes", true,  21,  true,  fztranslate_mark("FTPES - FTP over explicit TLS"),                         true  },
-	{ INSECURE_FTP, L"ftp",   false, 21,  true,  fztranslate_mark("FTP - Insecure File Transfer Protocol"),                 true  },
-	{ S3,           L"s3",    true, 443,  false,  "S3 - Amazon Simple Storage Service",	                                false },
-	{ STORJ,        L"storj", true, 443,  true,  fztranslate_mark("Storj - Decentralized Cloud Storage"),                   false },
-	{ UNKNOWN,      L"",      false, 21,  false, "", false }
+	{ FTP,          L"ftp",      false, 21, true,  fztranslate_mark("FTP - File Transfer Protocol with optional encryption"), L"" },
+	{ SFTP,         L"sftp",     true,  22, false, "SFTP - SSH File Transfer Protocol",                                       L"" },
+	{ HTTP,         L"http",     true,  80, false, "HTTP - Hypertext Transfer Protocol",                                      L"" },
+	{ HTTPS,        L"https",    true, 443, true,  fztranslate_mark("HTTPS - HTTP over TLS"),                                 L"" },
+	{ FTPS,         L"ftps",     true, 990, true,  fztranslate_mark("FTPS - FTP over implicit TLS"),                          L"" },
+	{ FTPES,        L"ftpes",    true,  21, true,  fztranslate_mark("FTPES - FTP over explicit TLS"),                         L"" },
+	{ INSECURE_FTP, L"ftp",      false, 21, true,  fztranslate_mark("FTP - Insecure File Transfer Protocol"),                 L"" },
+	{ S3,           L"s3",       true, 443, false, "S3 - Amazon Simple Storage Service",                                      L"" },
+	{ STORJ,        L"storj",    true, 443, true,  fztranslate_mark("Storj - Decentralized Cloud Storage"),                   L"" },
+	{ WEBDAV,       L"webdav",   true, 443, true,  "WebDAV",                                                                  L"https" },
+	{ AZURE_FILE,   L"azfile",   true, 443, false, "Microsoft Azure File Storage Service",                                    L"https" },
+	{ AZURE_BLOB,   L"azblob",   true, 443, false, "Microsoft Azure Blob Storage Service",                                    L"https" },
+	{ SWIFT,        L"swift",    true, 443, false, "OpenStack Swift",                                                         L"https" },
+	{ GOOGLE_CLOUD, L"google",   true, 443, false, "Google Cloud Storage",                                                    L"https" },
+	{ GOOGLE_DRIVE, L"gdrive",   true, 443, false, "Google Drive",                                                            L"https" },
+	{ DROPBOX,      L"dropbox",  true, 443, false, "Dropbox",                                                                 L"https" },
+	{ ONEDRIVE,     L"onedrive", true, 443, false, "Microsoft OneDrive",                                                      L"https" },
+	{ B2,           L"b2",       true, 443, false, "Backblaze B2",                                                            L"https" },
+	{ UNKNOWN,      L"",         false, 21, false, "", L"" }
 };
 
 static std::vector<ServerProtocol> const defaultProtocols = {
@@ -105,6 +114,7 @@ CServer& CServer::operator=(const CServer &op)
 	m_postLoginCommands = op.m_postLoginCommands;
 	m_bypassProxy = op.m_bypassProxy;
 	m_name = op.m_name;
+	extraParameters_ = op.extraParameters_;
 
 	return *this;
 }
@@ -146,6 +156,9 @@ bool CServer::operator==(const CServer &op) const
 	if (m_bypassProxy != op.m_bypassProxy) {
 		return false;
 	}
+	if (extraParameters_ != op.extraParameters_) {
+		return false;
+	}
 
 	// Do not compare number of allowed multiple connections
 
@@ -226,6 +239,14 @@ bool CServer::operator<(const CServer &op) const
 		return false;
 	}
 
+	if (extraParameters_ < op.extraParameters_) {
+		return true;
+	}
+	else if (extraParameters_ > op.extraParameters_) {
+		return false;
+	}
+
+
 	// Do not compare number of allowed multiple connections
 
 	return false;
@@ -241,7 +262,12 @@ CServer::CServer(ServerProtocol protocol, ServerType type, std::wstring const& h
 	m_protocol = protocol;
 	m_type = type;
 	m_host = host;
-	m_port = port;
+	if (port) {
+		m_port = port;
+	}
+	else {
+		port = GetDefaultPort(protocol);
+	}
 }
 
 void CServer::SetType(ServerType type)
@@ -253,11 +279,18 @@ void CServer::SetProtocol(ServerProtocol serverProtocol)
 {
 	assert(serverProtocol != UNKNOWN);
 
-	if (!GetProtocolInfo(serverProtocol).supportsPostlogin) {
+	if (!ProtocolHasFeature(serverProtocol, ProtocolFeature::PostLoginCommands)) {
 		m_postLoginCommands.clear();
 	}
 
 	m_protocol = serverProtocol;
+
+	// Clear out parameters not supported by the current protocol
+	std::map<std::string, std::wstring> oldParams;
+	std::swap(extraParameters_, oldParams);
+	for (auto const& param : oldParams) {
+		SetExtraParameter(param.first, param.second);
+	}
 }
 
 bool CServer::SetHost(std::wstring const& host, unsigned int port)
@@ -349,7 +382,7 @@ std::wstring CServer::Format(ServerFormat formatType, Credentials const& credent
 	}
 
 	auto user = GetUser();
-	if (user != L"anonymous" || credentials.logonType_ != LogonType::anonymous) {
+	if (credentials.logonType_ != LogonType::anonymous) {
 		// For now, only escape if formatting for URL.
 		// Open question: Do we need some form of escapement for presentation within the GUI,
 		// that deals e.g. with whitespace but does not touch Unicode characters?
@@ -489,7 +522,7 @@ ServerProtocol CServer::GetProtocolFromName(std::wstring const& name)
 
 bool CServer::SetPostLoginCommands(const std::vector<std::wstring>& postLoginCommands)
 {
-	if (!SupportsPostLoginCommands(m_protocol)) {
+	if (!ProtocolHasFeature(m_protocol, ProtocolFeature::PostLoginCommands)) {
 		m_postLoginCommands.clear();
 		return false;
 	}
@@ -498,14 +531,17 @@ bool CServer::SetPostLoginCommands(const std::vector<std::wstring>& postLoginCom
 	return true;
 }
 
-bool CServer::SupportsPostLoginCommands(ServerProtocol const protocol)
+ServerProtocol CServer::GetProtocolFromPrefix(std::wstring const& prefix, ServerProtocol const hint)
 {
-	return protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP;
-}
+	std::wstring const lower = fz::str_tolower_ascii(prefix);
+
+	if (hint != UNKNOWN && !lower.empty()) {
+		auto const& info = GetProtocolInfo(hint);
+		if (info.prefix == lower || info.alternative_prefix == lower) {
+			return hint;
+		}
+	}
 
-ServerProtocol CServer::GetProtocolFromPrefix(std::wstring const& prefix)
-{
-	std::wstring lower = fz::str_tolower_ascii(prefix);
 	for (unsigned int i = 0; protocolInfos[i].protocol != UNKNOWN; ++i) {
 		if (protocolInfos[i].prefix == lower) {
 			return protocolInfos[i].protocol;
@@ -537,12 +573,57 @@ bool CServer::GetBypassProxy() const
 	return m_bypassProxy;
 }
 
-bool CServer::ProtocolHasDataTypeConcept(const ServerProtocol protocol)
+bool CServer::HasFeature(ProtocolFeature const feature) const
 {
-	if (protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP) {
-		return true;
-	}
+	return ProtocolHasFeature(m_protocol, feature);
+}
 
+bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature const feature)
+{
+	switch (feature) {
+	case ProtocolFeature::DataTypeConcept:
+	case ProtocolFeature::TransferMode:
+	case ProtocolFeature::EnterCommand:
+	case ProtocolFeature::PostLoginCommands:
+		if (protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::Charset:
+		if (protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP ||
+			protocol == STORJ || protocol == SFTP) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::PreserveTimestamp:
+	case ProtocolFeature::ServerType:
+		if (protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP ||
+			protocol == SFTP) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::DirectoryRename:
+		if (protocol != AZURE_FILE) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::S3Lifecycle:
+		if (protocol == S3) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::RecursiveDelete:
+		if (protocol == GOOGLE_DRIVE || protocol == DROPBOX || protocol == ONEDRIVE) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::ServerAssignedHome:
+		if (protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP ||
+			protocol == SFTP) {
+			return true;
+		}
+		break;
+	}
 	return false;
 }
 
@@ -564,6 +645,46 @@ ServerType CServer::GetServerTypeFromName(std::wstring const& name)
 	return DEFAULT;
 }
 
+void CServer::ClearExtraParameters()
+{
+	extraParameters_.clear();
+}
+
+std::wstring CServer::GetExtraParameter(std::string const& name) const
+{
+	auto it = extraParameters_.find(name);
+	if (it != extraParameters_.cend()) {
+		return it->second;
+	}
+	return std::wstring();
+}
+
+std::map<std::string, std::wstring> const& CServer::GetExtraParameters() const
+{
+	return extraParameters_;
+}
+
+void CServer::SetExtraParameter(std::string const& name, std::wstring const& value)
+{
+	if (value.empty()) {
+		extraParameters_.erase(name);
+	}
+	else {
+		bool found = false;
+		auto const& traits  = ExtraServerParameterTraits(m_protocol);
+		for (auto const& trait : traits) {
+			if (trait.section_ != ParameterSection::credentials && name == trait.name_) {
+				found = true;
+				break;
+			}
+		}
+
+		if (found) {
+			extraParameters_[name] = value;
+		}
+	}
+}
+
 LogonType GetLogonTypeFromName(std::wstring const& name)
 {
 	if (name == _("Normal")) {
@@ -617,9 +738,141 @@ void Credentials::SetPass(std::wstring const& password)
 std::wstring Credentials::GetPass() const
 {
 	if (logonType_ == LogonType::anonymous) {
-		return L"anonymous@example.com";
+		return L"";
 	}
 	else {
 		return password_;
 	}
 }
+
+void Credentials::ClearExtraParameters()
+{
+	extraParameters_.clear();
+}
+
+std::wstring Credentials::GetExtraParameter(std::string const& name) const
+{
+	auto it = extraParameters_.find(name);
+	if (it != extraParameters_.cend()) {
+		return it->second;
+	}
+	return std::wstring();
+}
+
+std::map<std::string, std::wstring> const& Credentials::GetExtraParameters() const
+{
+	return extraParameters_;
+}
+
+void Credentials::SetExtraParameter(ServerProtocol protocol, std::string const& name, std::wstring const& value)
+{
+	bool found = false;
+	auto const& traits = ExtraServerParameterTraits(protocol);
+	for (auto const& trait : traits) {
+		if (trait.section_ != ParameterSection::credentials && name == trait.name_) {
+			found = true;
+			break;
+		}
+	}
+
+	if (found) {
+		extraParameters_[name] = value;
+	}
+}
+
+
+std::vector<LogonType> GetSupportedLogonTypes(ServerProtocol protocol)
+{
+	switch (protocol) {
+	case FTP:
+	case HTTP:
+	case FTPS:
+	case FTPES:
+	case INSECURE_FTP:
+		return {LogonType::anonymous, LogonType::normal, LogonType::ask, LogonType::interactive, LogonType::account};
+	case SFTP:
+		return {LogonType::anonymous, LogonType::normal, LogonType::ask, LogonType::interactive, LogonType::key};
+	case S3:
+		return {LogonType::anonymous, LogonType::normal, LogonType::ask};
+	case STORJ:
+	case AZURE_FILE:
+	case AZURE_BLOB:
+	case SWIFT:
+	case B2:
+		return {LogonType::normal, LogonType::ask};
+	case WEBDAV:
+		return {LogonType::anonymous, LogonType::normal, LogonType::ask};
+	case GOOGLE_CLOUD:
+	case GOOGLE_DRIVE:
+	case DROPBOX:
+	case ONEDRIVE:
+		return {LogonType::interactive};
+	case HTTPS:
+	case UNKNOWN:
+		return {LogonType::anonymous};
+	}
+
+	return {LogonType::anonymous};
+}
+
+std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol protocol)
+{
+	switch (protocol) {
+	case GOOGLE_CLOUD:
+		{
+			static std::vector<ParameterTraits> ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"email", ParameterSection::user, ParameterTraits::optional, std::wstring(), std::wstring()});
+				return ret;
+			}();
+			return ret;
+		}
+	case SWIFT:
+		{
+			static std::vector<ParameterTraits> ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"identpath", ParameterSection::host, 0, std::wstring(), _("Path of identity service")});
+				ret.emplace_back(ParameterTraits{"identuser", ParameterSection::user, ParameterTraits::optional, std::wstring(), std::wstring()});
+				return ret;
+			}();
+			return ret;
+		}
+	default:
+		break;
+	}
+
+	static std::vector<ParameterTraits> empty;
+	return empty;
+}
+
+std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
+{
+	switch (protocol)
+	{
+	case AZURE_FILE:
+		return std::tuple<std::wstring, std::wstring>{L"file.core.windows.net", L""};
+	case AZURE_BLOB:
+		return std::tuple<std::wstring, std::wstring>{L"blob.core.windows.net", L""};
+	case GOOGLE_CLOUD:
+		return std::tuple<std::wstring, std::wstring>{L"storage.googleapis.com", L""};
+	case GOOGLE_DRIVE:
+		return std::tuple<std::wstring, std::wstring>{L"www.googleapis.com", L""};
+	case S3:
+		return std::tuple<std::wstring, std::wstring>{L"s3.amazonaws.com", L""};
+	case DROPBOX:
+		return std::tuple<std::wstring, std::wstring>{L"api.dropboxapi.com", L""};
+	case ONEDRIVE:
+		return std::tuple<std::wstring, std::wstring>{L"graph.microsoft.com", L""};
+	case B2:
+		return std::tuple<std::wstring, std::wstring>{L"api.backblazeb2.com", L""};
+	default:
+		break;
+	}
+
+	return std::tuple<std::wstring, std::wstring>{};
+}
+
+bool ProtocolHasUser(ServerProtocol protocol)
+{
+	return protocol != DROPBOX && protocol != ONEDRIVE;
+}
