@@ -9,8 +9,6 @@
 
 int CSftpConnectOpData::Send()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CSftpConnectOpData::Send() in state %d", opState);
-	
 	switch (opState)
 	{
 	case connect_init:
@@ -77,7 +75,10 @@ int CSftpConnectOpData::Send()
 	case connect_keys:
 		return controlSocket_.SendCommand(L"keyfile \"" + *(keyfile_++) + L"\"");
 	case connect_open:
-		return controlSocket_.SendCommand(fz::sprintf(L"open \"%s@%s\" %d", currentServer_.GetUser(), controlSocket_.ConvertDomainName(currentServer_.GetHost()), currentServer_.GetPort()));
+		{
+			std::wstring user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+			return controlSocket_.SendCommand(fz::sprintf(L"open \"%s@%s\" %d", user, controlSocket_.ConvertDomainName(currentServer_.GetHost()), currentServer_.GetPort()));
+		}
 	default:
 		LogMessage(MessageType::Debug_Warning, L"Unknown op state: %d", opState);
 		break;
@@ -88,8 +89,6 @@ int CSftpConnectOpData::Send()
 
 int CSftpConnectOpData::ParseResponse()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CSftpConnectOpData::ParseResponse() in state %d", opState);
-
 	if (controlSocket_.result_ != FZ_REPLY_OK) {
 		return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 	}
@@ -137,8 +136,6 @@ int CSftpConnectOpData::ParseResponse()
 
 int CSftpConnectOpData::Reset(int result)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CSftpConnectOpData::Reset(%d) in state %d", result, opState);
-
 	if (opState == connect_init && (result & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED) {
 		LogMessage(MessageType::Error, _("fzsftp could not be started"));
 	}
