@@ -14,12 +14,11 @@ enum mkdStates
 
 int CSftpMkdirOpData::Send()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CSftpMkdirOpData::Send() in state %d", opState);
-	
-	if (!holdsLock_) {
-		if (!controlSocket_.TryLock(locking_reason::mkdir, path_)) {
-			return FZ_REPLY_WOULDBLOCK;
-		}
+	if (!opLock_) {
+		opLock_ = controlSocket_.Lock(locking_reason::mkdir, path_);
+	}
+	if (opLock_.waiting()) {
+		return FZ_REPLY_WOULDBLOCK;
 	}
 
 	switch (opState)
@@ -71,8 +70,6 @@ int CSftpMkdirOpData::Send()
 
 int CSftpMkdirOpData::ParseResponse()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CSftpMkdirOpData::ParseResponse() in state %d", opState);
-	
 	bool successful = controlSocket_.result_ == FZ_REPLY_OK;
 	switch (opState)
 	{
@@ -99,7 +96,7 @@ int CSftpMkdirOpData::ParseResponse()
 				return FZ_REPLY_INTERNALERROR;
 			}
 			engine_.GetDirectoryCache().UpdateFile(currentServer_, currentMkdPath_, segments_.back(), true, CDirectoryCache::dir);
-			controlSocket_.SendDirectoryListingNotification(currentMkdPath_, false, false);
+			controlSocket_.SendDirectoryListingNotification(currentMkdPath_, false);
 
 			currentMkdPath_.AddSegment(segments_.back());
 			segments_.pop_back();
