@@ -6,8 +6,6 @@
 
 int CSftpRemoveDirOpData::Send()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CSftpRemoveDirOpData::Send() in state %d", opState);
-	
 	CServerPath fullPath = engine_.GetPathCache().Lookup(currentServer_, path_, subDir_);
 	if (fullPath.empty()) {
 		fullPath = path_;
@@ -29,8 +27,6 @@ int CSftpRemoveDirOpData::Send()
 
 int CSftpRemoveDirOpData::ParseResponse()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CSftpRemoveDirOpData::ParseResponse() in state %d", opState);
-
 	if (controlSocket_.result_ != FZ_REPLY_OK) {
 		return controlSocket_.result_;
 	}
@@ -41,7 +37,7 @@ int CSftpRemoveDirOpData::ParseResponse()
 	}
 
 	engine_.GetDirectoryCache().RemoveDir(currentServer_, path_, subDir_, engine_.GetPathCache().Lookup(currentServer_, path_, subDir_));
-	controlSocket_.SendDirectoryListingNotification(path_, false, false);
+	controlSocket_.SendDirectoryListingNotification(path_, false);
 
 	return FZ_REPLY_OK;
 }
\ No newline at end of file
