@@ -130,18 +130,6 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 	case sftpEvent::Send:
 		SetActive(CFileZillaEngine::send);
 		break;
-	case sftpEvent::Listentry:
-		if (operations_.empty() || operations_.back()->opId != Command::list) {
-			LogMessage(MessageType::Debug_Warning, L"sftpEvent::Listentry outside list operation, ignoring.");
-			break;
-		}
-		else {
-			int res = static_cast<CSftpListOpData&>(*operations_.back()).ParseEntry(std::move(message.text[0]), message.text[1], std::move(message.text[2]));
-			if (res != FZ_REPLY_WOULDBLOCK) {
-				ResetOperation(res);
-			}
-		}
-		break;
 	case sftpEvent::Transfer:
 		{
 			auto value = fz::to_integral<int64_t>(message.text[0]);
@@ -227,7 +215,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 					return;
 				}
 
-				std::wstring const pass = data.credentials_.GetPass();
+				std::wstring const pass = (data.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : data.credentials_.GetPass();
 				std::wstring show = L"Pass: ";
 				show.append(pass.size(), '*');
 				SendCommand(pass, show);
@@ -294,6 +282,28 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 	}
 }
 
+void CSftpControlSocket::OnSftpListEvent(sftp_list_message const& message)
+{
+	if (!currentServer_) {
+		return;
+	}
+
+	if (!input_thread_) {
+		return;
+	}
+
+	if (operations_.empty() || operations_.back()->opId != Command::list) {
+		LogMessage(MessageType::Debug_Warning, L"sftpEvent::Listentry outside list operation, ignoring.");
+		return;
+	}
+	else {
+		int res = static_cast<CSftpListOpData&>(*operations_.back()).ParseEntry(std::move(message.text), message.mtime, std::move(message.name));
+		if (res != FZ_REPLY_WOULDBLOCK) {
+			ResetOperation(res);
+		}
+	}
+}
+
 void CSftpControlSocket::OnTerminate(std::wstring const& error)
 {
 	if (!error.empty()) {
@@ -447,7 +457,7 @@ void CSftpControlSocket::List(CServerPath const& path, std::wstring const& subDi
 		LogMessage(MessageType::Status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
 	}
 
-	Push(std::make_unique<CSftpListOpData>(*this, path, subDir, flags, operations_.empty()));
+	Push(std::make_unique<CSftpListOpData>(*this, path, subDir, flags));
 }
 
 void CSftpControlSocket::ChangeDir(CServerPath const& path, std::wstring const& subDir, bool link_discovery)
@@ -478,6 +488,7 @@ void CSftpControlSocket::ProcessReply(int result, std::wstring const& reply)
 	}
 
 	auto & data = *operations_.back();
+	LogMessage(MessageType::Debug_Verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
 	int res = data.ParseResponse();
 	if (res == FZ_REPLY_OK) {
 		ResetOperation(FZ_REPLY_OK);
@@ -501,8 +512,7 @@ void CSftpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 									std::wstring const& remoteFile, bool download,
 									CFileTransferCommand::t_transferSettings const& transferSettings)
 {
-	auto pData = std::make_unique<CSftpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath);
-	pData->transferSettings_ = transferSettings;
+	auto pData = std::make_unique<CSftpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, transferSettings);
 	Push(std::move(pData));
 }
 
@@ -637,21 +647,22 @@ void CSftpControlSocket::OnQuotaRequest(CRateLimiter::rate_direction direction)
 		else {
 			b = static_cast<int>(bytes);
 		}
-		AddToStream(fz::sprintf(L"-%d%d,%d\n", direction, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(direction))));
+		AddToStream(fz::sprintf("-%d%d,%d\n", direction, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(direction))));
 		UpdateUsage(direction, b);
 	}
 	else if (bytes == 0) {
 		Wait(direction);
 	}
 	else if (bytes < 0) {
-		AddToStream(fz::sprintf(L"-%d-\n", direction));
+		AddToStream(fz::sprintf("-%d-\n", direction));
 	}
 }
 
 void CSftpControlSocket::operator()(fz::event_base const& ev)
 {
-	if (fz::dispatch<CSftpEvent, CTerminateEvent>(ev, this,
+	if (fz::dispatch<CSftpEvent, CSftpListEvent, CTerminateEvent>(ev, this,
 		&CSftpControlSocket::OnSftpEvent,
+		&CSftpControlSocket::OnSftpListEvent,
 		&CSftpControlSocket::OnTerminate)) {
 		return;
 	}
