@@ -41,8 +41,7 @@
 #define WAIT_READ	 0x02
 #define WAIT_WRITE	 0x04
 #define WAIT_ACCEPT  0x08
-#define WAIT_CLOSE	 0x10
-#define WAIT_EVENTCOUNT 5
+#define WAIT_EVENTCOUNT 4
 
 namespace fz {
 
@@ -188,11 +187,98 @@ int last_socket_error()
 #else
 inline int last_socket_error() { return errno; }
 #endif
+
+int set_nonblocking(int fd)
+{
+	// Set socket to non-blocking.
+#ifdef FZ_WINDOWS
+	unsigned long nonblock = 1;
+	int res = ioctlsocket(fd, FIONBIO, &nonblock);
+	if (!res) {
+		return 0;
+	}
+	else {
+		return convert_msw_error_code(WSAGetLastError());
+	}
+#else
+	int flags = fcntl(fd, F_GETFL);
+	if (flags == -1) {
+		return errno;
+	}
+	int res = fcntl(fd, F_SETFL, flags | O_NONBLOCK);
+	if (res == -1) {
+		return errno;
+	}
+	return 0;
+#endif
+}
+
+int do_set_flags(int fd, int flags, int flags_mask, duration const& keepalive_interval)
+{
+	if (flags_mask & socket::flag_nodelay) {
+		const int value = (flags & socket::flag_nodelay) ? 1 : 0;
+		int res = setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (const char*)&value, sizeof(value));
+		if (res != 0) {
+			return last_socket_error();
+		}
+	}
+	if (flags_mask & socket::flag_keepalive) {
+#if FZ_WINDOWS
+		tcp_keepalive v{};
+		v.onoff = (flags & socket::flag_keepalive) ? 1 : 0;
+		v.keepalivetime = static_cast<ULONG>(keepalive_interval.get_milliseconds());
+		v.keepaliveinterval = 1000;
+		DWORD tmp{};
+		int res = WSAIoctl(fd, SIO_KEEPALIVE_VALS, &v, sizeof(v), nullptr, 0, &tmp, nullptr, nullptr);
+		if (res != 0) {
+			return last_socket_error();
+		}
+#else
+		const int value = (flags & socket::flag_keepalive) ? 1 : 0;
+		int res = setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (const char*)&value, sizeof(value));
+		if (res != 0) {
+			return last_socket_error();
+		}
+#ifdef TCP_KEEPIDLE
+		int const idle = keepalive_interval.get_seconds();
+		res = setsockopt(fd, IPPROTO_TCP, TCP_KEEPINTVL, (const char*)&idle, sizeof(idle));
+		if (res != 0) {
+			return last_socket_error();
+		}
+#endif
+#endif
+	}
+
+	return 0;
+}
+
+int do_set_buffer_sizes(int fd, int size_read, int size_write)
+{
+	int ret = 0;
+	if (size_read != -1) {
+		int res = setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (const char*)&size_read, sizeof(size_read));
+		if (res != 0) {
+			ret = last_socket_error();
+		}
+	}
+
+	if (size_write != -1) {
+		int res = setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (const char*)&size_write, sizeof(size_write));
+		if (res != 0) {
+			ret = last_socket_error();
+		}
+	}
+
+	return ret;
+}
+
 }
 
 class socket_thread final
 {
+	friend class socket_base;
 	friend class socket;
+	friend class listen_socket;
 public:
 	socket_thread()
 		: mutex_(false)
@@ -225,13 +311,13 @@ public:
 #endif
 	}
 
-	void set_socket(socket* pSocket)
+	void set_socket(socket_base* pSocket)
 	{
 		scoped_lock l(mutex_);
 		set_socket(pSocket, l);
 	}
 
-	void set_socket(socket* pSocket, scoped_lock const&)
+	void set_socket(socket_base* pSocket, scoped_lock const&)
 	{
 		socket_ = pSocket;
 
@@ -241,14 +327,14 @@ public:
 		waiting_ = 0;
 	}
 
-	int connect(std::string const& bind)
+	int connect(std::string const& bind, std::string const& host, unsigned int port)
 	{
 		assert(socket_);
 		if (!socket_) {
 			return EINVAL;
 		}
 
-		host_ = to_utf8(socket_->host_);
+		host_ = host;
 		if (host_.empty()) {
 			return EINVAL;
 		}
@@ -256,7 +342,7 @@ public:
 		bind_ = bind;
 
 		// Connect method of socket ensures port is in range
-		port_ = sprintf("%u", socket_->port_);
+		port_ = sprintf("%u", port);
 
 		start();
 
@@ -342,7 +428,7 @@ protected:
 			const int value = 1;
 			setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &value, sizeof(int));
 #endif
-			socket::set_nonblocking(fd);
+			set_nonblocking(fd);
 		}
 
 		return fd;
@@ -379,8 +465,8 @@ protected:
 			(void)bind(fd, &bindAddr.sockaddr_, sizeof(bindAddr));
 		}
 
-		socket::do_set_flags(fd, socket_->flags_, socket_->flags_, socket_->keepalive_interval_);
-		socket::do_set_buffer_sizes(fd, socket_->buffer_sizes_[0], socket_->buffer_sizes_[1]);
+		do_set_flags(fd, socket_->flags_, socket_->flags_, socket_->keepalive_interval_);
+		do_set_buffer_sizes(fd, socket_->buffer_sizes_[0], socket_->buffer_sizes_[1]);
 
 		int res = ::connect(fd, addr.ai_addr, addr.ai_addrlen);
 		if (res == -1) {
@@ -562,12 +648,12 @@ protected:
 
 		for (;;) {
 #ifdef FZ_WINDOWS
-			int wait_events = FD_CLOSE;
+			int wait_events{};
 			if (waiting_ & WAIT_CONNECT) {
 				wait_events |= FD_CONNECT;
 			}
 			if (waiting_ & WAIT_READ) {
-				wait_events |= FD_READ;
+				wait_events |= FD_READ | FD_CLOSE;
 			}
 			if (waiting_ & WAIT_WRITE) {
 				wait_events |= FD_WRITE;
@@ -575,9 +661,6 @@ protected:
 			if (waiting_ & WAIT_ACCEPT) {
 				wait_events |= FD_ACCEPT;
 			}
-			if (waiting_ & WAIT_CLOSE) {
-				wait_events |= FD_CLOSE;
-			}
 			WSAEventSelect(socket_->fd_, sync_event_, wait_events);
 			l.unlock();
 			WSAWaitForMultipleEvents(1, &sync_event_, false, WSA_INFINITE, false);
@@ -607,6 +690,14 @@ protected:
 					triggered_errors_[1] = convert_msw_error_code(events.iErrorCode[FD_READ_BIT]);
 					waiting_ &= ~WAIT_READ;
 				}
+				if (events.lNetworkEvents & FD_CLOSE) {
+					triggered_ |= WAIT_READ;
+					int err = convert_msw_error_code(events.iErrorCode[FD_CLOSE_BIT]);
+					if (err) {
+						triggered_errors_[1] = err;
+					}
+					waiting_ &= ~WAIT_READ;
+				}
 			}
 			if (waiting_ & WAIT_WRITE) {
 				if (events.lNetworkEvents & FD_WRITE) {
@@ -622,13 +713,6 @@ protected:
 					waiting_ &= ~WAIT_ACCEPT;
 				}
 			}
-			if (waiting_ & WAIT_CLOSE) {
-				if (events.lNetworkEvents & FD_CLOSE) {
-					triggered_ |= WAIT_CLOSE;
-					triggered_errors_[4] = convert_msw_error_code(events.iErrorCode[FD_CLOSE_BIT]);
-					waiting_ &= ~WAIT_CLOSE;
-				}
-			}
 
 			if (triggered_ || !waiting_) {
 				return true;
@@ -735,36 +819,6 @@ protected:
 			socket_->evt_handler_->send_event<socket_event>(socket_, socket_event_flag::connection, triggered_errors_[3]);
 			triggered_ &= ~WAIT_ACCEPT;
 		}
-		if (triggered_ & WAIT_CLOSE) {
-			send_close_event();
-		}
-	}
-
-	void send_close_event()
-	{
-		if (!socket_ || !socket_->evt_handler_) {
-			return;
-		}
-
-#ifdef FZ_WINDOWS
-		// MSDN says this:
-		//   FD_CLOSE being posted after all data is read from a socket.
-		//   An application should check for remaining data upon receipt
-		//   of FD_CLOSE to avoid any possibility of losing data.
-		// First half is actually plain wrong.
-		char buf;
-		if (!triggered_errors_[4] && recv( socket_->fd_, &buf, 1, MSG_PEEK ) > 0) {
-			if (!(waiting_ & WAIT_READ)) {
-				return;
-			}
-			socket_->evt_handler_->send_event<socket_event>(socket_, socket_event_flag::read, 0);
-		}
-		else
-#endif
-		{
-			socket_->evt_handler_->send_event<socket_event>(socket_, socket_event_flag::close, triggered_errors_[4]);
-			triggered_ &= ~WAIT_CLOSE;
-		}
 	}
 
 	// Call only while locked
@@ -793,7 +847,7 @@ protected:
 				break;
 			}
 
-			if (socket_->state_ == socket::listening) {
+			if (dynamic_cast<listen_socket*>(socket_)) {
 				while (idle_loop(l)) {
 					if (socket_->fd_ == -1) {
 						waiting_ = 0;
@@ -812,10 +866,6 @@ protected:
 					}
 				}
 
-#ifdef FZ_WINDOWS
-				waiting_ |= WAIT_CLOSE;
-				int wait_close = WAIT_CLOSE;
-#endif
 				while (idle_loop(l)) {
 					if (socket_->fd_ == -1) {
 						waiting_ = 0;
@@ -823,21 +873,11 @@ protected:
 					}
 					bool res = do_wait(0, l);
 
-					if (triggered_ & WAIT_CLOSE && socket_) {
-						socket_->state_ = socket::closing;
-#ifdef FZ_WINDOWS
-						wait_close = 0;
-#endif
-					}
-
 					if (!res) {
 						break;
 					}
 
 					send_events();
-#ifdef FZ_WINDOWS
-					waiting_ |= wait_close;
-#endif
 				}
 			}
 		}
@@ -852,7 +892,7 @@ protected:
 		return;
 	}
 
-	socket* socket_{};
+	socket_base* socket_{};
 
 	std::string host_;
 	std::string port_;
@@ -886,7 +926,7 @@ protected:
 	async_task thread_;
 };
 
-socket::socket(thread_pool & pool, event_handler* evt_handler)
+socket_base::socket_base(thread_pool& pool, event_handler* evt_handler)
 	: thread_pool_(pool)
 	, evt_handler_(evt_handler)
 	, keepalive_interval_(duration::from_hours(2))
@@ -897,19 +937,7 @@ socket::socket(thread_pool & pool, event_handler* evt_handler)
 	buffer_sizes_[1] = -1;
 }
 
-socket::~socket()
-{
-	if (state_ != none) {
-		close();
-	}
-
-	if (socket_thread_) {
-		scoped_lock l(socket_thread_->mutex_);
-		detach_thread(l);
-	}
-}
-
-void socket::detach_thread(scoped_lock & l)
+void socket_base::detach_thread(scoped_lock & l)
 {
 	if (!socket_thread_) {
 		return;
@@ -938,67 +966,7 @@ void socket::detach_thread(scoped_lock & l)
 	}
 }
 
-int socket::connect(native_string const& host, unsigned int port, address_type family, std::string const& bind)
-{
-	if (state_ != none) {
-		return EISCONN;
-	}
-
-	if (port < 1 || port > 65535) {
-		return EINVAL;
-	}
-
-	if (host.empty()) {
-		return EINVAL;
-	}
-
-	int af{};
-
-	switch (family)
-	{
-	case address_type::unknown:
-		af = AF_UNSPEC;
-		break;
-	case address_type::ipv4:
-		af = AF_INET;
-		break;
-	case address_type::ipv6:
-		af = AF_INET6;
-		break;
-	default:
-		return EINVAL;
-	}
-
-	if (socket_thread_ && socket_thread_->started_) {
-		scoped_lock l(socket_thread_->mutex_);
-		if (!socket_thread_->threadwait_) {
-			// Possibly inside a blocking call, e.g. getaddrinfo.
-			// Detach the thread so that we can continue.
-			detach_thread(l);
-		}
-	}
-	if (!socket_thread_) {
-		socket_thread_ = new socket_thread();
-		socket_thread_->set_socket(this);
-	}
-
-	family_ = af;
-	state_ = connecting;
-
-	host_ = host;
-	port_ = port;
-	int res = socket_thread_->connect(bind);
-	if (res) {
-		state_ = none;
-		delete socket_thread_;
-		socket_thread_ = nullptr;
-		return res;
-	}
-
-	return EINPROGRESS;
-}
-
-void socket::set_event_handler(event_handler* pEvtHandler)
+void socket_base::set_event_handler(event_handler* pEvtHandler)
 {
 	if (socket_thread_) {
 		scoped_lock l(socket_thread_->mutex_);
@@ -1011,31 +979,13 @@ void socket::set_event_handler(event_handler* pEvtHandler)
 
 		evt_handler_ = pEvtHandler;
 
-		if (pEvtHandler && state_ == connected) {
-#ifdef FZ_WINDOWS
-			// If a graceful shutdown is going on in background already,
-			// no further events are recorded. Send out events we're not
-			// waiting for (i.e. they got triggered already) manually.
-
-			if (!(socket_thread_->waiting_ & WAIT_WRITE)) {
+		if (pEvtHandler && dynamic_cast<socket*>(this) && state_ == socket::connected) {
+			if (!(socket_thread_->waiting_ & WAIT_WRITE) && !has_pending_event(evt_handler_, this, socket_event_flag::write)) {
 				pEvtHandler->send_event<socket_event>(this, socket_event_flag::write, 0);
 			}
-
-			pEvtHandler->send_event<socket_event>(this, socket_event_flag::read, 0);
-			if (socket_thread_->waiting_ & WAIT_READ) {
-				socket_thread_->waiting_ &= ~WAIT_READ;
-				socket_thread_->wakeup_thread(l);
+			if (!(socket_thread_->waiting_ & WAIT_READ) && !has_pending_event(evt_handler_, this, socket_event_flag::read)) {
+				pEvtHandler->send_event<socket_event>(this, socket_event_flag::read, 0);
 			}
-#else
-			socket_thread_->waiting_ |= WAIT_READ | WAIT_WRITE;
-			socket_thread_->wakeup_thread(l);
-#endif
-		}
-		else if (pEvtHandler && state_ == closing) {
-			if (!(socket_thread_->triggered_ & WAIT_READ)) {
-				socket_thread_->waiting_ |= WAIT_READ;
-			}
-			socket_thread_->send_events();
 		}
 	}
 	else {
@@ -1044,116 +994,7 @@ void socket::set_event_handler(event_handler* pEvtHandler)
 	}
 }
 
-#define ERRORDECL(c, desc) { c, #c, desc },
-
-struct Error_table
-{
-	int code;
-	char const* const name;
-	char const* const description;
-};
-
-static Error_table const error_table[] =
-{
-	ERRORDECL(EACCES, fztranslate_mark("Permission denied"))
-	ERRORDECL(EADDRINUSE, fztranslate_mark("Local address in use"))
-	ERRORDECL(EAFNOSUPPORT, fztranslate_mark("The specified address family is not supported"))
-	ERRORDECL(EINPROGRESS, fztranslate_mark("Operation in progress"))
-	ERRORDECL(EINVAL, fztranslate_mark("Invalid argument passed"))
-	ERRORDECL(EMFILE, fztranslate_mark("Process file table overflow"))
-	ERRORDECL(ENFILE, fztranslate_mark("System limit of open files exceeded"))
-	ERRORDECL(ENOBUFS, fztranslate_mark("Out of memory"))
-	ERRORDECL(ENOMEM, fztranslate_mark("Out of memory"))
-	ERRORDECL(EPERM, fztranslate_mark("Permission denied"))
-	ERRORDECL(EPROTONOSUPPORT, fztranslate_mark("Protocol not supported"))
-	ERRORDECL(EAGAIN, fztranslate_mark("Resource temporarily unavailable"))
-	ERRORDECL(EALREADY, fztranslate_mark("Operation already in progress"))
-	ERRORDECL(EBADF, fztranslate_mark("Bad file descriptor"))
-	ERRORDECL(ECONNREFUSED, fztranslate_mark("Connection refused by server"))
-	ERRORDECL(EFAULT, fztranslate_mark("Socket address outside address space"))
-	ERRORDECL(EINTR, fztranslate_mark("Interrupted by signal"))
-	ERRORDECL(EISCONN, fztranslate_mark("Socket already connected"))
-	ERRORDECL(ENETUNREACH, fztranslate_mark("Network unreachable"))
-	ERRORDECL(ENOTSOCK, fztranslate_mark("File descriptor not a socket"))
-	ERRORDECL(ETIMEDOUT, fztranslate_mark("Connection attempt timed out"))
-	ERRORDECL(EHOSTUNREACH, fztranslate_mark("No route to host"))
-	ERRORDECL(ENOTCONN, fztranslate_mark("Socket not connected"))
-	ERRORDECL(ENETRESET, fztranslate_mark("Connection reset by network"))
-	ERRORDECL(EOPNOTSUPP, fztranslate_mark("Operation not supported"))
-	ERRORDECL(ESHUTDOWN, fztranslate_mark("Socket has been shut down"))
-	ERRORDECL(EMSGSIZE, fztranslate_mark("Message too large"))
-	ERRORDECL(ECONNABORTED, fztranslate_mark("Connection aborted"))
-	ERRORDECL(ECONNRESET, fztranslate_mark("Connection reset by peer"))
-	ERRORDECL(EPIPE, fztranslate_mark("Local endpoint has been closed"))
-	ERRORDECL(EHOSTDOWN, fztranslate_mark("Host is down"))
-
-	// Getaddrinfo related
-#ifdef EAI_ADDRFAMILY
-	ERRORDECL(EAI_ADDRFAMILY, fztranslate_mark("Network host does not have any network addresses in the requested address family"))
-#endif
-	ERRORDECL(EAI_AGAIN, fztranslate_mark("Temporary failure in name resolution"))
-	ERRORDECL(EAI_BADFLAGS, fztranslate_mark("Invalid value for ai_flags"))
-#ifdef EAI_BADHINTS
-	ERRORDECL(EAI_BADHINTS, fztranslate_mark("Invalid value for hints"))
-#endif
-	ERRORDECL(EAI_FAIL, fztranslate_mark("Nonrecoverable failure in name resolution"))
-	ERRORDECL(EAI_FAMILY, fztranslate_mark("The ai_family member is not supported"))
-	ERRORDECL(EAI_MEMORY, fztranslate_mark("Memory allocation failure"))
-#ifdef EAI_NODATA
-	ERRORDECL(EAI_NODATA, fztranslate_mark("No address associated with nodename"))
-#endif
-	ERRORDECL(EAI_NONAME, fztranslate_mark("Neither nodename nor servname provided, or not known"))
-#ifdef EAI_OVERFLOW
-	ERRORDECL(EAI_OVERFLOW, fztranslate_mark("Argument buffer overflow"))
-#endif
-#ifdef EAI_PROTOCOL
-	ERRORDECL(EAI_PROTOCOL, fztranslate_mark("Resolved protocol is unknown"))
-#endif
-	ERRORDECL(EAI_SERVICE, fztranslate_mark("The servname parameter is not supported for ai_socktype"))
-	ERRORDECL(EAI_SOCKTYPE, fztranslate_mark("The ai_socktype member is not supported"))
-#ifdef EAI_SYSTEM
-	ERRORDECL(EAI_SYSTEM, fztranslate_mark("Other system error"))
-#endif
-
-	// Codes that have no POSIX equivalence
-#ifdef FZ_WINDOWS
-	ERRORDECL(WSANOTINITIALISED, fztranslate_mark("Not initialized, need to call WSAStartup"))
-	ERRORDECL(WSAENETDOWN, fztranslate_mark("System's network subsystem has failed"))
-	ERRORDECL(WSAEPROTOTYPE, fztranslate_mark("Protocol not supported on given socket type"))
-	ERRORDECL(WSAESOCKTNOSUPPORT, fztranslate_mark("Socket type not supported for address family"))
-	ERRORDECL(WSAEADDRNOTAVAIL, fztranslate_mark("Cannot assign requested address"))
-	ERRORDECL(ERROR_NETNAME_DELETED, fztranslate_mark("The specified network name is no longer available"))
-#endif
-	{ 0, nullptr, nullptr }
-};
-
-std::string socket::error_string(int error)
-{
-	for (int i = 0; error_table[i].code; ++i) {
-		if (error != error_table[i].code) {
-			continue;
-		}
-
-		return error_table[i].name;
-	}
-
-	return sprintf("%d", error);
-}
-
-native_string socket::error_description(int error)
-{
-	for (int i = 0; error_table[i].code; ++i) {
-		if (error != error_table[i].code) {
-			continue;
-		}
-
-		return to_native(to_native(std::string(error_table[i].name)) + fzT(" - ") + to_native(translate(error_table[i].description)));
-	}
-
-	return sprintf(fzT("%d"), error);
-}
-
-int socket::close()
+int socket_base::close()
 {
 	if (socket_thread_) {
 		scoped_lock l(socket_thread_->mutex_);
@@ -1166,7 +1007,7 @@ int socket::close()
 		socket_thread_->wakeup_thread(l);
 
 		socket_thread::close_socket_fd(fd);
-		state_ = none;
+		state_ = 0;
 
 		socket_thread_->triggered_ = 0;
 		for (int i = 0; i < WAIT_EVENTCOUNT; ++i) {
@@ -1182,7 +1023,7 @@ int socket::close()
 		int fd = fd_;
 		fd_ = -1;
 		socket_thread::close_socket_fd(fd);
-		state_ = none;
+		state_ = 0;
 
 		if (evt_handler_) {
 			remove_socket_events(evt_handler_, this);
@@ -1193,101 +1034,8 @@ int socket::close()
 	return 0;
 }
 
-socket::socket_state socket::get_state()
-{
-	socket_state state;
-	if (socket_thread_) {
-		socket_thread_->mutex_.lock();
-	}
-	state = state_;
-	if (socket_thread_) {
-		socket_thread_->mutex_.unlock();
-	}
 
-	return state;
-}
-
-int socket::read(void* buffer, unsigned int size, int& error)
-{
-	int res = recv(fd_, (char*)buffer, size, 0);
-
-	if (res == -1) {
-		error = last_socket_error();
-		if (error == EAGAIN) {
-			if (socket_thread_) {
-				scoped_lock l(socket_thread_->mutex_);
-				if (!(socket_thread_->waiting_ & WAIT_READ)) {
-					socket_thread_->waiting_ |= WAIT_READ;
-					socket_thread_->wakeup_thread(l);
-				}
-			}
-		}
-	}
-	else {
-		error = 0;
-	}
-
-	return res;
-}
-
-int socket::peek(void* buffer, unsigned int size, int& error)
-{
-	int res = recv(fd_, (char*)buffer, size, MSG_PEEK);
-
-	if (res == -1) {
-		error = last_socket_error();
-	}
-	else {
-		error = 0;
-	}
-
-	return res;
-}
-
-int socket::write(const void* buffer, unsigned int size, int& error)
-{
-#ifdef MSG_NOSIGNAL
-	const int flags = MSG_NOSIGNAL;
-#else
-	const int flags = 0;
-
-#if !defined(SO_NOSIGPIPE) && !defined(FZ_WINDOWS)
-	// Some systems have neither. Need to block signal
-	sigaction old_action;
-	sigaction action = {};
-	action.sa_handler = SIG_IGN;
-	int signal_set = sigaction(SIGPIPE, &action, &old_action);
-#endif
-
-#endif
-
-	int res = send(fd_, (const char*)buffer, size, flags);
-
-#if !defined(MSG_NOSIGNAL) && !defined(SO_NOSIGPIPE) && !defined(FZ_WINDOWS)
-	// Restore previous signal handler
-	if (!signal_set)
-		sigaction(SIGPIPE, &old_action, 0);
-#endif
-
-	if (res == -1) {
-		error = last_socket_error();
-		if (error == EAGAIN) {
-			if (socket_thread_) {
-				scoped_lock l (socket_thread_->mutex_);
-				if (!(socket_thread_->waiting_ & WAIT_WRITE)) {
-					socket_thread_->waiting_ |= WAIT_WRITE;
-					socket_thread_->wakeup_thread(l);
-				}
-			}
-		}
-	}
-	else
-		error = 0;
-
-	return res;
-}
-
-std::string socket::address_to_string(sockaddr const* addr, int addr_len, bool with_port, bool strip_zone_index)
+std::string socket_base::address_to_string(sockaddr const* addr, int addr_len, bool with_port, bool strip_zone_index)
 {
 	char hostbuf[NI_MAXHOST];
 	char portbuf[NI_MAXSERV];
@@ -1322,7 +1070,7 @@ std::string socket::address_to_string(sockaddr const* addr, int addr_len, bool w
 	}
 }
 
-std::string socket::address_to_string(char const* buf, int buf_len)
+std::string socket_base::address_to_string(char const* buf, int buf_len)
 {
 	if (buf_len != 4 && buf_len != 16) {
 		return std::string();
@@ -1341,7 +1089,7 @@ std::string socket::address_to_string(char const* buf, int buf_len)
 	return address_to_string(&addr.sockaddr_, sizeof(addr), false, true);
 }
 
-std::string socket::local_ip(bool strip_zone_index) const
+std::string socket_base::local_ip(bool strip_zone_index) const
 {
 	sockaddr_storage addr;
 	socklen_t addr_len = sizeof(addr);
@@ -1353,19 +1101,7 @@ std::string socket::local_ip(bool strip_zone_index) const
 	return address_to_string((sockaddr *)&addr, addr_len, false, strip_zone_index);
 }
 
-std::string socket::peer_ip(bool strip_zone_index) const
-{
-	sockaddr_storage addr;
-	socklen_t addr_len = sizeof(addr);
-	int res = getpeername(fd_, (sockaddr*)&addr, &addr_len);
-	if (res) {
-		return std::string();
-	}
-
-	return address_to_string((sockaddr *)&addr, addr_len, false, strip_zone_index);
-}
-
-address_type socket::address_family() const
+address_type socket_base::address_family() const
 {
 	sockaddr_u addr;
 	socklen_t addr_len = sizeof(addr);
@@ -1385,7 +1121,107 @@ address_type socket::address_family() const
 	}
 }
 
-int socket::listen(address_type family, int port)
+int socket_base::local_port(int& error)
+{
+	sockaddr_u addr;
+	socklen_t addr_len = sizeof(addr);
+	error = getsockname(fd_, &addr.sockaddr_, &addr_len);
+	if (error) {
+#ifdef FZ_WINDOWS
+		error = convert_msw_error_code(error);
+#endif
+		return -1;
+	}
+
+	if (addr.storage.ss_family == AF_INET) {
+		return ntohs(addr.in4.sin_port);
+	}
+	else if (addr.storage.ss_family == AF_INET6) {
+		return ntohs(addr.in6.sin6_port);
+	}
+
+	error = EINVAL;
+	return -1;
+}
+
+void socket_base::set_flags(int flags)
+{
+	if (socket_thread_) {
+		socket_thread_->mutex_.lock();
+	}
+
+	if (fd_ != -1) {
+		do_set_flags(fd_, flags, flags ^ flags_, keepalive_interval_);
+	}
+	flags_ = flags;
+
+	if (socket_thread_) {
+		socket_thread_->mutex_.unlock();
+	}
+}
+
+int socket_base::set_buffer_sizes(int size_receive, int size_send)
+{
+	int ret = 0;
+
+	if (socket_thread_) {
+		socket_thread_->mutex_.lock();
+	}
+
+	buffer_sizes_[0] = size_receive;
+	buffer_sizes_[1] = size_send;
+
+	if (fd_ != -1) {
+		ret = do_set_buffer_sizes(fd_, size_receive, size_send);
+	}
+
+	if (socket_thread_) {
+		socket_thread_->mutex_.unlock();
+	}
+
+	return ret;
+}
+
+void socket_base::set_keepalive_interval(duration const& d)
+{
+	if (d < duration::from_minutes(1)) {
+		return;
+	}
+
+	if (socket_thread_) {
+		socket_thread_->mutex_.lock();
+	}
+
+	keepalive_interval_ = d;
+	if (fd_ != -1) {
+		do_set_flags(fd_, flags_, flag_keepalive, keepalive_interval_);
+	}
+
+	if (socket_thread_) {
+		socket_thread_->mutex_.unlock();
+	}
+}
+
+
+
+listen_socket::listen_socket(thread_pool & pool, event_handler* evt_handler)
+	: socket_base(pool, evt_handler)
+{
+}
+
+listen_socket::~listen_socket()
+{
+	if (state_ != none) {
+		close();
+	}
+
+	if (socket_thread_) {
+		scoped_lock l(socket_thread_->mutex_);
+		detach_thread(l);
+	}
+}
+
+int listen_socket::listen(address_type family, int port)
 {
 	if (state_ != none) {
 		return EALREADY;
@@ -1475,27 +1311,241 @@ int socket::listen(address_type family, int port)
 	return 0;
 }
 
-int socket::local_port(int& error)
+socket* listen_socket::accept(int &error)
 {
-	sockaddr_u addr;
-	socklen_t addr_len = sizeof(addr);
-	error = getsockname(fd_, &addr.sockaddr_, &addr_len);
-	if (error) {
-#ifdef FZ_WINDOWS
-		error = convert_msw_error_code(error);
+	if (socket_thread_) {
+		scoped_lock l(socket_thread_->mutex_);
+		socket_thread_->waiting_ |= WAIT_ACCEPT;
+		socket_thread_->wakeup_thread(l);
+	}
+	// TODO: accept4 for SOCK_CLOEXEC
+	int fd = ::accept(fd_, nullptr, nullptr);
+	if (fd == -1) {
+		error = last_socket_error();
+		return nullptr;
+	}
+
+#if defined(SO_NOSIGPIPE) && !defined(MSG_NOSIGNAL)
+	// We do not want SIGPIPE if writing to socket.
+	const int value = 1;
+	setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &value, sizeof(int));
 #endif
-		return -1;
+
+	set_nonblocking(fd);
+
+	do_set_buffer_sizes(fd, buffer_sizes_[0], buffer_sizes_[1]);
+
+	socket* pSocket = new socket(thread_pool_, nullptr);
+	pSocket->state_ = socket::connected;
+	pSocket->fd_ = fd;
+	pSocket->socket_thread_ = new socket_thread();
+	pSocket->socket_thread_->set_socket(pSocket);
+	pSocket->socket_thread_->waiting_ = WAIT_READ | WAIT_WRITE;
+	pSocket->socket_thread_->start();
+
+	return pSocket;
+}
+
+listen_socket::listen_socket_state listen_socket::get_state()
+{
+	listen_socket_state state;
+	if (socket_thread_) {
+		socket_thread_->mutex_.lock();
+	}
+	state = static_cast<listen_socket_state>(state_);
+	if (socket_thread_) {
+		socket_thread_->mutex_.unlock();
 	}
 
-	if (addr.storage.ss_family == AF_INET) {
-		return ntohs(addr.in4.sin_port);
-	}
-	else if (addr.storage.ss_family == AF_INET6) {
-		return ntohs(addr.in6.sin6_port);
+	return state;
+}
+
+
+
+
+socket::socket(thread_pool & pool, event_handler* evt_handler)
+	: socket_base(pool, evt_handler)
+{
+}
+
+socket::~socket()
+{
+	if (state_ != none) {
+		close();
 	}
 
-	error = EINVAL;
-	return -1;
+	if (socket_thread_) {
+		scoped_lock l(socket_thread_->mutex_);
+		detach_thread(l);
+	}
+}
+
+int socket::connect(native_string const& host, unsigned int port, address_type family, std::string const& bind)
+{
+	if (state_ != none) {
+		return EISCONN;
+	}
+
+	if (port < 1 || port > 65535) {
+		return EINVAL;
+	}
+
+	if (host.empty()) {
+		return EINVAL;
+	}
+
+	int af{};
+
+	switch (family)
+	{
+	case address_type::unknown:
+		af = AF_UNSPEC;
+		break;
+	case address_type::ipv4:
+		af = AF_INET;
+		break;
+	case address_type::ipv6:
+		af = AF_INET6;
+		break;
+	default:
+		return EINVAL;
+	}
+
+	if (socket_thread_ && socket_thread_->started_) {
+		scoped_lock l(socket_thread_->mutex_);
+		if (!socket_thread_->threadwait_) {
+			// Possibly inside a blocking call, e.g. getaddrinfo.
+			// Detach the thread so that we can continue.
+			detach_thread(l);
+		}
+	}
+	if (!socket_thread_) {
+		socket_thread_ = new socket_thread();
+		socket_thread_->set_socket(this);
+	}
+
+	family_ = af;
+	state_ = connecting;
+
+	host_ = host;
+	port_ = port;
+	int res = socket_thread_->connect(bind, to_utf8(host_), port_);
+	if (res) {
+		state_ = none;
+		delete socket_thread_;
+		socket_thread_ = nullptr;
+		return res;
+	}
+
+	return EINPROGRESS;
+}
+
+socket::socket_state socket::get_state()
+{
+	socket_state state;
+	if (socket_thread_) {
+		socket_thread_->mutex_.lock();
+	}
+	state = static_cast<socket_state>(state_);
+	if (socket_thread_) {
+		socket_thread_->mutex_.unlock();
+	}
+
+	return state;
+}
+
+int socket::read(void* buffer, unsigned int size, int& error)
+{
+	int res = recv(fd_, (char*)buffer, size, 0);
+
+	if (res == -1) {
+		error = last_socket_error();
+		if (error == EAGAIN) {
+			if (socket_thread_) {
+				scoped_lock l(socket_thread_->mutex_);
+				if (!(socket_thread_->waiting_ & WAIT_READ)) {
+					socket_thread_->waiting_ |= WAIT_READ;
+					socket_thread_->wakeup_thread(l);
+				}
+			}
+		}
+	}
+	else {
+		error = 0;
+	}
+
+	return res;
+}
+
+int socket::peek(void* buffer, unsigned int size, int& error)
+{
+	int res = recv(fd_, (char*)buffer, size, MSG_PEEK);
+
+	if (res == -1) {
+		error = last_socket_error();
+	}
+	else {
+		error = 0;
+	}
+
+	return res;
+}
+
+int socket::write(const void* buffer, unsigned int size, int& error)
+{
+#ifdef MSG_NOSIGNAL
+	const int flags = MSG_NOSIGNAL;
+#else
+	const int flags = 0;
+
+#if !defined(SO_NOSIGPIPE) && !defined(FZ_WINDOWS)
+	// Some systems have neither. Need to block signal
+	sigaction old_action;
+	sigaction action = {};
+	action.sa_handler = SIG_IGN;
+	int signal_set = sigaction(SIGPIPE, &action, &old_action);
+#endif
+
+#endif
+
+	int res = send(fd_, (const char*)buffer, size, flags);
+
+#if !defined(MSG_NOSIGNAL) && !defined(SO_NOSIGPIPE) && !defined(FZ_WINDOWS)
+	// Restore previous signal handler
+	if (!signal_set) {
+		sigaction(SIGPIPE, &old_action, 0);
+	}
+#endif
+
+	if (res == -1) {
+		error = last_socket_error();
+		if (error == EAGAIN) {
+			if (socket_thread_) {
+				scoped_lock l (socket_thread_->mutex_);
+				if (!(socket_thread_->waiting_ & WAIT_WRITE)) {
+					socket_thread_->waiting_ |= WAIT_WRITE;
+					socket_thread_->wakeup_thread(l);
+				}
+			}
+		}
+	}
+	else {
+		error = 0;
+	}
+
+	return res;
+}
+
+std::string socket::peer_ip(bool strip_zone_index) const
+{
+	sockaddr_storage addr;
+	socklen_t addr_len = sizeof(addr);
+	int res = getpeername(fd_, (sockaddr*)&addr, &addr_len);
+	if (res) {
+		return std::string();
+	}
+
+	return address_to_string((sockaddr *)&addr, addr_len, false, strip_zone_index);
 }
 
 int socket::remote_port(int& error)
@@ -1521,143 +1571,6 @@ int socket::remote_port(int& error)
 	return -1;
 }
 
-socket* socket::accept(int &error)
-{
-	if (socket_thread_) {
-		scoped_lock l(socket_thread_->mutex_);
-		socket_thread_->waiting_ |= WAIT_ACCEPT;
-		socket_thread_->wakeup_thread(l);
-	}
-	// TODO: accept4 for SOCK_CLOEXEC
-	int fd = ::accept(fd_, nullptr, nullptr);
-	if (fd == -1) {
-		error = last_socket_error();
-		return nullptr;
-	}
-
-#if defined(SO_NOSIGPIPE) && !defined(MSG_NOSIGNAL)
-	// We do not want SIGPIPE if writing to socket.
-	const int value = 1;
-	setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &value, sizeof(int));
-#endif
-
-	set_nonblocking(fd);
-
-	do_set_buffer_sizes(fd, buffer_sizes_[0], buffer_sizes_[1]);
-
-	socket* pSocket = new socket(thread_pool_, nullptr);
-	pSocket->state_ = connected;
-	pSocket->fd_ = fd;
-	pSocket->socket_thread_ = new socket_thread();
-	pSocket->socket_thread_->set_socket(pSocket);
-	pSocket->socket_thread_->waiting_ = WAIT_READ | WAIT_WRITE;
-	pSocket->socket_thread_->start();
-
-	return pSocket;
-}
-
-int socket::set_nonblocking(int fd)
-{
-	// Set socket to non-blocking.
-#ifdef FZ_WINDOWS
-	unsigned long nonblock = 1;
-	int res = ioctlsocket(fd, FIONBIO, &nonblock);
-	if (!res) {
-		return 0;
-	}
-	else {
-		return convert_msw_error_code(WSAGetLastError());
-	}
-#else
-	int flags = fcntl(fd, F_GETFL);
-	if (flags == -1) {
-		return errno;
-	}
-	int res = fcntl(fd, F_SETFL, flags | O_NONBLOCK);
-	if (res == -1) {
-		return errno;
-	}
-	return 0;
-#endif
-}
-
-void socket::set_flags(int flags)
-{
-	if (socket_thread_) {
-		socket_thread_->mutex_.lock();
-	}
-
-	if (fd_ != -1) {
-		do_set_flags(fd_, flags, flags ^ flags_, keepalive_interval_);
-	}
-	flags_ = flags;
-
-	if (socket_thread_) {
-		socket_thread_->mutex_.unlock();
-	}
-}
-
-int socket::do_set_flags(int fd, int flags, int flags_mask, duration const& keepalive_interval)
-{
-	if (flags_mask & flag_nodelay) {
-		const int value = (flags & flag_nodelay) ? 1 : 0;
-		int res = setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (const char*)&value, sizeof(value));
-		if (res != 0) {
-			return last_socket_error();
-		}
-	}
-	if (flags_mask & flag_keepalive) {
-#if FZ_WINDOWS
-		tcp_keepalive v{};
-		v.onoff = (flags & flag_keepalive) ? 1 : 0;
-		v.keepalivetime = static_cast<ULONG>(keepalive_interval.get_milliseconds());
-		v.keepaliveinterval = 1000;
-		DWORD tmp{};
-		int res = WSAIoctl(fd, SIO_KEEPALIVE_VALS, &v, sizeof(v), nullptr, 0, &tmp, nullptr, nullptr);
-		if (res != 0) {
-			return last_socket_error();
-		}
-#else
-		const int value = (flags & flag_keepalive) ? 1 : 0;
-		int res = setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (const char*)&value, sizeof(value));
-		if (res != 0) {
-			return last_socket_error();
-		}
-#ifdef TCP_KEEPIDLE
-		int const idle = keepalive_interval.get_seconds();
-		res = setsockopt(fd, IPPROTO_TCP, TCP_KEEPINTVL, (const char*)&idle, sizeof(idle));
-		if (res != 0) {
-			return last_socket_error();
-		}
-#endif
-#endif
-	}
-
-	return 0;
-}
-
-int socket::set_buffer_sizes(int size_receive, int size_send)
-{
-	int ret = 0;
-
-	if (socket_thread_) {
-		socket_thread_->mutex_.lock();
-	}
-
-	buffer_sizes_[0] = size_receive;
-	buffer_sizes_[1] = size_send;
-
-	if (fd_ != -1) {
-		ret = do_set_buffer_sizes(fd_, size_receive, size_send);
-	}
-
-	if (socket_thread_) {
-		socket_thread_->mutex_.unlock();
-	}
-
-	return ret;
-}
-
 int socket::ideal_send_buffer_size()
 {
 	int size = -1;
@@ -1670,7 +1583,7 @@ int socket::ideal_send_buffer_size()
 	if (fd_ != -1) {
 		// MSDN says this:
 		// "Dynamic send buffering for TCP was added on Windows 7 and Windows
-		// Server 2008 R2.By default, dynamic send buffering for TCP is
+		// Server 2008 R2. By default, dynamic send buffering for TCP is
 		// enabled unless an application sets the SO_SNDBUF socket option on
 		// the stream socket"
 		//
@@ -1695,52 +1608,11 @@ int socket::ideal_send_buffer_size()
 	return size;
 }
 
-
-int socket::do_set_buffer_sizes(int fd, int size_read, int size_write)
-{
-	int ret = 0;
-	if (size_read != -1) {
-		int res = setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (const char*)&size_read, sizeof(size_read));
-		if (res != 0) {
-			ret = last_socket_error();
-		}
-	}
-
-	if (size_write != -1) {
-		int res = setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (const char*)&size_write, sizeof(size_write));
-		if (res != 0) {
-			ret = last_socket_error();
-		}
-	}
-
-	return ret;
-}
-
 native_string socket::peer_host() const
 {
 	return host_;
 }
 
-void socket::set_keepalive_interval(duration const& d)
-{
-	if (d < duration::from_minutes(1)) {
-		return;
-	}
-
-	if (socket_thread_) {
-		socket_thread_->mutex_.lock();
-	}
-
-	keepalive_interval_ = d;
-	if (fd_ != -1) {
-		do_set_flags(fd_, flags_, flag_keepalive, keepalive_interval_);
-	}
-
-	if (socket_thread_) {
-		socket_thread_->mutex_.unlock();
-	}
-}
-
 void socket::retrigger(socket_event_flag event)
 {
 	if (!socket_thread_) {
@@ -1753,15 +1625,32 @@ void socket::retrigger(socket_event_flag event)
 	
 	fz::scoped_lock l(socket_thread_->mutex_);
 
-	if (has_pending_event(evt_handler_, this, event)) {
+	if (state_ != socket::connected) {
+		return;
+	}
+
+	if (!evt_handler_ || has_pending_event(evt_handler_, this, event)) {
 		return;
 	}
 
 	int const wait_flag = (event == socket_event_flag::read) ? WAIT_READ : WAIT_WRITE;
 	if (!(socket_thread_->waiting_ & wait_flag)) {
-		socket_thread_->waiting_ |= wait_flag;
-		socket_thread_->wakeup_thread(l);
+		evt_handler_->send_event<socket_event>(this, event, 0);
 	}
 }
 
+int socket::shutdown()
+{
+#ifdef FZ_WINDOWS
+	int res = ::shutdown(fd_, SD_SEND);
+#else
+	int res = ::shutdown(fd_, SHUT_WR);
+#endif
+	if (res != 0) {
+		return last_socket_error();
+	}
+
+	return 0;
+}
+
 }
