@@ -1,7 +1,9 @@
 #include <filezilla.h>
 #include "engineprivate.h"
+#include "socket_errors.h"
 #include "tlssocket.h"
 #include "tlssocket_impl.h"
+#include "tls_system_trust_store_impl.h"
 #include "ControlSocket.h"
 
 #include <libfilezilla/iputils.hpp>
@@ -15,7 +17,11 @@
 #if FZ_USE_GNUTLS_SYSTEM_CIPHERS
 char const ciphers[] = "@SYSTEM";
 #else
-char const ciphers[] = "SECURE256:+SECURE128:-ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-CTYPE-OPENPGP:-VERS-SSL3.0";
+	#if GNUTLS_VERSION_NUMBER >= 0x030600
+		char const ciphers[] = "SECURE256:+SECURE128:-ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-VERS-SSL3.0";
+	#else
+		char const ciphers[] = "SECURE256:+SECURE128:-ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-CTYPE-OPENPGP:-VERS-SSL3.0";
+	#endif
 #endif
 
 #define TLSDEBUG 0
@@ -168,9 +174,6 @@ bool CTlsSocketImpl::Init()
 		return false;
 	}
 
-	// Disable time checks. We allow expired/not yet valid certificates, though only after explicit user confirmation
-	gnutls_certificate_set_verify_flags(m_certCredentials, gnutls_certificate_get_verify_flags(m_certCredentials) | GNUTLS_VERIFY_DISABLE_TIME_CHECKS | GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS);
-
 	if (!InitSession()) {
 		return false;
 	}
@@ -286,18 +289,18 @@ void CTlsSocketImpl::LogError(int code, std::wstring const& function, MessageTyp
 	}
 	else if (code == GNUTLS_E_PULL_ERROR) {
 		if (function.empty()) {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not read from socket: %s", fz::socket::error_description(m_socket_error));
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not read from socket: %s", fz::socket_error_description(m_socket_error));
 		}
 		else {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not read from socket in %s: %s", function, fz::socket::error_description(m_socket_error));
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not read from socket in %s: %s", function, fz::socket_error_description(m_socket_error));
 		}
 	}
 	else if (code == GNUTLS_E_PUSH_ERROR) {
 		if (function.empty()) {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not write to socket: %s", fz::socket::error_description(m_socket_error));
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not write to socket: %s", fz::socket_error_description(m_socket_error));
 		}
 		else {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not write to socket in %s: %s", function, fz::socket::error_description(m_socket_error));
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not write to socket in %s: %s", function, fz::socket_error_description(m_socket_error));
 		}
 	}
 	else {
@@ -354,7 +357,7 @@ ssize_t CTlsSocketImpl::PushFunction(const void* data, size_t len)
 	}
 
 	int error;
-	int written = socketBackend_->Write(data, len, error);
+	int written = socketBackend_->Write(data, static_cast<unsigned int>(len), error);
 
 	if (written < 0) {
 		m_canWriteToSocket = false;
@@ -380,9 +383,6 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 #if TLSDEBUG
 	m_pOwner->LogMessage(MessageType::Debug_Debug, L"CTlsSocketImpl::PullFunction(%d)",  (int)len);
 #endif
-	if (m_socketClosed) {
-		return 0;
-	}
 
 	if (!m_canReadFromSocket) {
 		gnutls_transport_set_errno(m_session, EAGAIN);
@@ -390,15 +390,10 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 	}
 
 	int error;
-	int read = socketBackend_->Read(data, len, error);
+	int read = socketBackend_->Read(data, static_cast<unsigned int>(len), error);
 	if (read < 0) {
 		m_canReadFromSocket = false;
-		if (error == EAGAIN) {
-			if (m_canCheckCloseSocket && !socketBackend_->IsWaiting(CRateLimiter::inbound)) {
-				tlsSocket_.send_event<fz::socket_event>(socketBackend_.get(), fz::socket_event_flag::close, 0);
-			}
-		}
-		else {
+		if (error != EAGAIN) {
 			m_socket_error = error;
 		}
 		gnutls_transport_set_errno(m_session, error);
@@ -408,10 +403,6 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 		return -1;
 	}
 
-	if (m_canCheckCloseSocket) {
-		tlsSocket_.send_event<fz::socket_event>(socketBackend_.get(), fz::socket_event_flag::close, 0);
-	}
-
 	if (!read) {
 		m_socket_eof = true;
 	}
@@ -434,6 +425,12 @@ void CTlsSocketImpl::OnSocketEvent(fz::socket_event_source*, fz::socket_event_fl
 		return;
 	}
 
+	if (error) {
+		m_socket_error = error;
+		Failure(0, true);
+		return;
+	}
+
 	switch (t)
 	{
 	case fz::socket_event_flag::read:
@@ -442,31 +439,6 @@ void CTlsSocketImpl::OnSocketEvent(fz::socket_event_source*, fz::socket_event_fl
 	case fz::socket_event_flag::write:
 		OnSend();
 		break;
-	case fz::socket_event_flag::close:
-		{
-			m_canCheckCloseSocket = true;
-			char tmp[100];
-			int peeked = socketBackend_->Peek(&tmp, 100, error);
-			if (peeked >= 0) {
-				if (peeked > 0) {
-					m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::OnSocketEvent(): pending data, postponing close event");
-				}
-				else {
-					m_socket_eof = true;
-					m_socketClosed = true;
-				}
-				OnRead();
-
-				if (peeked) {
-					return;
-				}
-			}
-
-			m_pOwner->LogMessage(MessageType::Debug_Info, L"CTlsSocketImpl::OnSocketEvent(): close event received");
-
-			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::close, 0);
-		}
-		break;
 	default:
 		break;
 	}
@@ -661,7 +633,7 @@ int CTlsSocketImpl::ContinueHandshake()
 		if (m_shutdown_requested) {
 			int error = Shutdown();
 			if (!error || error != EAGAIN) {
-				tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::close, 0);
+				tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, error);
 			}
 		}
 
@@ -699,7 +671,7 @@ int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
 		TriggerEvents();
 
 		error = 0;
-		return min;
+		return static_cast<int>(min);
 	}
 
 	int res = DoCallGnutlsRecordRecv(buffer, len);
@@ -738,6 +710,13 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 		error = EAGAIN;
 		return -1;
 	}
+	else if (m_tlsState == CTlsSocket::TlsState::closing) {
+		error = EAGAIN;
+		return -1;
+	}
+	else if (m_tlsState == CTlsSocket::TlsState::closed) {
+		return 0;
+	}
 	else if (m_tlsState != CTlsSocket::TlsState::conn) {
 		error = ENOTCONN;
 		return -1;
@@ -756,14 +735,15 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 	len -= m_writeSkip;
 	buffer = (char*)buffer + m_writeSkip;
 
-	int res = gnutls_record_send(m_session, buffer, len);
+	ssize_t res = gnutls_record_send(m_session, buffer, len);
 
-	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket)
+	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
 		res = gnutls_record_send(m_session, nullptr, 0);
+	}
 
 	if (res >= 0) {
 		error = 0;
-		int written = res + m_writeSkip;
+		int written = static_cast<int>(res) + m_writeSkip;
 		m_writeSkip = 0;
 
 		TriggerEvents();
@@ -784,7 +764,7 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 		}
 	}
 	else {
-		Failure(res, false, L"gnutls_record_send");
+		Failure(static_cast<int>(res), false, L"gnutls_record_send");
 		error = m_socket_error;
 		return -1;
 	}
@@ -792,8 +772,9 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 
 void CTlsSocketImpl::TriggerEvents()
 {
-	if (m_tlsState != CTlsSocket::TlsState::conn)
+	if (m_tlsState != CTlsSocket::TlsState::conn) {
 		return;
+	}
 
 	if (m_canTriggerRead) {
 		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, 0);
@@ -809,7 +790,7 @@ void CTlsSocketImpl::TriggerEvents()
 void CTlsSocketImpl::CheckResumeFailedReadWrite()
 {
 	if (m_lastWriteFailed) {
-		int res = GNUTLS_E_AGAIN;
+		ssize_t res = GNUTLS_E_AGAIN;
 		while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
 			res = gnutls_record_send(m_session, nullptr, 0);
 		}
@@ -819,11 +800,11 @@ void CTlsSocketImpl::CheckResumeFailedReadWrite()
 		}
 
 		if (res < 0) {
-			Failure(res, true);
+			Failure(static_cast<int>(res), true);
 			return;
 		}
 
-		m_writeSkip += res;
+		m_writeSkip += static_cast<int>(res);
 		m_lastWriteFailed = false;
 		m_canTriggerWrite = true;
 	}
@@ -859,10 +840,22 @@ void CTlsSocketImpl::Failure(int code, bool send_close, std::wstring const& func
 			}
 		}
 	}
+
+	auto const oldState = m_tlsState;
+
 	Uninit();
 
 	if (send_close) {
-		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::close, m_socket_error);
+		int error = m_socket_error;
+		if (!error) {
+			error = ECONNABORTED;
+		}
+		if (oldState == CTlsSocket::TlsState::handshake) {
+			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::connection, error);
+		}
+		else {
+			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, error);
+		}
 	}
 }
 
@@ -873,7 +866,7 @@ int CTlsSocketImpl::Peek(void *buffer, unsigned int len, int& error)
 		memcpy(buffer, peekBuffer_.get(), min);
 
 		error = 0;
-		return min;
+		return static_cast<int>(min);
 	}
 
 	int read = Read(peekBuffer_.get(len), len, error);
@@ -890,11 +883,13 @@ int CTlsSocketImpl::Shutdown()
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::Shutdown()");
 
-	if (m_tlsState == CTlsSocket::TlsState::closed)
+	if (m_tlsState == CTlsSocket::TlsState::closed) {
 		return 0;
+	}
 
-	if (m_tlsState == CTlsSocket::TlsState::closing)
+	if (m_tlsState == CTlsSocket::TlsState::closing) {
 		return EAGAIN;
+	}
 
 	if (m_tlsState == CTlsSocket::TlsState::handshake || m_tlsState == CTlsSocket::TlsState::verifycert) {
 		// Shutdown during handshake is not a good idea.
@@ -903,8 +898,9 @@ int CTlsSocketImpl::Shutdown()
 		return EAGAIN;
 	}
 
-	if (m_tlsState != CTlsSocket::TlsState::conn)
+	if (m_tlsState != CTlsSocket::TlsState::conn) {
 		return ECONNABORTED;
+	}
 
 	m_tlsState = CTlsSocket::TlsState::closing;
 
@@ -936,7 +932,7 @@ void CTlsSocketImpl::ContinueShutdown()
 	if (!res) {
 		m_tlsState = CTlsSocket::TlsState::closed;
 
-		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::close, 0);
+		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
 
 		return;
 	}
@@ -956,8 +952,9 @@ void CTlsSocketImpl::TrustCurrentCert(bool trusted)
 	if (trusted) {
 		m_tlsState = CTlsSocket::TlsState::conn;
 
-		if (m_lastWriteFailed)
+		if (m_lastWriteFailed) {
 			m_lastWriteFailed = false;
+		}
 		CheckResumeFailedReadWrite();
 
 		if (m_tlsState == CTlsSocket::TlsState::conn) {
@@ -1047,7 +1044,7 @@ bool CTlsSocketImpl::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& ou
 		return false;
 	}
 
-	std::vector<std::wstring> alt_subject_names = GetCertSubjectAltNames(cert);
+	std::vector<CCertificate::SubjectName> alt_subject_names = GetCertSubjectAltNames(cert);
 
 	size = 0;
 	res = gnutls_x509_crt_get_issuer_dn(cert, nullptr, &size);
@@ -1109,9 +1106,9 @@ bool CTlsSocketImpl::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& ou
 }
 
 
-std::vector<std::wstring> CTlsSocketImpl::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
+std::vector<CCertificate::SubjectName> CTlsSocketImpl::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
 {
-	std::vector<std::wstring> ret;
+	std::vector<CCertificate::SubjectName> ret;
 
 	char san[4096];
 	for (unsigned int i = 0; i < 10000; ++i) { // I assume this is a sane limit
@@ -1127,13 +1124,13 @@ std::vector<std::wstring> CTlsSocketImpl::GetCertSubjectAltNames(gnutls_x509_crt
 		if (type_or_error == GNUTLS_SAN_DNSNAME || type_or_error == GNUTLS_SAN_RFC822NAME) {
 			std::wstring dns = fz::to_wstring_from_utf8(san);
 			if (!dns.empty()) {
-				ret.emplace_back(std::move(dns));
+				ret.emplace_back(CCertificate::SubjectName{std::move(dns), type_or_error == GNUTLS_SAN_DNSNAME});
 			}
 		}
 		else if (type_or_error == GNUTLS_SAN_IPADDRESS) {
-			std::wstring ip = fz::to_wstring(fz::socket::address_to_string(san, san_size));
+			std::wstring ip = fz::to_wstring(fz::socket::address_to_string(san, static_cast<int>(san_size)));
 			if (!ip.empty()) {
-				ret.emplace_back(std::move(ip));
+				ret.emplace_back(CCertificate::SubjectName{std::move(ip), false});
 			}
 		}
 	}
@@ -1324,47 +1321,103 @@ int CTlsSocketImpl::VerifyCertificate()
 		return FZ_REPLY_ERROR;
 	}
 
-	gnutls_x509_crt_t root{};
-	clone_cert(certs.certs[certs.certs_size - 1], root);
-	if (!root) {
-		m_pOwner->LogMessage(MessageType::Error, _("Could not copy certificate"));
-		Failure(0, true);
-		return FZ_REPLY_ERROR;
-	}
+	bool const uses_hostname = !hostname_.empty() && fz::get_address_type(hostname_) == fz::address_type::unknown;
+
+	bool systemTrust = false;
+	bool hostnameMismatch = false;
 
 	// Our trust-model is user-guided TOFU on the host's certificate.
+	// 
+	// First we verify it against the system trust store.
 	//
-	// Here we validate the certificate chain sent by the server on the assumption
-	// that it's signed by a trusted CA.
+	// If that fails, we validate the certificate chain sent by the server
+	// allowing three impairments:
+	// - Hostname mismatch
+	// - Out of validity
+	// - Signer not found
 	//
-	// For now, add the highest certificate from the chain to trust list. Otherweise
-	// gnutls_certificate_verify_peers2 always stops with GNUTLS_CERT_SIGNER_NOT_FOUND
-	// at the highest certificate in the chain.
-	//
-	// Actual trust decision is done later by the user.
-	gnutls_x509_trust_list_t tlist;
-	gnutls_certificate_get_trust_list(m_certCredentials, &tlist);
-	if (gnutls_x509_trust_list_add_cas(tlist, &root, 1, 0) != 1) {
-		m_pOwner->LogMessage(MessageType::Error, _("Could not add certificate to temporary trust list"));
-		Failure(0, true);
-		return FZ_REPLY_ERROR;
+	// In any case, actual trust decision is done later by the user.
+
+
+	// First, check system trust
+	if (uses_hostname) {
+
+		auto lease = m_pOwner->GetEngine().GetContext().GetTlsSystemTrustStore().impl_->lease();
+		auto cred = std::get<0>(lease);
+		if (cred) {
+			gnutls_credentials_set(m_session, GNUTLS_CRD_CERTIFICATE, cred);
+			unsigned int status = 0;
+			int verifyResult = gnutls_certificate_verify_peers3(m_session, fz::to_utf8(hostname_).c_str(), &status);
+			gnutls_credentials_set(m_session, GNUTLS_CRD_CERTIFICATE, m_certCredentials);
+			std::get<1>(lease).unlock();
+
+			if (verifyResult < 0) {
+				m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_certificate_verify_peers2 returned %d with status %u", verifyResult, status);
+				m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
+				Failure(0, true);
+				return FZ_REPLY_ERROR;
+			}
+
+			if (!status) {
+				systemTrust = true;
+			}
+		}
+		else {
+			std::get<1>(lease).unlock();
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"System trust store could not be loaded");
+		}
 	}
 
-	unsigned int status = 0;
-	int const verifyResult = gnutls_certificate_verify_peers2(m_session, &status);
+	if (!systemTrust) {
+		// System trust store cannot verify this certificate. Allow three impairments:
+		//
+		// 1. For now, add the highest certificate from the chain to trust list. Otherwise
+		// gnutls_certificate_verify_peers2 always stops with GNUTLS_CERT_SIGNER_NOT_FOUND
+		// at the highest certificate in the chain.
+		gnutls_x509_crt_t root{};
+		clone_cert(certs.certs[certs.certs_size - 1], root);
+		if (!root) {
+			m_pOwner->LogMessage(MessageType::Error, _("Could not copy certificate"));
+			Failure(0, true);
+			return FZ_REPLY_ERROR;
+		}
 
-	if (verifyResult < 0) {
-		m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_certificate_verify_peers2 returned %d with status %u", verifyResult, status);
-		m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
-		Failure(0, true);
-		return FZ_REPLY_ERROR;
-	}
+		gnutls_x509_trust_list_t tlist;
+		gnutls_certificate_get_trust_list(m_certCredentials, &tlist);
+		if (gnutls_x509_trust_list_add_cas(tlist, &root, 1, 0) != 1) {
+			m_pOwner->LogMessage(MessageType::Error, _("Could not add certificate to temporary trust list"));
+			Failure(0, true);
+			return FZ_REPLY_ERROR;
+		}
 
-	if (status != 0) {
-		PrintVerificationError(status);
+		// 2. Also disable time checks. We allow expired/not yet valid certificates, though only
+		// after explicit user confirmation.
+		gnutls_certificate_set_verify_flags(m_certCredentials, gnutls_certificate_get_verify_flags(m_certCredentials) | GNUTLS_VERIFY_DISABLE_TIME_CHECKS | GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS);
 
-		Failure(0, true);
-		return FZ_REPLY_ERROR;
+		unsigned int status = 0;
+		int verifyResult = gnutls_certificate_verify_peers2(m_session, &status);
+
+		if (verifyResult < 0) {
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_certificate_verify_peers2 returned %d with status %u", verifyResult, status);
+			m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
+			Failure(0, true);
+			return FZ_REPLY_ERROR;
+		}
+
+		if (status != 0) {
+			PrintVerificationError(status);
+
+			Failure(0, true);
+			return FZ_REPLY_ERROR;
+		}
+
+		// 3. Hostname mismatch
+		if (uses_hostname) {
+			if (!gnutls_x509_crt_check_hostname(certs.certs[0], fz::to_utf8(hostname_).c_str())) {
+				hostnameMismatch = true;
+				m_pOwner->LogMessage(MessageType::Debug_Warning, L"Hostname does not match certificate SANs");
+			}
+		}
 	}
 
 	datum_holder cert_der{};
@@ -1384,8 +1437,9 @@ int CTlsSocketImpl::VerifyCertificate()
 
 		TrustCurrentCert(true);
 
-		if (m_tlsState != CTlsSocket::TlsState::conn)
+		if (m_tlsState != CTlsSocket::TlsState::conn) {
 			return FZ_REPLY_ERROR;
+		}
 		return FZ_REPLY_OK;
 	}
 
@@ -1419,7 +1473,9 @@ int CTlsSocketImpl::VerifyCertificate()
 		GetCipherName(),
 		GetMacName(),
 		algorithmWarnings,
-		std::move(certificates));
+		std::move(certificates),
+		systemTrust,
+		hostnameMismatch);
 
 	// Finally, ask user to verify the certificate chain
 	m_pOwner->SendAsyncRequest(pNotification);
@@ -1511,7 +1567,7 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 		return list;
 	}
 	else {
-		for (size_t i = 0; ; ++i) {
+		for (unsigned int i = 0; ; ++i) {
 			unsigned int idx;
 			ret = gnutls_priority_get_cipher_suite_index(pcache, i, &idx);
 			if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
@@ -1541,7 +1597,7 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 
 int CTlsSocketImpl::DoCallGnutlsRecordRecv(void* data, size_t len)
 {
-	int res = gnutls_record_recv(m_session, data, len);
+	ssize_t res = gnutls_record_recv(m_session, data, len);
 	while( (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) && m_canReadFromSocket && !gnutls_record_get_direction(m_session)) {
 		// Spurious EAGAIN. Can happen if GnuTLS gets a partial
 		// record and the socket got closed.
@@ -1553,7 +1609,7 @@ int CTlsSocketImpl::DoCallGnutlsRecordRecv(void* data, size_t len)
 		res = gnutls_record_recv(m_session, data, len);
 	}
 
-	return res;
+	return static_cast<int>(res);
 }
 
 std::wstring CTlsSocketImpl::GetGnutlsVersion()
