@@ -1,5 +1,7 @@
-#ifndef __COMMANDS_H__
-#define __COMMANDS_H__
+#ifndef FILEZILLA_ENGINE_COMMANDS_HEADER
+#define FILEZILLA_ENGINE_COMMANDS_HEADER
+
+#include <libfilezilla/uri.hpp>
 
 // See below for actual commands and their parameters
 
@@ -18,10 +20,16 @@ enum class Command
 	rename,
 	chmod,
 	raw,
+	httprequest, // Only used by HTTP protocol
 
 	// Only used internally
 	cwd,
-	rawtransfer
+	private1,
+	private2,
+	private3,
+	private4,
+	private5,
+	private6,
 };
 
 // Reply codes
@@ -44,7 +52,9 @@ enum class Command
 #define FZ_REPLY_TIMEOUT		(0x0800 | FZ_REPLY_ERROR)
 #define FZ_REPLY_NOTSUPPORTED	(0x1000 | FZ_REPLY_ERROR) // Will be returned if command not supported by that protocol
 #define FZ_REPLY_WRITEFAILED	(0x2000 | FZ_REPLY_ERROR) // Happens if local file could not be written during transfer
-#define FZ_REPLY_LINKNOTDIR		(0x4000)
+#define FZ_REPLY_LINKNOTDIR		(0x4000 | FZ_REPLY_ERROR)
+
+#define FZ_REPLY_CONTINUE 0x8000 // Used internally
 
 // --------------- //
 // Actual commands //
@@ -54,7 +64,7 @@ class CCommand
 {
 public:
 	CCommand() = default;
-	virtual ~CCommand() {}; // TODO: One GCC >= 4.8 is in Debian Stable (Jessie by then), make default and add testcase to configure.
+	virtual ~CCommand() = default;
 
 	virtual Command GetId() const = 0;
 	virtual CCommand *Clone() const = 0;
@@ -90,13 +100,17 @@ class CBasicCommand final : public CCommandHelper<CBasicCommand<id>, id>
 class CConnectCommand final : public CCommandHelper<CConnectCommand, Command::connect>
 {
 public:
-	explicit CConnectCommand(CServer const& server, bool retry_conncting = true);
+	explicit CConnectCommand(CServer const& server, Credentials const& credentials, bool retry_conncting = true);
 
-	CServer const& GetServer() const;
-	bool RetryConnecting() const { return m_retry_connecting; }
+	CServer const& GetServer() const { return server_; }
+	Credentials const& GetCredentials() const { return credentials_; }
+	bool RetryConnecting() const { return retry_connecting_; }
+
+	virtual bool valid() const override;
 protected:
-	CServer const m_Server;
-	bool const m_retry_connecting;
+	CServer const server_;
+	Credentials const credentials_;
+	bool const retry_connecting_;
 };
 
 typedef CBasicCommand<Command::disconnect> CDisconnectCommand;
@@ -105,6 +119,7 @@ typedef CBasicCommand<Command::disconnect> CDisconnectCommand;
 #define LIST_FLAG_AVOID 2
 #define LIST_FLAG_FALLBACK_CURRENT 4
 #define LIST_FLAG_LINK 8
+#define LIST_FLAG_CLEARCACHE 16
 class CListCommand final : public CCommandHelper<CListCommand, Command::list>
 {
 	// Without a given directory, the current directory will be listed.
@@ -147,11 +162,8 @@ public:
 	class t_transferSettings final
 	{
 	public:
-		t_transferSettings()
-			: binary(true)
-		{}
-
-		bool binary;
+		bool binary{true};
+		bool fsync{};
 	};
 
 	// For uploads, set download to false.
@@ -174,6 +186,21 @@ protected:
 	t_transferSettings const m_transferSettings;
 };
 
+class CHttpRequestCommand final : public CCommandHelper<CHttpRequestCommand, Command::httprequest>
+{
+public:
+	// Reply body is delivered through nId_data notifications
+	CHttpRequestCommand(fz::uri const& uri, std::string const& verb = std::string("GET"), std::string const& body = std::string())
+		: uri_(uri)
+		, verb_(verb)
+		, body_(body)
+	{}
+
+	fz::uri const uri_;
+	std::string const verb_;
+	std::string const body_;
+};
+
 class CRawCommand final : public CCommandHelper<CRawCommand, Command::raw>
 {
 public:
