@@ -7,6 +7,8 @@ class CDirectoryCache;
 class COptionsBase;
 class CPathCache;
 class CRateLimiter;
+class OpLockManager;
+class TlsSystemTrustStore;
 
 namespace fz {
 class event_loop;
@@ -37,6 +39,8 @@ public:
 	CDirectoryCache& GetDirectoryCache();
 	CPathCache& GetPathCache();
 	CustomEncodingConverterBase const& GetCustomEncodingConverter() { return customEncodingConverter_; }
+	OpLockManager& GetOpLockManager();
+	TlsSystemTrustStore& GetTlsSystemTrustStore();
 
 protected:
 	COptionsBase& options_;
@@ -46,4 +50,4 @@ protected:
 	std::unique_ptr<Impl> impl_;
 };
 
-#endif
\ No newline at end of file
+#endif
