@@ -53,8 +53,10 @@ enum RequestId
 	reqId_interactiveLogin, // gives a challenge prompt for a password
 	reqId_hostkey,			// used only by SSH/SFTP to indicate new host key
 	reqId_hostkeyChanged,	// used only by SSH/SFTP to indicate changed host key
-	reqId_certificate		// sent after a successful TLS handshake to allow certificate
+	reqId_certificate,		// sent after a successful TLS handshake to allow certificate
 							// validation
+	reqId_insecure_ftp		// If using opportunistic FTP over TLS, ask user whether he really wants
+							// to use plaintext FTP
 };
 
 class CNotification
@@ -111,17 +113,20 @@ public:
 // You get this type of notification everytime a directory listing has been
 // requested explicitely or when a directory listing was retrieved implicitely
 // during another operation, e.g. file transfers.
+//
+// Primary notifications are those resulting from a CListCommand, other ones
+// can happen spontanously through other actions.
 class CDirectoryListing;
 class CDirectoryListingNotification final : public CNotificationHelper<nId_listing>
 {
 public:
-	explicit CDirectoryListingNotification(const CServerPath& path, const bool modified = false, const bool failed = false);
-	bool Modified() const { return m_modified; }
+	explicit CDirectoryListingNotification(CServerPath const& path, bool const primary, bool const failed = false);
+	bool Primary() const { return primary_; }
 	bool Failed() const { return m_failed; }
 	const CServerPath GetPath() const { return m_path; }
 
 protected:
-	bool m_modified{};
+	bool const primary_{};
 	bool m_failed{};
 	CServerPath m_path;
 };
@@ -313,19 +318,32 @@ protected:
 class CDataNotification final : public CNotificationHelper<nId_data>
 {
 public:
-	CDataNotification(char* pData, int len);
+	CDataNotification(char* pData, size_t len);
 	virtual ~CDataNotification();
 
-	char* Detach(int& len);
+	CDataNotification(CDataNotification const&) = delete;
+	CDataNotification& operator=(CDataNotification const&) = delete;
+
+	char* Detach(size_t& len);
+
+	char const* data() const { return m_pData; }
+	size_t size() const { return m_len; }
 
 protected:
 	char* m_pData;
-	unsigned int m_len;
+	size_t m_len;
 };
 
 class CCertificate final
 {
 public:
+	class SubjectName final
+	{
+	public:
+		std::wstring name;
+		bool isDns{};
+	};
+
 	CCertificate() = default;
 	CCertificate(CCertificate const& op) = default;
 
@@ -339,7 +357,7 @@ public:
 		std::wstring const& fingerprint_sha1,
 		std::wstring const& issuer,
 		std::wstring const& subject,
-		std::vector<std::wstring> const& altSubjectNames);
+		std::vector<SubjectName> const& altSubjectNames);
 
 	CCertificate(
 		std::vector<uint8_t> && rawdata,
@@ -351,7 +369,7 @@ public:
 		std::wstring const& fingerprint_sha1,
 		std::wstring const& issuer,
 		std::wstring const& subject,
-		std::vector<std::wstring> && altSubjectNames);
+		std::vector<SubjectName> && altSubjectNames);
 
 
 	std::vector<uint8_t> GetRawData() const { return m_rawData; }
@@ -370,7 +388,7 @@ public:
 	std::wstring const& GetSubject() const { return m_subject; }
 	std::wstring const& GetIssuer() const { return m_issuer; }
 
-	std::vector<std::wstring> const& GetAltSubjectNames() const { return m_altSubjectNames; }
+	std::vector<SubjectName> const& GetAltSubjectNames() const { return m_altSubjectNames; }
 
 private:
 	fz::datetime m_activationTime;
@@ -390,7 +408,7 @@ private:
 	std::wstring m_issuer;
 	std::wstring m_subject;
 
-	std::vector<std::wstring> m_altSubjectNames;
+	std::vector<SubjectName> m_altSubjectNames;
 };
 
 class CCertificateNotification final : public CAsyncRequestNotification
@@ -402,7 +420,9 @@ public:
 		std::wstring const& sessionCipher,
 		std::wstring const& sessionMac,
 		int algorithmWarnings,
-		std::vector<CCertificate> && certificates);
+		std::vector<CCertificate> && certificates,
+		bool systemTrust,
+		bool hostnameMismatch);
 	virtual RequestId GetRequestID() const { return reqId_certificate; }
 
 	std::wstring const& GetHost() const { return m_host; }
@@ -428,6 +448,9 @@ public:
 
 	int GetAlgorithmWarnings() const { return m_algorithmWarnings; }
 
+	bool SystemTrust() const { return systemTrust_; }
+	bool MismatchedHostname() const { return hostnameMismatch_; }
+
 private:
 	std::wstring m_host;
 	unsigned int m_port{};
@@ -439,6 +462,9 @@ private:
 	int m_algorithmWarnings{};
 
 	std::vector<CCertificate> m_certificates;
+
+	bool systemTrust_{};
+	bool hostnameMismatch_{};
 };
 
 class CSftpEncryptionNotification final : public CNotificationHelper<nId_sftp_encryption>, public CSftpEncryptionDetails
@@ -451,4 +477,14 @@ public:
 	CLocalPath dir;
 };
 
+class CInsecureFTPNotification final : public CAsyncRequestNotification
+{
+public:
+	CInsecureFTPNotification(CServer const& server);
+	virtual RequestId GetRequestID() const { return reqId_insecure_ftp; }
+
+	CServer const server_;
+	bool allow_{};
+};
+
 #endif
