@@ -18,7 +18,23 @@ enum ServerProtocol
 
 	STORJ,
 
-	MAX_VALUE = STORJ
+	WEBDAV,
+
+	AZURE_FILE,
+	AZURE_BLOB,
+
+	SWIFT,
+
+	GOOGLE_CLOUD,
+	GOOGLE_DRIVE,
+
+	DROPBOX,
+
+	ONEDRIVE,
+
+	B2,
+
+	MAX_VALUE = B2
 };
 
 enum ServerType
@@ -61,6 +77,21 @@ enum CharsetEncoding
 	ENCODING_CUSTOM
 };
 
+enum class ProtocolFeature
+{
+	DataTypeConcept,		// Some protocol distinguish between ASCII and binary files for line-ending conversion.
+	TransferMode,
+	PreserveTimestamp,
+	Charset,
+	ServerType,
+	EnterCommand,
+	DirectoryRename,
+	PostLoginCommands,
+	S3Lifecycle,
+	RecursiveDelete,
+	ServerAssignedHome
+};
+
 class Credentials;
 class CServerPath;
 class CServer final
@@ -113,18 +144,16 @@ public:
 	static std::wstring GetProtocolName(ServerProtocol protocol);
 	static ServerProtocol GetProtocolFromName(std::wstring const& name);
 
-	static ServerProtocol GetProtocolFromPrefix(std::wstring const& prefix);
+	static ServerProtocol GetProtocolFromPrefix(std::wstring const& prefix, ServerProtocol const hint = UNKNOWN);
 	static std::wstring GetPrefixFromProtocol(ServerProtocol const protocol);
 	static std::vector<ServerProtocol> const& GetDefaultProtocols();
 
-	// Some protocol distinguish between ASCII and binary files for line-ending
-	// conversion.
-	static bool ProtocolHasDataTypeConcept(ServerProtocol const protocol);
+	bool HasFeature(ProtocolFeature const feature) const;
+	static bool ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature const feature);
 
 	// These commands will be executed after a successful login.
 	std::vector<std::wstring> const& GetPostLoginCommands() const { return m_postLoginCommands; }
 	bool SetPostLoginCommands(std::vector<std::wstring> const& postLoginCommands);
-	static bool SupportsPostLoginCommands(ServerProtocol const protocol);
 
 	void SetBypassProxy(bool val);
 
@@ -139,6 +168,11 @@ public:
 	bool empty() const { return m_host.empty(); }
 	explicit operator bool() const { return !empty(); }
 
+	void ClearExtraParameters();
+	std::wstring GetExtraParameter(std::string const& name) const;
+	std::map<std::string, std::wstring> const& GetExtraParameters() const;
+	void SetExtraParameter(std::string const& name, std::wstring const& value);
+
 protected:
 	ServerProtocol m_protocol{UNKNOWN};
 	ServerType m_type{DEFAULT};
@@ -154,6 +188,8 @@ protected:
 
 	std::vector<std::wstring> m_postLoginCommands;
 	bool m_bypassProxy{};
+
+	std::map<std::string, std::wstring> extraParameters_;
 };
 
 
@@ -171,6 +207,40 @@ enum class LogonType
 std::wstring GetNameFromLogonType(LogonType type);
 LogonType GetLogonTypeFromName(std::wstring const& name);
 
+std::vector<LogonType> GetSupportedLogonTypes(ServerProtocol protocol);
+
+
+namespace ParameterSection {
+    enum type {
+		host,
+		user,
+		credentials,
+		extra,
+
+		section_count
+	};
+}
+
+struct ParameterTraits
+{
+	std::string name_;
+
+	ParameterSection::type section_;
+
+	enum flags : unsigned char {
+		optional = 0x01,
+		numeric = 0x02
+	};
+	unsigned char flags_;
+	std::wstring default_;
+	std::wstring hint_;
+};
+
+std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol protocol);
+
+std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol);
+
+bool ProtocolHasUser(ServerProtocol protocol);
 
 class Credentials
 {
@@ -192,8 +262,14 @@ public:
 	std::wstring account_;
 	std::wstring keyFile_;
 
+	void ClearExtraParameters();
+	std::wstring GetExtraParameter(std::string const& name) const;
+	std::map<std::string, std::wstring> const& GetExtraParameters() const;
+	void SetExtraParameter(ServerProtocol protocol, std::string const& name, std::wstring const& value);
+
 protected:
 	std::wstring password_;
+	std::map<std::string, std::wstring> extraParameters_;
 };
 
 #endif
