@@ -20,8 +20,7 @@ enum class socket_event_flag
 
 	connection,
 	read,
-	write,
-	close
+	write
 };
 
 /**
@@ -45,6 +44,16 @@ struct socket_event_type;
  * All socket events are sent through this.
  *
  * \sa \ref fz::socket_event_flag
+ *
+ * If the error value is non-zero for the connection, read and write events,
+ * the socket has failed and needs to be closed. Doing anything else with
+ * failed sockets is undefined behavior. 
+ * Failure events can be received at any time.
+ *
+ * Read and write events are edge-triggered:
+ * - After receiving a read event for a socket, it will not be sent again unless
+ * a subsequent call to socket::read has returned EAGAIN.
+ * - The same holds for the write event and socket::write
  */
 typedef simple_event<socket_event_type, socket_event_source*, socket_event_flag, int> socket_event;
 
@@ -79,117 +88,10 @@ void change_socket_event_handler(event_handler * old_handler, event_handler * ne
 /// \private
 class socket_thread;
 
-/**
- * \brief IPv6 capable, non-blocking socket class
- *
- * Uses and edge-triggered socket events.
- *
- * Error codes are the same as used by the POSIX socket functions,
- * see 'man 2 socket', 'man 2 connect', ...
- */
-class socket final : public socket_event_source
+/// \private
+class socket_base : public socket_event_source
 {
-	friend class socket_thread;
 public:
-	socket(thread_pool& pool, event_handler* evt_handler);
-	virtual ~socket();
-
-	socket(socket const&) = delete;
-	socket& operator=(socket const&) = delete;
-
-	enum socket_state
-	{
-		// How the socket is initially
-		none,
-
-		// Only in listening and connecting states you can get a connection event.
-		// After sending the event, socket is in connected state
-		listening,
-		connecting,
-
-		// Only in this state you can get send or receive events
-		connected,
-
-		// Graceful shutdown, you get close event once done
-		closing,
-		closed
-	};
-	socket_state get_state();
-
-	// Connects to the given host, given as name, IPv4 or IPv6 address.
-	// Returns 0 on success, else an error code. Note: EINPROGRESS is
-	// not really an error. On success, you should still wait for the
-	// connection event.
-	// If host is a name that can be resolved, a hostaddress socket event gets sent.
-	// Once connections got established, a connection event gets sent. If
-	// connection could not be established, a close event gets sent.
-	int connect(native_string const& host, unsigned int port, address_type family = address_type::unknown, std::string const& bind = std::string());
-
-	// After receiving a send or receive event, you can call these functions
-	// as long as their return value is positive.
-	int read(void *buffer, unsigned int size, int& error);
-	int peek(void *buffer, unsigned int size, int& error);
-	int write(const void *buffer, unsigned int size, int& error);
-
-	int close();
-
-	/**
-	 * \brief Returns local address of a connected socket
-	 *
-	 * \return empty string on error
-	 */
-	std::string local_ip(bool strip_zone_index = false) const;
-
-	/**
-	* \brief Returns remote address of a connected socket
-	*
-	* \return empty string on error
-	*/
-	std::string peer_ip(bool strip_zone_index = false) const;
-
-	/// Returns the hostname passed to Connect()
-	native_string peer_host() const;
-
-	/**
-	* \brief Returns local port of a connected socket
-	*
-	* \return -1 on error
-	*/
-	int local_port(int& error);
-
-	/**
-	* \brief Returns remote port of a connected socket
-	*
-	* \return -1 on error
-	*/
-	int remote_port(int& error);
-
-	/// If connected, either ipv4 or ipv6, unknown otherwise
-	address_type address_family() const;
-
-	/**
-	 * \brief Gets a symbolic name for socket errors.
-	 *
-	 * \example error_string(EAGAIN) == "EAGAIN"
-	 *
-	 * \return name if the error code is known
-	 * \return number as string if the error code is not known
-	 */
-	static std::string error_string(int error);
-
-	/**
-	 * \brief Gets a human-readable, translated description of the error
-	 */
-	static native_string error_description(int error);
-
-	void set_event_handler(event_handler* pEvtHandler);
-
-	static std::string address_to_string(sockaddr const* addr, int addr_len, bool with_port = true, bool strip_zone_index = false);
-	static std::string address_to_string(char const* buf, int buf_len);
-
-	int listen(address_type family, int port = 0);
-	socket* accept(int& error);
-
 	enum
 	{
 		/// flag_nodelay disables Nagle's algorithm
@@ -206,7 +108,7 @@ public:
 	 * \brief Sets socket buffer sizes.
 	 *
 	 * Internally this sets SO_RCVBUF and SO_SNDBUF on the socket.
-	 * 
+	 *
 	 * If called on listen socket, sizes will be inherited by accepted sockets.
 	 */
 	int set_buffer_sizes(int size_receive, int size_send);
@@ -218,6 +120,151 @@ public:
 	 */
 	void set_keepalive_interval(duration const& d);
 
+	/// If connected, either ipv4 or ipv6, unknown otherwise
+	address_type address_family() const;
+
+	/**
+	 * \brief Returns local address of a connected socket
+	 *
+	 * \return empty string on error
+	 */
+	std::string local_ip(bool strip_zone_index = false) const;
+
+	/**
+	* \brief Returns local port of a connected socket
+	*
+	* \return -1 on error
+	*/
+	int local_port(int& error);
+
+	static std::string address_to_string(sockaddr const* addr, int addr_len, bool with_port = true, bool strip_zone_index = false);
+	static std::string address_to_string(char const* buf, int buf_len);
+
+	int close();
+
+	void set_event_handler(event_handler* pEvtHandler);
+
+protected:
+	friend class socket_thread;
+
+	socket_base(thread_pool& pool, event_handler* evt_handler);
+
+	// Note: Unlocks the lock.
+	void detach_thread(scoped_lock & l);
+
+	thread_pool & thread_pool_;
+	event_handler* evt_handler_;
+
+	int fd_{-1};
+
+	socket_thread* socket_thread_{};
+
+	unsigned int port_{};
+
+	int family_;
+
+	int flags_{};
+	duration keepalive_interval_;
+
+	int buffer_sizes_[2];
+
+	int state_{};
+};
+
+class socket;
+
+class listen_socket final : public socket_base
+{
+	friend class socket_thread;
+public:
+	listen_socket(thread_pool& pool, event_handler* evt_handler);
+	virtual ~listen_socket();
+
+	listen_socket(listen_socket const&) = delete;
+	listen_socket& operator=(listen_socket const&) = delete;
+
+	int listen(address_type family, int port = 0);
+	socket* accept(int& error);
+
+	enum listen_socket_state
+	{
+		// How the socket is initially
+		none,
+
+		// Only in listening state you can get a connection event.
+		listening
+	};
+	listen_socket_state get_state();
+};
+
+/**
+ * \brief IPv6 capable, non-blocking socket class
+ *
+ * Uses and edge-triggered socket events.
+ *
+ * Error codes are the same as used by the POSIX socket functions,
+ * see 'man 2 socket', 'man 2 connect', ...
+ */
+class socket final : public socket_base
+{
+	friend class socket_thread;
+public:
+	socket(thread_pool& pool, event_handler* evt_handler);
+	virtual ~socket();
+
+	socket(socket const&) = delete;
+	socket& operator=(socket const&) = delete;
+
+	enum socket_state
+	{
+		// How the socket is initially
+		none,
+
+		// Only in connecting state you can get a connection event.
+		// After sending the event, socket is in connected state
+		connecting,
+
+		// Only in this state you can get send or receive events
+		connected,
+
+		closed
+	};
+	socket_state get_state();
+
+	// Connects to the given host, given as name, IPv4 or IPv6 address.
+	// Returns 0 on success, else an error code. Note: EINPROGRESS is
+	// not really an error. On success, you should still wait for the
+	// connection event.
+	// If host is a name that can be resolved, a hostaddress socket event gets
+	// sent.
+	// Once connections got established or establishment fails, a connection
+	// event gets sent, with the error parameter indicating success or failur.
+	// connection could not be established, a connection event with an error event gets sent.
+	int connect(native_string const& host, unsigned int port, address_type family = address_type::unknown, std::string const& bind = std::string());
+
+	// After receiving a send or receive event, you can call these functions
+	// as long as their return value is positive.
+	int read(void *buffer, unsigned int size, int& error);
+	int peek(void *buffer, unsigned int size, int& error);
+	int write(const void *buffer, unsigned int size, int& error);
+
+	/**
+	* \brief Returns remote address of a connected socket
+	*
+	* \return empty string on error
+	*/
+	std::string peer_ip(bool strip_zone_index = false) const;
+
+	/// Returns the hostname passed to Connect()
+	native_string peer_host() const;
+
+	/**
+	* \brief Returns remote port of a connected socket
+	*
+	* \return -1 on error
+	*/
+	int remote_port(int& error);
+
 	/**
 	 * On a connected socket, gets the ideal send buffer size or
 	 * -1 if it cannot be determined.
@@ -232,31 +279,16 @@ public:
 	 */
 	void retrigger(socket_event_flag event);
 
+	/**
+	 * \brief Signals peers that we want to close the connections.
+	 *
+	 * Implicitly done through close.
+	 */
+	int shutdown();
+
 private:
-	static int do_set_flags(int fd, int flags, int flags_mask, duration const& keepalive_interval);
-	static int do_set_buffer_sizes(int fd, int size_read, int size_write);
-	static int set_nonblocking(int fd);
-
-	// Note: Unlocks the lock.
-	void detach_thread(scoped_lock & l);
-
-	thread_pool & thread_pool_;
-	event_handler* evt_handler_;
-
-	int fd_{-1};
-
-	socket_state state_{none};
-
-	socket_thread* socket_thread_{};
-
+	friend class listen_socket;
 	native_string host_;
-	unsigned int port_{};
-	int family_;
-
-	int flags_{};
-	duration keepalive_interval_;
-
-	int buffer_sizes_[2];
 };
 
 #ifdef FZ_WINDOWS
