@@ -10,13 +10,13 @@
 #include "cmdline.h"
 #include "welcome_dialog.h"
 #include "msgbox.h"
+#include "themeprovider.h"
+#include "wxfilesystem_blob_handler.h"
 
 #include <libfilezilla/local_filesys.hpp>
 
 #include <wx/evtloop.h>
 
-#include "wxfilesystem_blob_handler.h"
-#include "xrc_helper.h"
 #ifdef __WXMSW__
 #include <wx/dynlib.h>
 #endif
@@ -70,8 +70,9 @@ extern "C"
 static void SetAppId()
 {
 	wxDynamicLibrary dll;
-	if (!dll.Load(_T("shell32.dll")))
+	if (!dll.Load(_T("shell32.dll"))) {
 		return;
+	}
 
 	if (!dll.HasSymbol(_T("SetCurrentProcessExplicitAppUserModelID"))) {
 		return;
@@ -232,6 +233,16 @@ bool CFileZillaApp::OnInit()
 		return false;
 	}
 
+#if USE_MAC_SANDBOX
+	// Set PUTTYDIR so that fzsftp uses the sandboxed home to put settings.
+	wxString home;
+	if (wxGetEnv(L"HOME", &home) && !home.empty()) {
+		if (home[home.size() - 1] != '/') {
+			home += '/';
+		}
+		wxSetEnv("PUTTYDIR", home + L".config/putty");
+	}
+#endif
 	InitDefaultsDir();
 
 	COptions::Init();
@@ -277,19 +288,36 @@ USE AT OWN RISK"), _T("Important Information"));
 	CSessionManager::Init();
 #endif
 
+	themeProvider_ = std::make_unique<CThemeProvider>();
+
 	// Load the text wrapping engine
 	m_pWrapEngine = std::make_unique<CWrapEngine>();
 	m_pWrapEngine->LoadCache();
 
+	bool welcome_skip = false;
 #ifdef USE_MAC_SANDBOX
 	OSXSandboxUserdirs::Get().Load();
+
+    if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
+		CWelcomeDialog welcome;
+		welcome.Run(nullptr, false);
+		welcome_skip = true;
+
+        OSXSandboxUserdirsDialog dlg;
+        dlg.Run(nullptr, true);
+	    if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
+			return false;
+		}
+    }
 #endif
 
 	CMainFrame *frame = new CMainFrame();
 	frame->Show(true);
 	SetTopWindow(frame);
 
-	CWelcomeDialog::RunDelayed(frame);
+	if (!welcome_skip) {
+		CWelcomeDialog::RunDelayed(frame);
+	}
 
 	frame->ProcessCommandLine();
 	frame->PostInitialize();
@@ -445,26 +473,6 @@ bool CFileZillaApp::LoadResourceFiles()
 
 	m_resourceDir.AddSegment(_T("resources"));
 
-	wxXmlResource *pResource = wxXmlResource::Get();
-
-#ifndef __WXDEBUG__
-	pResource->SetFlags(pResource->GetFlags() | wxXRC_NO_RELOADING);
-#endif
-
-	InitHandlers(*pResource);
-
-	fz::local_filesys fs;
-	std::wstring dir = m_resourceDir.GetPath() + L"xrc/";
-	pResource->LoadFile(wxString(dir + L"certificate.xrc"));
-	pResource->LoadFile(wxString(dir + L"dialogs.xrc"));
-	pResource->LoadFile(wxString(dir + L"inputdialog.xrc"));
-	pResource->LoadFile(wxString(dir + L"menus.xrc"));
-	pResource->LoadFile(wxString(dir + L"netconfwizard.xrc"));
-	pResource->LoadFile(wxString(dir + L"settings.xrc"));
-	pResource->LoadFile(wxString(dir + L"sitemanager.xrc"));
-	pResource->LoadFile(wxString(dir + L"update.xrc"));
-	pResource->LoadFile(wxString(dir + L"storj.xrc"));
-
 	// Useful for XRC files with embedded image data.
 	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
 
