@@ -2,6 +2,7 @@
 
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
+#include "view.h"
 #include "LocalListView.h"
 #include "queue.h"
 #include "filezillaapp.h"
@@ -28,18 +29,13 @@
 #include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/recursive_remove.hpp>
 
-class CLocalListViewDropTarget final : public CScrollableDropTarget<wxListCtrlEx>
+class CLocalListViewDropTarget final : public CFileDropTarget<wxListCtrlEx>
 {
 public:
 	CLocalListViewDropTarget(CLocalListView* pLocalListView)
-		: CScrollableDropTarget<wxListCtrlEx>(pLocalListView)
-		, m_pLocalListView(pLocalListView), m_pFileDataObject(new wxFileDataObject()),
-		m_pRemoteDataObject(new CRemoteDataObject())
+		: CFileDropTarget<wxListCtrlEx>(pLocalListView)
+		, m_pLocalListView(pLocalListView)
 	{
-		m_pDataObject = new wxDataObjectComposite;
-		m_pDataObject->Add(m_pRemoteDataObject, true);
-		m_pDataObject->Add(m_pFileDataObject, false);
-		SetDataObject(m_pDataObject);
 	}
 
 	void ClearDropHighlight()
@@ -105,9 +101,13 @@ public:
 			return wxDragError;
 		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
+		auto const format = m_pDataObject->GetReceivedFormat();
+		if (format == m_pFileDataObject->GetFormat()) {
 			m_pLocalListView->m_state.HandleDroppedFiles(m_pFileDataObject, dir, def == wxDragCopy);
 		}
+		else if (format == m_pLocalDataObject->GetFormat()) {
+			m_pLocalListView->m_state.HandleDroppedFiles(m_pLocalDataObject, dir, def == wxDragCopy);
+		}
 		else {
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
@@ -132,8 +132,9 @@ public:
 		CScrollableDropTarget<wxListCtrlEx>::OnDrop(x, y);
 		ClearDropHighlight();
 
-		if (m_pLocalListView->m_fileData.empty())
+		if (m_pLocalListView->m_fileData.empty()) {
 			return false;
+		}
 
 		return true;
 	}
@@ -150,33 +151,33 @@ public:
 
 		int flags;
 		int hit = m_pLocalListView->HitTest(point, flags, 0);
-		if (!(flags & wxLIST_HITTEST_ONITEM))
+		if (!(flags & wxLIST_HITTEST_ONITEM)) {
 			hit = -1;
+		}
 
-		if (hit != -1)
-		{
+		if (hit != -1) {
 			const CLocalFileData* const data = m_pLocalListView->GetData(hit);
-			if (!data || !data->dir)
+			if (!data || !data->dir) {
 				hit = -1;
-			else
-			{
+			}
+			else {
 				const CDragDropManager* pDragDropManager = CDragDropManager::Get();
-				if (pDragDropManager && pDragDropManager->pDragSource == m_pLocalListView)
-				{
-					if (m_pLocalListView->GetItemState(hit, wxLIST_STATE_SELECTED))
+				if (pDragDropManager && pDragDropManager->pDragSource == m_pLocalListView) {
+					if (m_pLocalListView->GetItemState(hit, wxLIST_STATE_SELECTED)) {
 						hit = -1;
-					else
+					}
+					else {
 						subDir = data->name;
+					}
 				}
-				else
+				else {
 					subDir = data->name;
+				}
 			}
 		}
-		if (hit != m_pLocalListView->m_dropTarget)
-		{
+		if (hit != m_pLocalListView->m_dropTarget) {
 			ClearDropHighlight();
-			if (hit != -1)
-			{
+			if (hit != -1) {
 				m_pLocalListView->m_dropTarget = hit;
 #ifdef __WXMSW__
 				m_pLocalListView->SetItemState(hit, wxLIST_STATE_DROPHILITED, wxLIST_STATE_DROPHILITED);
@@ -241,10 +242,7 @@ public:
 	}
 
 protected:
-	CLocalListView *m_pLocalListView;
-	wxFileDataObject* m_pFileDataObject;
-	CRemoteDataObject* m_pRemoteDataObject;
-	wxDataObjectComposite* m_pDataObject;
+	CLocalListView *m_pLocalListView{};
 };
 
 BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
@@ -269,9 +267,10 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CLocalListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQueue)
+CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue)
 	: CFileListCtrl<CLocalFileData>(pParent, pQueue),
-	CStateEventHandler(state)
+	CStateEventHandler(state),
+	m_parentView(pParent)
 {
 	wxGetApp().AddStartupProfileRecord("CLocalListView::CLocalListView");
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
@@ -279,8 +278,6 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQu
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
 	m_state.RegisterHandler(this, STATECHANGE_SERVER);
 
-	m_dropTarget = -1;
-
 	const unsigned long widths[4] = { 120, 80, 100, 120 };
 
 	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
@@ -291,10 +288,6 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQu
 
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
-#ifdef __WXMSW__
-	m_pVolumeEnumeratorThread = 0;
-#endif
-
 	InitHeaderSortImageList();
 
 	InitSort(OPTION_LOCALFILELIST_SORTORDER);
@@ -312,7 +305,7 @@ CLocalListView::~CLocalListView()
 	COptions::Get()->SetOption(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
 
 #ifdef __WXMSW__
-	delete m_pVolumeEnumeratorThread;
+	volumeEnumeratorThread_.reset();
 #endif
 }
 
@@ -321,8 +314,9 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 
 	CancelLabelEdit();
 
-	wxString focused;
-	std::list<wxString> selectedNames;
+	std::wstring focused;
+	int focusedItem = -1;
+	std::vector<std::wstring> selectedNames;
 	bool ensureVisible = false;
 	if (m_dir != dirname) {
 		ResetSearchPrefix();
@@ -345,7 +339,7 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 	}
 	else {
 		// Remember which items were selected
-		selectedNames = RememberSelectedItems(focused);
+		selectedNames = RememberSelectedItems(focused, focusedItem);
 	}
 
 	if (m_pFilelistStatusBar) {
@@ -387,7 +381,7 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 #ifdef __WXMSW__
 regular_dir:
 #endif
-		CFilterManager filter;
+		CStateFilterManager const& filter = m_state.GetStateFilterManager();
 		fz::local_filesys local_filesys;
 
 		if (!local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false)) {
@@ -414,24 +408,29 @@ regular_dir:
 
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
-				if (data.dir)
+				if (data.dir) {
 					totalDirCount++;
+				}
 				else {
-					if (data.size != -1)
+					if (data.size != -1) {
 						totalSize += data.size;
-					else
+					}
+					else {
 						unknown_sizes++;
+					}
 					totalFileCount++;
 				}
 				m_indexMapping.push_back(num);
 			}
-			else
+			else {
 				hidden++;
+			}
 			num++;
 		}
 
-		if (m_pFilelistStatusBar)
+		if (m_pFilelistStatusBar) {
 			m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
+		}
 	}
 
 	if (m_dropTarget != -1) {
@@ -443,8 +442,9 @@ regular_dir:
 	}
 
 	const int count = m_indexMapping.size();
-	if (oldItemCount != count)
+	if (oldItemCount != count) {
 		SetItemCount(count);
+	}
 
 	SortList(-1, -1, false);
 
@@ -453,7 +453,7 @@ regular_dir:
 		RefreshComparison();
 	}
 
-	ReselectItems(selectedNames, focused, ensureVisible);
+	ReselectItems(selectedNames, focused, focusedItem, ensureVisible);
 
 	RefreshListOnly();
 
@@ -465,21 +465,23 @@ int CLocalListView::OnGetItemImage(long item) const
 {
 	CLocalListView *pThis = const_cast<CLocalListView *>(this);
 	CLocalFileData *data = pThis->GetData(item);
-	if (!data)
+	if (!data) {
 		return -1;
+	}
 	int &icon = data->icon;
 
-	if (icon == -2)
-	{
-		wxString path = _T("");
-		if (data->name != _T(".."))
-		{
+	if (icon == -2) {
+		std::wstring path;
+		if (data->name != L"..") {
 #ifdef __WXMSW__
-			if (m_dir.GetPath() == _T("\\"))
-				path = data->name + _T("\\");
+			if (m_dir.GetPath() == L"\\") {
+				path = data->name + L"\\";
+			}
 			else
 #endif
+			{
 				path = m_dir.GetPath() + data->name;
+			}
 		}
 
 		icon = pThis->GetIconIndex(data->dir ? iconType::dir : iconType::file, path);
@@ -625,31 +627,31 @@ void CLocalListView::DisplayDrives()
 
 	std::list<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 	for (auto it = drives.begin(); it != drives.end(); ++it) {
-		wxString drive = *it;
-		if (drive.Right(1) == _T("\\")) {
-			drive.RemoveLast();
+		std::wstring drive = *it;
+		if (!drive.empty() && drive.back() == '\\') {
+			drive.pop_back();
 		}
 
 		CLocalFileData data;
 		data.name = drive;
-		data.label = fz::sparse_optional<wxString>(data.name);
+		data.label = fz::sparse_optional<std::wstring>(data.name);
 		data.dir = true;
 		data.size = -1;
-		data.icon = GetIconIndex(iconType::dir, _T(""), false);
+		data.icon = GetIconIndex(iconType::dir, std::wstring(), false);
 
 		m_fileData.push_back(data);
 		m_indexMapping.push_back(count);
-		count++;
+		++count;
 	}
 
-	if (m_pFilelistStatusBar)
+	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->SetDirectoryContents(0, drives.size(), 0, false, 0);
+	}
 
-	if (!m_pVolumeEnumeratorThread) {
-		m_pVolumeEnumeratorThread = new CVolumeDescriptionEnumeratorThread(this);
-		if (m_pVolumeEnumeratorThread->Failed()) {
-			delete m_pVolumeEnumeratorThread;
-			m_pVolumeEnumeratorThread = 0;
+	if (!volumeEnumeratorThread_) {
+		volumeEnumeratorThread_ = std::make_unique<CVolumeDescriptionEnumeratorThread>(this);
+		if (volumeEnumeratorThread_->Failed()) {
+			volumeEnumeratorThread_.reset();
 		}
 	}
 }
@@ -666,30 +668,31 @@ void CLocalListView::DisplayShares(wxString computer)
 	DWORD read, total;
 	DWORD resume_handle = 0;
 
-	if (!computer.empty() && computer.Last() == '\\')
+	if (!computer.empty() && computer.Last() == '\\') {
 		computer.RemoveLast();
+	}
 
 	int j = m_fileData.size();
 	int share_count = 0;
 	int res = 0;
-	do
-	{
+	do {
 		const wxWX2WCbuf buf = computer.wc_str(wxConvLocal);
 		res = NetShareEnum((wchar_t*)(const wchar_t*)buf, 1, &si.pShareInfoBlob, MAX_PREFERRED_LENGTH, &read, &total, &resume_handle);
 
-		if (res != ERROR_SUCCESS && res != ERROR_MORE_DATA)
+		if (res != ERROR_SUCCESS && res != ERROR_MORE_DATA) {
 			break;
+		}
 
 		SHARE_INFO_1* p = si.pShareInfo;
-		for (unsigned int i = 0; i < read; i++, p++)
-		{
-			if (p->shi1_type != STYPE_DISKTREE)
+		for (unsigned int i = 0; i < read; ++i, ++p) {
+			if (p->shi1_type != STYPE_DISKTREE) {
 				continue;
+			}
 
 			CLocalFileData data;
 			data.name = p->shi1_netname;
 #ifdef __WXMSW__
-			data.label = fz::sparse_optional<wxString>(data.name);
+			data.label = fz::sparse_optional<std::wstring>(data.name);
 #endif
 			data.dir = true;
 			data.size = -1;
@@ -704,56 +707,68 @@ void CLocalListView::DisplayShares(wxString computer)
 	}
 	while (res == ERROR_MORE_DATA);
 
-	if (m_pFilelistStatusBar)
+	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->SetDirectoryContents(0, share_count, 0, false, 0);
+	}
 }
 
 #endif //__WXMSW__
 
 CLocalFileData* CLocalListView::GetData(unsigned int item)
 {
-	if (!IsItemValid(item))
+	if (!IsItemValid(item)) {
 		return 0;
+	}
 
 	return &m_fileData[m_indexMapping[item]];
 }
 
 bool CLocalListView::IsItemValid(unsigned int item) const
 {
-	if (item >= m_indexMapping.size())
+	if (item >= m_indexMapping.size()) {
 		return false;
+	}
 
 	unsigned int index = m_indexMapping[item];
-	if (index >= m_fileData.size())
+	if (index >= m_fileData.size()) {
 		return false;
+	}
 
 	return true;
 }
 
-CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComparisonObject()
+std::unique_ptr<CFileListCtrlSortBase> CLocalListView::GetSortComparisonObject()
 {
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
 	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
 	if (!m_sortDirection) {
-		if (m_sortColumn == 1)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 2)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 3)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
+		if (m_sortColumn == 1) {
+			return std::make_unique<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 2) {
+			return std::make_unique<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 3) {
+			return std::make_unique<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else {
+			return std::make_unique<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
 	}
 	else {
-		if (m_sortColumn == 1)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 2)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 3)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
+		if (m_sortColumn == 1) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 2) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 3) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else {
+			return std::make_unique<CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
 	}
 }
 
@@ -764,15 +779,34 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		return;
 	}
 
-	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_LOCALFILELIST"));
-	if (!pMenu)
-		return;
+	wxMenu menu;
 
+	auto item = new wxMenuItem(&menu, XRCID("ID_UPLOAD"), _("&Upload"), _("Upload selected files and directories"));
+	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOAD"), wxART_MENU));
+	menu.Append(item);
+	item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add files to queue"), _("Add selected files and folders to the transfer queue"));
+	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOADADD"), wxART_MENU));
+	menu.Append(item);
+	menu.Append(XRCID("ID_ENTER"), _("E&nter directory"), _("Enter selected directory"));
+	
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_OPEN"), _("&Open"), _("Open the file."));		
+	menu.Append(XRCID("ID_EDIT"), _("&Edit"), _("Edit the file with the configured editor and upload changes to the server."));
+		
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));		
+	menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
+	menu.Append(XRCID("ID_CONTEXT_REFRESH"), _("Re&fresh"));
+		
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_DELETE"),_("&Delete"), _("Delete selected files and directories"));
+	menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected files and directories"));
+		
 	const bool connected = m_state.IsRemoteConnected();
 	if (!connected) {
-		pMenu->Enable(XRCID("ID_EDIT"), COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL) == 0);
-		pMenu->Enable(XRCID("ID_UPLOAD"), false);
-		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
+		menu.Enable(XRCID("ID_EDIT"), COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL) == 0);
+		menu.Enable(XRCID("ID_UPLOAD"), false);
+		menu.Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
 
 	int index = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -783,43 +817,45 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		++count;
 		const CLocalFileData* const data = GetData(index);
 		if (!data || (!index && m_hasParent)) {
-			pMenu->Enable(XRCID("ID_OPEN"), false);
-			pMenu->Enable(XRCID("ID_RENAME"), false);
-			pMenu->Enable(XRCID("ID_EDIT"), false);
+			menu.Enable(XRCID("ID_OPEN"), false);
+			menu.Enable(XRCID("ID_RENAME"), false);
+			menu.Enable(XRCID("ID_EDIT"), false);
 		}
-		if ((data && data->comparison_flags == fill) || (!index && m_hasParent))
+		if ((data && data->comparison_flags == fill) || (!index && m_hasParent)) {
 			fillCount++;
-		if (data && data->dir)
+		}
+		if (data && data->dir) {
 			selectedDir = true;
+		}
 		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
 	if (!count || fillCount == count) {
-		pMenu->Delete(XRCID("ID_ENTER"));
-		pMenu->Enable(XRCID("ID_UPLOAD"), false);
-		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
-		pMenu->Enable(XRCID("ID_DELETE"), false);
-		pMenu->Enable(XRCID("ID_RENAME"), false);
-		pMenu->Enable(XRCID("ID_EDIT"), false);
+		menu.Delete(XRCID("ID_ENTER"));
+		menu.Enable(XRCID("ID_UPLOAD"), false);
+		menu.Enable(XRCID("ID_ADDTOQUEUE"), false);
+		menu.Enable(XRCID("ID_DELETE"), false);
+		menu.Enable(XRCID("ID_RENAME"), false);
+		menu.Enable(XRCID("ID_EDIT"), false);
 	}
 	else if (count > 1) {
-		pMenu->Delete(XRCID("ID_ENTER"));
-		pMenu->Enable(XRCID("ID_RENAME"), false);
+		menu.Delete(XRCID("ID_ENTER"));
+		menu.Enable(XRCID("ID_RENAME"), false);
 	}
 	else {
 		// Exactly one item selected
-		if (!selectedDir)
-			pMenu->Delete(XRCID("ID_ENTER"));
+		if (!selectedDir) {
+			menu.Delete(XRCID("ID_ENTER"));
+		}
 	}
 	if (selectedDir) {
-		pMenu->Enable(XRCID("ID_EDIT"), false);
+		menu.Enable(XRCID("ID_EDIT"), false);
 		if (m_state.GetLocalRecursiveOperation() && m_state.GetLocalRecursiveOperation()->IsActive()) {
-			pMenu->Enable(XRCID("ID_UPLOAD"), false);
-			pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
+			menu.Enable(XRCID("ID_UPLOAD"), false);
+			menu.Enable(XRCID("ID_ADDTOQUEUE"), false);
 		}
 	}
 
-	PopupMenu(pMenu);
-	delete pMenu;
+	PopupMenu(&menu);
 }
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
@@ -942,7 +978,7 @@ wxString CLocalListView::MenuMkdir()
 		return wxString();
 	}
 
-	wxFileName fn(dlg.GetValue(), _T(""));
+	wxFileName fn(dlg.GetValue(), wxString());
 	fn.Normalize(wxPATH_NORM_ALL, m_dir.GetPath());
 
 	bool res;
@@ -966,15 +1002,18 @@ void CLocalListView::OnMenuDelete(wxCommandEvent&)
 	std::list<fz::native_string> pathsToDelete;
 	long item = -1;
 	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			continue;
+		}
 
 		CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			continue;
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		pathsToDelete.push_back(fz::to_native(m_dir.GetPath() + data->name));
 	}
@@ -1046,21 +1085,30 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 		wxCommandEvent cmdEvent;
 		OnMenuMkdir(cmdEvent);
 	}
-	else
+	else if (code == 'F' && event.GetModifiers() == wxMOD_CONTROL) {
+		if (m_parentView) {
+			m_parentView->ShowSearchPanel();
+		}
+	}
+	else {
 		event.Skip();
+	}
 }
 
 bool CLocalListView::OnBeginRename(const wxListEvent& event)
 {
-	if (!m_state.GetLocalDir().IsWriteable())
+	if (!m_state.GetLocalDir().IsWriteable()) {
 		return false;
+	}
 
-	if (event.GetIndex() == 0 && m_hasParent)
+	if (event.GetIndex() == 0 && m_hasParent) {
 		return false;
+	}
 
 	const CLocalFileData * const data = GetData(event.GetIndex());
-	if (!data || data->comparison_flags == fill)
+	if (!data || data->comparison_flags == fill) {
 		return false;
+	}
 
 	return true;
 }
@@ -1085,9 +1133,9 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 		return false;
 	}
 
-	wxString newname = event.GetLabel();
+	std::wstring newname = event.GetLabel().ToStdWstring();
 #ifdef __WXMSW__
-	newname = newname.Left(255);
+	newname = newname.substr(0, 255);
 #endif
 
 	if (newname == data->name) {
@@ -1109,8 +1157,8 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 
 void CLocalListView::ApplyCurrentFilter()
 {
-	CFilterManager filter;
-
+	CStateFilterManager const& filter = m_state.GetStateFilterManager();
+	
 	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing()) {
 		ExitComparisonMode();
 	}
@@ -1120,8 +1168,9 @@ void CLocalListView::ApplyCurrentFilter()
 		return;
 	}
 
-	wxString focused;
-	const std::list<wxString>& selectedNames = RememberSelectedItems(focused);
+	int focusedItem = -1;
+	std::wstring focused;
+	std::vector<std::wstring> const& selectedNames = RememberSelectedItems(focused, focusedItem);
 
 	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->UnselectAll();
@@ -1134,24 +1183,29 @@ void CLocalListView::ApplyCurrentFilter()
 	int hidden = 0;
 
 	m_indexMapping.clear();
-	if (m_hasParent)
+	if (m_hasParent) {
 		m_indexMapping.push_back(0);
+	}
 	for (unsigned int i = min; i < m_fileData.size(); ++i) {
 		const CLocalFileData& data = m_fileData[i];
-		if (data.comparison_flags == fill)
+		if (data.comparison_flags == fill) {
 			continue;
+		}
 		if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 			++hidden;
 			continue;
 		}
 
-		if (data.dir)
+		if (data.dir) {
 			++totalDirCount;
+		}
 		else {
-			if (data.size != -1)
+			if (data.size != -1) {
 				totalSize += data.size;
-			else
+			}
+			else {
 				++unknown_sizes;
+			}
 			++totalFileCount;
 		}
 
@@ -1159,8 +1213,9 @@ void CLocalListView::ApplyCurrentFilter()
 	}
 	SetItemCount(m_indexMapping.size());
 
-	if (m_pFilelistStatusBar)
+	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
+	}
 
 	SortList(-1, -1, false);
 
@@ -1169,15 +1224,16 @@ void CLocalListView::ApplyCurrentFilter()
 		RefreshComparison();
 	}
 
-	ReselectItems(selectedNames, focused);
+	ReselectItems(selectedNames, focused, focusedItem);
 
-	if (!IsComparing())
+	if (!IsComparing()) {
 		RefreshListOnly();
+	}
 }
 
-std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
+std::vector<std::wstring> CLocalListView::RememberSelectedItems(std::wstring& focused, int & focusedItem)
 {
-	std::list<wxString> selectedNames;
+	std::vector<std::wstring> selectedNames;
 	// Remember which items were selected
 #ifndef __WXMSW__
 	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
@@ -1193,31 +1249,35 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 			const CLocalFileData &data = m_fileData[m_indexMapping[item]];
 			if (data.comparison_flags != fill) {
 				if (data.dir) {
-					selectedNames.push_back(_T("d") + data.name);
+					selectedNames.push_back(L"d" + data.name);
 				}
 				else {
-					selectedNames.push_back(_T("-") + data.name);
+					selectedNames.push_back(L"-" + data.name);
 				}
 			}
 			SetSelection(item, false);
 		}
 	}
 
+	focusedItem = -1;
 	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
 	if (item >= 0 && static_cast<size_t>(item) < m_indexMapping.size()) {
 		const CLocalFileData &data = m_fileData[m_indexMapping[item]];
 		if (data.comparison_flags != fill) {
 			focused = data.name;
 		}
-
-		SetItemState(item, 0, wxLIST_STATE_FOCUSED);
+		focusedItem = item;
 	}
 
 	return selectedNames;
 }
 
-void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxString focused, bool ensureVisible)
+void CLocalListView::ReselectItems(const std::vector<std::wstring>& selectedNames, std::wstring focused, int focusedItem, bool ensureVisible)
 {
+	if (!GetItemCount()) {
+		return;
+	}
+
 	// Reselect previous items if neccessary.
 	// Sorting direction did not change. We just have to scan through items once
 
@@ -1235,6 +1295,13 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 				return;
 			}
 		}
+
+		if (focusedItem != -1 && GetItemCount() != 0) {
+			if (focusedItem >= GetItemCount()) {
+				--focusedItem;
+			}
+			SetItemState(focusedItem, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+		}
 		return;
 	}
 
@@ -1250,6 +1317,7 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 					EnsureVisible(i);
 				}
 				focused.clear();
+				focusedItem = -1;
 			}
 			if (data.dir && selectedName == (_T("d") + data.name)) {
 				if (firstSelected == -1) {
@@ -1281,7 +1349,15 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 			SetItemState(firstSelected, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
 		}
 		else {
-			SetItemState(0, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+			if (GetItemCount() != 0) {
+				if (focusedItem == -1) {
+					focusedItem = 0;
+				}
+				else if (focusedItem >= GetItemCount()) {
+					focusedItem = GetItemCount() - 1;
+				}
+				SetItemState(focusedItem, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+			}
 		}
 	}
 }
@@ -1299,7 +1375,7 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, con
 	}
 	else {
 		wxASSERT(notification == STATECHANGE_LOCAL_REFRESH_FILE);
-		RefreshFile(data);
+		RefreshFile(data.ToStdWstring());
 	}
 }
 
@@ -1312,38 +1388,52 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			return;
+		}
 	}
 
+#ifdef __WXMAC__
+	// Don't use wxFileDataObject on Mac, crashes on Mojave, wx bug #18232
+	CLocalDataObject obj;
+#else
 	wxFileDataObject obj;
+#endif
 
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
 	pDragDropManager->localParent = m_dir;
 
+	auto const path = m_dir.GetPath();
+
+	bool added = false;
+
 	item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
 		CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			continue;
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
-		wxString const name = m_dir.GetPath() + data->name;
-		pDragDropManager->m_localFiles.push_back(name);
+		std::wstring name = path + data->name;
 		obj.AddFile(name);
+		added = true;
 	}
 
-	if (obj.GetFilenames().empty()) {
+	if (!added) {
 		pDragDropManager->Release();
 		return;
 	}
@@ -1365,21 +1455,23 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	}
 }
 
-void CLocalListView::RefreshFile(const wxString& file)
+void CLocalListView::RefreshFile(std::wstring const& file)
 {
 	CLocalFileData data;
 
 	bool wasLink;
 	fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(m_dir.GetPath() + file), wasLink, &data.size, &data.time, &data.attributes);
-	if (type == fz::local_filesys::unknown)
+	if (type == fz::local_filesys::unknown) {
 		return;
+	}
 
 	data.name = file;
 	data.dir = type == fz::local_filesys::dir;
 
-	CFilterManager filter;
-	if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time))
+	CStateFilterManager const& filter = m_state.GetStateFilterManager();
+	if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 		return;
+	}
 
 	CancelLabelEdit();
 
@@ -1387,8 +1479,9 @@ void CLocalListView::RefreshFile(const wxString& file)
 	unsigned int i = 0;
 	for (auto iter = m_fileData.begin(); iter != m_fileData.end(); ++iter, ++i) {
 		const CLocalFileData& oldData = *iter;
-		if (oldData.name != file)
+		if (oldData.name != file) {
 			continue;
+		}
 
 		// Update file list status bar
 		if (m_pFilelistStatusBar) {
@@ -1400,31 +1493,41 @@ void CLocalListView::RefreshFile(const wxString& file)
 				int item = -1;
 				for (;;) {
 					item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-					if (item == -1)
+					if (item == -1) {
 						break;
-					if (m_indexMapping[item] != i)
+					}
+					if (m_indexMapping[item] != i) {
 						continue;
+					}
 
-					if (oldData.dir)
+					if (oldData.dir) {
 						m_pFilelistStatusBar->UnselectDirectory();
-					else
+					}
+					else {
 						m_pFilelistStatusBar->UnselectFile(oldData.size);
-					if (data.dir)
+					}
+					if (data.dir) {
 						m_pFilelistStatusBar->SelectDirectory();
-					else
+					}
+					else {
 						m_pFilelistStatusBar->SelectFile(data.size);
+					}
 					break;
 				}
 			}
 
-			if (oldData.dir)
+			if (oldData.dir) {
 				m_pFilelistStatusBar->RemoveDirectory();
-			else
+			}
+			else {
 				m_pFilelistStatusBar->RemoveFile(oldData.size);
-			if (data.dir)
+			}
+			if (data.dir) {
 				m_pFilelistStatusBar->AddDirectory();
-			else
+			}
+			else {
 				m_pFilelistStatusBar->AddFile(data.size);
+			}
 		}
 
 		// Update the data
@@ -1436,22 +1539,26 @@ void CLocalListView::RefreshFile(const wxString& file)
 			RefreshComparison();
 		}
 		else {
-			if (m_sortColumn)
+			if (m_sortColumn) {
 				SortList();
+			}
 			RefreshListOnly(false);
 		}
 		return;
 	}
 
-	if (data.dir)
+	if (data.dir) {
 		m_pFilelistStatusBar->AddDirectory();
-	else
+	}
+	else {
 		m_pFilelistStatusBar->AddFile(data.size);
+	}
 
-	wxString focused;
-	std::list<wxString> selectedNames;
+	std::wstring focused;
+	int focusedItem = -1;
+	std::vector<std::wstring> selectedNames;
 	if (IsComparing()) {
-		selectedNames = RememberSelectedItems(focused);
+		selectedNames = RememberSelectedItems(focused, focusedItem);
 		if (!m_originalIndexMapping.empty()) {
 			m_indexMapping.clear();
 			m_originalIndexMapping.swap(m_indexMapping);
@@ -1464,11 +1571,11 @@ void CLocalListView::RefreshFile(const wxString& file)
 
 	// Find correct position in index mapping
 	std::vector<unsigned int>::iterator start = m_indexMapping.begin();
-	if (m_hasParent)
+	if (m_hasParent) {
 		++start;
-	CFileListCtrl<CLocalFileData>::CSortComparisonObject compare = GetSortComparisonObject();
-	std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), index, compare);
-	compare.Destroy();
+	}
+	std::unique_ptr<CFileListCtrlSortBase> compare = GetSortComparisonObject();
+	std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), index, SortPredicate(compare));
 
 	const int item = insertPos - m_indexMapping.begin();
 	m_indexMapping.insert(insertPos, index);
@@ -1490,9 +1597,10 @@ void CLocalListView::RefreshFile(const wxString& file)
 	}
 	else {
 		RefreshComparison();
-		if (m_pFilelistStatusBar)
+		if (m_pFilelistStatusBar) {
 			m_pFilelistStatusBar->UnselectAll();
-		ReselectItems(selectedNames, focused);
+		}
+		ReselectItems(selectedNames, focused, focusedItem);
 	}
 }
 
@@ -1501,12 +1609,14 @@ wxListItemAttr* CLocalListView::OnGetItemAttr(long item) const
 	CLocalListView *pThis = const_cast<CLocalListView *>(this);
 	const CLocalFileData* const data = pThis->GetData((unsigned int)item);
 
-	if (!data)
+	if (!data) {
 		return 0;
+	}
 
 #ifndef __WXMSW__
-	if (item == m_dropTarget)
+	if (item == m_dropTarget) {
 		return &pThis->m_dropHighlightAttribute;
+	}
 #endif
 
 	switch (data->comparison_flags)
@@ -1531,16 +1641,17 @@ void CLocalListView::StartComparison()
 
 	ComparisonRememberSelections();
 
-	if (m_originalIndexMapping.empty())
+	if (m_originalIndexMapping.empty()) {
 		m_originalIndexMapping.swap(m_indexMapping);
-	else
+	}
+	else {
 		m_indexMapping.clear();
+	}
 
 	m_comparisonIndex = -1;
 
 	const CLocalFileData& last = m_fileData[m_fileData.size() - 1];
-	if (last.comparison_flags != fill)
-	{
+	if (last.comparison_flags != fill) {
 		CLocalFileData data;
 		data.dir = false;
 		data.icon = -1;
@@ -1552,12 +1663,14 @@ void CLocalListView::StartComparison()
 
 bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz::datetime& date)
 {
-	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size())
+	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
+	}
 
 	const unsigned int index = m_originalIndexMapping[m_comparisonIndex];
-	if (index >= m_fileData.size())
+	if (index >= m_fileData.size()) {
 		return false;
+	}
 
 	const CLocalFileData& data = m_fileData[index];
 
@@ -1578,8 +1691,9 @@ void CLocalListView::FinishComparison()
 	RefreshListOnly();
 
 	CComparableListing* pOther = GetOther();
-	if (!pOther)
+	if (!pOther) {
 		return;
+	}
 
 	pOther->ScrollTopItem(GetTopItem());
 }
@@ -1663,16 +1777,18 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 		}
 
 		const CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			continue;
+		}
 
 		if (data->dir) {
 			wxBell();
 			return;
 		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		selected_item.push_back({m_dir.GetPath() + data->name, data->size});
 	}
@@ -1699,11 +1815,13 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 		}
 
 		const CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			continue;
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		selected_item_list.push_back(*data);
 	}
@@ -1724,8 +1842,9 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 		dlg.SetTitle(_("Confirmation needed"));
 		dlg.AddText(_("You have selected more than 10 files or directories to open, do you really want to continue?"));
 
-		if (!dlg.Run())
+		if (!dlg.Run()) {
 			return;
+		}
 	}
 
 	for (auto const& data : selected_item_list) {
@@ -1762,8 +1881,9 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 			continue;
 		}
 
-		if (wxExecute(cmd))
+		if (wxExecute(cmd)) {
 			continue;
+		}
 
 		wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 	}
@@ -1783,16 +1903,15 @@ int64_t CLocalListView::ItemGetSize(int index) const
 
 void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 {
-	if (!m_pVolumeEnumeratorThread) {
+	if (!volumeEnumeratorThread_) {
 		return;
 	}
 
 	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
-	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
+	volumeInfo = volumeEnumeratorThread_->GetVolumes();
 
 	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED) {
-		delete m_pVolumeEnumeratorThread;
-		m_pVolumeEnumeratorThread = 0;
+		volumeEnumeratorThread_.reset();
 	}
 
 	if (m_dir.GetPath() != _T("\\")) {
@@ -1800,12 +1919,12 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 	}
 
 	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
-		wxString drive = iter->volume;
+		std::wstring drive = iter->volume;
 
 		unsigned int item, index;
 		for (item = m_hasParent ? 1 : 0; item < m_indexMapping.size(); ++item) {
 			index = m_indexMapping[item];
-			if (m_fileData[index].name == drive || m_fileData[index].name.substr(0, drive.Len() + 1) == drive + _T(" ")) {
+			if (m_fileData[index].name == drive || m_fileData[index].name.substr(0, drive.size() + 1) == drive + _T(" ")) {
 				break;
 			}
 		}
@@ -1814,7 +1933,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		}
 
 		if (!iter->volumeName.empty()) {
-			m_fileData[index].label = fz::sparse_optional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
+			m_fileData[index].label = fz::sparse_optional<std::wstring>(drive + _T(" (") + iter->volumeName + _T(")"));
 		}
 		if (iter->icon != -1) {
 			m_fileData[index].icon = iter->icon;
