@@ -35,37 +35,31 @@ public:
 	std::wstring m_known_subdir;
 };
 
-class CLocalTreeViewDropTarget final : public CScrollableDropTarget<wxTreeCtrlEx>
+class CLocalTreeViewDropTarget final : public CFileDropTarget<wxTreeCtrlEx>
 {
 public:
 	CLocalTreeViewDropTarget(CLocalTreeView* pLocalTreeView)
-		: CScrollableDropTarget(pLocalTreeView)
-		, m_pLocalTreeView(pLocalTreeView), m_pFileDataObject(new wxFileDataObject()),
-		m_pRemoteDataObject(new CRemoteDataObject())
+		: CFileDropTarget(pLocalTreeView)
+		, m_pLocalTreeView(pLocalTreeView)
 	{
-		m_pDataObject = new wxDataObjectComposite;
-		m_pDataObject->Add(m_pRemoteDataObject, true);
-		m_pDataObject->Add(m_pFileDataObject, false);
-		SetDataObject(m_pDataObject);
 	}
 
 	void ClearDropHighlight()
 	{
 		const wxTreeItemId dropHighlight = m_pLocalTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId())
-		{
+		if (dropHighlight != wxTreeItemId()) {
 			m_pLocalTreeView->SetItemDropHighlight(dropHighlight, false);
 			m_pLocalTreeView->m_dropHighlight = wxTreeItemId();
 		}
 	}
 
-	wxString GetDirFromItem(const wxTreeItemId& item)
+	std::wstring GetDirFromItem(wxTreeItemId const& item)
 	{
-		const wxString& dir = m_pLocalTreeView->GetDirFromItem(item);
+		std::wstring const dir = m_pLocalTreeView->GetDirFromItem(item);
 
 #ifdef __WXMSW__
-		if (dir == _T("/")) {
-			return wxString();
+		if (dir == L"/") {
+			return std::wstring();
 		}
 #endif
 
@@ -89,8 +83,10 @@ public:
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
+		{
 			return def;
-		if( def == wxDragLink ) {
+		}
+		if (def == wxDragLink) {
 			def = wxDragCopy;
 		}
 
@@ -99,7 +95,7 @@ public:
 			return wxDragNone;
 		}
 
-		const CLocalPath path(GetDirFromItem(hit).ToStdWstring());
+		const CLocalPath path(GetDirFromItem(hit));
 		if (path.empty() || !path.IsWriteable()) {
 			return wxDragNone;
 		}
@@ -113,9 +109,13 @@ public:
 			pDragDropManager->pDropTarget = m_pLocalTreeView;
 		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
+		auto const format = m_pDataObject->GetReceivedFormat();
+		if (format == m_pFileDataObject->GetFormat()) {
 			m_pLocalTreeView->m_state.HandleDroppedFiles(m_pFileDataObject, path, def == wxDragCopy);
 		}
+		else if (format == m_pLocalDataObject->GetFormat()) {
+			m_pLocalTreeView->m_state.HandleDroppedFiles(m_pLocalDataObject, path, def == wxDragCopy);
+		}
 		else {
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
@@ -148,7 +148,7 @@ public:
 			return false;
 		}
 
-		std::wstring const dir = GetDirFromItem(hit).ToStdWstring();
+		std::wstring const dir = GetDirFromItem(hit);
 		if (dir.empty() || !CLocalPath(dir).IsWriteable()) {
 			return false;
 		}
@@ -164,7 +164,7 @@ public:
 			return hit;
 		}
 
-		wxString dir = GetDirFromItem(hit);
+		std::wstring dir = GetDirFromItem(hit);
 
 		if (dir.empty()) {
 			ClearDropHighlight();
@@ -172,13 +172,13 @@ public:
 		}
 
 		const wxTreeItemId dropHighlight = m_pLocalTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId())
+		if (dropHighlight != wxTreeItemId()) {
 			m_pLocalTreeView->SetItemDropHighlight(dropHighlight, false);
+		}
 
 		m_pLocalTreeView->SetItemDropHighlight(hit, true);
 		m_pLocalTreeView->m_dropHighlight = hit;
 
-
 		return hit;
 	}
 
@@ -195,11 +195,13 @@ public:
 		}
 
 		wxTreeItemId hit = DisplayDropHighlight(wxPoint(x, y));
-		if (!hit.IsOk())
+		if (!hit.IsOk()) {
 			return wxDragNone;
+		}
 
-		if (def == wxDragLink)
+		if (def == wxDragLink) {
 			def = wxDragCopy;
+		}
 
 		return def;
 	}
@@ -217,10 +219,7 @@ public:
 	}
 
 protected:
-	CLocalTreeView *m_pLocalTreeView;
-	wxFileDataObject* m_pFileDataObject;
-	CRemoteDataObject* m_pRemoteDataObject;
-	wxDataObjectComposite* m_pDataObject;
+	CLocalTreeView *m_pLocalTreeView{};
 };
 
 BEGIN_EVENT_TABLE(CLocalTreeView, wxTreeCtrlEx)
@@ -309,7 +308,7 @@ void CLocalTreeView::SetDir(wxString const& localDir)
 
 	if (localDir.Left(2) == _T("\\\\")) {
 		// TODO: UNC path, don't display it yet
-		m_currentDir = _T("");
+		m_currentDir.clear();
 		SafeSelectItem(wxTreeItemId());
 		return;
 	}
@@ -333,7 +332,7 @@ void CLocalTreeView::SetDir(wxString const& localDir)
 		SafeSelectItem(parent);
 		return;
 	}
-	wxTreeItemId item = MakeSubdirs(parent, localDir.Left(localDir.Length() - subDirs.Length()), subDirs);
+	wxTreeItemId item = MakeSubdirs(parent, localDir.Left(localDir.Length() - subDirs.Length()).ToStdWstring(), subDirs);
 	if (!item) {
 		return;
 	}
@@ -346,8 +345,9 @@ wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
 	const wxString separator = wxFileName::GetPathSeparator();
 #ifdef __WXMSW__
 	int pos = localDir.Find(separator);
-	if (pos == -1)
+	if (pos == -1) {
 		return wxTreeItemId();
+	}
 
 	wxString drive = localDir.Left(pos);
 	localDir = localDir.Mid(pos + 1);
@@ -355,36 +355,43 @@ wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
 	wxTreeItemIdValue value;
 	wxTreeItemId root = GetFirstChild(m_drives, value);
 	while (root) {
-		if (!GetItemText(root).Left(drive.Length()).CmpNoCase(drive))
+		if (!GetItemText(root).Left(drive.Length()).CmpNoCase(drive)) {
 			break;
+		}
 
 		root = GetNextSibling(root);
 	}
 	if (!root) {
-		if (drive[1] == ':')
+		if (drive[1] == ':') {
 			return AddDrive(drive[0]);
+		}
 		return wxTreeItemId();
 	}
 #else
-		if (localDir[0] == '/')
-			localDir = localDir.Mid(1);
-		wxTreeItemId root = GetRootItem();
+	if (localDir[0] == '/') {
+		localDir = localDir.Mid(1);
+	}
+	wxTreeItemId root = GetRootItem();
 #endif
 
 	while (!localDir.empty()) {
 		wxString subDir;
 		int pos2 = localDir.Find(separator);
-		if (pos2 == -1)
+		if (pos2 == -1) {
 			subDir = localDir;
-		else
+		}
+		else {
 			subDir = localDir.Left(pos2);
+		}
 
 		wxTreeItemId child = GetSubdir(root, subDir);
-		if (!child)
+		if (!child) {
 			return root;
+		}
 
-		if (!pos2)
+		if (!pos2) {
 			return child;
+		}
 
 		root = child;
 		localDir = localDir.Mid(pos2 + 1);
@@ -404,7 +411,9 @@ wxTreeItemId CLocalTreeView::GetSubdir(wxTreeItemId parent, const wxString& subD
 #else
 		if (GetItemText(child) == subDir)
 #endif
+		{
 			return child;
+		}
 
 		child = GetNextSibling(child);
 	}
@@ -427,8 +436,9 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 
 	for (auto it = drives.begin(); it != drives.end(); ++it) {
 		wxString drive = *it;
-		if (drive.Right(1) == _T("\\"))
+		if (drive.Right(1) == _T("\\")) {
 			drive = drive.RemoveLast();
+		}
 
 		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(iconType::dir, _T(""), false));
 		AppendItem(item, _T(""));
@@ -442,36 +452,33 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 
 #endif
 
-void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, std::wstring const& knownSubdir)
+void CLocalTreeView::DisplayDir(wxTreeItemId parent, std::wstring const& dirname, std::wstring const& knownSubdir)
 {
 	fz::local_filesys local_filesys;
 
-	{
-		wxLogNull log;
-		if (!local_filesys.begin_find_files(fz::to_native(dirname), true)) {
-			if (!knownSubdir.empty()) {
-				wxTreeItemId item = GetSubdir(parent, knownSubdir);
-				if (item != wxTreeItemId()) {
-					return;
-				}
+	if (!local_filesys.begin_find_files(fz::to_native(dirname), true)) {
+		if (!knownSubdir.empty()) {
+			wxTreeItemId item = GetSubdir(parent, knownSubdir);
+			if (item != wxTreeItemId()) {
+				return;
+			}
 
-				const wxString fullName = dirname + knownSubdir;
-				item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
+			std::wstring const fullName = dirname + knownSubdir;
+			item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
-						-1
+					-1
 #else
-						GetIconIndex(iconType::opened_dir, fullName)
+					GetIconIndex(iconType::opened_dir, fullName)
 #endif
-					);
-				CheckSubdirStatus(item, fullName);
-			}
-			else {
-				++m_setSelection;
-				DeleteChildren(parent);
-				--m_setSelection;
-			}
-			return;
+				);
+			CheckSubdirStatus(item, fullName);
 		}
+		else {
+			++m_setSelection;
+			DeleteChildren(parent);
+			--m_setSelection;
+		}
+		return;
 	}
 
 	wxASSERT(parent);
@@ -497,7 +504,7 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, st
 			continue;
 		}
 
-		std::wstring fullName = dirname.ToStdWstring() + wfile;
+		std::wstring fullName = dirname + wfile;
 #ifdef __WXMSW__
 		if (fz::stricmp(wfile, knownSubdir))
 #else
@@ -524,7 +531,7 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, st
 	}
 
 	if (!matchedKnown && !knownSubdir.empty()) {
-		const wxString fullName = dirname + knownSubdir;
+		std::wstring const& fullName = dirname + knownSubdir;
 		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 				-1
@@ -539,7 +546,7 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, st
 	SortChildren(parent);
 }
 
-wxString CLocalTreeView::HasSubdir(const wxString& dirname)
+std::wstring CLocalTreeView::HasSubdir(std::wstring const& dirname)
 {
 	wxLogNull nullLog;
 
@@ -547,7 +554,7 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 
 	fz::local_filesys local_filesys;
 	if (!local_filesys.begin_find_files(fz::to_native(dirname), true)) {
-		return wxString();
+		return std::wstring();
 	}
 
 	fz::native_string file;
@@ -571,37 +578,37 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 		return wfile;
 	}
 
-	return wxString();
+	return std::wstring();
 }
 
-wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname, wxString subDir)
+wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, std::wstring dirname, wxString subDir)
 {
-	const wxString& separator = wxFileName::GetPathSeparator();
-
-	wxString segment;
+	std::wstring segment;
 	while (!subDir.empty()) {
-		int pos = subDir.Find(separator);
+		int pos = subDir.Find(fz::local_filesys::path_separator);
 		if (!pos) {
 			subDir = subDir.Mid(1);
 			continue;
 		}
 		else if (pos == -1) {
-			segment = subDir;
-			subDir = _T("");
+			segment = subDir.ToStdWstring();
+			subDir.clear();
 		}
 		else {
-			segment = subDir.Left(pos);
+			segment = subDir.Left(pos).ToStdWstring();
 			subDir = subDir.Mid(pos + 1);
 		}
 
-		DisplayDir(parent, dirname, segment.ToStdWstring());
+		DisplayDir(parent, dirname, segment);
 
 		wxTreeItemId item = GetSubdir(parent, segment);
-		if (!item)
+		if (!item) {
 			return wxTreeItemId();
+		}
 
 		parent = item;
-		dirname += segment + separator;
+		dirname += segment;
+		dirname += fz::local_filesys::path_separator;
 	}
 
 	// Not needed, stays unexpanded by default
@@ -621,66 +628,67 @@ void CLocalTreeView::OnItemExpanding(wxTreeEvent& event)
 	}
 }
 
-wxString CLocalTreeView::GetDirFromItem(wxTreeItemId item)
+std::wstring CLocalTreeView::GetDirFromItem(wxTreeItemId item)
 {
-	const wxString separator = wxFileName::GetPathSeparator();
-	wxString dir;
-	while (item)
-	{
+	wchar_t const separator = fz::local_filesys::path_separator;
+	std::wstring dir;
+	while (item) {
 #ifdef __WXMSW__
 		if (item == m_desktop) {
 			wxChar path[MAX_PATH + 1];
 			if (SHGetFolderPath(0, CSIDL_DESKTOPDIRECTORY, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
 				if (SHGetFolderPath(0, CSIDL_DESKTOP, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
 					wxMessageBoxEx(_("Failed to get desktop path"));
-					return _T("/");
+					return L"/";
 				}
 			}
 			dir = path;
-			if (dir.empty() || dir.Last() != separator)
+			if (dir.empty() || dir.back() != separator) {
 				dir += separator;
+			}
 			return dir;
 		}
 		else if (item == m_documents) {
 			wxChar path[MAX_PATH + 1];
 			if (SHGetFolderPath(0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
 				wxMessageBoxEx(_("Failed to get 'My Documents' path"));
-				return _T("/");
+				return L"/";
 			}
 			dir = path;
-			if (dir.empty() || dir.Last() != separator)
+			if (dir.empty() || dir.back() != separator) {
 				dir += separator;
+			}
 			return dir;
 		}
-		else if (item == m_drives)
-			return _T("/");
+		else if (item == m_drives) {
+			return L"/";
+		}
 		else if (GetItemParent(item) == m_drives) {
-			wxString text = GetItemText(item);
-			int pos = text.Find(_T(" "));
-			if (pos == -1)
+			std::wstring text = GetItemText(item).ToStdWstring();
+			size_t pos = text.find(' ');
+			if (pos == std::wstring::npos) {
 				return text + separator + dir;
-			else
-				return text.Left(pos) + separator + dir;
+			}
+			else {
+				return text.substr(0, pos) + separator + dir;
+			}
 		}
 		else
 #endif
-		if (item == GetRootItem())
-			return _T("/") + dir;
+		{
+			if (item == GetRootItem()) {
+				return L"/" + dir;
+			}
+		}
 
-		dir = GetItemText(item) + separator + dir;
+		dir = GetItemText(item).ToStdWstring() + separator + dir;
 
 		item = GetItemParent(item);
 	}
 
-	return separator;
+	return std::wstring(1, separator);
 }
 
-struct t_dir
-{
-	wxString dir;
-	wxTreeItemId item;
-};
-
 void CLocalTreeView::UpdateSortMode()
 {
 	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
@@ -698,11 +706,19 @@ void CLocalTreeView::UpdateSortMode()
 	}
 }
 
+namespace {
+struct t_dir final
+{
+	std::wstring dir;
+	wxTreeItemId item;
+};
+}
+
 void CLocalTreeView::RefreshListing()
 {
 	wxLogNull nullLog;
 
-	const wxString separator = wxFileName::GetPathSeparator();
+	wchar_t const separator = fz::local_filesys::path_separator;
 
 	std::list<t_dir> dirsToCheck;
 
@@ -712,10 +728,11 @@ void CLocalTreeView::RefreshListing()
 	wxTreeItemIdValue tmp;
 	for (auto child = GetFirstChild(m_drives, tmp); child; child = GetNextSibling(child)) {
 		if (IsExpanded(child)) {
-			wxString drive = GetItemText(child);
-			int pos = drive.Find(_T(" "));
-			if (pos != -1)
-				drive = drive.Left(pos);
+			std::wstring drive = GetItemText(child).ToStdWstring();
+			size_t pos = drive.find(' ' );
+			if (pos != std::wstring::npos) {
+				drive = drive.substr(0, pos);
+			}
 
 			t_dir dir;
 			dir.dir = drive + separator;
@@ -744,7 +761,7 @@ void CLocalTreeView::RefreshListing()
 			wxTreeItemIdValue value;
 			for (auto child = GetFirstChild(dir.item, value); child; child = GetNextSibling(child)) {
 				t_dir subdir;
-				subdir.dir = dir.dir + GetItemText(child) + separator;
+				subdir.dir = dir.dir + GetItemText(child).ToStdWstring() + separator;
 				subdir.item = child;
 				dirsToCheck.push_back(subdir);
 			}
@@ -783,21 +800,24 @@ void CLocalTreeView::RefreshListing()
 		auto iter = dirs.rbegin();
 		while (child || iter != dirs.rend()) {
 			int cmp;
-			if (child && iter != dirs.rend())
+			if (child && iter != dirs.rend()) {
 #ifdef __WXMSW__
 				cmp = GetItemText(child).CmpNoCase(*iter);
 #else
 				cmp = GetItemText(child).Cmp(*iter);
 #endif
-			else if (child)
+			}
+			else if (child) {
 				cmp = 1;
-			else
+			}
+			else {
 				cmp = -1;
+			}
 
 			if (!cmp) {
 				// Found item with same name. Mark it for further processing
 				if (!IsExpanded(child)) {
-					wxString path = dir.dir + *iter + separator;
+					std::wstring path = dir.dir + *iter + separator;
 					if (!CheckSubdirStatus(child, path)) {
 						t_dir subdir;
 						subdir.dir = path;
@@ -819,16 +839,18 @@ void CLocalTreeView::RefreshListing()
 				// Delete child from tree, unless current selection
 				// is in the subtree.
 				wxTreeItemId sel = GetSelection();
-				while (sel && sel != child)
+				while (sel && sel != child) {
 					sel = GetItemParent(sel);
+				}
 				wxTreeItemId prev = GetPrevSibling(child);
-				if (!sel)
+				if (!sel) {
 					Delete(child);
+				}
 				child = prev;
 			}
 			else if (cmp < 0) {
 				// New subdirectory, add treeitem
-				wxString fullname = dir.dir + *iter + separator;
+				std::wstring fullname = dir.dir + *iter + separator;
 				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(iconType::dir, fullname),
 #ifdef __WXMSW__
 						-1
@@ -842,8 +864,9 @@ void CLocalTreeView::RefreshListing()
 				inserted = true;
 			}
 		}
-		if (inserted)
+		if (inserted) {
 			SortChildren(dir.item);
+		}
 	}
 
 #ifdef __WXMSW__
@@ -863,7 +886,7 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 		return;
 	}
 
-	std::wstring dir = GetDirFromItem(item).ToStdWstring();
+	std::wstring dir = GetDirFromItem(item);
 
 	std::wstring error;
 	if (!m_state.SetLocalDir(dir, &error)) {
@@ -900,36 +923,47 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 	}
 
 	wxTreeItemId item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 #ifdef __WXMSW__
-	if (item == m_drives || item == m_desktop || item == m_documents)
+	if (item == m_drives || item == m_desktop || item == m_documents) {
 		return;
+	}
 #endif
 
-	wxString dir = GetDirFromItem(item);
-	if (dir == _T("/"))
+	std::wstring dir = GetDirFromItem(item);
+	if (dir == L"/") {
 		return;
+	}
 
 #ifdef __WXMSW__
-	if (!dir.empty() && dir.Last() == '\\')
-		dir.RemoveLast();
+	if (!dir.empty() && dir.back() == '\\') {
+		dir.pop_back();
+	}
 #endif
-	if (!dir.empty() && dir.Last() == '/')
-		dir.RemoveLast();
+	if (!dir.empty() && dir.back() == '/') {
+		dir.pop_back();
+	}
 
 #ifdef __WXMSW__
-	if (!dir.empty() && dir.Last() == ':')
+	if (!dir.empty() && dir.back() == ':') {
 		return;
+	}
 #endif
 
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
 
-	wxFileDataObject obj;
-
+#ifdef __WXMAC__
+	// Don't use wxFileDataObject on Mac, crashes on Mojave, wx bug #18232
+	CLocalDataObject obj;
 	obj.AddFile(dir);
+#else
+	wxFileDataObject obj;
+	obj.AddFile(dir);
+#endif
 
 	wxDropSource source(this);
 	source.SetData(obj);
@@ -939,8 +973,7 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	pDragDropManager->Release();
 
-	if (!handled_internally && (res == wxDragCopy || res == wxDragMove))
-	{
+	if (!handled_internally && (res == wxDragCopy || res == wxDragMove)) {
 		// We only need to refresh local side if the operation got handled
 		// externally, the internal handlers do this for us already
 		m_state.RefreshLocal();
@@ -981,8 +1014,7 @@ bool CLocalTreeView::CreateRoot()
 {
 	int iconIndex, openIconIndex;
 	wxString name = GetSpecialFolder(CSIDL_DESKTOP, iconIndex, openIconIndex);
-	if (name.empty())
-	{
+	if (name.empty()) {
 		name = _("Desktop");
 		iconIndex = openIconIndex = -1;
 	}
@@ -990,8 +1022,7 @@ bool CLocalTreeView::CreateRoot()
 	m_desktop = AddRoot(name, iconIndex, openIconIndex);
 
 	name = GetSpecialFolder(CSIDL_PERSONAL, iconIndex, openIconIndex);
-	if (name.empty())
-	{
+	if (name.empty()) {
 		name = _("My Documents");
 		iconIndex = openIconIndex = -1;
 	}
@@ -1000,8 +1031,7 @@ bool CLocalTreeView::CreateRoot()
 
 
 	name = GetSpecialFolder(CSIDL_DRIVES, iconIndex, openIconIndex);
-	if (name.empty())
-	{
+	if (name.empty()) {
 		name = _("My Computer");
 		iconIndex = openIconIndex = -1;
 	}
@@ -1060,12 +1090,26 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 		return;
 	}
 
-	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_LOCALTREE"));
-	if (!pMenu) {
-		return;
-	}
+	wxMenu menu;
+	auto item = new wxMenuItem(&menu, XRCID("ID_UPLOAD"), _("&Upload"), _("Upload selected directory"));
+	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOAD"), wxART_MENU));
+	menu.Append(item);
+	item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add to queue"), _("Add selected directory to the transfer queue"));
+	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOADADD"), wxART_MENU));
+	menu.Append(item);
 
-	CLocalPath const path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_OPEN"), _("&Open"), _("Open directory in system's file manager"));
+
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
+	menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
+
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_DELETE"), _("&Delete"), _("Delete selected directory"));
+	menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected directory"));
+
+	CLocalPath const path(GetDirFromItem(m_contextMenuItem));
 
 	const bool hasParent = path.HasParent();
 	const bool writeable = path.IsWriteable();
@@ -1074,14 +1118,13 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 
 	bool const idle = m_state.GetLocalRecursiveOperation() && !m_state.GetLocalRecursiveOperation()->IsActive();
 
-	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected && idle);
-	pMenu->Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected && idle);
-	pMenu->Enable(XRCID("ID_MKDIR"), writeable);
-	pMenu->Enable(XRCID("ID_DELETE"), writeable && hasParent);
-	pMenu->Enable(XRCID("ID_RENAME"), writeable && hasParent);
+	menu.Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected && idle);
+	menu.Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected && idle);
+	menu.Enable(XRCID("ID_MKDIR"), writeable);
+	menu.Enable(XRCID("ID_DELETE"), writeable && hasParent);
+	menu.Enable(XRCID("ID_RENAME"), writeable && hasParent);
 
-	PopupMenu(pMenu);
-	delete pMenu;
+	PopupMenu(&menu);
 }
 
 void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
@@ -1095,7 +1138,7 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 		return;
 	}
 
-	CLocalPath path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
+	CLocalPath path(GetDirFromItem(m_contextMenuItem));
 
 	if (!path.HasParent()) {
 		return;
@@ -1167,7 +1210,7 @@ wxString CLocalTreeView::MenuMkdir()
 		return wxString();
 	}
 
-	std::wstring path = GetDirFromItem(m_contextMenuItem).ToStdWstring();
+	std::wstring path = GetDirFromItem(m_contextMenuItem);
 	if (!path.empty() && path.back() != wxFileName::GetPathSeparator()) {
 		path += wxFileName::GetPathSeparator();
 	}
@@ -1222,7 +1265,7 @@ void CLocalTreeView::OnMenuRename(wxCommandEvent&)
 	}
 #endif
 
-	CLocalPath path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
+	CLocalPath path(GetDirFromItem(m_contextMenuItem));
 	if (!path.HasParent() || !path.IsWriteable()) {
 		wxBell();
 		return;
@@ -1237,7 +1280,7 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent&)
 		return;
 	}
 
-	std::wstring path = GetDirFromItem(m_contextMenuItem).ToStdWstring();
+	std::wstring path = GetDirFromItem(m_contextMenuItem);
 
 	CLocalPath local_path(path);
 	if (!local_path.HasParent() || !local_path.IsWriteable()) {
@@ -1290,7 +1333,7 @@ void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 	}
 #endif
 
-	CLocalPath path(GetDirFromItem(item).ToStdWstring());
+	CLocalPath path(GetDirFromItem(item));
 
 	if (!path.HasParent() || !path.IsWriteable())
 	{
@@ -1317,7 +1360,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	}
 #endif
 
-	std::wstring path = GetDirFromItem(item).ToStdWstring();
+	std::wstring path = GetDirFromItem(item);
 
 	CLocalPath local_path(path);
 	if (!local_path.HasParent() || !local_path.IsWriteable()) {
@@ -1393,15 +1436,18 @@ void CLocalTreeView::OnChar(wxKeyEvent& event)
 	m_contextMenuItem = GetSelection();
 
 	wxCommandEvent cmdEvt;
-	if (event.GetKeyCode() == WXK_F2)
+	if (event.GetKeyCode() == WXK_F2) {
 		OnMenuRename(cmdEvt);
-	else if (event.GetKeyCode() == WXK_DELETE || event.GetKeyCode() == WXK_NUMPAD_DELETE)
+	}
+	else if (event.GetKeyCode() == WXK_DELETE || event.GetKeyCode() == WXK_NUMPAD_DELETE) {
 		OnMenuDelete(cmdEvt);
-	else
+	}
+	else {
 		event.Skip();
+	}
 }
 
-bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
+bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, std::wstring const& path)
 {
 	wxTreeItemIdValue value;
 	wxTreeItemId child = GetFirstChild(item, value);
@@ -1411,15 +1457,14 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 #ifdef __WXMAC__
 	// By default, OS X has a list of servers mounted into /net,
 	// listing that directory is slow.
-	if (GetItemParent(item) == GetRootItem() && (path == _T("/net") || path == _T("/net/")))
-	{
+	if (GetItemParent(item) == GetRootItem() && (path == L"/net" || path == L"/net/")) {
 		CFilterManager filter;
 
 		const int attributes = S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
-		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, fz::datetime()))
-		{
-			if (!child)
-				AppendItem(item, _T(""));
+		if (!filter.FilenameFiltered(L"localhost", path, true, size, true, attributes, fz::datetime())) {
+			if (!child) {
+				AppendItem(item, L"");
+			}
 			return true;
 		}
 	}
@@ -1436,7 +1481,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 			int attributes;
 			fz::local_filesys::type type;
 			fz::datetime date;
-			if (!path.empty() && path.Last() == fz::local_filesys::path_separator) {
+			if (!path.empty() && path.back() == fz::local_filesys::path_separator) {
 				type = fz::local_filesys::get_file_info(fz::to_native(path + pData->m_known_subdir), wasLink, 0, &date, &attributes);
 			}
 			else {
@@ -1444,19 +1489,21 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 			}
 			if (type == fz::local_filesys::dir) {
 				CFilterManager filter;
-				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date))
+				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date)) {
 					return true;
+				}
 			}
 		}
 	}
 
-	wxString sub = HasSubdir(path);
+	std::wstring sub = HasSubdir(path);
 	if (!sub.empty()) {
-		wxTreeItemId subItem = AppendItem(item, _T(""));
-		SetItemData(subItem, new CTreeItemData(sub.ToStdWstring()));
+		wxTreeItemId subItem = AppendItem(item, L"");
+		SetItemData(subItem, new CTreeItemData(sub));
 	}
-	else if (child)
+	else if (child) {
 		Delete(child);
+	}
 
 	return true;
 }
@@ -1464,13 +1511,15 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 #ifdef __WXMSW__
 void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 {
-	if (!m_drives)
+	if (!m_drives) {
 		return;
+	}
 
 	if (wParam == DBT_DEVICEARRIVAL || wParam == DBT_DEVICEREMOVECOMPLETE) {
 		DEV_BROADCAST_HDR* pHdr = (DEV_BROADCAST_HDR*)lParam;
-		if (pHdr->dbch_devicetype != DBT_DEVTYP_VOLUME)
+		if (pHdr->dbch_devicetype != DBT_DEVTYP_VOLUME) {
 			return;
+		}
 
 		// Added or removed volume
 		DEV_BROADCAST_VOLUME* pVolume = (DEV_BROADCAST_VOLUME*)lParam;
@@ -1494,8 +1543,9 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 			mask <<= 1;
 		}
 
-		if (GetSelection() == m_drives)
+		if (GetSelection() == m_drives) {
 			m_state.RefreshLocal();
+		}
 	}
 }
 
@@ -1524,17 +1574,16 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar letter)
 
 	wxTreeItemIdValue value;
 	wxTreeItemId driveItem = GetFirstChild(m_drives, value);
-	while (driveItem)
-	{
-		if (!GetItemText(driveItem).Left(2).CmpNoCase(drive))
+	while (driveItem) {
+		if (!GetItemText(driveItem).Left(2).CmpNoCase(drive)) {
 			break;
+		}
 
 		driveItem = GetNextSibling(driveItem);
 	}
-	if (driveItem)
-	{
+	if (driveItem) {
 		SetItemText(driveItem, itemLabel);
-		int icon = GetIconIndex(iconType::dir, drive + _T("\\"));
+		int icon = GetIconIndex(iconType::dir, drive.ToStdWstring() + L"\\");
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Normal);
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Selected);
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Expanded);
@@ -1543,8 +1592,8 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar letter)
 		return driveItem;
 	}
 
-	wxTreeItemId item = AppendItem(m_drives, itemLabel, GetIconIndex(iconType::dir, drive + _T("\\")));
-	AppendItem(item, _T(""));
+	wxTreeItemId item = AppendItem(m_drives, itemLabel, GetIconIndex(iconType::dir, drive.ToStdWstring() + L"\\"));
+	AppendItem(item, L"");
 	SortChildren(m_drives);
 
 	return item;
@@ -1553,18 +1602,19 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar letter)
 void CLocalTreeView::RemoveDrive(wxChar drive)
 {
 	wxString driveName = drive;
-	driveName += _T(":");
+	driveName += L":";
 	wxTreeItemIdValue value;
 	wxTreeItemId driveItem = GetFirstChild(m_drives, value);
-	while (driveItem)
-	{
-		if (!GetItemText(driveItem).Left(2).CmpNoCase(driveName))
+	while (driveItem) {
+		if (!GetItemText(driveItem).Left(2).CmpNoCase(driveName)) {
 			break;
+		}
 
 		driveItem = GetNextSibling(driveItem);
 	}
-	if (!driveItem)
+	if (!driveItem) {
 		return;
+	}
 
 	Delete(driveItem);
 }
@@ -1574,11 +1624,11 @@ void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
 	// On-demand open icon for selected items
 	wxTreeItemId item = event.GetItem();
 
-	if (!item)
+	if (!item) {
 		return;
+	}
 
-	if (GetItemImage(item, wxTreeItemIcon_Selected) == -1)
-	{
+	if (GetItemImage(item, wxTreeItemIcon_Selected) == -1) {
 		int icon = GetIconIndex(iconType::opened_dir, GetDirFromItem(item));
 		SetItemImage(item, icon, wxTreeItemIcon_Selected);
 		SetItemImage(item, icon, wxTreeItemIcon_SelectedExpanded);
@@ -1589,12 +1639,14 @@ void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
 
 void CLocalTreeView::OnMenuOpen(wxCommandEvent&)
 {
-	if (!m_contextMenuItem.IsOk())
+	if (!m_contextMenuItem.IsOk()) {
 		return;
+	}
 
-	wxString path = GetDirFromItem(m_contextMenuItem);
-	if (path.empty())
+	std::wstring path = GetDirFromItem(m_contextMenuItem);
+	if (path.empty()) {
 		return;
+	}
 
 	OpenInFileManager(path);
 }
