@@ -28,7 +28,6 @@ class Site;
 class CSplitterWindowEx;
 class CStatusView;
 class CState;
-class CThemeProvider;
 class CToolBar;
 class CWindowStateManager;
 
@@ -65,7 +64,7 @@ public:
 
 	CContextControl* GetContextControl() { return m_pContextControl; }
 
-	bool ConnectToSite(Site & data, Bookmark const& bookmark);
+	bool ConnectToSite(Site & data, Bookmark const& bookmark, CState* pState = 0);
 
 	CFileZillaEngineContext& GetEngineContext() { return m_engineContext; }
 private:
@@ -95,7 +94,6 @@ private:
 	CStatusView* m_pStatusView{};
 	CQueueView* m_pQueueView{};
 	CLed* m_pActivityLed[2];
-	CThemeProvider* m_pThemeProvider{};
 #if FZ_MANUALUPDATECHECK
 	CUpdater* m_pUpdater{};
 	virtual void UpdaterStateChanged( UpdaterState s, build const& v );
