@@ -36,19 +36,13 @@
 #include <powrprof.h>
 #endif
 
-class CQueueViewDropTarget final : public CScrollableDropTarget<wxListCtrlEx>
+class CQueueViewDropTarget final : public CFileDropTarget<wxListCtrlEx>
 {
 public:
 	CQueueViewDropTarget(CQueueView* pQueueView)
-		: CScrollableDropTarget<wxListCtrlEx>(pQueueView)
+		: CFileDropTarget<wxListCtrlEx>(pQueueView)
 		, m_pQueueView(pQueueView)
-		, m_pFileDataObject(new wxFileDataObject())
-		, m_pRemoteDataObject(new CRemoteDataObject())
 	{
-		m_pDataObject = new wxDataObjectComposite;
-		m_pDataObject->Add(m_pRemoteDataObject, true);
-		m_pDataObject->Add(m_pFileDataObject, false);
-		SetDataObject(m_pDataObject);
 	}
 
 	virtual wxDragResult OnData(wxCoord, wxCoord, wxDragResult def)
@@ -57,16 +51,21 @@ public:
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
+		{
 			return def;
+		}
 
-		if (!GetData())
+		if (!GetData()) {
 			return wxDragError;
+		}
 
 		CDragDropManager* pDragDropManager = CDragDropManager::Get();
-		if (pDragDropManager)
+		if (pDragDropManager) {
 			pDragDropManager->pDropTarget = m_pQueueView;
+		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
+		auto const format = m_pDataObject->GetReceivedFormat();
+		if (format == m_pFileDataObject->GetFormat() || format == m_pLocalDataObject->GetFormat()) {
 			CState* const pState = CContextManager::Get()->GetCurrentContext();
 			if (!pState) {
 				return wxDragNone;
@@ -81,7 +80,12 @@ public:
 				return wxDragNone;
 			}
 
-			pState->UploadDroppedFiles(m_pFileDataObject, path, true);
+			if (format == m_pFileDataObject->GetFormat()) {
+				pState->UploadDroppedFiles(m_pFileDataObject, path, true);
+			}
+			else {
+				pState->UploadDroppedFiles(m_pLocalDataObject, path, true);
+			}
 		}
 		else {
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
@@ -103,7 +107,7 @@ public:
 				return wxDragNone;
 			}
 
-			const CLocalPath& target = pState->GetLocalDir();
+			CLocalPath const& target = pState->GetLocalDir();
 			if (!target.IsWriteable()) {
 				wxBell();
 				return wxDragNone;
@@ -133,8 +137,7 @@ public:
 		}
 
 		CDragDropManager* pDragDropManager = CDragDropManager::Get();
-		if (pDragDropManager && !pDragDropManager->remoteParent.empty())
-		{
+		if (pDragDropManager && !pDragDropManager->remoteParent.empty()) {
 			// Drag from remote to queue, check if local path is writeable
 			CState* const pState = CContextManager::Get()->GetCurrentContext();
 			if (!pState) {
@@ -162,10 +165,7 @@ public:
 
 	int DisplayDropHighlight(wxPoint) { return -1; }
 protected:
-	CQueueView *m_pQueueView;
-	wxFileDataObject* m_pFileDataObject;
-	CRemoteDataObject* m_pRemoteDataObject;
-	wxDataObjectComposite* m_pDataObject;
+	CQueueView *m_pQueueView{};
 };
 
 BEGIN_EVENT_TABLE(CQueueView, CQueueViewBase)
@@ -178,6 +178,7 @@ EVT_MENU(XRCID("ID_ACTIONAFTER_NONE"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_SHOW_NOTIFICATION_BUBBLE"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_REQUEST_ATTENTION"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_CLOSE"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_CLOSE_ONCE"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_DISCONNECT"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_PLAYSOUND"), CQueueView::OnActionAfter)
@@ -197,6 +198,9 @@ EVT_MENU(XRCID("ID_PRIORITY_LOWEST"), CQueueView::OnSetPriority)
 EVT_COMMAND(wxID_ANY, fzEVT_GRANTEXCLUSIVEENGINEACCESS, CQueueView::OnExclusiveEngineRequestGranted)
 
 EVT_SIZE(CQueueView::OnSize)
+
+EVT_LIST_COL_CLICK(wxID_ANY, CQueueView::OnColumnClicked)
+
 END_EVENT_TABLE()
 
 CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsyncRequestQueue *pAsyncRequestQueue)
@@ -204,6 +208,8 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	m_pMainFrame(pMainFrame),
 	m_pAsyncRequestQueue(pAsyncRequestQueue)
 {
+	wxGetApp().AddStartupProfileRecord("CQueueView::CQueueView");
+
 	if (m_pAsyncRequestQueue) {
 		m_pAsyncRequestQueue->SetQueue(this);
 	}
@@ -217,8 +223,7 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	}
 	m_actionAfterState = static_cast<ActionAfterState::type>(action);
 
-	std::list<ColumnId> extraCols;
-	extraCols.push_back(colTransferStatus);
+	std::vector<ColumnId> extraCols({colTransferStatus});
 	CreateColumns(extraCols);
 
 	RegisterOption(OPTION_NUMTRANSFERS);
@@ -267,11 +272,13 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download,
 	}
 	else {
 		fileItem = new CFileItem(pServerItem, queueOnly, download, sourceFile, targetFile, localPath, remotePath, size);
-		if (download) {
-			fileItem->SetAscii(CAutoAsciiFiles::TransferRemoteAsAscii(sourceFile, remotePath.GetType()));
-		}
-		else {
-			fileItem->SetAscii(CAutoAsciiFiles::TransferLocalAsAscii(sourceFile, remotePath.GetType()));
+		if (server.server.HasFeature(ProtocolFeature::DataTypeConcept)) {
+			if (download) {
+				fileItem->SetAscii(CAutoAsciiFiles::TransferRemoteAsAscii(sourceFile, remotePath.GetType()));
+			}
+			else {
+				fileItem->SetAscii(CAutoAsciiFiles::TransferLocalAsAscii(sourceFile, remotePath.GetType()));
+			}
 		}
 		fileItem->m_edit = edit;
 		if (edit != CEditHandler::none) {
@@ -318,7 +325,9 @@ bool CQueueView::QueueFiles(const bool queueOnly, const CLocalPath& localPath, c
 {
 	CServerItem* pServerItem = CreateServerItem(dataObject.GetServer());
 
-	const std::list<CRemoteDataObject::t_fileInfo>& files = dataObject.GetFiles();
+	std::vector<CRemoteDataObject::t_fileInfo> const& files = dataObject.GetFiles();
+
+	bool const hasDataTypeConcept = dataObject.GetServer().server.HasFeature(ProtocolFeature::DataTypeConcept);
 
 	for (auto const& fileInfo : files) {
 		if (fileInfo.dir) {
@@ -333,7 +342,9 @@ bool CQueueView::QueueFiles(const bool queueOnly, const CLocalPath& localPath, c
 		CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, true,
 			fileInfo.name, (fileInfo.name != localFile) ? localFile : std::wstring(),
 			localPath, dataObject.GetServerPath(), fileInfo.size);
-		fileItem->SetAscii(CAutoAsciiFiles::TransferRemoteAsAscii(fileInfo.name, dataObject.GetServerPath().GetType()));
+		if (hasDataTypeConcept) {
+			fileItem->SetAscii(CAutoAsciiFiles::TransferRemoteAsAscii(fileInfo.name, dataObject.GetServerPath().GetType()));
+		}
 
 		InsertItem(pServerItem, fileItem);
 	}
@@ -354,11 +365,15 @@ bool CQueueView::QueueFiles(const bool queueOnly, ServerWithCredentials const& s
 		InsertItem(pServerItem, fileItem);
 	}
 	else {
+		bool const hasDataTypeConcept = server.server.HasFeature(ProtocolFeature::DataTypeConcept);
+
 		for (auto const& file : files) {
 			CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, false,
 				file.name, std::wstring(),
 				listing.localPath, listing.remotePath, file.size);
-			fileItem->SetAscii(CAutoAsciiFiles::TransferLocalAsAscii(file.name, listing.remotePath.GetType()));
+			if (hasDataTypeConcept) {
+				fileItem->SetAscii(CAutoAsciiFiles::TransferLocalAsAscii(file.name, listing.remotePath.GetType()));
+			}
 
 			InsertItem(pServerItem, fileItem);
 		}
@@ -1120,7 +1135,7 @@ bool CQueueView::RemoveItem(CQueueItem* item, bool destroy, bool updateItemCount
 
 	if (item->GetType() == QueueItemType::File) {
 		// Update size information
-		const CFileItem* const pFileItem = (const CFileItem* const)item;
+		const CFileItem* const pFileItem = static_cast<CFileItem const*>(item);
 		int64_t size = pFileItem->GetSize();
 		if (size < 0) {
 			--m_filesWithUnknownSize;
@@ -1492,7 +1507,13 @@ void CQueueView::UpdateStatusLinePositions()
 #endif
 
 	for (auto pCtrl : m_statusLineList) {
-		int index = GetItemIndex(pCtrl->GetItem()) + 1;
+		CFileItem const* pItem = pCtrl->GetItem();
+		if (!pItem) {
+			pCtrl->Show(false);
+			continue;
+		}
+
+		int index = GetItemIndex(pItem) + 1;
 		if (index < m_lastTopItem || index > bottomItem) {
 			pCtrl->Show(false);
 			continue;
@@ -1583,6 +1604,7 @@ void CQueueView::LoadQueueFromXML()
 
 void CQueueView::LoadQueue()
 {
+	wxGetApp().AddStartupProfileRecord("CQueueView::LoadQueue");
 	// We have to synchronize access to queue.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_QUEUE);
@@ -1787,36 +1809,66 @@ void CQueueView::OnPostScroll()
 
 void CQueueView::OnContextMenu(wxContextMenuEvent&)
 {
-	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_QUEUE"));
-	if (!pMenu) {
-		return;
-	}
+	wxMenu menu;
+
+	menu.Append(XRCID("ID_PROCESSQUEUE"), _("Process &Queue"), wxString(), wxITEM_CHECK);
+	menu.Append(XRCID("ID_REMOVEALL"), _("Stop and remove &all"));
+
+    menu.AppendSeparator();
+	menu.Append(XRCID("ID_REMOVE"), _("&Remove selected"));
+	menu.Append(XRCID("ID_DEFAULT_FILEEXISTSACTION"), _("&Default file exists action..."));
+
+	auto menuPriority = new wxMenu;
+	menu.AppendSubMenu(menuPriority, _("Set &Priority"))->SetId(XRCID("ID_PRIORITY"));
+	menuPriority->Append(XRCID("ID_PRIORITY_HIGHEST"), _("&Highest"), wxString(), wxITEM_CHECK);
+	menuPriority->Append(XRCID("ID_PRIORITY_HIGH"), _("H&igh"), wxString(), wxITEM_CHECK);
+    menuPriority->Append(XRCID("ID_PRIORITY_NORMAL"), _("&Normal"), wxString(), wxITEM_CHECK);
+    menuPriority->Append(XRCID("ID_PRIORITY_LOW"), _("&Low"), wxString(), wxITEM_CHECK);
+    menuPriority->Append(XRCID("ID_PRIORITY_LOWEST"), _("L&owest"), wxString(), wxITEM_CHECK);
+  
+	auto menuAfter = new wxMenu;
+	menu.AppendSubMenu(menuAfter, _("Action after queue &completion"))->SetId(XRCID("ID_ACTIONAFTER"));
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_NONE"), _("&None"), wxString(), wxITEM_CHECK);
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_SHOW_NOTIFICATION_BUBBLE"), _("Sh&ow notification bubble"), wxString(), wxITEM_CHECK);
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_REQUEST_ATTENTION"), _("&Request attention"), wxString(), wxITEM_CHECK);
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_CLOSE"), _("&Close FileZilla"), wxString(), wxITEM_CHECK);
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), _("&Run command..."), wxString(), wxITEM_CHECK);
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_PLAYSOUND"), _("&Play sound"), wxString(), wxITEM_CHECK);
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_CLOSE_ONCE"), _("&Close FileZilla once"), wxString(), wxITEM_CHECK);
+#if defined(FZ_WINDOWS) || defined(FZ_MAC)
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_REBOOT"), _("R&eboot system once"), wxString(), wxITEM_CHECK);
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_SHUTDOWN"), _("S&hutdown system once"), wxString(), wxITEM_CHECK);
+    menuAfter->Append(XRCID("ID_ACTIONAFTER_SLEEP"), _("S&uspend system once"), wxString(), wxITEM_CHECK);
+#endif
+
+	menu.Append(XRCID("ID_EXPORT"), _("E&xport..."));
 
 	bool has_selection = HasSelection();
 
-	pMenu->Check(XRCID("ID_PROCESSQUEUE"), IsActive() ? true : false);
-	pMenu->Check(XRCID("ID_ACTIONAFTER_NONE"), IsActionAfter(ActionAfterState::None));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_SHOW_NOTIFICATION_BUBBLE"), IsActionAfter(ActionAfterState::ShowNotification));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_REQUEST_ATTENTION"), IsActionAfter(ActionAfterState::RequestAttention));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_CLOSE"), IsActionAfter(ActionAfterState::Close));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_CLOSE_ONCE"), IsActionAfter(ActionAfterState::CloseOnce));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), IsActionAfter(ActionAfterState::RunCommand));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_PLAYSOUND"), IsActionAfter(ActionAfterState::PlaySound));
+	menu.Check(XRCID("ID_PROCESSQUEUE"), IsActive() ? true : false);
+	menu.Check(XRCID("ID_ACTIONAFTER_NONE"), IsActionAfter(ActionAfterState::None));
+	menu.Check(XRCID("ID_ACTIONAFTER_SHOW_NOTIFICATION_BUBBLE"), IsActionAfter(ActionAfterState::ShowNotification));
+	menu.Check(XRCID("ID_ACTIONAFTER_REQUEST_ATTENTION"), IsActionAfter(ActionAfterState::RequestAttention));
+	menu.Check(XRCID("ID_ACTIONAFTER_CLOSE"), IsActionAfter(ActionAfterState::Close));
+	menu.Check(XRCID("ID_ACTIONAFTER_CLOSE_ONCE"), IsActionAfter(ActionAfterState::CloseOnce));
+	menu.Check(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), IsActionAfter(ActionAfterState::RunCommand));
+	menu.Check(XRCID("ID_ACTIONAFTER_PLAYSOUND"), IsActionAfter(ActionAfterState::PlaySound));
 #if defined(__WXMSW__) || defined(__WXMAC__)
-	pMenu->Check(XRCID("ID_ACTIONAFTER_REBOOT"), IsActionAfter(ActionAfterState::Reboot));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_SHUTDOWN"), IsActionAfter(ActionAfterState::Shutdown));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_SLEEP"), IsActionAfter(ActionAfterState::Sleep));
+	menu.Check(XRCID("ID_ACTIONAFTER_REBOOT"), IsActionAfter(ActionAfterState::Reboot));
+	menu.Check(XRCID("ID_ACTIONAFTER_SHUTDOWN"), IsActionAfter(ActionAfterState::Shutdown));
+	menu.Check(XRCID("ID_ACTIONAFTER_SLEEP"), IsActionAfter(ActionAfterState::Sleep));
 #endif
-	pMenu->Enable(XRCID("ID_REMOVE"), has_selection);
+	menu.Enable(XRCID("ID_REMOVE"), has_selection);
 
-	pMenu->Enable(XRCID("ID_PRIORITY"), has_selection);
-	pMenu->Enable(XRCID("ID_DEFAULT_FILEEXISTSACTION"), has_selection);
+	menu.Enable(XRCID("ID_PRIORITY"), has_selection);
+	menu.Enable(XRCID("ID_DEFAULT_FILEEXISTSACTION"), has_selection);
 #if defined(__WXMSW__) || defined(__WXMAC__)
-	pMenu->Enable(XRCID("ID_ACTIONAFTER"), m_actionAfterWarnDialog == NULL);
+	menu.Enable(XRCID("ID_ACTIONAFTER"), m_actionAfterWarnDialog == NULL);
 #endif
 
-	PopupMenu(pMenu);
-	delete pMenu;
+	menu.Enable(XRCID("ID_EXPORT"), GetItemCount() != 0);
+
+	PopupMenu(&menu);
 }
 
 void CQueueView::OnProcessQueue(wxCommandEvent& event)
@@ -2751,8 +2803,9 @@ void CQueueView::ActionAfter(bool warned)
 #if defined(__WXMSW__) || defined(__WXMAC__)
 void CQueueView::ActionAfterWarnUser(ActionAfterState::type s)
 {
-	if (m_actionAfterWarnDialog != NULL)
+	if (m_actionAfterWarnDialog != NULL) {
 		return;
+	}
 
 	wxString message;
 	wxString label;
@@ -3172,3 +3225,16 @@ void CQueueView::OnStateChange(CState*, t_statechange_notifications notification
 		}
 	}
 }
+
+void CQueueView::OnColumnClicked(wxListEvent &event)
+{
+	int const col = event.GetColumn();
+	bool const reverse = wxGetKeyState(WXK_SHIFT);
+
+	for (auto * serverItem : m_serverList) {
+		serverItem->Sort(col, reverse);
+	}
+
+	RefreshListOnly();
+	UpdateStatusLinePositions();
+}
