@@ -9,13 +9,14 @@ class CChmodDialog;
 class CInfoText;
 class CQueueView;
 class CRemoteListViewDropTarget;
+class CView;
 class CWindowTinter;
 
 class CRemoteListView final : public CFileListCtrl<CGenericFileData>, CStateEventHandler
 {
 	friend class CRemoteListViewDropTarget;
 public:
-	CRemoteListView(wxWindow* pParent, CState& state, CQueueView* pQueue);
+	CRemoteListView(CView* pParent, CState& state, CQueueView* pQueue);
 	virtual ~CRemoteListView();
 
 	virtual bool CanStartComparison();
@@ -25,16 +26,30 @@ public:
 	virtual void OnExitComparisonMode();
 
 	void LinkIsNotDir(CServerPath const& path, std::wstring const& link);
+
+	struct ChmodUICommand 
+	{
+		wxWindow		*parentWindow;
+		char			*permissions;
+		int				fileCount;
+		int				dirCount;
+		std::wstring	const &name;
+	};
+
+	typedef std::function<void(ChmodUICommand &, CState &)> ChmodHandler;
+
+	static std::map<ServerProtocol, CRemoteListView::ChmodHandler> chmodHandlers;
+	
 protected:
 	virtual wxString GetItemText(int item, unsigned int column);
 
 	// Clears all selections and returns the list of items that were selected
-	std::list<wxString> RememberSelectedItems(wxString& focused);
+	std::vector<std::wstring> RememberSelectedItems(std::wstring & focused, int & focusItem);
 
 	// Select a list of items based in their names.
 	// Sort order may not change between call to RememberSelectedItems and
 	// ReselectItems
-	void ReselectItems(std::list<wxString>& selectedNames, wxString focused, bool ensureVisible = false);
+	void ReselectItems(std::vector<std::wstring>& selectedNames, std::wstring focused, int focusedItem, bool ensureVisible = false);
 
 
 	// Declared const due to design error in wxWidgets.
@@ -50,7 +65,7 @@ protected:
 	bool IsItemValid(unsigned int item) const;
 	int GetItemIndex(unsigned int item) const;
 
-	virtual CSortComparisonObject GetSortComparisonObject();
+	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() override;
 
 	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 	void ApplyCurrentFilter();
@@ -70,6 +85,8 @@ protected:
 	// Caller is responsible to check selection is valid!
 	void TransferSelectedFiles(const CLocalPath& local_parent, bool queue_only);
 
+	void HandleGenericChmod(ChmodUICommand &command);
+
 	// Cache icon for directories, no need to calculate it multiple times
 	int m_dirIcon;
 
@@ -102,6 +119,8 @@ protected:
 
 	std::unique_ptr<CWindowTinter> m_windowTinter;
 
+	CView *m_parentView{};
+	
 	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent &event);
 	void OnContextMenu(wxContextMenuEvent& event);
