@@ -28,19 +28,13 @@ public:
 	CServerPath m_path;
 };
 
-class CRemoteTreeViewDropTarget final : public CScrollableDropTarget<wxTreeCtrlEx>
+class CRemoteTreeViewDropTarget final : public CFileDropTarget<wxTreeCtrlEx>
 {
 public:
 	CRemoteTreeViewDropTarget(CRemoteTreeView* pRemoteTreeView)
-		: CScrollableDropTarget<wxTreeCtrlEx>(pRemoteTreeView)
+		: CFileDropTarget<wxTreeCtrlEx>(pRemoteTreeView)
 		, m_pRemoteTreeView(pRemoteTreeView)
-		, m_pFileDataObject(new wxFileDataObject())
-		, m_pRemoteDataObject(new CRemoteDataObject())
 	{
-		m_pDataObject = new wxDataObjectComposite;
-		m_pDataObject->Add(m_pRemoteDataObject, true);
-		m_pDataObject->Add(m_pFileDataObject, false);
-		SetDataObject(m_pDataObject);
 	}
 
 	void ClearDropHighlight()
@@ -69,7 +63,9 @@ public:
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
+		{
 			return def;
+		}
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
 		if (!hit) {
@@ -90,9 +86,13 @@ public:
 			pDragDropManager->pDropTarget = m_pRemoteTreeView;
 		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
+		auto const format = m_pDataObject->GetReceivedFormat();
+		if (format == m_pFileDataObject->GetFormat()) {
 			m_pRemoteTreeView->m_state.UploadDroppedFiles(m_pFileDataObject, path, false);
 		}
+		else if (format == m_pLocalDataObject->GetFormat()) {
+			m_pRemoteTreeView->m_state.UploadDroppedFiles(m_pLocalDataObject, path, false);
+		}
 		else {
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
@@ -111,7 +111,7 @@ public:
 				return wxDragNone;
 			}
 
-			std::list<CRemoteDataObject::t_fileInfo> const& files = m_pRemoteDataObject->GetFiles();
+			std::vector<CRemoteDataObject::t_fileInfo> const& files = m_pRemoteDataObject->GetFiles();
 			for (auto const& info : files) {
 				if (info.dir) {
 					CServerPath dir = m_pRemoteDataObject->GetServerPath();
@@ -146,12 +146,14 @@ public:
 		ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
-		if (!hit)
+		if (!hit) {
 			return false;
+		}
 
 		const CServerPath& path = m_pRemoteTreeView->GetPathFromItem(hit);
-		if (path.empty())
+		if (path.empty()) {
 			return false;
+		}
 
 		return true;
 	}
@@ -172,8 +174,9 @@ public:
 		}
 
 		const wxTreeItemId dropHighlight = m_pRemoteTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId())
+		if (dropHighlight != wxTreeItemId()) {
 			m_pRemoteTreeView->SetItemDropHighlight(dropHighlight, false);
+		}
 
 		m_pRemoteTreeView->SetItemDropHighlight(hit, true);
 		m_pRemoteTreeView->m_dropHighlight = hit;
@@ -194,11 +197,13 @@ public:
 		}
 
 		wxTreeItemId hit = DisplayDropHighlight(wxPoint(x, y));
-		if (!hit.IsOk())
+		if (!hit.IsOk()) {
 			return wxDragNone;
+		}
 
-		if (def == wxDragLink)
+		if (def == wxDragLink) {
 			def = wxDragCopy;
+		}
 
 		return def;
 	}
@@ -216,10 +221,7 @@ public:
 	}
 
 protected:
-	CRemoteTreeView *m_pRemoteTreeView;
-	wxFileDataObject* m_pFileDataObject;
-	CRemoteDataObject* m_pRemoteDataObject;
-	wxDataObjectComposite* m_pDataObject;
+	CRemoteTreeView *m_pRemoteTreeView{};
 };
 
 BEGIN_EVENT_TABLE(CRemoteTreeView, wxTreeCtrlEx)
@@ -279,7 +281,7 @@ CRemoteTreeView::~CRemoteTreeView()
 void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_DIR) {
-		SetDirectoryListing(m_state.GetRemoteDir(), data2 ? *reinterpret_cast<bool const*>(data2) : false);
+		SetDirectoryListing(m_state.GetRemoteDir(), data2 ? *reinterpret_cast<bool const*>(data2) : true);
 	}
 	else if (notification == STATECHANGE_APPLYFILTER) {
 		ApplyFilters(false);
@@ -289,14 +291,14 @@ void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, co
 	}
 }
 
-void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified)
+void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool primary)
 {
 	m_busy = true;
 
 	if (!pListing) {
 		m_ExpandAfterList = wxTreeItemId();
 		DeleteAllItems();
-		AddRoot(_T(""));
+		AddRoot(wxString());
 		m_busy = false;
 		if (FindFocus() == this) {
 			wxNavigationKeyEvent *evt = new wxNavigationKeyEvent();
@@ -320,10 +322,12 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 		return;
 	}
 
+	bool const wasEmpty = GetChildrenCount(GetRootItem()) == 0;
+
 #ifndef __WXMSW__
 	Freeze();
 #endif
-	wxTreeItemId parent = MakeParent(pListing->path, !modified);
+	wxTreeItemId parent = MakeParent(pListing->path, primary);
 	if (!parent) {
 		m_busy = false;
 #ifndef __WXMSW__
@@ -332,19 +336,21 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 		return;
 	}
 
-	if (!IsExpanded(parent) && parent != m_ExpandAfterList)
-	{
+	if (wasEmpty && GetItemParent(parent) == GetRootItem() && !m_ExpandAfterList) {
+		m_ExpandAfterList = parent;
+	}
+
+	if (!IsExpanded(parent) && parent != m_ExpandAfterList) {
 		DeleteChildren(parent);
 		CFilterManager filter;
-		if (HasSubdirs(*pListing, filter))
-			AppendItem(parent, _T(""), -1, -1);
+		if (HasSubdirs(*pListing, filter)) {
+			AppendItem(parent, wxString(), -1, -1);
+		}
 	}
-	else
-	{
-		RefreshItem(parent, *pListing, !modified);
+	else {
+		RefreshItem(parent, *pListing, primary);
 
-		if (m_ExpandAfterList == parent)
-		{
+		if (m_ExpandAfterList == parent) {
 #ifndef __WXMSW__
 			// Prevent CalculatePositions from being called
 			wxGenericTreeItem *anchor = m_anchor;
@@ -363,11 +369,13 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 #ifndef __WXMSW__
 	Thaw();
 #endif
-	if (!modified)
+	if (primary) {
 		SafeSelectItem(parent);
+	}
 #ifndef __WXMSW__
-	else
+	else {
 		Refresh();
+	}
 #endif
 
 	m_busy = false;
@@ -375,11 +383,11 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 
 wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 {
-	std::vector<wxString> pieces;
+	std::vector<std::wstring> pieces;
 	pieces.reserve(path.SegmentCount() + 1);
 	while (path.HasParent()) {
 		pieces.push_back(path.GetLastSegment());
-		path = path.GetParent();
+		path.MakeParent();
 	}
 	wxASSERT(!path.GetPath().empty());
 	pieces.push_back(path.GetPath());
@@ -387,9 +395,9 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 	const wxTreeItemId root = GetRootItem();
 	wxTreeItemId parent = root;
 
-	for (std::vector<wxString>::const_reverse_iterator iter = pieces.rbegin(); iter != pieces.rend(); ++iter) {
+	for (auto iter = pieces.crbegin(); iter != pieces.crend(); ++iter) {
 		if (iter != pieces.rbegin()) {
-			path.AddSegment(iter->ToStdWstring());
+			path.AddSegment(*iter);
 		}
 
 		wxTreeItemIdValue cookie;
@@ -397,13 +405,15 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 		if (child && GetItemText(child).empty()) {
 			Delete(child);
 			child = wxTreeItemId();
-			if (parent != root)
+			if (parent != root) {
 				ListExpand(parent);
+			}
 		}
 		for (child = GetFirstChild(parent, cookie); child; child = GetNextSibling(child)) {
 			const wxString& text = GetItemText(child);
-			if (text == *iter)
+			if (text == *iter) {
 				break;
+			}
 		}
 		if (!child) {
 			CDirectoryListing listing;
@@ -420,8 +430,9 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 
 			auto nextIter = iter;
 			++nextIter;
-			if (nextIter != pieces.rend())
+			if (nextIter != pieces.rend()) {
 				DisplayItem(child, listing);
+			}
 		}
 		if (select && iter != pieces.rbegin()) {
 #ifndef __WXMSW__
@@ -493,20 +504,24 @@ void CRemoteTreeView::CreateImageList()
 
 bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilterManager& filter)
 {
-	if (!listing.has_dirs())
+	if (!listing.has_dirs()) {
 		return false;
+	}
 
-	if (!filter.HasActiveFilters())
+	if (!filter.HasActiveFilters()) {
 		return true;
+	}
 
-	const wxString path = listing.path.GetPath();
-	for (unsigned int i = 0; i < listing.GetCount(); i++)
-	{
-		if (!listing[i].is_dir())
+	std::wstring const path = listing.path.GetPath();
+	for (size_t i = 0; i < listing.size(); ++i) {
+		auto const& entry = listing[i];
+		if (!entry.is_dir()) {
 			continue;
+		}
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
+		if (filter.FilenameFiltered(entry.name, path, true, -1, false, 0, entry.time)) {
 			continue;
+		}
 
 		return true;
 	}
@@ -518,17 +533,20 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 {
 	DeleteChildren(parent);
 
-	const wxString path = listing.path.GetPath();
+	std::wstring const path = listing.path.GetPath();
 
 	CFilterDialog filter;
-	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
-		if (!listing[i].is_dir())
+	for (size_t i = 0; i < listing.size(); ++i) {
+		auto const& entry = listing[i];
+		if (!entry.is_dir()) {
 			continue;
+		}
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
+		if (filter.FilenameFiltered(entry.name, path, true, -1, false, 0, entry.time)) {
 			continue;
+		}
 
-		std::wstring const& name = listing[i].name;
+		std::wstring const& name = entry.name;
 		CServerPath subdir = listing.path;
 		subdir.AddSegment(name);
 
@@ -538,8 +556,9 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 			wxTreeItemId child = AppendItem(parent, name, 0, 2, 0);
 			SetItemImages(child, false);
 
-			if (HasSubdirs(subListing, filter))
-				AppendItem(child, _T(""), -1, -1);
+			if (HasSubdirs(subListing, filter)) {
+				AppendItem(child, L"", -1, -1);
+			}
 		}
 		else {
 			wxTreeItemId child = AppendItem(parent, name, 1, 3, 0);
@@ -562,16 +581,17 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 	CFilterManager filter;
 
-	wxString const path = listing.path.GetPath();
+	std::wstring const path = listing.path.GetPath();
 
 	std::vector<std::wstring> dirs;
-	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
-		if (!listing[i].is_dir()) {
+	for (size_t i = 0; i < listing.size(); ++i) {
+		auto const& entry = listing[i];
+		if (!entry.is_dir()) {
 			continue;
 		}
 
-		if (!filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time)) {
-			dirs.push_back(listing[i].name);
+		if (!filter.FilenameFiltered(entry.name, path, true, -1, false, 0, entry.time)) {
+			dirs.push_back(entry.name);
 		}
 	}
 
@@ -605,11 +625,13 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		else if (cmp > 0) {
 			// Child no longer exists
 			wxTreeItemId sel = GetSelection();
-			while (sel && sel != child)
+			while (sel && sel != child) {
 				sel = GetItemParent(sel);
+			}
 			wxTreeItemId prev = GetPrevSibling(child);
-			if (!sel || will_select_parent)
+			if (!sel || will_select_parent) {
 				Delete(child);
+			}
 			child = prev;
 		}
 		else if (cmp < 0) {
@@ -642,11 +664,13 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 	while (child) {
 		// Child no longer exists
 		wxTreeItemId sel = GetSelection();
-		while (sel && sel != child)
+		while (sel && sel != child) {
 			sel = GetItemParent(sel);
+		}
 		wxTreeItemId prev = GetPrevSibling(child);
-		if (!sel || will_select_parent)
+		if (!sel || will_select_parent) {
 			Delete(child);
+		}
 		child = prev;
 	}
 	while (iter != dirs.rend()) {
@@ -802,7 +826,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	}
 
 	const CServerPath& parent = path.GetParent();
-	const wxString& lastSegment = path.GetLastSegment();
+	std::wstring const lastSegment = path.GetLastSegment();
 	if (lastSegment.empty()) {
 		return;
 	}
@@ -876,34 +900,49 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 {
 	m_contextMenuItem = event.GetItem();
-	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_REMOTETREE"));
-	if (!pMenu) {
-		return;
-	}
+
+	wxMenu menu;
+	auto item = new wxMenuItem(&menu, XRCID("ID_DOWNLOAD"), _("&Download"), _("Download selected directory"));
+	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_DOWNLOAD"), wxART_MENU));
+	menu.Append(item);
+    item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add to queue"), _("Add selected directory to the transfer queue"));
+	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_DOWNLOADADD"), wxART_MENU));
+	menu.Append(item);
+
+    menu.AppendSeparator();
+    menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
+    menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
+
+    menu.AppendSeparator();
+    menu.Append(XRCID("ID_DELETE"), _("D&elete"), _("Delete selected directory"));
+    menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected directory"));
+    menu.Append(XRCID("ID_GETURL"), _("C&opy URL(s) to clipboard"), _("Copy the URLs of the selected items to clipboard."));
+    menu.Append(XRCID("ID_GETURL_PASSWORD"), _("C&opy URL(s) with password to clipboard"), _("Copy the URLs of the selected items to clipboard, including password."));
+    menu.Append(XRCID("ID_CHMOD"), _("&File Attributes..."), _("Change the file permissions."));
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
 	if (!m_state.IsRemoteIdle() || path.empty()) {
-		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
-		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
-		pMenu->Enable(XRCID("ID_MKDIR"), false);
-		pMenu->Enable(XRCID("ID_MKDIR_CHGDIR"), false);
-		pMenu->Enable(XRCID("ID_DELETE"), false);
-		pMenu->Enable(XRCID("ID_CHMOD"), false);
-		pMenu->Enable(XRCID("ID_RENAME"), false);
-		pMenu->Enable(XRCID("ID_GETURL"), false);
+		menu.Enable(XRCID("ID_DOWNLOAD"), false);
+		menu.Enable(XRCID("ID_ADDTOQUEUE"), false);
+		menu.Enable(XRCID("ID_MKDIR"), false);
+		menu.Enable(XRCID("ID_MKDIR_CHGDIR"), false);
+		menu.Enable(XRCID("ID_DELETE"), false);
+		menu.Enable(XRCID("ID_CHMOD"), false);
+		menu.Enable(XRCID("ID_RENAME"), false);
+		menu.Enable(XRCID("ID_GETURL"), false);
+	}
+	else if (!path.HasParent()) {
+		menu.Enable(XRCID("ID_RENAME"), false);
 	}
-	else if (!path.HasParent())
-		pMenu->Enable(XRCID("ID_RENAME"), false);
 
 	if (!m_state.GetLocalDir().IsWriteable()) {
-		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
-		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
+		menu.Enable(XRCID("ID_DOWNLOAD"), false);
+		menu.Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
 
-	pMenu->Delete(XRCID(wxGetKeyState(WXK_SHIFT) ? "ID_GETURL" : "ID_GETURL_PASSWORD"));
+	menu.Delete(XRCID(wxGetKeyState(WXK_SHIFT) ? "ID_GETURL" : "ID_GETURL_PASSWORD"));
 
-	PopupMenu(pMenu);
-	delete pMenu;
+	PopupMenu(&menu);
 }
 
 void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
@@ -938,12 +977,13 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		// ... and it needs to be cached
 		cached = m_state.m_pEngine->CacheLookup(parentPath, listing) == FZ_REPLY_OK;
 		if (cached) {
-			for (unsigned int i = 0; i < listing.GetCount(); ++i) {
-				if (listing[i].name != name) {
+			for (size_t i = 0; i < listing.size(); ++i) {
+				auto const& entry = listing[i];
+				if (entry.name != name) {
 					continue;
 				}
 
-				pChmodDlg->ConvertPermissions(*listing[i].permissions, permissions);
+				pChmodDlg->ConvertPermissions(*entry.permissions, permissions);
 			}
 		}
 	}
@@ -1057,49 +1097,66 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 {
-	if (!m_state.IsRemoteIdle())
+	if (!m_state.IsRemoteIdle()) {
 		return;
+	}
 
-	if (!m_contextMenuItem)
+	if (!m_contextMenuItem) {
 		return;
+	}
 
 	CServerPath const& pathToDelete = GetPathFromItem(m_contextMenuItem);
-	if (pathToDelete.empty())
+	if (pathToDelete.empty()) {
 		return;
+	}
 
-	if (wxMessageBoxEx(_("Really delete all selected files and/or directories from the server?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
+	if (wxMessageBoxEx(_("Really delete all selected files and/or directories from the server?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES) {
 		return;
-
-	const bool hasParent = pathToDelete.HasParent();
-
-	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
-
-	recursion_root root;
-	CServerPath startDir;
-	if (hasParent) {
-		std::wstring const name = GetItemText(m_contextMenuItem).ToStdWstring();
-		startDir = pathToDelete.GetParent();
-		root = recursion_root(startDir, !hasParent);
-		root.add_dir_to_visit(startDir, name);
 	}
-	else {
-		startDir = pathToDelete;
-		root = recursion_root(startDir, !hasParent);
-		root.add_dir_to_visit(startDir, std::wstring());
-	}
-	pRecursiveOperation->AddRecursionRoot(std::move(root));
 
-	CServerPath currentPath;
-	const wxTreeItemId selected = GetSelection();
-	if (selected)
-		currentPath = GetPathFromItem(selected);
-	if (!currentPath.empty() && (pathToDelete == currentPath || pathToDelete.IsParentOf(currentPath, false))) {
-		currentPath = startDir;
-		m_state.ChangeRemoteDir(startDir);
-	}
+	bool const hasParent = pathToDelete.HasParent();
 
 	CFilterManager filter;
-	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters(), currentPath);
+	if (CServer::ProtocolHasFeature(m_state.GetServer().server.GetProtocol(), ProtocolFeature::RecursiveDelete) && !filter.HasActiveRemoteFilters()) {
+		if (hasParent) {
+			std::wstring const name = GetItemText(m_contextMenuItem).ToStdWstring();
+			m_state.m_pCommandQueue->ProcessCommand(new CRemoveDirCommand(pathToDelete.GetParent(), name));
+		}
+		else {
+			m_state.m_pCommandQueue->ProcessCommand(new CRemoveDirCommand(pathToDelete, std::wstring()));
+		}
+	}
+	else {
+
+		CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
+
+		recursion_root root;
+		CServerPath startDir;
+		if (hasParent) {
+			std::wstring const name = GetItemText(m_contextMenuItem).ToStdWstring();
+			startDir = pathToDelete.GetParent();
+			root = recursion_root(startDir, !hasParent);
+			root.add_dir_to_visit(startDir, name);
+		}
+		else {
+			startDir = pathToDelete;
+			root = recursion_root(startDir, !hasParent);
+			root.add_dir_to_visit(startDir, std::wstring());
+		}
+		pRecursiveOperation->AddRecursionRoot(std::move(root));
+
+		CServerPath currentPath;
+		const wxTreeItemId selected = GetSelection();
+		if (selected) {
+			currentPath = GetPathFromItem(selected);
+		}
+		if (!currentPath.empty() && (pathToDelete == currentPath || pathToDelete.IsParentOf(currentPath, false))) {
+			currentPath = startDir;
+			m_state.ChangeRemoteDir(startDir);
+		}
+
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters(), currentPath);
+	}
 }
 
 void CRemoteTreeView::OnMenuRename(wxCommandEvent&)
