@@ -19,7 +19,7 @@ public:
 
 protected:
 	wxTreeItemId MakeParent(CServerPath path, bool select);
-	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified);
+	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool primary);
 	virtual void OnStateChange(t_statechange_notifications notification, const wxString&, const void*);
 
 	void DisplayItem(wxTreeItemId parent, const CDirectoryListing& listing);
@@ -38,7 +38,7 @@ protected:
 	CQueueView* m_pQueue;
 
 	void CreateImageList();
-	wxBitmap CreateIcon(int index, const wxString& overlay = _T(""));
+	wxBitmap CreateIcon(int index, wxString const& overlay = wxString());
 	wxImageList* m_pImageList{};
 
 	// Set to true in SetDirectoryListing.
