@@ -1,5 +1,5 @@
-#ifndef __DNDOBJECTS_H__
-#define __DNDOBJECTS_H__
+#ifndef FILEZILLA_INTERFACE_DNDOBJECTS_HEADER
+#define FILEZILLA_INTERFACE_DNDOBJECTS_HEADER
 
 #ifdef __WXMSW__
 #define FZ3_USESHELLEXT 1
@@ -7,25 +7,37 @@
 #define FZ3_USESHELLEXT 0
 #endif
 
+#include "drop_target_ex.h"
 #include "xmlfunctions.h"
 
 #include <wx/dnd.h>
 
 #include <memory>
 
-class wxRemoteDataFormat : public wxDataFormat
+class CLocalDataObject final : public wxDataObjectSimple
 {
 public:
-	wxRemoteDataFormat()
-		: wxDataFormat(_T("FileZilla3 remote data format v1"))
-	{
-	}
+	CLocalDataObject();
+
+	virtual size_t GetDataSize() const;
+	virtual bool GetDataHere(void *buf) const;
+
+	virtual bool SetData(size_t len, const void* buf);
+
+	std::vector<std::string> const& GetFiles() const { return files_; }
+	std::vector<std::wstring> GetFilesW() const;
+
+	void Reserve(size_t count);
+	void AddFile(std::wstring const& file);
+
+protected:
+	std::vector<std::string> files_;
 };
 
-class CRemoteDataObject : public wxDataObjectSimple
+class CRemoteDataObject final : public wxDataObjectSimple
 {
 public:
-	CRemoteDataObject(const CServer& server, const CServerPath& path);
+	CRemoteDataObject(ServerWithCredentials const& server, const CServerPath& path);
 	CRemoteDataObject();
 
 	virtual size_t GetDataSize() const;
@@ -38,33 +50,34 @@ public:
 
 	bool DidSendData() const { return m_didSendData; }
 
-	const CServer& GetServer() const { return m_server; }
+	ServerWithCredentials const& GetServer() const { return server_; }
 	const CServerPath& GetServerPath() const { return m_path; }
 	int GetProcessId() const { return m_processId; }
 
 	struct t_fileInfo
 	{
 		std::wstring name;
-		bool dir;
 		int64_t size;
+		bool dir;
 		bool link;
 	};
 
-	const std::list<t_fileInfo>& GetFiles() const { return m_fileList; }
+	const std::vector<t_fileInfo>& GetFiles() const { return m_fileList; }
 
-	void AddFile(const wxString& name, bool dir, int64_t size, bool link);
+	void Reserve(size_t count);
+	void AddFile(std::wstring const& name, bool dir, int64_t size, bool link);
 
 protected:
-	CServer m_server;
+	ServerWithCredentials server_;
 	CServerPath m_path;
 
 	mutable CXmlFile m_xmlFile;
 
-	bool m_didSendData;
+	bool m_didSendData{};
 
 	int m_processId;
 
-	std::list<t_fileInfo> m_fileList;
+	std::vector<t_fileInfo> m_fileList;
 
 	mutable size_t m_expectedSize{};
 };
@@ -99,6 +112,18 @@ protected:
 	std::wstring m_dragDirectory;
 };
 
-#endif //__WXMSW__
+#endif
 
-#endif //__DNDOBJECTS_H__
+template<typename Control>
+class CFileDropTarget : public CScrollableDropTarget<Control>
+{
+protected:
+	CFileDropTarget(Control* ctrl);
+
+	CLocalDataObject* m_pLocalDataObject{};
+	wxFileDataObject* m_pFileDataObject{};
+	CRemoteDataObject* m_pRemoteDataObject{};
+	wxDataObjectComposite* m_pDataObject{};
+};
+
+#endif
