@@ -9,6 +9,7 @@
 #include "window_state_manager.h"
 #include "xrc_helper.h"
 
+#include <libfilezilla/file.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
 class CChangedFileDialog : public wxDialogEx
@@ -61,8 +62,9 @@ CEditHandler::CEditHandler()
 
 CEditHandler* CEditHandler::Create()
 {
-	if (!m_pEditHandler)
+	if (!m_pEditHandler) {
 		m_pEditHandler = new CEditHandler();
+	}
 
 	return m_pEditHandler;
 }
@@ -75,12 +77,14 @@ CEditHandler* CEditHandler::Get()
 void CEditHandler::RemoveTemporaryFiles(wxString const& temp)
 {
 	wxDir dir(temp);
-	if (!dir.IsOpened())
+	if (!dir.IsOpened()) {
 		return;
+	}
 
 	wxString file;
-	if (!dir.GetFirst(&file, _T("fz3temp-*"), wxDIR_DIRS))
+	if (!dir.GetFirst(&file, _T("fz3temp-*"), wxDIR_DIRS)) {
 		return;
+	}
 
 	wxChar const& sep = wxFileName::GetPathSeparator();
 	do {
@@ -89,18 +93,17 @@ void CEditHandler::RemoveTemporaryFiles(wxString const& temp)
 			continue;
 		}
 
-		RemoveTemporaryFilesInSpecificDir(temp + file + sep);
+		RemoveTemporaryFilesInSpecificDir((temp + file + sep).ToStdWstring());
 	} while (dir.GetNext(&file));
 }
 
-void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
+void CEditHandler::RemoveTemporaryFilesInSpecificDir(std::wstring const& temp)
 {
-	const wxString lockfile = temp + _("fz3temp-lockfile");
+	std::wstring const lockfile = temp + L"fz3temp-lockfile";
 	if (wxFileName::FileExists(lockfile)) {
 #ifndef __WXMSW__
-		int fd = open(lockfile.mb_str(), O_RDWR | O_CLOEXEC, 0);
-		if (fd >= 0)
-		{
+		int fd = open(fz::to_string(lockfile).c_str(), O_RDWR | O_CLOEXEC, 0);
+		if (fd >= 0) {
 			// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
 			struct flock f = {};
 			f.l_type = F_WRLCK;
@@ -116,12 +119,11 @@ void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
 			close(fd);
 		}
 #endif
-		{
-			wxLogNull log;
-			wxRemoveFile(lockfile);
-		}
-		if (wxFileName::FileExists(lockfile))
+		fz::remove_file(fz::to_native(lockfile));
+
+		if (wxFileName::FileExists(lockfile)) {
 			return;
+		}
 	}
 
 	wxLogNull log;
@@ -139,21 +141,24 @@ void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
 
 }
 
-wxString CEditHandler::GetLocalDirectory()
+std::wstring CEditHandler::GetLocalDirectory()
 {
-	if (!m_localDir.empty())
+	if (!m_localDir.empty()) {
 		return m_localDir;
+	}
 
 	wxFileName tmpdir(wxFileName::GetTempDir(), _T(""));
 	// Need to call GetLongPath on MSW, GetTempDir can return short path
 	// which will cause problems when calculating maximum allowed file
 	// length
 	wxString dir = tmpdir.GetLongPath();
-	if (dir.empty() || !wxFileName::DirExists(dir))
-		return wxString();
+	if (dir.empty() || !wxFileName::DirExists(dir)) {
+		return std::wstring();
+	}
 
-	if (dir.Last() != wxFileName::GetPathSeparator())
+	if (dir.Last() != wxFileName::GetPathSeparator()) {
 		dir += wxFileName::GetPathSeparator();
+	}
 
 	// On POSIX, the permissions of the created directory (700) ensure
 	// that this is a safe operation.
@@ -161,14 +166,16 @@ wxString CEditHandler::GetLocalDirectory()
 	// already has the correct permissions which get inherited.
 	int i = 1;
 	do {
-		wxString newDir = dir + wxString::Format(_T("fz3temp-%d"), i++);
-		if (wxFileName::FileExists(newDir) || wxFileName::DirExists(newDir))
+		wxString newDir = dir + wxString::Format(_T("fz3temp-%d"), ++i);
+		if (wxFileName::FileExists(newDir) || wxFileName::DirExists(newDir)) {
 			continue;
+		}
 
-		if (!wxMkdir(newDir, 0700))
-			return wxString();
+		if (!wxMkdir(newDir, 0700)) {
+			return std::wstring();
+		}
 
-		m_localDir = newDir + wxFileName::GetPathSeparator();
+		m_localDir = (newDir + wxFileName::GetPathSeparator()).ToStdWstring();
 		break;
 	} while (true);
 
@@ -179,17 +186,15 @@ wxString CEditHandler::GetLocalDirectory()
 	RemoveTemporaryFiles(dir);
 
 #ifdef __WXMSW__
-	m_lockfile_handle = ::CreateFile((m_localDir + _T("fz3temp-lockfile")).wc_str(), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
-	if (m_lockfile_handle == INVALID_HANDLE_VALUE)
-	{
+	m_lockfile_handle = ::CreateFile((m_localDir + L"fz3temp-lockfile").c_str(), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
+	if (m_lockfile_handle == INVALID_HANDLE_VALUE) {
 		wxRmdir(m_localDir);
-		m_localDir = _T("");
+		m_localDir.clear();
 	}
 #else
-	wxString file = m_localDir + _T("fz3temp-lockfile");
-	m_lockfile_descriptor = open(file.mb_str(), O_CREAT | O_RDWR | O_CLOEXEC, 0600);
-	if (m_lockfile_descriptor >= 0)
-	{
+	auto file = fz::to_native(m_localDir) + "fz3temp-lockfile";
+	m_lockfile_descriptor = open(file.c_str(), O_CREAT | O_RDWR | O_CLOEXEC, 0600);
+	if (m_lockfile_descriptor >= 0) {
 		// Lock 1 byte region in the lockfile.
 		struct flock f = {};
 		f.l_type = F_WRLCK;
@@ -206,20 +211,24 @@ wxString CEditHandler::GetLocalDirectory()
 
 void CEditHandler::Release()
 {
-	if (m_timer.IsRunning())
+	if (m_timer.IsRunning()) {
 		m_timer.Stop();
-	if (m_busyTimer.IsRunning())
+	}
+	if (m_busyTimer.IsRunning()) {
 		m_busyTimer.Stop();
+	}
 
 	if (!m_localDir.empty()) {
 #ifdef __WXMSW__
-		if (m_lockfile_handle != INVALID_HANDLE_VALUE)
+		if (m_lockfile_handle != INVALID_HANDLE_VALUE) {
 			CloseHandle(m_lockfile_handle);
+		}
 		wxRemoveFile(m_localDir + _T("fz3temp-lockfile"));
 #else
 		wxRemoveFile(m_localDir + _T("fz3temp-lockfile"));
-		if (m_lockfile_descriptor >= 0)
+		if (m_lockfile_descriptor >= 0) {
 			close(m_lockfile_descriptor);
+		}
 #endif
 
 		wxLogNull log;
@@ -232,45 +241,51 @@ void CEditHandler::Release()
 	delete this;
 }
 
-CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName) const
+CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName) const
 {
 	std::list<t_fileData>::const_iterator iter = GetFile(fileName);
-	if (iter == m_fileDataList[local].end())
+	if (iter == m_fileDataList[local].end()) {
 		return unknown;
+	}
 
 	return iter->state;
 }
 
-CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
+CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server) const
 {
 	std::list<t_fileData>::const_iterator iter = GetFile(fileName, remotePath, server);
-	if (iter == m_fileDataList[remote].end())
+	if (iter == m_fileDataList[remote].end()) {
 		return unknown;
+	}
 
 	return iter->state;
 }
 
-int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileState state, const CServer* pServer /*=0*/) const
+int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileState state, ServerWithCredentials const& server) const
 {
 	int count = 0;
 	if (state == unknown) {
-		wxASSERT(!pServer);
-		if (type != remote)
+		wxASSERT(!server);
+		if (type != remote) {
 			count += m_fileDataList[local].size();
-		if (type != local)
+		}
+		if (type != local) {
 			count += m_fileDataList[remote].size();
+		}
 	}
 	else {
-		auto f = [state, pServer](decltype(m_fileDataList[0]) & items) {
-			int count = 0;
+		auto f = [state, &server](decltype(m_fileDataList[0]) & items) {
+			int cnt = 0;
 			for (auto const& data : items) {
-				if (data.state != state)
+				if (data.state != state) {
 					continue;
+				}
 
-				if (!pServer || data.server == *pServer)
-					++count;
+				if (!server || data.server == server) {
+					++cnt;
+				}
 			}
-			return count;
+			return cnt;
 		};
 		if (type != remote) {
 			count += f(m_fileDataList[local]);
@@ -283,15 +298,17 @@ int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileSt
 	return count;
 }
 
-bool CEditHandler::AddFile(CEditHandler::fileType type, wxString& fileName, const CServerPath& remotePath, const CServer& server)
+bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
 {
 	wxASSERT(type != none);
 
 	fileState state;
-	if (type == local)
+	if (type == local) {
 		state = GetFileState(fileName);
-	else
+	}
+	else {
 		state = GetFileState(fileName, remotePath, server);
+	}
 	if (state != unknown) {
 		wxFAIL_MSG(_T("File state not unknown"));
 		return false;
@@ -306,17 +323,19 @@ bool CEditHandler::AddFile(CEditHandler::fileType type, wxString& fileName, cons
 	}
 	else {
 		data.file = fileName;
-		data.name = wxFileName(fileName).GetFullName();
+		data.name = wxFileName(fileName).GetFullName().ToStdWstring();
 		data.state = edit;
 	}
 	data.remotePath = remotePath;
 	data.server = server;
 
-	if (type == local && !COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL))
+	if (type == local && !COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
 		return StartEditing(local, data);
+	}
 
-	if (type == remote || StartEditing(type, data))
+	if (type == remote || StartEditing(type, data)) {
 		m_fileDataList[type].push_back(data);
+	}
 
 	return true;
 }
@@ -324,32 +343,34 @@ bool CEditHandler::AddFile(CEditHandler::fileType type, wxString& fileName, cons
 bool CEditHandler::Remove(const wxString& fileName)
 {
 	std::list<t_fileData>::iterator iter = GetFile(fileName);
-	if (iter == m_fileDataList[local].end())
+	if (iter == m_fileDataList[local].end()) {
 		return true;
+	}
 
 	wxASSERT(iter->state != upload && iter->state != upload_and_remove);
-	if (iter->state == upload || iter->state == upload_and_remove)
+	if (iter->state == upload || iter->state == upload_and_remove) {
 		return false;
+	}
 
 	m_fileDataList[local].erase(iter);
 
 	return true;
 }
 
-bool CEditHandler::Remove(const wxString& fileName, const CServerPath& remotePath, const CServer& server)
+bool CEditHandler::Remove(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
 {
 	std::list<t_fileData>::iterator iter = GetFile(fileName, remotePath, server);
-	if (iter == m_fileDataList[remote].end())
+	if (iter == m_fileDataList[remote].end()) {
 		return true;
+	}
 
 	wxASSERT(iter->state != download && iter->state != upload && iter->state != upload_and_remove);
-	if (iter->state == download || iter->state == upload || iter->state == upload_and_remove)
+	if (iter->state == download || iter->state == upload || iter->state == upload_and_remove) {
 		return false;
+	}
 
-	if (wxFileName::FileExists(iter->file))
-	{
-		if (!wxRemoveFile(iter->file))
-		{
+	if (wxFileName::FileExists(iter->file)) {
+		if (!wxRemoveFile(iter->file)) {
 			iter->state = removing;
 			return false;
 		}
@@ -382,8 +403,9 @@ bool CEditHandler::RemoveAll(bool force)
 	keep.clear();
 
 	for (auto iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
-		if (force)
+		if (force) {
 			continue;
+		}
 
 		if (iter->state == upload || iter->state == upload_and_remove) {
 			keep.push_back(*iter);
@@ -395,12 +417,13 @@ bool CEditHandler::RemoveAll(bool force)
 	return m_fileDataList[local].empty() && m_fileDataList[remote].empty();
 }
 
-bool CEditHandler::RemoveAll(fileState state, const CServer* pServer /*=0*/)
+bool CEditHandler::RemoveAll(fileState state, ServerWithCredentials const& server)
 {
 	// Others not implemented
 	wxASSERT(state == upload_and_remove_failed);
-	if (state != upload_and_remove_failed)
+	if (state != upload_and_remove_failed) {
 		return false;
+	}
 
 	std::list<t_fileData> keep;
 
@@ -410,7 +433,7 @@ bool CEditHandler::RemoveAll(fileState state, const CServer* pServer /*=0*/)
 			continue;
 		}
 
-		if (pServer && iter->server != *pServer) {
+		if (server && iter->server != server) {
 			keep.push_back(*iter);
 			continue;
 		}
@@ -432,8 +455,9 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxStri
 {
 	std::list<t_fileData>::iterator iter;
 	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
-		if (iter->file == fileName)
+		if (iter->file == fileName) {
 			break;
+		}
 	}
 
 	return iter;
@@ -443,25 +467,29 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 {
 	std::list<t_fileData>::const_iterator iter;
 	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
-		if (iter->file == fileName)
+		if (iter->file == fileName) {
 			break;
+		}
 	}
 
 	return iter;
 }
 
-std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxString& fileName, const CServerPath& remotePath, const CServer& server)
+std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
 {
 	std::list<t_fileData>::iterator iter;
 	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
-		if (iter->name != fileName)
+		if (iter->name != fileName) {
 			continue;
+		}
 
-		if (iter->server != server)
+		if (iter->server != server) {
 			continue;
+		}
 
-		if (iter->remotePath != remotePath)
+		if (iter->remotePath != remotePath) {
 			continue;
+		}
 
 		return iter;
 	}
@@ -469,18 +497,21 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxStri
 	return iter;
 }
 
-std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
+std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server) const
 {
 	std::list<t_fileData>::const_iterator iter;
 	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
-		if (iter->name != fileName)
+		if (iter->name != fileName) {
 			continue;
+		}
 
-		if (iter->server != server)
+		if (iter->server != server) {
 			continue;
+		}
 
-		if (iter->remotePath != remotePath)
+		if (iter->remotePath != remotePath) {
 			continue;
+		}
 
 		return iter;
 	}
@@ -488,11 +519,12 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 	return iter;
 }
 
-void CEditHandler::FinishTransfer(bool, const wxString& fileName)
+void CEditHandler::FinishTransfer(bool, wxString const& fileName)
 {
 	auto iter = GetFile(fileName);
-	if (iter == m_fileDataList[local].end())
+	if (iter == m_fileDataList[local].end()) {
 		return;
+	}
 
 	wxASSERT(iter->state == upload || iter->state == upload_and_remove);
 
@@ -502,10 +534,12 @@ void CEditHandler::FinishTransfer(bool, const wxString& fileName)
 		m_fileDataList[local].erase(iter);
 		break;
 	case upload:
-		if (wxFileName::FileExists(fileName))
+		if (wxFileName::FileExists(fileName)) {
 			iter->state = edit;
-		else
+		}
+		else {
 			m_fileDataList[local].erase(iter);
+		}
 		break;
 	default:
 		return;
@@ -514,11 +548,12 @@ void CEditHandler::FinishTransfer(bool, const wxString& fileName)
 	SetTimerState();
 }
 
-void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, const CServerPath& remotePath, const CServer& server)
+void CEditHandler::FinishTransfer(bool successful, wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
 {
 	auto iter = GetFile(fileName, remotePath, server);
-	if (iter == m_fileDataList[remote].end())
+	if (iter == m_fileDataList[remote].end()) {
 		return;
+	}
 
 	wxASSERT(iter->state == download || iter->state == upload || iter->state == upload_and_remove);
 
@@ -526,34 +561,43 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, con
 	{
 	case upload_and_remove:
 		if (successful) {
-			if (wxFileName::FileExists(iter->file) && !wxRemoveFile(iter->file))
+			if (wxFileName::FileExists(iter->file) && !wxRemoveFile(iter->file)) {
 				iter->state = removing;
-			else
+			}
+			else {
 				m_fileDataList[remote].erase(iter);
+			}
 		}
 		else {
-			if (!wxFileName::FileExists(iter->file))
+			if (!wxFileName::FileExists(iter->file)) {
 				m_fileDataList[remote].erase(iter);
-			else
+			}
+			else {
 				iter->state = upload_and_remove_failed;
+			}
 		}
 		break;
 	case upload:
-		if (wxFileName::FileExists(iter->file))
+		if (wxFileName::FileExists(iter->file)) {
 			iter->state = edit;
-		else
+		}
+		else {
 			m_fileDataList[remote].erase(iter);
+		}
 		break;
 	case download:
 		if (wxFileName::FileExists(iter->file)) {
 			iter->state = edit;
-			if (StartEditing(remote, *iter))
+			if (StartEditing(remote, *iter)) {
 				break;
+			}
 		}
-		if (wxFileName::FileExists(iter->file) && !wxRemoveFile(iter->file))
+		if (wxFileName::FileExists(iter->file) && !wxRemoveFile(iter->file)) {
 			iter->state = removing;
-		else
+		}
+		else {
 			m_fileDataList[remote].erase(iter);
+		}
 		break;
 	default:
 		return;
@@ -562,20 +606,22 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, con
 	SetTimerState();
 }
 
-bool CEditHandler::StartEditing(const wxString& file)
+bool CEditHandler::StartEditing(wxString const& file)
 {
 	auto iter = GetFile(file);
-	if (iter == m_fileDataList[local].end())
+	if (iter == m_fileDataList[local].end()) {
 		return false;
+	}
 
 	return StartEditing(local, *iter);
 }
 
-bool CEditHandler::StartEditing(const wxString& file, const CServerPath& remotePath, const CServer& server)
+bool CEditHandler::StartEditing(wxString const& file, CServerPath const& remotePath, ServerWithCredentials const& server)
 {
 	auto iter = GetFile(file, remotePath, server);
-	if (iter == m_fileDataList[remote].end())
+	if (iter == m_fileDataList[remote].end()) {
 		return false;
+	}
 
 	return StartEditing(remote, *iter);
 }
@@ -729,7 +775,7 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 	return res;
 }
 
-bool CEditHandler::UploadFile(const wxString& file, const CServerPath& remotePath, const CServer& server, bool unedit)
+bool CEditHandler::UploadFile(wxString const& file, CServerPath const& remotePath, ServerWithCredentials const& server, bool unedit)
 {
 	std::list<t_fileData>::iterator iter = GetFile(file, remotePath, server);
 	return UploadFile(remote, iter, unedit);
@@ -745,12 +791,14 @@ bool CEditHandler::UploadFile(fileType type, std::list<t_fileData>::iterator ite
 {
 	wxCHECK(type != none, false);
 
-	if (iter == m_fileDataList[type].end())
+	if (iter == m_fileDataList[type].end()) {
 		return false;
+	}
 
 	wxASSERT(iter->state == edit || iter->state == upload_and_remove_failed);
-	if (iter->state != edit && iter->state != upload_and_remove_failed)
+	if (iter->state != edit && iter->state != upload_and_remove_failed) {
 		return false;
+	}
 
 	iter->state = unedit ? upload_and_remove : upload;
 
@@ -763,17 +811,17 @@ bool CEditHandler::UploadFile(fileType type, std::list<t_fileData>::iterator ite
 		return false;
 	}
 
-	if (mtime.empty())
+	if (mtime.empty()) {
 		mtime = fz::datetime::now();
+	}
 
 	iter->modificationTime = mtime;
 
 	wxASSERT(m_pQueue);
 
-	wxString file;
+	std::wstring file;
 	CLocalPath localPath(iter->file, &file);
-	if (file.empty())
-	{
+	if (file.empty()) {
 		m_fileDataList[type].erase(iter);
 		return false;
 	}
@@ -789,8 +837,9 @@ void CEditHandler::OnTimerEvent(wxTimerEvent&)
 #ifdef __WXMSW__
 	// Don't check for changes if mouse is captured,
 	// e.g. if user is dragging a file
-	if (GetCapture())
+	if (GetCapture()) {
 		return;
+	}
 #endif
 
 	CheckForModifications();
@@ -800,13 +849,14 @@ void CEditHandler::SetTimerState()
 {
 	bool editing = GetFileCount(none, edit) != 0;
 
-	if (m_timer.IsRunning())
-	{
-		if (!editing)
+	if (m_timer.IsRunning()) {
+		if (!editing) {
 			m_timer.Stop();
+		}
 	}
-	else if (editing)
+	else if (editing) {
 		m_timer.Start(15000);
+	}
 }
 
 wxString CEditHandler::CanOpen(CEditHandler::fileType type, const wxString& fileName, bool &dangerous, bool &program_exists)
@@ -887,43 +937,47 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 	wxFileName fn(file);
 
 	wxString ext = fn.GetExt();
-	if (ext.empty())
-	{
-		if (fn.GetFullName()[0] == '.')
+	if (ext.empty()) {
+		if (fn.GetFullName()[0] == '.') {
 			ext = _T(".");
-		else
+		}
+		else {
 			ext = _T("/");
+		}
 	}
 
 	wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS) + _T("\n");
 	associations.Replace(_T("\r"), _T(""));
 	int pos;
-	while ((pos = associations.Find('\n')) != -1)
-	{
+	while ((pos = associations.Find('\n')) != -1) {
 		wxString assoc = associations.Left(pos);
 		associations = associations.Mid(pos + 1);
 
-		if (assoc.empty())
+		if (assoc.empty()) {
 			continue;
+		}
 
 		wxString command;
-		if (!UnquoteCommand(assoc, command))
+		if (!UnquoteCommand(assoc, command)) {
 			continue;
+		}
 
-		if (assoc != ext)
+		if (assoc != ext) {
 			continue;
+		}
 
 		wxString prog = command;
 
 		wxString args;
-		if (!UnquoteCommand(prog, args))
+		if (!UnquoteCommand(prog, args)) {
 			return wxString();
+		}
 
-		if (prog.empty())
+		if (prog.empty()) {
 			return wxString();
+		}
 
-		if (!ProgramExists(prog))
-		{
+		if (!ProgramExists(prog)) {
 			program_exists = false;
 			return prog;
 		}
@@ -940,72 +994,76 @@ void CEditHandler::OnChangedFileEvent(wxCommandEvent&)
 	CheckForModifications();
 }
 
-wxString CEditHandler::GetTemporaryFile(wxString name)
+std::wstring CEditHandler::GetTemporaryFile(std::wstring name)
 {
+	name = CQueueView::ReplaceInvalidCharacters(name);
 #ifdef __WXMSW__
 	// MAX_PATH - 1 is theoretical limit, we subtract another 4 to allow
 	// editors which create temporary files
-	int max = MAX_PATH - 5;
+	size_t max = MAX_PATH - 5;
 #else
-	int max = -1;
+	size_t max = std::wstring::npos;
 #endif
-	if (max != -1)
-	{
+	if (max != std::wstring::npos) {
 		name = TruncateFilename(m_localDir, name, max);
-		if (name.empty())
-			return wxString();
+		if (name.empty()) {
+			return std::wstring();
+		}
 	}
 
-	wxString file = m_localDir + name;
-	if (!FilenameExists(file))
+	std::wstring file = m_localDir + name;
+	if (!FilenameExists(file)) {
 		return file;
+	}
 
-	if (max != -1)
-		max--;
+	if (max != std::wstring::npos) {
+		--max;
+	}
 	int cutoff = 1;
 	int n = 1;
-	while (++n < 10000) // Just to give up eventually
-	{
+	while (++n < 10000) { // Just to give up eventually
 		// Further reduce length if needed
-		if (max != -1 && n >= cutoff)
-		{
+		if (max != std::wstring::npos && n >= cutoff) {
 			cutoff *= 10;
-			max--;
+			--max;
 			name = TruncateFilename(m_localDir, name, max);
-			if (name.empty())
-				return wxString();
+			if (name.empty()) {
+				return std::wstring();
+			}
 		}
 
-		int pos = name.Find('.', true);
-		if (pos < 1)
-			file = m_localDir + name + wxString::Format(_T(" %d"), n);
-		else
-			file = m_localDir + name.Left(pos) + wxString::Format(_T(" %d"), n) + name.Mid(pos);
+		size_t pos = name.rfind('.');
+		if (pos == std::wstring::npos || !pos) {
+			file = m_localDir + name + fz::sprintf(L" %d", n);
+		}
+		else {
+			file = m_localDir + name.substr(0, pos) + fz::sprintf(L" %d", n) + name.substr(pos);
+		}
 
-		if (!FilenameExists(file))
+		if (!FilenameExists(file)) {
 			return file;
+		}
 	}
 
-	return wxString();
+	return std::wstring();
 }
 
-wxString CEditHandler::TruncateFilename(const wxString path, const wxString& name, int max)
+std::wstring CEditHandler::TruncateFilename(std::wstring const& path, std::wstring const& name, size_t max)
 {
-	const int pathlen = path.Len();
-	const int namelen = name.Len();
-	if (namelen + pathlen > max)
-	{
-		int pos = name.Find('.', true);
-		if (pos != -1)
-		{
-			int extlen = namelen - pos;
+	size_t const pathlen = path.size();
+	size_t const namelen = name.size();
+
+	if (namelen + pathlen > max) {
+		size_t pos = name.rfind('.');
+		if (pos != std::wstring::npos) {
+			size_t extlen = namelen - pos;
 			if (pathlen + extlen >= max)
 			{
 				// Cannot truncate extension
-				return wxString();
+				return std::wstring();
 			}
 
-			return name.Left(max - pathlen - extlen) + name.Mid(pos);
+			return name.substr(0, max - pathlen - extlen) + name.substr(pos);
 		}
 	}
 
@@ -1017,8 +1075,9 @@ bool CEditHandler::FilenameExists(const wxString& file)
 	for (auto const& fileData : m_fileDataList[remote]) {
 		// Always ignore case, we don't know which type of filesystem the user profile
 		// is installed upon.
-		if (!fileData.file.CmpNoCase(file))
+		if (!wxString(fileData.file).CmpNoCase(file)) {
 			return true;
+		}
 	}
 
 	if (wxFileName::FileExists(file)) {
@@ -1056,8 +1115,7 @@ CEditHandlerStatusDialog::CEditHandlerStatusDialog(wxWindow* parent)
 
 CEditHandlerStatusDialog::~CEditHandlerStatusDialog()
 {
-	if (m_pWindowStateManager)
-	{
+	if (m_pWindowStateManager) {
 		m_pWindowStateManager->Remember(OPTION_EDITSTATUSDIALOG_SIZE);
 		delete m_pWindowStateManager;
 	}
@@ -1479,15 +1537,15 @@ void CNewAssociationDialog::OnOK(wxCommandEvent&)
 		}
 
 		if (always)
-			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("2") + cmd);
+			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("2") + cmd.ToStdWstring());
 		else {
 			wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS);
 			if (!associations.empty() && associations.Last() != '\n')
 				associations += '\n';
 			if (m_ext.empty())
 				m_ext = _T("/");
-			associations += m_ext + _T(" ") + cmd;
-			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations);
+			associations += m_ext.ToStdWstring() + _T(" ") + cmd.ToStdWstring();
+			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations.ToStdWstring());
 		}
 	}
 	else {
@@ -1520,7 +1578,7 @@ void CNewAssociationDialog::OnOK(wxCommandEvent&)
 			if (m_ext.empty())
 				m_ext = _T("/");
 			associations += m_ext + _T(" ") + cmd;
-			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations);
+			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations.ToStdWstring());
 		}
 	}
 
@@ -1539,12 +1597,14 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent&)
 #endif
 		wxFD_OPEN | wxFD_FILE_MUST_EXIST);
 
-	if (dlg.ShowModal() != wxID_OK)
+	if (dlg.ShowModal() != wxID_OK) {
 		return;
+	}
 
 	wxString editor = dlg.GetPath();
-	if (editor.empty())
+	if (editor.empty()) {
 		return;
+	}
 
 	if (!ProgramExists(editor)) {
 		XRCCTRL(*this, "ID_EDITOR", wxWindow)->SetFocus();
@@ -1552,13 +1612,14 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent&)
 		return;
 	}
 
-	if (editor.Find(' ') != -1)
+	if (editor.Find(' ') != -1) {
 		editor = _T("\"") + editor + _T("\"");
+	}
 
 	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->ChangeValue(editor);
 }
 
-bool CEditHandler::Edit(CEditHandler::fileType type, wxString const fileName, CServerPath const& path, CServer const& server, int64_t size, wxWindow * parent)
+bool CEditHandler::Edit(CEditHandler::fileType type, std::wstring const& fileName, CServerPath const& path, ServerWithCredentials const& server, int64_t size, wxWindow * parent)
 {
 	std::vector<FileData> data;
 	FileData d{fileName, size};
@@ -1567,7 +1628,7 @@ bool CEditHandler::Edit(CEditHandler::fileType type, wxString const fileName, CS
 	return Edit(type, data, path, server, parent);
 }
 
-bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, CServer const& server, wxWindow * parent)
+bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, ServerWithCredentials const& server, wxWindow * parent)
 {
 	if (type == CEditHandler::remote) {
 		wxString const& localDir = GetLocalDirectory();
@@ -1587,8 +1648,9 @@ bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const
 		dlg.SetTitle(_("Confirmation needed"));
 		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
 
-		if (!dlg.Run())
+		if (!dlg.Run()) {
 			return false;
+		}
 	}
 
 	bool success = true;
@@ -1602,7 +1664,7 @@ bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const
 	return success;
 }
 
-bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, CServer const& server, wxWindow * parent, size_t fileCount, int & already_editing_action)
+bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, ServerWithCredentials const& server, wxWindow * parent, size_t fileCount, int & already_editing_action)
 {
 	bool dangerous = false;
 	bool program_exists = false;
@@ -1632,10 +1694,12 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 	}
 
 	fileState state;
-	if (type == local)
+	if (type == local) {
 		state = GetFileState(file.name);
-	else
+	}
+	else {
 		state = GetFileState(file.name, path, server);
+	}
 	switch (state)
 	{
 	case CEditHandler::download:
@@ -1728,7 +1792,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 		break;
 	}
 
-	wxString localFile = file.name;
+	std::wstring localFile = file.name;
 	if (!AddFile(type, localFile, path, server)) {
 		if( type == CEditHandler::local) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), file.name), _("Opening failed"), wxICON_EXCLAMATION);
@@ -1741,10 +1805,10 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 	}
 
 	if (type == CEditHandler::remote) {
-		wxString localFileName;
+		std::wstring localFileName;
 		CLocalPath localPath(localFile, &localFileName);
 
-		m_pQueue->QueueFile(false, true, file.name, (localFileName != file.name) ? localFileName : wxString(),
+		m_pQueue->QueueFile(false, true, file.name, (localFileName != file.name) ? localFileName : std::wstring(),
 			localPath, path, server, file.size, type, QueuePriority::high);
 		m_pQueue->QueueFile_Finish(true);
 	}
