@@ -1,5 +1,5 @@
-#ifndef __FILELISTCTRL_H__
-#define __FILELISTCTRL_H__
+#ifndef FILEZILLA_INTERFACE_FILELISTCTRL_HEADER
+#define FILEZILLA_INTERFACE_FILELISTCTRL_HEADER
 
 #include "listctrlex.h"
 #include "systemimagelist.h"
@@ -29,6 +29,10 @@ public:
 class CFileListCtrlSortBase
 {
 public:
+	CFileListCtrlSortBase() = default;
+	CFileListCtrlSortBase(CFileListCtrlSortBase const&) = delete;
+	CFileListCtrlSortBase& operator=(CFileListCtrlSortBase const&) = delete;
+
 	enum DirSortMode
 	{
 		dirsort_ontop,
@@ -76,16 +80,18 @@ public:
 	static int CmpNoCase(std::wstring const& str1, std::wstring const& str2)
 	{
 		int cmp = fz::stricmp(str1, str2);
-		if (cmp)
+		if (cmp) {
 			return cmp;
+		}
 		return str1.compare(str2);
 	}
 
 	static int CmpNoCase(wxString const& str1, wxString const& str2)
 	{
 		int cmp = str1.CmpNoCase(str2);
-		if (cmp)
+		if (cmp) {
 			return cmp;
+		}
 		return str1.Cmp(str2);
 	}
 
@@ -99,7 +105,7 @@ public:
 		return CmpNatural(str1.c_str(), str2.c_str());
 	}
 
-	static int CmpNatural(wxChar const* p1, wxChar const* p2)
+	static int CmpNatural(wchar_t const* p1, wchar_t const* p2)
 	{
 		int res = 0;
 		int zeroCount = 0;
@@ -107,42 +113,56 @@ public:
 		for (; *p1 && *p2; ++p1, ++p2) {
 			int diff = static_cast<int>(wxTolower(*p1)) - static_cast<int>(wxTolower(*p2));
 			if (isNumber) {
-				if (res == 0)
+				if (res == 0) {
 					res = diff;
+				}
 				int nbDigits = (wxIsdigit(*p1) ? 1 : 0) + (wxIsdigit(*p2) ? 1 : 0);
 				if (nbDigits == 0 && res == 0) {
-					if (zeroCount)
+					if (zeroCount) {
 						break;
+					}
 					isNumber = false;
-				} else if (nbDigits != 2)
+				}
+				else if (nbDigits != 2) {
 					break;
-			} else if (wxIsdigit(*p1) && wxIsdigit(*p2)) {
+				}
+			}
+			else if (wxIsdigit(*p1) && wxIsdigit(*p2)) {
 				zeroCount = 0;
-				for(; *p1 == '0' && *(p1+1) && wxIsdigit(*(p1+1)); ++p1)
+				for (; *p1 == '0' && *(p1 + 1) && wxIsdigit(*(p1 + 1)); ++p1) {
 					zeroCount++;
-				for(; *p2 == '0' && *(p2+1) && wxIsdigit(*(p2+1)); ++p2)
+				}
+				for (; *p2 == '0' && *(p2 + 1) && wxIsdigit(*(p2 + 1)); ++p2) {
 					zeroCount--;
+				}
 				res = *p1 - *p2;
 				isNumber = true;
-			} else if (diff)
+			}
+			else if (diff) {
 				return diff;
+			}
 		}
 
-		if (res == 0 && isNumber)
+		if (res == 0 && isNumber) {
 			res = zeroCount;
+		}
 
-		if (!*p1 && !*p2)
+		if (!*p1 && !*p2) {
 			return res;
-		if (!isNumber || res == 0)
-			 return !*p1 ? -1 : 1;
-		if (*p1 && wxIsdigit(*p1))
+		}
+		if (!isNumber || res == 0) {
+			return !*p1 ? -1 : 1;
+		}
+		if (*p1 && wxIsdigit(*p1)) {
 			return 1;       //more digits
-		if (*p2 && wxIsdigit(*p2))
+		}
+		if (*p2 && wxIsdigit(*p2)) {
 			return -1;      //fewer digits
+		}
 		return res;         //same length, compare first different digit in the sequence
 	}
 
-	typedef int (* CompareFunction)(const wxString&, const wxString&);
+	typedef int (* CompareFunction)(std::wstring const&, std::wstring const&);
 	static CompareFunction GetCmpFunction(NameSortMode mode)
 	{
 		switch (mode)
@@ -197,29 +217,37 @@ public:
 		default:
 		case dirsort_ontop:
 			if (data1.is_dir()) {
-				if (!data2.is_dir())
+				if (!data2.is_dir()) {
 					return -1;
-				else
+				}
+				else {
 					return 0;
+				}
 			}
 			else {
-				if (data2.is_dir())
+				if (data2.is_dir()) {
 					return 1;
-				else
+				}
+				else {
 					return 0;
+				}
 			}
 		case dirsort_onbottom:
 			if (data1.is_dir()) {
-				if (!data2.is_dir())
+				if (!data2.is_dir()) {
 					return 1;
-				else
+				}
+				else {
 					return 0;
+				}
 			}
 			else {
-				if (data2.is_dir())
+				if (data2.is_dir()) {
 					return -1;
-				else
+				}
+				else {
 					return 0;
+				}
 			}
 		case dirsort_inline:
 			return 0;
@@ -235,12 +263,15 @@ public:
 	inline int CmpSize(const value_type &data1, const value_type &data2) const
 	{
 		int64_t const diff = data1.size - data2.size;
-		if (diff < 0)
+		if (diff < 0) {
 			return -1;
-		else if (diff > 0)
+		}
+		else if (diff > 0) {
 			return 1;
-		else
+		}
+		else {
 			return 0;
+		}
 	}
 
 	inline int CmpStringNoCase(const wxString &data1, const wxString &data2) const
@@ -250,10 +281,10 @@ public:
 
 	inline int CmpTime(const value_type &data1, const value_type &data2) const
 	{
-		if( data1.time < data2.time ) {
+		if (data1.time < data2.time) {
 			return -1;
 		}
-		else if( data1.time > data2.time ) {
+		else if (data1.time > data2.time) {
 			return 1;
 		}
 		else {
@@ -270,7 +301,7 @@ protected:
 
 template<class CFileData> class CFileListCtrl;
 
-template<class T, typename DataEntry> class CReverseSort : public T
+template<class T, typename DataEntry> class CReverseSort final : public T
 {
 public:
 	CReverseSort(typename T::List const& listing, std::vector<DataEntry>& fileData, CFileListCtrlSortBase::DirSortMode dirSortMode, CFileListCtrlSortBase::NameSortMode nameSortMode, CFileListCtrl<DataEntry>* const pListView)
@@ -344,10 +375,12 @@ public:
 
 		DataEntry &type1 = m_fileData[a];
 		DataEntry &type2 = m_fileData[b];
-		if (type1.fileType.empty())
-			type1.fileType = m_pListView->GetType(data1.name, data1.is_dir()).ToStdWstring();
-		if (type2.fileType.empty())
-			type2.fileType = m_pListView->GetType(data2.name, data2.is_dir()).ToStdWstring();
+		if (type1.fileType.empty()) {
+			type1.fileType = m_pListView->GetType(data1.name, data1.is_dir());
+		}
+		if (type2.fileType.empty()) {
+			type2.fileType = m_pListView->GetType(data2.name, data2.is_dir());
+		}
 
 		CMP(CmpStringNoCase, type1.fileType, type2.fileType);
 
@@ -440,10 +473,12 @@ public:
 		typename Listing::value_type const& data1 = this->m_listing[a];
 		typename Listing::value_type const& data2 = this->m_listing[b];
 
-		if (data1.path < data2.path)
+		if (data1.path < data2.path) {
 			return true;
-		if (data1.path != data2.path)
+		}
+		if (data1.path != data2.path) {
 			return false;
+		}
 
 		CMP_LESS(CmpName, data1, data2);
 	}
@@ -468,10 +503,12 @@ public:
 		CMP(CmpDir, data1, data2);
 		CMP(CmpName, data1, data2);
 
-		if (data1.path < data2.path)
+		if (data1.path < data2.path) {
 			return true;
-		if (data1.path != data2.path)
+		}
+		if (data1.path != data2.path) {
 			return false;
+		}
 
 		CMP_LESS(CmpName, data1, data2);
 	}
@@ -490,28 +527,7 @@ template<class CFileData> class CFileListCtrl : public wxListCtrlEx, public CCom
 	template<typename Listing, typename DataEntry> friend class CFileListCtrlSortType;
 public:
 	CFileListCtrl(wxWindow* pParent, CQueueView *pQueue, bool border = false);
-	virtual ~CFileListCtrl();
-
-	class CSortComparisonObject : public std::binary_function<int,int,bool>
-	{
-	public:
-		CSortComparisonObject(CFileListCtrlSortBase* pObject)
-			: m_pObject(pObject)
-		{
-		}
-
-		void Destroy()
-		{
-			delete m_pObject;
-		}
-
-		inline bool operator()(int a, int b)
-		{
-			return m_pObject->operator ()(a, b);
-		}
-	protected:
-		CFileListCtrlSortBase* m_pObject;
-	};
+	virtual ~CFileListCtrl() = default;
 
 	void SetFilelistStatusBar(CFilelistStatusBar* pFilelistStatusBar) { m_pFilelistStatusBar = pFilelistStatusBar; }
 	CFilelistStatusBar* GetFilelistStatusBar() { return m_pFilelistStatusBar; }
@@ -523,7 +539,7 @@ public:
 	std::vector<unsigned int> const& indexMapping() const { return m_indexMapping; }
 
 protected:
-	CQueueView *m_pQueue;
+	CQueueView *m_pQueue{};
 
 	std::vector<CFileData> m_fileData;
 	std::vector<unsigned int> m_indexMapping;
@@ -532,10 +548,10 @@ protected:
 	virtual bool ItemIsDir(int index) const = 0;
 	virtual int64_t ItemGetSize(int index) const = 0;
 
-	std::map<wxString, wxString> m_fileTypeMap;
+	std::map<wxString, std::wstring> m_fileTypeMap;
 
 	// The .. item
-	bool m_hasParent;
+	bool m_hasParent{true};
 
 	int m_sortColumn{-1};
 	int m_sortDirection{};
@@ -544,10 +560,10 @@ protected:
 	void SortList(int column = -1, int direction = -1, bool updateSelections = true);
 	CFileListCtrlSortBase::DirSortMode GetDirSortMode();
 	CFileListCtrlSortBase::NameSortMode GetNameSortMode();
-	virtual CSortComparisonObject GetSortComparisonObject() = 0;
+	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() = 0;
 
 	// An empty path denotes a virtual file
-	wxString GetType(wxString name, bool dir, const wxString& path = _T(""));
+	std::wstring GetType(std::wstring name, bool dir, std::wstring const& path = std::wstring());
 
 	// Comparison related
 	virtual void ScrollTopItem(int item);
@@ -555,7 +571,7 @@ protected:
 	virtual void OnExitComparisonMode();
 	virtual void CompareAddFile(t_fileEntryFlags flags);
 
-	int m_comparisonIndex;
+	int m_comparisonIndex{-1};
 
 	// Remembers which non-fill items are selected if enabling/disabling comparison.
 	// Exploit fact that sort order doesn't change -> O(n)
@@ -563,7 +579,7 @@ protected:
 	void ComparisonRestoreSelections();
 	std::deque<int> m_comparisonSelections;
 
-	CFilelistStatusBar* m_pFilelistStatusBar;
+	CFilelistStatusBar* m_pFilelistStatusBar{};
 
 	// Indexes of the items added, sorted ascending.
 	void UpdateSelections_ItemsAdded(std::vector<int> const& added_indexes);
@@ -589,22 +605,22 @@ private:
 	void SortList_UpdateSelections(bool* selections, int focused_item, unsigned int focused_index);
 
 	// If this is set to true, don't process selection changed events
-	bool m_insideSetSelection;
+	bool m_insideSetSelection{};
 
 #ifdef __WXMSW__
 	virtual WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
 	virtual bool MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result);
 #else
-	int m_focusItem;
+	int m_focusItem{-1};
 	std::vector<bool> m_selections;
-	int m_pending_focus_processing;
+	int m_pending_focus_processing{};
 #endif
 
 #if defined(__WXGTK__) && !defined(__WXGTK3__)
 	std::unique_ptr<CGtkEventCallbackProxyBase> m_gtkEventCallbackProxy;
 #endif
 
-	wxString m_genericTypes[2];
+	std::wstring m_genericTypes[2];
 
 	DECLARE_EVENT_TABLE()
 	void OnColumnClicked(wxListEvent &event);
@@ -620,8 +636,24 @@ private:
 	void OnKeyDown(wxKeyEvent& event);
 };
 
+class SortPredicate
+{
+public:
+	SortPredicate() = delete;
+	SortPredicate(std::unique_ptr<CFileListCtrlSortBase> const& ref)
+		: p_(ref.get())
+	{}
+
+	inline bool operator()(int lhs, int rhs) {
+		return (*p_)(lhs, rhs);
+	}
+
+public:
+	CFileListCtrlSortBase const* p_;
+};
+
 #ifdef FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 #include "filelistctrl.cpp"
 #endif
 
-#endif //__FILELISTCTRL_H__
+#endif
