@@ -1,5 +1,5 @@
-#ifndef __FILTER_H__
-#define __FILTER_H__
+#ifndef FILEZILLA_INTERFACE_FILTER_HEADER
+#define FILEZILLA_INTERFACE_FILTER_HEADER
 
 #include "dialogex.h"
 
@@ -14,7 +14,6 @@ enum t_filterType
 	filter_permissions = 0x08,
 	filter_path = 0x10,
 	filter_date = 0x20,
-	filter_time = 0x40,
 #ifdef __WXMSW__
 	filter_meta = filter_attributes,
 	filter_foreign = filter_permissions,
@@ -27,16 +26,16 @@ enum t_filterType
 class CFilterCondition
 {
 public:
-	CFilterCondition();
+	bool set(t_filterType t, std::wstring const& v, int c, bool matchCase);
 
-	t_filterType type;
-	int condition;
-
-	wxString strValue; // All other types
-	int64_t value; // If type is size
+	std::wstring strValue; 
+	std::wstring lowerValue; // Name and path matches
 	fz::datetime date; // If type is date
-	bool matchCase;
+	int64_t value{}; // If type is size
 	std::shared_ptr<std::wregex> pRegEx;
+
+	t_filterType type{filter_name};
+	int condition{};
 };
 
 class CFilter
@@ -50,21 +49,26 @@ public:
 		not_all
 	};
 
-	wxString name;
+	bool empty() const { return filters.empty(); }
 
-	bool filterFiles{true};
-	bool filterDirs{true};
-	t_matchType matchType{all};
+	explicit operator bool() const { return !filters.empty(); }
+
+	std::vector<CFilterCondition> filters;
+
+	std::wstring name;
+
+	t_matchType matchType{ all };
+
+	bool filterFiles{ true };
+	bool filterDirs{ true };
 
 	// Filenames on Windows ignore case
 #ifdef __WXMSW__
 	bool matchCase{};
 #else
-	bool matchCase{true};
+	bool matchCase{ true };
 #endif
 
-	std::vector<CFilterCondition> filters;
-
 	bool HasConditionOfType(t_filterType type) const;
 	bool IsLocalFilter() const;
 };
@@ -72,7 +76,7 @@ public:
 class CFilterSet
 {
 public:
-	wxString name;
+	std::wstring name;
 	std::vector<bool> local;
 	std::vector<bool> remote;
 };
@@ -84,12 +88,12 @@ class CFilterManager
 {
 public:
 	CFilterManager();
-	virtual ~CFilterManager() {}
+	virtual ~CFilterManager() = default;
 
 	// Note: Under non-windows, attributes are permissions
-	bool FilenameFiltered(std::wstring const& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const;
-	bool FilenameFiltered(std::vector<CFilter> const& filters, std::wstring const& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date) const;
-	static bool FilenameFilteredByFilter(CFilter const& filter, std::wstring const& name, const wxString& path, bool dir, int64_t size, int attributes, fz::datetime const& date);
+	virtual bool FilenameFiltered(std::wstring const& name, std::wstring const& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const;
+	bool FilenameFiltered(std::vector<CFilter> const& filters, std::wstring const& name, std::wstring const& path, bool dir, int64_t size, int attributes, fz::datetime const& date) const;
+	static bool FilenameFilteredByFilter(CFilter const& filter, std::wstring const& name, std::wstring const& path, bool dir, int64_t size, int attributes, fz::datetime const& date);
 	static bool HasActiveFilters(bool ignore_disabled = false);
 
 	bool HasSameLocalAndRemoteFilters() const;
@@ -98,16 +102,14 @@ public:
 
 	ActiveFilters GetActiveFilters();
 
-	static bool CompileRegexes(std::vector<CFilter>& filters);
-	static bool CompileRegexes(CFilter& filter);
+	bool HasActiveLocalFilters() const;
+	bool HasActiveRemoteFilters() const;
 
 	static void Import(pugi::xml_node& element);
 	static bool LoadFilter(pugi::xml_node& element, CFilter& filter);
 	static void SaveFilter(pugi::xml_node& element, const CFilter& filter);
 
 protected:
-	static bool CompileRegexes();
-
 	static void LoadFilters();
 	static void LoadFilters(pugi::xml_node& element);
 	static void SaveFilters();
@@ -127,7 +129,6 @@ class CFilterDialog final : public wxDialogEx, public CFilterManager
 {
 public:
 	CFilterDialog();
-	virtual ~CFilterDialog() {}
 
 	bool Create(CMainFrame* parent);
 
@@ -150,13 +151,13 @@ protected:
 
 	void OnChangeAll(wxCommandEvent& event);
 
-	bool m_shiftClick;
+	bool m_shiftClick{};
 
-	CMainFrame* m_pMainFrame;
+	CMainFrame* m_pMainFrame{};
 
 	std::vector<CFilter> m_filters;
 	std::vector<CFilterSet> m_filterSets;
 	unsigned int m_currentFilterSet;
 };
 
-#endif //__FILTER_H__
+#endif
