@@ -62,20 +62,20 @@ int CFZPuttyGenInterface::IsKeyFileEncrypted()
 bool CFZPuttyGenInterface::LoadKeyFile(std::wstring& keyFile, bool silent, std::wstring& comment, std::wstring& data)
 {
 	if (!LoadProcess(silent)) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		return false;
 	}
 
 	int needs_conversion = NeedsConversion(keyFile, silent);
 	if (needs_conversion < 0) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		return false;
 	}
 
 	ReplyCode code;
 	if (needs_conversion) {
 		if (silent) {
-			comment = _("Could not load key file");
+			comment = fztranslate("Could not load key file");
 			return false;
 		}
 
@@ -141,12 +141,12 @@ bool CFZPuttyGenInterface::LoadKeyFile(std::wstring& keyFile, bool silent, std::
 	}
 
 	if (!Send(L"fingerprint")) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		return false;
 	}
 	code = GetReply(data);
 	if (code != success) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		data.clear();
 		return false;
 	}
@@ -154,7 +154,7 @@ bool CFZPuttyGenInterface::LoadKeyFile(std::wstring& keyFile, bool silent, std::
 	Send(L"comment");
 	code = GetReply(comment);
 	if (code != success) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		data.clear();
 		comment.clear();
 		return false;
