@@ -6,9 +6,11 @@
 #include "engine_context.h"
 #include "netconfwizard.h"
 #include "Options.h"
+#include "socket_errors.h"
 #include "dialogex.h"
 #include "filezillaapp.h"
 #include "externalipresolver.h"
+#include "xrc_helper.h"
 
 DECLARE_EVENT_TYPE(fzEVT_ON_EXTERNAL_IP_ADDRESS, -1)
 DEFINE_EVENT_TYPE(fzEVT_ON_EXTERNAL_IP_ADDRESS)
@@ -51,22 +53,26 @@ CNetConfWizard::~CNetConfWizard()
 
 bool CNetConfWizard::Load()
 {
-	if (!Create(m_parent, wxID_ANY, _("Firewall and router configuration wizard"), wxNullBitmap, wxPoint(0, 0)))
+	if (!Create(m_parent, wxID_ANY, _("Firewall and router configuration wizard"), wxNullBitmap, wxPoint(0, 0))) {
 		return false;
+	}
 
 	wxSize minPageSize = GetPageAreaSizer()->GetMinSize();
 
+	InitXrc();
 	for (int i = 1; i <= 7; ++i) {
 		wxWizardPageSimple* page = new wxWizardPageSimple();
 		bool res = wxXmlResource::Get()->LoadPanel(page, this, wxString::Format(_T("NETCONF_PANEL%d"), i));
-		if (!res)
+		if (!res) {
 			return false;
+		}
 		page->Show(false);
 
 		m_pages.push_back(page);
 	}
-	for (unsigned int i = 0; i < (m_pages.size() - 1); i++)
+	for (unsigned int i = 0; i < (m_pages.size() - 1); ++i) {
 		m_pages[i]->Chain(m_pages[i], m_pages[i + 1]);
+	}
 
 	GetPageAreaSizer()->Add(m_pages[0]);
 
@@ -217,7 +223,7 @@ void CNetConfWizard::OnPageChanging(wxWizardEvent& event)
 
 		int res = m_socket->connect(fzT("probe.filezilla-project.org"), 21);
 		if (res && res != EINPROGRESS) {
-			PrintMessage(wxString::Format(_("Connect failed: %s"), fz::socket::error_description(res)), 1);
+			PrintMessage(wxString::Format(_("Connect failed: %s"), fz::socket_error_description(res)), 1);
 			CloseSocket();
 		}
 	}
@@ -259,9 +265,6 @@ void CNetConfWizard::DoOnSocketEvent(fz::socket_event_source* s, fz::socket_even
 		case fz::socket_event_flag::write:
 			OnSend();
 			break;
-		case fz::socket_event_flag::close:
-			OnClose();
-			break;
 		case fz::socket_event_flag::connection:
 			OnConnect();
 			break;
@@ -276,10 +279,6 @@ void CNetConfWizard::DoOnSocketEvent(fz::socket_event_source* s, fz::socket_even
 			return;
 		}
 		switch (t) {
-		case fz::socket_event_flag::close:
-			PrintMessage(_("Listen socket closed"), 1);
-			CloseSocket();
-			break;
 		case fz::socket_event_flag::connection:
 			OnAccept();
 			break;
@@ -294,9 +293,6 @@ void CNetConfWizard::DoOnSocketEvent(fz::socket_event_source* s, fz::socket_even
 		}
 		switch (t)
 		{
-		case fz::socket_event_flag::close:
-			OnDataClose();
-			break;
 		case fz::socket_event_flag::read:
 			OnDataReceive();
 			break;
@@ -921,7 +917,7 @@ int CNetConfWizard::CreateListenSocket()
 
 int CNetConfWizard::CreateListenSocket(unsigned int port)
 {
-	m_pSocketServer = new fz::socket(engine_context_.GetThreadPool(), this);
+	m_pSocketServer = new fz::listen_socket(engine_context_.GetThreadPool(), this);
 	int res = m_pSocketServer->listen(m_socket ? m_socket->address_family() : fz::address_type::unknown, port);
 
 	if (res < 0) {
