@@ -8,6 +8,7 @@
 #include "Mainfrm.h"
 #include "asksavepassworddialog.h"
 #include "filezillaapp.h"
+#include "themeprovider.h"
 
 #include <wx/bmpbuttn.h>
 #include <wx/statline.h>
@@ -41,7 +42,9 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 #ifndef __WXMAC__
 	sizer->Add(new wxStaticLine(this, -1, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL), wxSizerFlags().Expand());
 #endif
-	auto mainSizer = new wxFlexGridSizer(1, 0, 0, 5);
+
+	DialogLayout layout(pParent);
+	auto mainSizer = layout.createFlex(0, 1);
 	sizer->Add(mainSizer, wxSizerFlags().Border(wxALL, ConvertDialogToPixels(wxPoint(2, 0)).x));
 
 	mainSizer->Add(new wxStaticText(this, -1, _("&Host:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
@@ -69,7 +72,7 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 	connect->SetDefault();
 	connectSizer->Add(connect, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
 
-	wxBitmap bmp(wxGetApp().GetResourceDir().GetPath() + _T("dropdown.png"), wxBITMAP_TYPE_PNG);
+	wxBitmap bmp = CThemeProvider::Get()->CreateBitmap(L"ART_DROPDOWN", wxART_OTHER, CThemeProvider::GetIconSize(iconSizeTiny));
 	auto flags = wxSizerFlags().Expand();
 #if defined(__WXMSW__)
 	wxSize dropdownSize = ConvertDialogToPixels(wxSize(12, -1));
@@ -89,6 +92,15 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 	}
 #endif
 
+	Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) {
+		if (event.GetKeyCode() == WXK_DOWN) {
+			wxCommandEvent evt(wxEVT_CHAR_HOOK);
+			OnQuickconnectDropdown(evt);
+		}
+		else {
+			event.Skip();
+		}
+	});
 	return true;
 }
 
@@ -132,7 +144,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 		break;
 	default:
 		{
-			const wxString prefix = server.server.GetPrefixFromProtocol(protocol);
+			std::wstring const prefix = server.server.GetPrefixFromProtocol(protocol);
 			if (!prefix.empty()) {
 				host = prefix + _T("://") + host;
 			}
@@ -185,7 +197,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	CRecentServerList::SetMostRecentServer(server);
 }
 
-void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent&)
+void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent& event)
 {
 	wxMenu* pMenu = new wxMenu;
 
@@ -214,7 +226,9 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent&)
 		pMenu->Enable(3, false);
 	}
 
-	XRCCTRL(*this, "ID_QUICKCONNECT_DROPDOWN", wxButton)->PopupMenu(pMenu);
+	auto * btn = XRCCTRL(*this, "ID_QUICKCONNECT_DROPDOWN", wxButton);
+	auto size = btn->GetSize() / 2;
+	btn->PopupMenu(pMenu, (event.GetEventType() == wxEVT_CHAR_HOOK) ? wxPoint(size.x, size.y) : wxDefaultPosition);
 	delete pMenu;
 	m_recentServers.clear();
 }
