@@ -125,10 +125,6 @@ BEGIN_EVENT_TABLE(CIconPreview, wxScrolledWindow)
 EVT_PAINT(CIconPreview::OnPaint)
 END_EVENT_TABLE()
 
-COptionsPageThemes::~COptionsPageThemes()
-{
-}
-
 bool COptionsPageThemes::CreatePage(COptions* pOptions, CSettingsDialog* pOwner, wxWindow* parent, wxSize& maxSize)
 {
 	bool success = COptionsPage::CreatePage(pOptions, pOwner, parent, maxSize);
@@ -267,7 +263,7 @@ bool COptionsPageThemes::OnDisplayedFirstTime()
 	if (pTheme->GetSelection() == wxNOT_FOUND) {
 		pTheme->SetSelection(pTheme->FindString(firstName));
 	}
-	activeTheme = ((wxStringClientData*)pTheme->GetClientObject(pTheme->GetSelection()))->GetData();
+	activeTheme = ((wxStringClientData*)pTheme->GetClientObject(pTheme->GetSelection()))->GetData().ToStdWstring();
 
 	if (!DisplayTheme(activeTheme)) {
 		failure = true;
