@@ -3,6 +3,7 @@
 
 #include "filezillaapp.h"
 #include "ipcmutex.h"
+#include "loginmanager.h"
 #include "Options.h"
 #include "xmlfunctions.h"
 
@@ -52,7 +53,7 @@ bool Bookmark::operator==(Bookmark const& b) const
 
 bool Site::operator==(Site const& s) const
 {
-	if (m_server != s.m_server) {
+	if (server_ != s.server_) {
 		return false;
 	}
 
@@ -151,14 +152,16 @@ bool CSiteManager::ReadBookmarkElement(Bookmark & bookmark, pugi::xml_node eleme
 
 std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 {
-	CServer server;
-	if (!::GetServer(element, server))
+	ServerWithCredentials server;
+	if (!::GetServer(element, server)) {
 		return 0;
-	if (server.GetName().empty())
+	}
+	if (server.server.GetName().empty()) {
 		return 0;
+	}
 
 	auto data = std::make_unique<Site>();
-	data->m_server = server;
+	data->server_ = server;
 
 	data->m_comments = GetTextElement(element, "Comments");
 	data->m_colour = GetColourFromIndex(GetTextElementInt(element, "Colour"));
@@ -167,7 +170,7 @@ std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 
 	// Bookmarks
 	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
-		wxString name = GetTextElement_Trimmed(bookmark, "Name");
+		std::wstring name = GetTextElement_Trimmed(bookmark, "Name");
 		if (name.empty()) {
 			continue;
 		}
@@ -234,12 +237,12 @@ public:
 
 	virtual bool AddSite(std::unique_ptr<Site> data)
 	{
-		wxString newName(data->m_server.GetName());
+		wxString newName(data->server_.server.GetName());
 		int i = GetInsertIndex(m_pMenu, newName);
 		newName.Replace(_T("&"), _T("&&"));
 		wxMenuItem* pItem = m_pMenu->Insert(i, wxID_ANY, newName);
 
-		data->m_path = path + _T("/") + CSiteManager::EscapeSegment(data->m_server.GetName());
+		data->m_path = path + _T("/") + CSiteManager::EscapeSegment(data->server_.server.GetName());
 
 		(*m_idMap)[pItem->GetId()] = std::move(data);
 
@@ -569,7 +572,7 @@ std::pair<std::unique_ptr<Site>, Bookmark> CSiteManager::DoGetSiteByPath(std::ws
 	return ret;
 }
 
-wxString CSiteManager::AddServer(CServer server)
+std::wstring CSiteManager::AddServer(ServerWithCredentials server)
 {
 	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
@@ -581,7 +584,7 @@ wxString CSiteManager::AddServer(CServer server)
 		wxString msg = file.GetError() + _T("\n") + _("The server could not be added.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
-		return wxString();
+		return std::wstring();
 	}
 
 	auto element = document.child("Servers");
@@ -589,9 +592,9 @@ wxString CSiteManager::AddServer(CServer server)
 		element = document.append_child("Servers");
 	}
 
-	std::list<wxString> names;
+	std::vector<std::wstring> names;
 	for (auto child = element.child("Server"); child; child = child.next_sibling("Server")) {
-		wxString name = GetTextElement(child, "Name");
+		std::wstring name = GetTextElement(child, "Name");
 		if (name.empty()) {
 			continue;
 		}
@@ -599,24 +602,24 @@ wxString CSiteManager::AddServer(CServer server)
 		names.push_back(name);
 	}
 
-	std::wstring name = _("New site").ToStdWstring();
+	std::wstring name = fztranslate("New site");
 	int i = 1;
 
 	for (;;) {
-		std::list<wxString>::const_iterator iter;
-		for (iter = names.begin(); iter != names.end(); ++iter) {
+		std::vector<std::wstring>::const_iterator iter;
+		for (iter = names.cbegin(); iter != names.cend(); ++iter) {
 			if (*iter == name) {
 				break;
 			}
 		}
-		if (iter == names.end()) {
+		if (iter == names.cend()) {
 			break;
 		}
 
-		name = _("New site") + wxString::Format(_T(" %d"), ++i);
+		name = _("New site").ToStdWstring() + fz::sprintf(L" %d", ++i);
 	}
 
-	server.SetName(name);
+	server.server.SetName(name);
 
 	auto xServer = element.append_child("Server");
 	SetServer(xServer, server);
@@ -624,15 +627,15 @@ wxString CSiteManager::AddServer(CServer server)
 
 	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
-			return wxString();
+			return std::wstring();
 		}
 
-		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName(), file.GetError());
+		std::wstring msg = fz::sprintf(fztranslate("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName(), file.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
-		return wxString();
+		return std::wstring();
 	}
 
-	return _T("0/") + EscapeSegment(name);
+	return L"0/" + EscapeSegment(name);
 }
 
 pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<std::wstring> const& segments)
@@ -640,20 +643,25 @@ pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<s
 	for (auto const& segment : segments) {
 		pugi::xml_node child;
 		for (child = node.first_child(); child; child = child.next_sibling()) {
-			if (strcmp(child.name(), "Server") && strcmp(child.name(), "Folder") && strcmp(child.name(), "Bookmark"))
+			if (strcmp(child.name(), "Server") && strcmp(child.name(), "Folder") && strcmp(child.name(), "Bookmark")) {
 				continue;
+			}
 
 			wxString name = GetTextElement_Trimmed(child, "Name");
-			if (name.empty())
+			if (name.empty()) {
 				name = GetTextElement_Trimmed(child);
-			if (name.empty())
+			}
+			if (name.empty()) {
 				continue;
+			}
 
-			if (name == segment)
+			if (name == segment) {
 				break;
+			}
 		}
-		if (!child)
+		if (!child) {
 			return pugi::xml_node();
+		}
 
 		node = child;
 		continue;
@@ -846,3 +854,78 @@ wxString CSiteManager::GetColourName(int i)
 	return wxGetTranslation(background_colors[i].name);
 }
 
+void CSiteManager::Rewrite(CLoginManager & loginManager, pugi::xml_node element, bool on_failure_set_to_ask)
+{
+	for (auto child = element.first_child(); child; child = child.next_sibling()) {
+		if (!strcmp(child.name(), "Folder")) {
+			Rewrite(loginManager, child, on_failure_set_to_ask);
+		}
+		else if (!strcmp(child.name(), "Server")) {
+			auto site = ReadServerElement(child);
+			if (site) {
+				loginManager.AskDecryptor(site->server_.credentials.encrypted_, true, false);
+				site->server_.credentials.Unprotect(loginManager.GetDecryptor(site->server_.credentials.encrypted_), on_failure_set_to_ask);
+				Save(child, *site);
+			}
+		}
+	}
+}
+
+void CSiteManager::Rewrite(CLoginManager & loginManager, bool on_failure_set_to_ask)
+{
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		return;
+	}
+	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
+
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	auto document = file.Load();
+	if (!document) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+		return;
+	}
+
+	auto element = document.child("Servers");
+	if (!element) {
+		return;
+	}
+
+	Rewrite(loginManager, element, on_failure_set_to_ask);
+
+	file.Save(true);
+}
+
+void CSiteManager::Save(pugi::xml_node element, Site const& site)
+{
+	SetServer(element, site.server_);
+
+	// Save comments
+	AddTextElement(element, "Comments", site.m_comments.ToStdWstring());
+
+	// Save colour
+	AddTextElement(element, "Colour", CSiteManager::GetColourIndex(site.m_colour));
+
+	// Save local dir
+	AddTextElement(element, "LocalDir", site.m_default_bookmark.m_localDir);
+
+	// Save remote dir
+	AddTextElement(element, "RemoteDir", site.m_default_bookmark.m_remoteDir.GetSafePath());
+
+	AddTextElementUtf8(element, "SyncBrowsing", site.m_default_bookmark.m_sync ? "1" : "0");
+	AddTextElementUtf8(element, "DirectoryComparison", site.m_default_bookmark.m_comparison ? "1" : "0");
+
+	for (auto const& bookmark : site.m_bookmarks) {
+		auto node = element.append_child("Bookmark");
+
+		AddTextElement(node, "Name", bookmark.m_name);
+
+		// Save local dir
+		AddTextElement(node, "LocalDir", bookmark.m_localDir);
+
+		// Save remote dir
+		AddTextElement(node, "RemoteDir", bookmark.m_remoteDir.GetSafePath());
+
+		AddTextElementUtf8(node, "SyncBrowsing", bookmark.m_sync ? "1" : "0");
+		AddTextElementUtf8(node, "DirectoryComparison", bookmark.m_comparison ? "1" : "0");
+	}
+}
