@@ -12,9 +12,26 @@
 #include "xrc_helper.h"
 
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/glue/wx.hpp>
 
 #include <algorithm>
 
+bool CStateFilterManager::FilenameFiltered(std::wstring const& name, std::wstring const& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const
+{
+	if (local) {
+		if (m_localFilter && FilenameFilteredByFilter(m_localFilter, name, path, dir, size, attributes, date)) {
+			return true;
+		}
+	}
+	else {
+		if (m_remoteFilter && FilenameFilteredByFilter(m_remoteFilter, name, path, dir, size, attributes, date)) {
+			return true;
+		}
+	}
+
+	return CFilterManager::FilenameFiltered(name, path, dir, size, local, attributes, date);
+}
+
 CContextManager CContextManager::m_the_context_manager;
 
 CContextManager::CContextManager()
@@ -41,15 +58,18 @@ CState* CContextManager::CreateState(CMainFrame &mainFrame)
 void CContextManager::DestroyState(CState* pState)
 {
 	for (unsigned int i = 0; i < m_contexts.size(); ++i) {
-		if (m_contexts[i] != pState)
+		if (m_contexts[i] != pState) {
 			continue;
+		}
 
 		m_contexts.erase(m_contexts.begin() + i);
-		if ((int)i < m_current_context)
-			m_current_context--;
+		if ((int)i < m_current_context) {
+			--m_current_context;
+		}
 		else if ((int)i == m_current_context) {
-			if (i >= m_contexts.size())
-				m_current_context--;
+			if (i >= m_contexts.size()) {
+				--m_current_context;
+			}
 			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 		}
 
@@ -62,12 +82,14 @@ void CContextManager::DestroyState(CState* pState)
 
 void CContextManager::SetCurrentContext(CState* pState)
 {
-	if (GetCurrentContext() == pState)
+	if (GetCurrentContext() == pState) {
 		return;
+	}
 
 	for (unsigned int i = 0; i < m_contexts.size(); ++i) {
-		if (m_contexts[i] != pState)
+		if (m_contexts[i] != pState) {
 			continue;
+		}
 
 		m_current_context = i;
 		NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
@@ -184,7 +206,7 @@ void CContextManager::ProcessDirectoryListing(CServer const& server, std::shared
 			continue;
 		}
 		if (state->GetServer() && state->GetServer().server == server) {
-			state->SetRemoteDir(listing, true);
+			state->SetRemoteDir(listing, false);
 		}
 	}
 }
@@ -295,40 +317,40 @@ bool CState::SetLocalDir(CLocalPath const& dir, std::wstring *error, bool rememb
 #ifdef __WXMSW__
 		if (dir.GetPath() == _T("\\")) {
 			m_previouslyVisitedLocalSubdir = m_localDir.GetPath();
-			m_previouslyVisitedLocalSubdir.RemoveLast();
+			m_previouslyVisitedLocalSubdir.pop_back();
 		}
 		else
 #endif
+		{
 			m_previouslyVisitedLocalSubdir = m_localDir.GetLastSegment();
+		}
 	}
 	else {
-		m_previouslyVisitedLocalSubdir = _T("");
+		m_previouslyVisitedLocalSubdir.clear();
 	}
 
 
 	m_localDir = dir;
 
-	COptions::Get()->SetOption(OPTION_LASTLOCALDIR, m_localDir.GetPath());
-
 	NotifyHandlers(STATECHANGE_LOCAL_DIR);
 
 	return true;
 }
 
-bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryListing, bool modified)
+bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryListing, bool primary)
 {
 	if (!pDirectoryListing) {
 		m_changeDirFlags.compare = false;
 		SetSyncBrowse(false);
-		if (modified) {
+		if (!primary) {
 			return false;
 		}
 
 		if (m_pDirectoryListing) {
 			m_pDirectoryListing = 0;
-			NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &modified);
+			NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &primary);
 		}
-		m_previouslyVisitedRemoteSubdir = _T("");
+		m_previouslyVisitedRemoteSubdir.clear();
 		return true;
 	}
 
@@ -340,10 +362,10 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 		m_previouslyVisitedRemoteSubdir = m_pDirectoryListing->path.GetLastSegment();
 	}
 	else {
-		m_previouslyVisitedRemoteSubdir = _T("");
+		m_previouslyVisitedRemoteSubdir.clear();
 	}
 
-	if (modified) {
+	if (!primary) {
 		if (!m_pDirectoryListing || m_pDirectoryListing->path != pDirectoryListing->path) {
 			// We aren't interested in these listings
 			return true;
@@ -362,14 +384,14 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 	m_pDirectoryListing = pDirectoryListing;
 
-	NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &modified);
+	NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &primary);
 
 	bool compare = m_changeDirFlags.compare;
-	if (!modified) {
+	if (primary) {
 		m_changeDirFlags.compare = false;
 	}
 
-	if (m_changeDirFlags.syncbrowse && !modified) {
+	if (m_changeDirFlags.syncbrowse && primary) {
 		m_changeDirFlags.syncbrowse = false;
 		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false)) {
 			SetSyncBrowse(false);
@@ -398,8 +420,6 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 				else {
 					m_localDir = local_path;
 
-					COptions::Get()->SetOption(OPTION_LASTLOCALDIR, m_localDir.GetPath());
-
 					NotifyHandlers(STATECHANGE_LOCAL_DIR);
 				}
 			}
@@ -420,8 +440,9 @@ std::shared_ptr<CDirectoryListing> CState::GetRemoteDir() const
 
 const CServerPath CState::GetRemotePath() const
 {
-	if (!m_pDirectoryListing)
+	if (!m_pDirectoryListing) {
 		return CServerPath();
+	}
 
 	return m_pDirectoryListing->path;
 }
@@ -485,7 +506,7 @@ void CState::SetSite(Site const& site, CServerPath const& path)
 			return;
 		}
 
-		SetRemoteDir(0);
+		SetRemoteDir(nullptr, true);
 		m_pCertificate.reset();
 		m_pSftpEncryptionInfo.reset();
 	}
@@ -538,7 +559,7 @@ bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 	SetSyncBrowse(false);
 
 	m_pCommandQueue->ProcessCommand(new CConnectCommand(site.server_.server, site.server_.credentials));
-	m_pCommandQueue->ProcessCommand(new CListCommand(path, _T(""), LIST_FLAG_FALLBACK_CURRENT));
+	m_pCommandQueue->ProcessCommand(new CListCommand(path, std::wstring(), LIST_FLAG_FALLBACK_CURRENT));
 
 	SetSite(site, path);
 
@@ -699,26 +720,49 @@ CGlobalStateEventHandler::~CGlobalStateEventHandler()
 	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
 }
 
-void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
+void CState::UploadDroppedFiles(CLocalDataObject const* pLocalDataObject, std::wstring const& subdir, bool queueOnly)
 {
 	if (!m_site.server_ || !m_pDirectoryListing) {
 		return;
 	}
 
 	CServerPath path = m_pDirectoryListing->path;
-	if (subdir == _T("..") && path.HasParent()) {
+	if (subdir == L".." && path.HasParent()) {
 		path = path.GetParent();
 	}
 	else if (!subdir.empty()) {
-		path.AddSegment(subdir.ToStdWstring());
+		path.AddSegment(subdir);
+	}
+
+	UploadDroppedFiles(pLocalDataObject, path, queueOnly);
+}
+
+void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, std::wstring const& subdir, bool queueOnly)
+{
+	if (!m_site.server_ || !m_pDirectoryListing) {
+		return;
+	}
+
+	CServerPath path = m_pDirectoryListing->path;
+	if (subdir == L".." && path.HasParent()) {
+		path = path.GetParent();
+	}
+	else if (!subdir.empty()) {
+		path.AddSegment(subdir);
 	}
 
 	UploadDroppedFiles(pFileDataObject, path, queueOnly);
 }
 
-void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const CServerPath& path, bool queueOnly)
+namespace {
+template <typename T>
+void DoUploadDroppedFiles(CState& state, CMainFrame & mainFrame, T const& files, const CServerPath& path, bool queueOnly)
 {
-	if (!m_site.server_) {
+	if (files.empty()) {
+		return;
+	}
+
+	if (!state.GetServer()) {
 		return;
 	}
 
@@ -726,26 +770,24 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 		return;
 	}
 
-	auto recursiveOperation = GetLocalRecursiveOperation();
+	auto recursiveOperation = state.GetLocalRecursiveOperation();
 	if (!recursiveOperation || recursiveOperation->IsActive()) {
 		wxBell();
 		return;
 	}
 
-	wxArrayString const& files = pFileDataObject->GetFilenames();
-
-	for (unsigned int i = 0; i < files.Count(); ++i) {
+	for (auto const& file : files) {
 		int64_t size;
 		bool is_link;
-		fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(files[i]), is_link, &size, 0, 0);
+		fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(file), is_link, &size, 0, 0);
 		if (type == fz::local_filesys::file) {
 			std::wstring localFile;
-			const CLocalPath localPath(files[i].ToStdWstring(), &localFile);
-			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, m_site.server_, size);
-			m_mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
+			CLocalPath const localPath(fz::to_wstring(file), &localFile);
+			mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, state.GetServer(), size);
+			mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
 		}
 		else if (type == fz::local_filesys::dir) {
-			CLocalPath localPath(files[i].ToStdWstring());
+			CLocalPath localPath(fz::to_wstring(file));
 			if (localPath.HasParent()) {
 
 				CServerPath remotePath = path;
@@ -763,55 +805,69 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 	CFilterManager filter;
 	recursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queueOnly);
 }
+}
 
-void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const CLocalPath& path, bool copy)
+void CState::UploadDroppedFiles(const CLocalDataObject* pLocalDataObject, const CServerPath& path, bool queueOnly)
 {
-	const wxArrayString &files = pFileDataObject->GetFilenames();
-	if (!files.Count()) {
+	auto const files = pLocalDataObject->GetFilesW();
+	DoUploadDroppedFiles(*this, m_mainFrame, files, path, queueOnly);
+}
+
+void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const CServerPath& path, bool queueOnly)
+{
+	wxArrayString const& files = pFileDataObject->GetFilenames();
+	DoUploadDroppedFiles(*this, m_mainFrame, files, path, queueOnly);
+}
+
+namespace {
+template<typename T>
+void DoHandleDroppedFiles(CState & state, CMainFrame & mainFrame, T const& files, CLocalPath const& path, bool copy)
+{
+	if (files.empty()) {
 		return;
 	}
 
 #ifdef __WXMSW__
 	int len = 1;
 
-	for (unsigned int i = 0; i < files.Count(); ++i) {
-		len += files[i].Len() + 1;
+	for (unsigned int i = 0; i < files.size(); ++i) {
+		len += files[i].size() + 1;
 	}
 
 	// SHFILEOPSTRUCT's pTo and pFrom accept null-terminated lists
 	// of null-terminated filenames.
-	wxChar* from = new wxChar[len];
-	wxChar* p = from;
-	for (unsigned int i = 0; i < files.Count(); ++i) {
-		wxStrcpy(p, files[i]);
-		p += files[i].Len() + 1;
+	wchar_t* from = new wchar_t[len];
+	wchar_t* p = from;
+	for (auto const& file : files) {
+		memcpy(p, static_cast<wchar_t const*>(file.c_str()), (file.size() + 1) * sizeof(wchar_t));
+		p += file.size() + 1;
 	}
 	*p = 0; // End of list
 
-	wxChar* to = new wxChar[path.GetPath().size() + 2];
-	wxStrcpy(to, path.GetPath());
+	wchar_t* to = new wchar_t[path.GetPath().size() + 2];
+	memcpy(to, static_cast<wchar_t const*>(path.GetPath().c_str()), (path.GetPath().size() + 1) * sizeof(wchar_t));
 	to[path.GetPath().size() + 1] = 0; // End of list
 
-	SHFILEOPSTRUCT op = {0};
+	SHFILEOPSTRUCT op = { 0 };
 	op.pFrom = from;
 	op.pTo = to;
 	op.wFunc = copy ? FO_COPY : FO_MOVE;
-	op.hwnd = (HWND)m_mainFrame.GetHandle();
+	op.hwnd = (HWND)mainFrame.GetHandle();
 	SHFileOperation(&op);
 
-	delete [] to;
-	delete [] from;
+	delete[] to;
+	delete[] from;
 #else
-	wxString error;
-	for (unsigned int i = 0; i < files.Count(); ++i) {
-		std::wstring const file(files[i].ToStdWstring());
+	(void)mainFrame;
 
+	wxString error;
+	for (auto const& file : files) {
 		int64_t size;
 		bool is_link;
 		fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(file), is_link, &size, 0, 0);
 		if (type == fz::local_filesys::file) {
 			std::wstring name;
-			CLocalPath sourcePath(file, &name);
+			CLocalPath sourcePath(fz::to_wstring(file), &name);
 			if (name.empty()) {
 				continue;
 			}
@@ -828,7 +884,7 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 			}
 		}
 		else if (type == fz::local_filesys::dir) {
-			CLocalPath sourcePath(file);
+			CLocalPath sourcePath(fz::to_wstring(file));
 			if (sourcePath == path || sourcePath.GetParent() == path) {
 				continue;
 			}
@@ -838,7 +894,7 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 			}
 
 			if (copy) {
-				RecursiveCopy(sourcePath, path);
+				state.RecursiveCopy(sourcePath, path);
 			}
 			else {
 				if (!sourcePath.HasParent()) {
@@ -853,7 +909,20 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 	}
 #endif
 
-	RefreshLocal();
+	state.RefreshLocal();
+}
+}
+
+void CState::HandleDroppedFiles(CLocalDataObject const* pLocalDataObject, CLocalPath const& path, bool copy)
+{
+	auto const files = pLocalDataObject->GetFilesW();
+	DoHandleDroppedFiles(*this, m_mainFrame, files, path, copy);
+}
+
+void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const CLocalPath& path, bool copy)
+{
+	wxArrayString const& files = pFileDataObject->GetFilenames();
+	DoHandleDroppedFiles(*this, m_mainFrame, files, path, copy);
 }
 
 bool CState::RecursiveCopy(CLocalPath source, const CLocalPath& target)
@@ -924,7 +993,7 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 {
 	bool hasDirs = false;
 	bool hasFiles = false;
-	const std::list<CRemoteDataObject::t_fileInfo>& files = pRemoteDataObject->GetFiles();
+	std::vector<CRemoteDataObject::t_fileInfo> const& files = pRemoteDataObject->GetFiles();
 	for (auto const& fileInfo : files) {
 		if (fileInfo.dir) {
 			hasDirs = true;
@@ -980,19 +1049,22 @@ bool CState::IsRemoteConnected() const
 
 bool CState::IsRemoteIdle(bool ignore_recursive) const
 {
-	if (!ignore_recursive && m_pRemoteRecursiveOperation && m_pRemoteRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+	if (!ignore_recursive && m_pRemoteRecursiveOperation && m_pRemoteRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none) {
 		return false;
+	}
 
-	if (!m_pCommandQueue)
+	if (!m_pCommandQueue) {
 		return true;
+	}
 
 	return m_pCommandQueue->Idle(ignore_recursive ? CCommandQueue::normal : CCommandQueue::any);
 }
 
 bool CState::IsLocalIdle(bool ignore_recursive) const
 {
-	if (!ignore_recursive && m_pLocalRecursiveOperation && m_pLocalRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+	if (!ignore_recursive && m_pLocalRecursiveOperation && m_pLocalRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none) {
 		return false;
+	}
 
 	return true;
 }
@@ -1214,7 +1286,7 @@ CServerPath CState::GetSynchronizedDirectory(CLocalPath local_path)
 	return remote_path;
 }
 
-bool CState::RefreshRemote()
+bool CState::RefreshRemote(bool clear_cache)
 {
 	if (!m_pCommandQueue) {
 		return false;
@@ -1224,7 +1296,12 @@ bool CState::RefreshRemote()
 		return false;
 	}
 
-	return ChangeRemoteDir(GetRemotePath(), _T(""), LIST_FLAG_REFRESH);
+	int flags = LIST_FLAG_REFRESH;
+	if (clear_cache) {
+		flags |= LIST_FLAG_CLEARCACHE;
+	}
+
+	return ChangeRemoteDir(GetRemotePath(), std::wstring(), flags);
 }
 
 bool CState::GetSecurityInfo(CCertificateNotification *& pInfo)
