@@ -3,6 +3,7 @@
 
 #include "local_path.h"
 #include "sitemanager.h"
+#include "filter.h"
 
 #include <memory>
 
@@ -50,6 +51,7 @@ enum t_statechange_notifications
 class CDirectoryListing;
 class CFileZillaEngine;
 class CCommandQueue;
+class CLocalDataObject; 
 class CLocalRecursiveOperation;
 class CMainFrame;
 class CGlobalStateEventHandler;
@@ -58,6 +60,22 @@ class CRemoteDataObject;
 class CRemoteRecursiveOperation;
 class CComparisonManager;
 
+class CStateFilterManager final : public CFilterManager
+{
+public:
+	virtual bool FilenameFiltered(std::wstring const& name, std::wstring const& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const override;
+
+	CFilter const& GetLocalFilter() const { return m_localFilter; }
+	void SetLocalFilter(CFilter const& filter) { m_localFilter = filter; }
+
+	CFilter const& GetRemoteFilter() const { return m_remoteFilter; }
+	void SetRemoteFilter(CFilter const& filter) { m_remoteFilter = filter; }
+
+private:
+	CFilter m_localFilter;
+	CFilter m_remoteFilter;
+};
+
 class CState;
 class CContextManager final
 {
@@ -121,11 +139,11 @@ public:
 	bool SetLocalDir(CLocalPath const& dir, std::wstring *error = 0, bool rememberPreviousSubdir = true);
 	bool SetLocalDir(std::wstring const& dir, std::wstring *error = 0, bool rememberPreviousSubdir = true);
 
-	bool Connect(Site const& site, const CServerPath& path = CServerPath(), bool compare = false);
+	bool Connect(Site const& site, CServerPath const& path = CServerPath(), bool compare = false);
 	bool Disconnect();
 
 	bool ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir = std::wstring(), int flags = 0, bool ignore_busy = false, bool compare = false);
-	bool SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryListing, bool modified = false);
+	bool SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryListing, bool primary);
 	std::shared_ptr<CDirectoryListing> GetRemoteDir() const;
 	const CServerPath GetRemotePath() const;
 
@@ -135,9 +153,9 @@ public:
 
 	void RefreshLocal();
 	void RefreshLocalFile(std::wstring const& file);
-	void LocalDirCreated(const CLocalPath& path);
+	void LocalDirCreated(CLocalPath const& path);
 
-	bool RefreshRemote();
+	bool RefreshRemote(bool clear_cache = false);
 
 	void RegisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification, CStateEventHandler* insertBefore = 0);
 	void UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification);
@@ -146,12 +164,15 @@ public:
 	CCommandQueue* m_pCommandQueue{};
 	CComparisonManager* GetComparisonManager() { return m_pComparisonManager; }
 
-	void UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly);
-	void UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const CServerPath& path, bool queueOnly);
-	void HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const CLocalPath& path, bool copy);
-	bool DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, const CLocalPath& path, bool queueOnly = false);
+	void UploadDroppedFiles(CLocalDataObject const* pLocalDataObject, std::wstring const& subdir, bool queueOnly);
+	void UploadDroppedFiles(wxFileDataObject const* pFileDataObject, std::wstring const& subdir, bool queueOnly);
+	void UploadDroppedFiles(CLocalDataObject const* pLocalDataObject, CServerPath const& path, bool queueOnly);
+	void UploadDroppedFiles(wxFileDataObject const* pFileDataObject, CServerPath const& path, bool queueOnly);
+	void HandleDroppedFiles(CLocalDataObject const* pLocalDataObject, CLocalPath const& path, bool copy);
+	void HandleDroppedFiles(wxFileDataObject const* pFileDataObject, CLocalPath const& path, bool copy);
+	bool DownloadDroppedFiles(CRemoteDataObject const* pRemoteDataObject, CLocalPath const& path, bool queueOnly = false);
 
-	static bool RecursiveCopy(CLocalPath source, const CLocalPath& targte);
+	static bool RecursiveCopy(CLocalPath source, CLocalPath const& targte);
 
 	bool IsRemoteConnected() const;
 	bool IsRemoteIdle(bool ignore_recursive = false) const;
@@ -160,15 +181,15 @@ public:
 	CLocalRecursiveOperation* GetLocalRecursiveOperation() { return m_pLocalRecursiveOperation; }
 	CRemoteRecursiveOperation* GetRemoteRecursiveOperation() { return m_pRemoteRecursiveOperation; }
 
-	void NotifyHandlers(t_statechange_notifications notification, wxString const& data = wxString(), const void* data2 = 0);
+	void NotifyHandlers(t_statechange_notifications notification, wxString const& data = wxString(), void const* data2 = 0);
 
 	bool SuccessfulConnect() const { return m_successful_connect; }
 	void SetSuccessfulConnect() { m_successful_connect = true; }
 
 	void ListingFailed(int error);
-	void LinkIsNotDir(const CServerPath& path, const wxString& subdir);
+	void LinkIsNotDir(CServerPath const& path, wxString const& subdir);
 
-	bool SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root = CServerPath());
+	bool SetSyncBrowse(bool enable, CServerPath const& assumed_remote_root = CServerPath());
 	bool GetSyncBrowse() const { return !m_sync_browse.local_root.empty(); }
 
 	Site GetLastSite() const { return m_last_site; }
@@ -183,13 +204,15 @@ public:
 
 	// If the previously selected directory was a direct child of the current directory, this
 	// returns the relative name of the subdirectory.
-	wxString GetPreviouslyVisitedLocalSubdir() const { return m_previouslyVisitedLocalSubdir; }
-	wxString GetPreviouslyVisitedRemoteSubdir() const { return m_previouslyVisitedRemoteSubdir; }
-	void ClearPreviouslyVisitedLocalSubdir() { m_previouslyVisitedLocalSubdir = _T(""); }
-	void ClearPreviouslyVisitedRemoteSubdir() { m_previouslyVisitedRemoteSubdir = _T(""); }
+	std::wstring GetPreviouslyVisitedLocalSubdir() const { return m_previouslyVisitedLocalSubdir; }
+	std::wstring GetPreviouslyVisitedRemoteSubdir() const { return m_previouslyVisitedRemoteSubdir; }
+	void ClearPreviouslyVisitedLocalSubdir() { m_previouslyVisitedLocalSubdir.clear(); }
+	void ClearPreviouslyVisitedRemoteSubdir() { m_previouslyVisitedRemoteSubdir.clear(); }
 
 	void UpdateSite(wxString const& oldPath, Site const& newSite);
 
+	CStateFilterManager& GetStateFilterManager() { return m_stateFilterManager; }
+
 protected:
 	void SetSite(Site const& site, CServerPath const& path = CServerPath());
 
@@ -212,7 +235,9 @@ protected:
 	CRemoteRecursiveOperation* m_pRemoteRecursiveOperation;
 
 	CComparisonManager* m_pComparisonManager;
-
+	
+	CStateFilterManager m_stateFilterManager;
+	
 	struct t_handlersForNotification
 	{
 		std::vector<CStateEventHandler*> handlers;
@@ -243,8 +268,8 @@ protected:
 	std::unique_ptr<CCertificateNotification> m_pCertificate;
 	std::unique_ptr<CSftpEncryptionNotification> m_pSftpEncryptionInfo;
 
-	wxString m_previouslyVisitedLocalSubdir;
-	wxString m_previouslyVisitedRemoteSubdir;
+	std::wstring m_previouslyVisitedLocalSubdir;
+	std::wstring m_previouslyVisitedRemoteSubdir;
 };
 
 class CGlobalStateEventHandler
