@@ -1,5 +1,5 @@
-#ifndef __SYSTEMIMAGELIST_H__
-#define __SYSTEMIMAGELIST_H__
+#ifndef FILEZILLA_INTERFACE_SYSTEMIMAGELIST_HEADER
+#define FILEZILLA_INTERFACE_SYSTEMIMAGELIST_HEADER
 
 #ifdef __WXMSW__
 #include <shellapi.h>
@@ -14,16 +14,15 @@ enum class iconType
 };
 
 // Required wxImageList extension
-class wxImageListEx : public wxImageList
+class wxImageListEx final : public wxImageList
 {
 public:
 	wxImageListEx();
 	wxImageListEx(int width, int height, const bool mask = true, int initialCount = 1);
-	virtual ~wxImageListEx() {}
 
 #ifdef __WXMSW__
 	wxImageListEx(WXHIMAGELIST hList) { m_hImageList = hList; }
-	HIMAGELIST GetHandle() const { return (HIMAGELIST)m_hImageList; }
+	HIMAGELIST GetHandle() const { return reinterpret_cast<HIMAGELIST>(m_hImageList); }
 	HIMAGELIST Detach();
 #endif
 };
@@ -32,28 +31,28 @@ class CSystemImageList
 {
 public:
 	CSystemImageList(int size = -1);
+	virtual ~CSystemImageList();
 
 	CSystemImageList(CSystemImageList const&) = delete;
 	CSystemImageList& operator=(CSystemImageList const&) = delete;
 
 	bool CreateSystemImageList(int size);
-	virtual ~CSystemImageList();
 
 	wxImageList* GetSystemImageList() { return m_pImageList; }
 
-	int GetIconIndex(iconType type, const wxString& fileName = _T(""), bool physical = true, bool symlink = false);
+	int GetIconIndex(iconType type, std::wstring const& fileName = std::wstring(), bool physical = true, bool symlink = false);
 
 #ifdef __WXMSW__
 	int GetLinkOverlayIndex();
 #endif
 
 private:
-	wxImageListEx *m_pImageList;
+	wxImageListEx *m_pImageList{};
 
 #ifndef __WXMSW__
-	std::map<wxString, int> m_iconCache;
-	std::map<wxString, int> m_iconSymlinkCache;
+	std::map<std::wstring, int> m_iconCache;
+	std::map<std::wstring, int> m_iconSymlinkCache;
 #endif
 };
 
-#endif //__SYSTEMIMAGELIST_H__
+#endif
