@@ -6,7 +6,6 @@
 #include "updater.h"
 #include "Options.h"
 #include "file_utils.h"
-#include <wx/tokenzr.h>
 #include <string>
 
 #ifdef __WXMSW__
@@ -14,12 +13,11 @@
 #endif
 
 #include <libfilezilla/file.hpp>
+#include <libfilezilla/hash.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
 #include <string>
 
-#include <nettle/sha2.h>
-
 BEGIN_EVENT_TABLE(CUpdater, wxEvtHandler)
 EVT_TIMER(wxID_ANY, CUpdater::OnTimer)
 END_EVENT_TABLE()
@@ -61,13 +59,13 @@ PqQuF7sJR6POArUVYkRD/2LIWsB7\n\
 
 void version_information::update_available()
 {
-	if( !nightly_.url_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) == 2 ) {
+	if (!nightly_.url_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) == 2) {
 		available_ = nightly_;
 	}
-	else if( !beta_.version_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) != 0 ) {
+	else if (!beta_.version_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) != 0) {
 		available_ = beta_;
 	}
-	else if( !stable_.version_.empty() ) {
+	else if (!stable_.version_.empty()) {
 		available_ = stable_;
 	}
 	else {
@@ -77,20 +75,23 @@ void version_information::update_available()
 
 static CUpdater* instance = 0;
 
-CUpdater::CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context)
+CUpdater::CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context, std::function<void(CActiveNotification const&)> const& activityNotificationHandler)
 	: state_(UpdaterState::idle)
-	, engine_(new CFileZillaEngine(engine_context, *this))
+	, engine_context_(engine_context)
+	, activityNotificationHandler_(activityNotificationHandler)
 {
 	AddHandler(parent);
 }
 
 void CUpdater::Init()
 {
-	if (state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading) {
+	if (Busy()) {
 		return;
 	}
 
-	raw_version_information_ = COptions::Get()->GetOption( OPTION_UPDATECHECK_NEWVERSION );
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) != 0 || !LongTimeSinceLastCheck()) {
+		raw_version_information_ = COptions::Get()->GetOption(OPTION_UPDATECHECK_NEWVERSION);
+	}
 
 	UpdaterState s = ProcessFinishedData(FZ_AUTOUPDATECHECK);
 
@@ -101,16 +102,18 @@ void CUpdater::Init()
 	update_timer_.SetOwner(this);
 	update_timer_.Start(1000 * 3600);
 
-	if( !instance ) {
+	if (!instance) {
 		instance = this;
 	}
 }
 
 CUpdater::~CUpdater()
 {
-	if( instance == this ) {
-		instance  =0;
+	if (instance == this) {
+		instance = 0;
+	
 	}
+
 	delete engine_;
 }
 
@@ -122,9 +125,9 @@ CUpdater* CUpdater::GetInstance()
 void CUpdater::AutoRunIfNeeded()
 {
 #if FZ_AUTOUPDATECHECK
-	if( state_ == UpdaterState::failed || state_ == UpdaterState::idle ) {
-		if( !COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK) != 0 && LongTimeSinceLastCheck() ) {
-			Run();
+	if (state_ == UpdaterState::failed || state_ == UpdaterState::idle) {
+		if (!COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK) != 0 && LongTimeSinceLastCheck()) {
+			Run(false);
 		}
 	}
 #endif
@@ -133,57 +136,64 @@ void CUpdater::AutoRunIfNeeded()
 void CUpdater::RunIfNeeded()
 {
 	build const b = AvailableBuild();
-	if( state_ == UpdaterState::idle || state_ == UpdaterState::failed ||
+	if (state_ == UpdaterState::idle || state_ == UpdaterState::failed ||
 		LongTimeSinceLastCheck() || (state_ == UpdaterState::newversion && !b.url_.empty()) ||
-		(state_ == UpdaterState::newversion_ready && !VerifyChecksum( DownloadedFile(), b.size_, b.hash_ ) ) )
+		(state_ == UpdaterState::newversion_ready && !VerifyChecksum(DownloadedFile(), b.size_, b.hash_)))
 	{
-		Run();
+		Run(true);
 	}
 }
 
 bool CUpdater::LongTimeSinceLastCheck() const
 {
-	wxString const lastCheckStr = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTDATE);
-	if (lastCheckStr.empty())
+	std::wstring const lastCheckStr = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTDATE);
+	if (lastCheckStr.empty()) {
 		return true;
+	}
 
-	fz::datetime lastCheck(lastCheckStr.ToStdWstring(), fz::datetime::utc);
-	if (lastCheck.empty())
+	fz::datetime lastCheck(lastCheckStr, fz::datetime::utc);
+	if (lastCheck.empty()) {
 		return true;
+	}
 
 	auto const span = fz::datetime::now() - lastCheck;
 
-	if (span.get_seconds() < 0)
+	if (span.get_seconds() < 0) {
 		// Last check in future
 		return true;
+	}
 
 	int days = 1;
-	if (!CBuildInfo::IsUnstable())
+	if (!CBuildInfo::IsUnstable()) {
 		days = COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_INTERVAL);
+	}
 	return span.get_days() >= days;
 }
 
-wxString CUpdater::GetUrl()
+fz::uri CUpdater::GetUrl()
 {
-	wxString host = CBuildInfo::GetHostname();
-	if (host.empty())
-		host = _T("unknown");
+	fz::uri uri("https://update.filezilla-project.org/update.php");
+	fz::query_string qs;
+	
+	std::string host = fz::to_utf8(CBuildInfo::GetHostname());
+	if (host.empty()) {
+		host = "unknown";
+	}
+	qs["platform"] = host;
+	qs["version"] = fz::to_utf8(CBuildInfo::GetVersion());
 
-	wxString version = CBuildInfo::GetVersion();
-	version.Replace(_T(" "), _T("%20"));
-
-	wxString url = wxString::Format(_T("https://update.filezilla-project.org/update.php?platform=%s&version=%s"), host, version);
 #if defined(__WXMSW__) || defined(__WXMAC__)
 	// Makes not much sense to submit OS version on Linux, *BSD and the likes, too many flavours.
-	wxString osVersion = wxString::Format(_T("&osversion=%d.%d"), wxPlatformInfo::Get().GetOSMajorVersion(), wxPlatformInfo::Get().GetOSMinorVersion());
-	url += osVersion;
+	qs["osversion"] = fz::sprintf("%d.%d", wxPlatformInfo::Get().GetOSMajorVersion(), wxPlatformInfo::Get().GetOSMinorVersion());
 #endif
 
 #ifdef __WXMSW__
-	if (wxIsPlatform64Bit())
-		url += _T("&osarch=64");
-	else
-		url += _T("&osarch=32");
+	if (wxIsPlatform64Bit()) {
+		qs["osarch"] = "64";
+	}
+	else {
+		qs["osarch"] = "32";
+	}
 
 	// Add information about package
 	{
@@ -198,56 +208,71 @@ wxString CUpdater::GetUrl()
 
 		long updated{};
 		if (key->GetValueType(_T("Updated")) == wxRegKey::Type_Dword && key->QueryValue(_T("Updated"), &updated)) {
-			url += wxString::Format(_T("&updated=%d"), updated);
+			qs["updated"] = fz::to_string(updated);
 		}
 
 		long package{};
 		if (key->GetValueType(_T("Package")) == wxRegKey::Type_Dword && key->QueryValue(_T("Package"), &package)) {
-			url += wxString::Format(_T("&package=%d"), package);
+			qs["package"] = fz::to_string(package);
+		}
+
+		wxString channel;
+		if (key->GetValueType(_T("Channel")) == wxRegKey::Type_String && key->QueryValue(_T("Channel"), channel)) {
+			qs["channel"] = fz::to_utf8(channel);
 		}
 	}
 #endif
 
-	wxString const cpuCaps = CBuildInfo::GetCPUCaps(',');
+	std::string const cpuCaps = fz::to_utf8(CBuildInfo::GetCPUCaps(','));
 	if (!cpuCaps.empty()) {
-		url += _T("&cpuid=") + cpuCaps;
+		qs["cpuid"] = cpuCaps;
 	}
 
-	wxString const lastVersion = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTVERSION);
+	std::wstring const lastVersion = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTVERSION);
 	if (lastVersion != CBuildInfo::GetVersion()) {
-		url += _T("&initial=1");
+		qs["initial"] = "1";
 	}
 	else {
-		url += _T("&initial=0");
+		qs["initial"] = "0";
 	}
 
-	return url;
+	if (manual_) {
+		qs["manual"] = "1";
+	}
+
+	wxString v;
+	if (wxGetEnv(_T("FZUPDATETEST"), &v) && v == _T("1")) {
+		qs["test"] = "1";
+	}
+	uri.query_ = qs.to_string(true);
+	return uri;
 }
 
-bool CUpdater::Run()
+bool CUpdater::Run(bool manual)
 {
-	if( state_ != UpdaterState::idle && state_ != UpdaterState::failed &&
-		state_ != UpdaterState::newversion && state_ != UpdaterState::newversion_ready )
+	if (state_ != UpdaterState::idle && state_ != UpdaterState::failed &&
+		state_ != UpdaterState::newversion && state_ != UpdaterState::newversion_ready)
 	{
 		return false;
 	}
 
-	auto  const t = fz::datetime::now();
-	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.format(_T("%Y-%m-%d %H:%M:%S"), fz::datetime::utc));
+	auto const t = fz::datetime::now();
+	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.format(L"%Y-%m-%d %H:%M:%S", fz::datetime::utc));
 
 	local_file_.clear();
-	log_ = wxString::Format(_("Started update check on %s\n"), t.format(_T("%Y-%m-%d %H:%M:%S"), fz::datetime::local));
+	log_ = fz::sprintf(_("Started update check on %s\n"), t.format(L"%Y-%m-%d %H:%M:%S", fz::datetime::local));
+	manual_ = manual;
 
-	wxString build = CBuildInfo::GetBuildType();
-	if( build.empty() ) {
-		build = _("custom");
+	std::wstring build = CBuildInfo::GetBuildType();
+	if (build.empty())  {
+		build = _("custom").ToStdWstring();
 	}
-	log_ += wxString::Format(_("Own build type: %s\n"), build);
+	log_ += fz::sprintf(_("Own build type: %s\n"), build);
 
 	SetState(UpdaterState::checking);
 
 	m_use_internal_rootcert = true;
-	int res = Download(GetUrl(), std::wstring());
+	int res = Request(GetUrl());
 
 	if (res != FZ_REPLY_WOULDBLOCK) {
 		SetState(UpdaterState::failed);
@@ -257,7 +282,7 @@ bool CUpdater::Run()
 	return state_ == UpdaterState::checking;
 }
 
-int CUpdater::Download(wxString const& url, std::wstring const& local_file)
+int CUpdater::Download(std::wstring const& url, std::wstring const& local_file)
 {
 	wxASSERT(pending_commands_.empty());
 	pending_commands_.clear();
@@ -269,12 +294,29 @@ int CUpdater::Download(wxString const& url, std::wstring const& local_file)
 	return ContinueDownload();
 }
 
+int CUpdater::Request(fz::uri const& uri)
+{
+	wxASSERT(pending_commands_.empty());
+	pending_commands_.clear();
+	pending_commands_.emplace_back(new CDisconnectCommand);
+
+	CServer server(fz::equal_insensitive_ascii(uri.scheme_, std::string("http")) ? HTTP : HTTPS, DEFAULT, fz::to_wstring_from_utf8(uri.host_), uri.port_);
+	pending_commands_.emplace_back(new CConnectCommand(server, Credentials()));
+	pending_commands_.emplace_back(new CHttpRequestCommand(uri));
+
+	return ContinueDownload();
+}
+
 int CUpdater::ContinueDownload()
 {
 	if (pending_commands_.empty()) {
 		return FZ_REPLY_OK;
 	}
 
+	if (!engine_) {
+		engine_ = new CFileZillaEngine(engine_context_, *this);
+	}
+
 	int res = engine_->Execute(*pending_commands_.front());
 	if (res == FZ_REPLY_OK) {
 		pending_commands_.pop_front();
@@ -284,12 +326,12 @@ int CUpdater::ContinueDownload()
 	return res;
 }
 
-bool CUpdater::CreateConnectCommand(wxString const& url)
+bool CUpdater::CreateConnectCommand(std::wstring const& url)
 {
 	ServerWithCredentials s;
 	CServerPath path;
 	std::wstring error;
-	if (!s.ParseUrl(url.ToStdWstring(), 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
+	if (!s.ParseUrl(url, 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
 		return false;
 	}
 
@@ -297,14 +339,15 @@ bool CUpdater::CreateConnectCommand(wxString const& url)
 	return true;
 }
 
-bool CUpdater::CreateTransferCommand(wxString const& url, std::wstring const& local_file)
+bool CUpdater::CreateTransferCommand(std::wstring const& url, std::wstring const& local_file)
 {
 	CFileTransferCommand::t_transferSettings transferSettings;
+	transferSettings.fsync = true;
 
 	ServerWithCredentials s;
 	CServerPath path;
 	std::wstring error;
-	if (!s.ParseUrl(url.ToStdWstring(), 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
+	if (!s.ParseUrl(url, 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
 		return false;
 	}
 	std::wstring file = path.GetLastSegment();
@@ -326,13 +369,20 @@ void CUpdater::DoOnEngineEvent(CFileZillaEngine* engine)
 	}
 
 	std::unique_ptr<CNotification> notification;
-	while( (notification = engine_->GetNextNotification()) ) {
+	while ((notification = engine_->GetNextNotification())) {
 		ProcessNotification(std::move(notification));
 	}
 }
 
 void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notification)
 {
+	if (notification->GetID() == nId_active) {
+		if (activityNotificationHandler_) {
+			auto const& activityNotification = static_cast<CActiveNotification const&>(*notification.get());
+			activityNotificationHandler_(activityNotification);
+		}
+		return;
+	}
 	if (state_ != UpdaterState::checking && state_ != UpdaterState::newversion_downloading) {
 		return;
 	}
@@ -354,7 +404,7 @@ void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notificatio
 						std::vector<uint8_t> ca_data = ca.GetRawData();
 
 						std::string updater_root = fz::base64_decode(s_update_cert);
-						if (ca_data.size() == updater_root.size() && !memcmp(&ca_data[0], updater_root.c_str(), ca_data.size()) ) {
+						if (ca_data.size() == updater_root.size() && !memcmp(&ca_data[0], updater_root.c_str(), ca_data.size())) {
 							certNotification.m_trusted = true;
 						}
 					}
@@ -375,7 +425,7 @@ void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notificatio
 	case nId_logmsg:
 		{
 			auto const& msg = static_cast<CLogmsgNotification const&>(*notification.get());
-			log_ += msg.msg + _T("\n");
+			log_ += msg.msg + L"\n";
 		}
 		break;
 	default:
@@ -395,10 +445,10 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 	else if (!version_information_.available_.url_.empty()) {
 
 		std::wstring const temp = GetTempFile();
-		wxString const local_file = GetLocalFile(version_information_.available_, true);
+		std::wstring const local_file = GetLocalFile(version_information_.available_, true);
 		if (!local_file.empty() && fz::local_filesys::get_file_type(fz::to_native(local_file)) != fz::local_filesys::unknown) {
 			local_file_ = local_file;
-			log_ += wxString::Format(_("Local file is %s\n"), local_file);
+			log_ += fz::sprintf(_("Local file is %s\n"), local_file);
 			s = UpdaterState::newversion_ready;
 		}
 		else {
@@ -414,7 +464,7 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 				if (size >= 0 && size >= version_information_.available_.size_) {
 					s = ProcessFinishedDownload();
 				}
-				else if (!can_download || Download(version_information_.available_.url_, temp) != FZ_REPLY_WOULDBLOCK ) {
+				else if (!can_download || Download(version_information_.available_.url_, temp) != FZ_REPLY_WOULDBLOCK) {
 					s = UpdaterState::newversion;
 				}
 			}
@@ -474,55 +524,54 @@ UpdaterState CUpdater::ProcessFinishedDownload()
 		s = UpdaterState::newversion;
 	}
 	else if (!VerifyChecksum(temp, version_information_.available_.size_, version_information_.available_.hash_)) {
-		wxLogNull log;
-		wxRemoveFile(temp);
+		fz::remove_file(fz::to_native(temp));
 		s = UpdaterState::newversion;
 	}
 	else {
 		s = UpdaterState::newversion_ready;
 
-		wxString local_file = GetLocalFile(version_information_.available_, false);
+		std::wstring local_file = GetLocalFile(version_information_.available_, false);
 
 		wxLogNull log;
-		if (local_file.empty() || !wxRenameFile( temp, local_file, false ) ) {
+		if (local_file.empty() || !wxRenameFile(temp, local_file, false)) {
 			s = UpdaterState::newversion;
-			wxRemoveFile( temp );
-			log_ += wxString::Format(_("Could not create local file %s\n"), local_file);
+			fz::remove_file(fz::to_native(temp));
+			log_ += fz::sprintf(_("Could not create local file %s\n"), local_file);
 		}
 		else {
 			local_file_ = local_file;
-			log_ += wxString::Format(_("Local file is %s\n"), local_file);
+			log_ += fz::sprintf(_("Local file is %s\n"), local_file);
 		}
 	}
 	return s;
 }
 
-wxString CUpdater::GetLocalFile(build const& b, bool allow_existing)
+std::wstring CUpdater::GetLocalFile(build const& b, bool allow_existing)
 {
-	wxString const fn = GetFilename( b.url_ );
-	wxString const dl = GetDownloadDir().GetPath();
+	std::wstring const fn = GetFilename(b.url_);
+	std::wstring const dl = GetDownloadDir().GetPath();
 
 	int i = 1;
-	wxString f = dl + fn;
+	std::wstring f = dl + fn;
 
 	while (fz::local_filesys::get_file_type(fz::to_native(f)) != fz::local_filesys::unknown && (!allow_existing || !VerifyChecksum(f, b.size_, b.hash_))) {
 		if (++i > 99) {
-			return wxString();
+			return std::wstring();
 		}
-		wxString ext;
-		int pos;
-		if (!fn.Right(8).CmpNoCase(_T(".tar.bz2"))) {
+
+		size_t pos;
+		if (fn.size() > 8 && fz::str_tolower_ascii(fn.substr(fn.size() - 8)) == L".tar.bz2") {
 			pos = fn.size() - 8;
 		}
 		else {
-			pos = fn.Find('.', true);
+			pos = fn.rfind('.');
 		}
 
-		if (pos == -1) {
-			f = dl + fn + wxString::Format(_T(" (%d)"), i);
+		if (pos == std::wstring::npos) {
+			f = dl + fn + fz::sprintf(L" (%d)", i);
 		}
 		else {
-			f = dl + fn.Left(pos) + wxString::Format(_T(" (%d)"), i) + fn.Mid(pos);
+			f = dl + fn.substr(0, pos) + fz::sprintf(L" (%d)", i) + fn.substr(pos);
 		}
 	}
 
@@ -535,31 +584,35 @@ void CUpdater::ProcessData(CDataNotification& dataNotification)
 		return;
 	}
 
-	int len;
+	size_t len;
 	char* data = dataNotification.Detach(len);
 
 	if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
-		log_ += wxString::Format(_T("ProcessData %d\n"), len);
+		log_ += fz::sprintf(_T("ProcessData %u\n"), len);
 	}
 
 	if (raw_version_information_.size() + len > 0x40000) {
-		log_ += _("Received version information is too large");
-		engine_->Cancel();
+		log_ += fztranslate("Received version information is too large") + L"\n";
+		if (engine_) {
+			engine_->Cancel();
+		}
 		SetState(UpdaterState::failed);
 	}
 	else {
-		for (int i = 0; i < len; ++i) {
-			if (data[i] < 10 || (unsigned char)data[i] > 127) {
-				log_ += _("Received invalid character in version information");
+		for (size_t i = 0; i < len; ++i) {
+			if (data[i] < 10 || static_cast<unsigned char>(data[i]) > 127) {
+				log_ += fztranslate("Received invalid character in version information") + L"\n";
 				SetState(UpdaterState::failed);
-				engine_->Cancel();
+				if (engine_) {
+					engine_->Cancel();
+				}
 				break;
 			}
 		}
 	}
 
 	if (state_ == UpdaterState::checking) {
-		raw_version_information_ += wxString(data, wxConvUTF8, len);
+		raw_version_information_ += fz::to_wstring_from_utf8(std::string(data, data + len));
 	}
 	delete [] data;
 }
@@ -569,129 +622,148 @@ void CUpdater::ParseData()
 	int64_t const ownVersionNumber = CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion().c_str());
 	version_information_ = version_information();
 
-	wxString raw_version_information = raw_version_information_;
+	std::wstring raw_version_information = raw_version_information_;
 
-	log_ += wxString::Format(_("Parsing %d bytes of version information.\n"), static_cast<int>(raw_version_information.size()));
+	log_ += fz::sprintf(_("Parsing %d bytes of version information.\n"), static_cast<int>(raw_version_information.size()));
 
-	while( !raw_version_information.empty() ) {
-		wxString line;
-		int pos = raw_version_information.Find('\n');
-		if (pos != -1) {
-			line = raw_version_information.Left(pos);
-			raw_version_information = raw_version_information.Mid(pos + 1);
+	while (!raw_version_information.empty()) {
+		std::wstring line;
+		size_t pos = raw_version_information.find('\n');
+		if (pos != std::wstring::npos) {
+			line = raw_version_information.substr(0, pos);
+			raw_version_information = raw_version_information.substr(pos + 1);
 		}
 		else {
 			line = raw_version_information;
 			raw_version_information.clear();
 		}
 
-		wxStringTokenizer tokens(line, _T(" \t\n"),  wxTOKEN_STRTOK);
-		if (!tokens.CountTokens()) {
+		auto const tokens = fz::strtok(line, L" \t\r\n");
+		if (tokens.empty()) {
 			// After empty line, changelog follows
 			version_information_.changelog_ = raw_version_information;
-			version_information_.changelog_.Trim(true);
-			version_information_.changelog_.Trim(false);
+			fz::trim(version_information_.changelog_);
 
 			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
-				log_ += wxString::Format(_T("Changelog: %s\n"), version_information_.changelog_);
+				log_ += fz::sprintf(L"Changelog: %s\n", version_information_.changelog_);
 			}
 			break;
 		}
 
-		wxString const type = tokens.GetNextToken();
-		if (type == _T("resources")) {
-			if (tokens.HasMoreTokens()) {
-				if (UpdatableBuild()) {
-					version_information_.resources_ = tokens.GetNextToken();
-				}
-			}
-			continue;
-		}
-
-		if (tokens.CountTokens() != 1 && tokens.CountTokens() != 5) {
+		std::wstring const& type = tokens[0];
+		if (tokens.size() < 2) {
 			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
-				log_ += wxString::Format(_T("Skipping line with %d tokens\n"), static_cast<int>(tokens.CountTokens() + 1));
+				log_ += fz::sprintf(L"Skipping line with one token of type %s\n", type);
 			}
 			continue;
 		}
 
-		wxString versionOrDate = tokens.GetNextToken();
+		if (type == L"resources") {
+			if (UpdatableBuild()) {
+				version_information_.resources_ = tokens[1];
+			}
+			continue;
+		}
 
-		if (type == _T("nightly")) {
-			fz::datetime nightlyDate(versionOrDate.ToStdWstring(), fz::datetime::utc);
+
+		std::wstring const& versionOrDate = tokens[1];
+
+		if (type == L"nightly") {
+			fz::datetime nightlyDate(versionOrDate, fz::datetime::utc);
 			if (nightlyDate.empty()) {
 				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
-					log_ += _T("Could not parse nightly date\n");
+					log_ += L"Could not parse nightly date\n";
 				}
 				continue;
 			}
 
 			fz::datetime buildDate = CBuildInfo::GetBuildDate();
 			if (buildDate.empty() || nightlyDate.empty() || nightlyDate <= buildDate) {
-				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-					log_ += _T("Nightly isn't newer\n");
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+					log_ += L"Nightly isn't newer\n";
 				}
 				continue;
 			}
 		}
+		else if (type == L"release" || type == L"beta") {
+			int64_t v = CBuildInfo::ConvertToVersionNumber(versionOrDate.c_str());
+			if (v <= ownVersionNumber) {
+				continue;
+			}
+		}
 		else {
-			int64_t v = CBuildInfo::ConvertToVersionNumber(versionOrDate.c_str());
-			if (v <= ownVersionNumber)
-				continue;
-		}
-
-		build* b = 0;
-		if( type == _T("nightly") && UpdatableBuild() ) {
-			b = &version_information_.nightly_;
-		}
-		else if( type == _T("release") ) {
-			b = &version_information_.stable_;
-		}
-		else if( type == _T("beta") ) {
-			b = &version_information_.beta_;
-		}
-
-		if( b ) {
-			b->version_ = versionOrDate;
-
-			if( UpdatableBuild() && tokens.CountTokens() == 4 ) {
-				wxString const url = tokens.GetNextToken();
-				wxString const sizestr = tokens.GetNextToken();
-				wxString const hash_algo = tokens.GetNextToken();
-				wxString const hash = tokens.GetNextToken();
-
-				if( GetFilename(url).empty() ) {
-					if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-						log_ += wxString::Format(_T("Could not extract filename from URL: %s\n"), url);
-					}
-					continue;
-				}
-
-				if( hash_algo.CmpNoCase(_T("sha512")) ) {
-					continue;
-				}
-
-				unsigned long long l = 0;
-				if( !sizestr.ToULongLong(&l) ) {
-					if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-						log_ += wxString::Format(_T("Could not parse size: %s"), sizestr);
-					}
-					continue;
-				}
-
-				b->url_ = url;
-				b->size_ = l;
-				b->hash_ = hash;
-
-				// @translator: Two examples: Found new nightly 2014-04-03\n, Found new release 3.9.0.1\n
-				log_ += wxString::Format(_("Found new %s %s\n"), type, b->version_);
+			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				log_ += fz::sprintf(L"Skipping line with unknown type %s\n", type);
 			}
+			continue;
+		}
+
+		build b;
+		b.version_ = versionOrDate;
+
+		if (tokens.size() < 6) {
+			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				log_ += fz::sprintf(L"Not parsing build line with only %d tokens", tokens.size());
+			}
+		}
+		else if (UpdatableBuild()) {
+			std::wstring const& url = tokens[2];
+			std::wstring const& sizestr = tokens[3];
+			std::wstring const& hash_algo = tokens[4];
+			std::wstring const& hash = tokens[5];
+
+			if (GetFilename(url).empty()) {
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+					log_ += fz::sprintf(L"Could not extract filename from URL: %s\n", url);
+				}
+				continue;
+			}
+
+			if (!fz::equal_insensitive_ascii(hash_algo, std::wstring(L"sha512"))) {
+				continue;
+			}
+
+			auto const size = fz::to_integral<uint64_t>(sizestr);
+			if (!size) {
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+					log_ += fz::sprintf(L"Could not parse size: %s", sizestr) + L"\n";
+				}
+				continue;
+			}
+
+			b.url_ = url;
+			b.size_ = size;
+			b.hash_ = fz::str_tolower_ascii(hash);
+			bool valid_hash = true;
+			for (auto const& c : b.hash_) {
+				if ((c < 'a' || c > 'f') && (c < '0' || c > '9')) {
+					valid_hash = false;
+					break;
+				}
+			}
+			if (!valid_hash) {
+				log_ += fz::sprintf(_("Invalid hash: %s\n"), hash);
+				continue;
+			}
+
+			// @translator: Two examples: Found new nightly 2014-04-03\n, Found new release 3.9.0.1\n
+			log_ += fz::sprintf(_("Found new %s %s\n"), type, b.version_);
+		}
+
+		if (type == _T("nightly") && UpdatableBuild()) {
+			version_information_.nightly_ = b;
+		}
+		else if (type == _T("release")) {
+			version_information_.stable_ = b;
+		}
+		else if (type == _T("beta")) {
+			version_information_.beta_ = b;
 		}
 	}
 
 	version_information_.update_available();
 
-	COptions::Get()->SetOption(OPTION_UPDATECHECK_NEWVERSION, raw_version_information_.ToStdWstring());
+	COptions::Get()->SetOption(OPTION_UPDATECHECK_NEWVERSION, raw_version_information_);
 }
 
 void CUpdater::OnTimer(wxTimerEvent&)
@@ -699,131 +771,120 @@ void CUpdater::OnTimer(wxTimerEvent&)
 	AutoRunIfNeeded();
 }
 
-bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const& checksum)
+bool CUpdater::VerifyChecksum(std::wstring const& file, int64_t size, std::wstring const& checksum)
 {
 	if (file.empty() || checksum.empty()) {
 		return false;
 	}
 
 	auto filesize = fz::local_filesys::get_size(fz::to_native(file));
-	if (filesize < 0 || filesize != size) {
+	if (filesize < 0) {
+		log_ += fz::sprintf(_("Could not obtain size of '%s'"), file) + L"\n";
+		return false;
+	}
+	else if (filesize != size) {
+		log_ += fz::sprintf(_("Local size of '%s' does not match expected size: %d != %d"), file, filesize, size) + L"\n";
 		return false;
 	}
 
-	sha512_ctx state;
-	sha512_init(&state);
+	fz::hash_accumulator acc(fz::hash_algorithm::sha512);
 
 	{
 		fz::file f(fz::to_native(file), fz::file::reading);
 		if (!f.opened()) {
+			log_ += fz::sprintf(_("Could not open '%s'"), file) + L"\n";
 			return false;
 		}
 		unsigned char buffer[65536];
 		int64_t read;
 		while ((read = f.read(buffer, sizeof(buffer))) > 0) {
-			sha512_update(&state, static_cast<size_t>(read), buffer);
+			acc.update(buffer, static_cast<size_t>(read));
 		}
 		if (read < 0) {
+			log_ += fz::sprintf(_("Could not read from '%s'"), file) + L"\n";
 			return false;
 		}
 	}
 
-	unsigned char raw_digest[64];
-	sha512_digest(&state, 64, raw_digest);
+	auto const digest = fz::hex_encode<std::wstring>(acc.digest());
 
-	wxString digest;
-	for (unsigned int i = 0; i < sizeof(raw_digest); ++i) {
-		unsigned char l = raw_digest[i] >> 4;
-		unsigned char r = raw_digest[i] & 0x0F;
-
-		if (l > 9)
-			digest += 'a' + l - 10;
-		else
-			digest += '0' + l;
-
-		if (r > 9)
-			digest += 'a' + r - 10;
-		else
-			digest += '0' + r;
-	}
-
-	if (checksum.CmpNoCase(digest)) {
-		log_ += wxString::Format(_("Checksum mismatch on file %s\n"), file);
+	if (digest != checksum) {
+		log_ += fz::sprintf(_("Checksum mismatch on file %s\n"), file);
 		return false;
 	}
 
-	log_ += wxString::Format(_("Checksum match on file %s\n"), file);
+	log_ += fz::sprintf(_("Checksum match on file %s\n"), file);
 	return true;
 }
 
 std::wstring CUpdater::GetTempFile() const
 {
-	wxASSERT( !version_information_.available_.hash_.empty() );
-	wxString ret = wxFileName::GetTempDir();
+	wxASSERT(!version_information_.available_.hash_.empty());
+	std::wstring ret = wxFileName::GetTempDir().ToStdWstring();
 	if (!ret.empty()) {
-		if (ret.Last() != wxFileName::GetPathSeparator()) {
+		if (ret.back() != wxFileName::GetPathSeparator()) {
 			ret += wxFileName::GetPathSeparator();
 		}
 
-		ret += _T("fzupdate_") + version_information_.available_.hash_.Left(16) + _T(".tmp");
+		ret += L"fzupdate_" + version_information_.available_.hash_.substr(0, 16) + L".tmp";
 	}
 
-	return ret.ToStdWstring();
+	return ret;
 }
 
-wxString CUpdater::GetFilename( wxString const& url) const
+std::wstring CUpdater::GetFilename(std::wstring const& url) const
 {
-	wxString ret;
-	int pos = url.Find('/', true);
-	if (pos != -1) {
-		ret = url.Mid(pos + 1);
+	std::wstring ret;
+	size_t pos = url.rfind('/');
+	if (pos != std::wstring::npos) {
+		ret = url.substr(pos + 1);
 	}
 	size_t p = ret.find_first_of(_T("?#"));
-	if( p != std::string::npos ) {
+	if (p != std::string::npos) {
 		ret = ret.substr(0, p);
 	}
 #ifdef __WXMSW__
-	ret.Replace(_T(":"), _T("_"));
+	fz::replace_substrings(ret, L":", L"_");
 #endif
 
 	return ret;
 }
 
-void CUpdater::SetState( UpdaterState s )
+void CUpdater::SetState(UpdaterState s)
 {
-	if( s != state_ ) {
+	if (s != state_) {
 		state_ = s;
 
 		if (s != UpdaterState::checking && s != UpdaterState::newversion_downloading) {
 			pending_commands_.clear();
 		}
 		build b = version_information_.available_;
-		for (auto const& handler : handlers_ ) {
-			if( handler ) {
-				handler->UpdaterStateChanged( s, b );
+		for (auto const& handler : handlers_) {
+			if (handler) {
+				handler->UpdaterStateChanged(s, b);
 			}
 		}
 	}
 }
 
-wxString CUpdater::DownloadedFile() const
+std::wstring CUpdater::DownloadedFile() const
 {
-	wxString ret;
-	if( state_ == UpdaterState::newversion_ready ) {
+	std::wstring ret;
+	if (state_ == UpdaterState::newversion_ready) {
 		ret = local_file_;
 	}
 	return ret;
 }
 
-void CUpdater::AddHandler( CUpdateHandler& handler )
+void CUpdater::AddHandler(CUpdateHandler& handler)
 {
-	for( auto const& h : handlers_ ) {
+	for (auto const& h : handlers_) {
 		if (h == &handler) {
 			return;
 		}
 	}
-	for( auto& h : handlers_ ) {
-		if( !h ) {
+	for (auto& h : handlers_) {
+		if (!h) {
 			h = &handler;
 			return;
 		}
@@ -831,7 +892,7 @@ void CUpdater::AddHandler( CUpdateHandler& handler )
 	handlers_.push_back(&handler);
 }
 
-void CUpdater::RemoveHandler( CUpdateHandler& handler )
+void CUpdater::RemoveHandler(CUpdateHandler& handler)
 {
 	for (auto& h : handlers_) {
 		if (h == &handler) {
@@ -864,4 +925,8 @@ bool CUpdater::UpdatableBuild() const
 	return CBuildInfo::GetBuildType() == _T("nightly") || CBuildInfo::GetBuildType() == _T("official");
 }
 
+bool CUpdater::Busy() const
+{
+	return state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading;
+}
 #endif
