@@ -3,24 +3,52 @@
 
 #include "xmlfunctions.h"
 
-class wxDialogEx;
-class CVerifyCertDialog final : protected wxEvtHandler
+#include <set>
+
+class CertStore final
 {
 public:
-	CVerifyCertDialog();
+	CertStore();
 
 	bool IsTrusted(CCertificateNotification const& notification);
-	void ShowVerificationDialog(CCertificateNotification& notification, bool displayOnly = false);
+	void SetTrusted(CCertificateNotification const& notification, bool permanent, bool trustAllHostnames);
+
+	void SetInsecure(std::wstring const& host, unsigned int port, bool permanent);
+
+	bool IsInsecure(std::wstring const& host, unsigned int port, bool permanentOnly = false);
+
+	bool HasCertificate(std::wstring const& host, unsigned int port);
 
 private:
 	struct t_certData {
 		std::wstring host;
+		bool trustSans{};
 		unsigned int port{};
 		std::vector<uint8_t> data;
 	};
 
-	bool IsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, bool permanentOnly);
-	bool DoIsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, std::list<t_certData> const& trustedCerts);
+	bool IsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, bool permanentOnly, bool allowSans);
+	bool DoIsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, std::list<t_certData> const& trustedCerts, bool allowSans);
+
+	void LoadTrustedCerts();
+
+	std::list<t_certData> trustedCerts_;
+	std::list<t_certData> sessionTrustedCerts_;
+	std::set<std::tuple<std::wstring, unsigned int>> insecureHosts_;
+	std::set<std::tuple<std::wstring, unsigned int>> sessionInsecureHosts_;
+
+	CXmlFile m_xmlFile;
+};
+
+class wxDialogEx;
+class CVerifyCertDialog final : protected wxEvtHandler
+{
+public:
+	CVerifyCertDialog(CertStore & certStore);
+
+	void ShowVerificationDialog(CCertificateNotification& notification, bool displayOnly = false);
+
+private:
 
 	bool DisplayAlgorithm(int controlId, wxString name, bool insecure);
 
@@ -31,15 +59,6 @@ private:
 
 	wxString DecodeValue(const wxString& value);
 
-	void SetPermanentlyTrusted(CCertificateNotification const& notification);
-
-	void LoadTrustedCerts();
-
-	std::list<t_certData> m_trustedCerts;
-	std::list<t_certData> m_sessionTrustedCerts;
-
-	CXmlFile m_xmlFile;
-
 	std::vector<CCertificate> m_certificates;
 	wxDialogEx* m_pDlg{};
 	wxSizer* m_pSubjectSizer{};
@@ -47,6 +66,10 @@ private:
 	int line_height_{};
 
 	void OnCertificateChoice(wxCommandEvent& event);
+
+	CertStore & certStore_;
 };
 
+void ConfirmInsecureConection(CertStore & certStore, CInsecureFTPNotification & notification);
+
 #endif
