@@ -1,5 +1,5 @@
-#ifndef __WELCOME_DIALOG_H__
-#define __WELCOME_DIALOG_H__
+#ifndef FILEZILLA_INTERFACE_WELCOME_DIALOG_HEADER
+#define FILEZILLA_INTERFACE_WELCOME_DIALOG_HEADER
 
 #include "dialogex.h"
 
@@ -8,10 +8,11 @@
 class CWelcomeDialog final : public wxDialogEx
 {
 public:
-	CWelcomeDialog() {}
-	virtual ~CWelcomeDialog() {}
+	CWelcomeDialog() = default;
 
-	bool Run(wxWindow* parent, bool force = false, bool delay = false);
+	bool Run(wxWindow* parent, bool force = false);
+
+	static void RunDelayed(wxWindow* parent);
 
 protected:
 
@@ -19,8 +20,10 @@ protected:
 
 	wxTimer m_delayedShowTimer;
 
+	wxWindow* parent_{};
+
 	DECLARE_EVENT_TABLE()
 	void OnTimer(wxTimerEvent& event);
 };
 
-#endif //__WELCOME_DIALOG_H__
+#endif
