@@ -6,6 +6,7 @@
 #include "buildinfo.h"
 #include "Options.h"
 
+#include <libfilezilla/file.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
 #include <wx/statbox.h>
@@ -120,34 +121,37 @@ bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned lon
 				}
 			}
 
-			if (lineLength > maxLength && wrappable)
-			{
+			if (lineLength > maxLength && wrappable) {
 				wxString tmp = wxString(str, wrappable - str);
-				if (!tmp.empty() && tmp.Last() == ' ')
+				if (!tmp.empty() && tmp.Last() == ' ') {
 					tmp.RemoveLast();
+				}
 				wrappedText += tmp + _T("\n");
-				if (*wrappable != ' ')
+				if (*wrappable != ' ') {
 					str = wrappable;
-				else
+				}
+				else {
 					str = wrappable + 1;
+				}
 				break;
 			}
 
 			p++;
 		}
-		if (!*p)
-		{
-			if (lineLength > maxLength)
-			{
-				if (!wrappable)
+		if (!*p) {
+			if (lineLength > maxLength) {
+				if (!wrappable) {
 					return false;
+				}
 
 				const wxString& tmp = wxString(str, wrappable - str);
 				wrappedText += tmp + _T("\n");
-				if (*wrappable != ' ')
+				if (*wrappable != ' ') {
 					str = wrappable;
-				else
+				}
+				else {
 					str = wrappable + 1;
+				}
 			}
 			wrappedText += str;
 			break;
@@ -162,14 +166,12 @@ bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned lon
 	wxASSERT(temp.empty() || temp.Last() != ' ');
 	wxASSERT(temp.empty() || temp.Last() != '\n');
 	temp.Replace(_T("&"), _T(""));
-	while (!temp.empty())
-	{
+	while (!temp.empty()) {
 		wxString piece;
 		int pos = temp.Find(_T("\n"));
-		if (pos == -1)
-		{
+		if (pos == -1) {
 			piece = temp;
-			temp = _T("");
+			temp.clear();
 		}
 		else
 		{
@@ -338,7 +340,7 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 
 bool CWrapEngine::WrapText(wxWindow* parent, int id, unsigned long maxLength)
 {
-	wxStaticText* pText = wxDynamicCast(parent->FindWindow(id), wxStaticText);
+	wxStaticText* pText = dynamic_cast<wxStaticText*>(parent->FindWindow(id));
 	if (!pText)
 		return false;
 
@@ -399,7 +401,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 		wxWindow* window;
 		wxSizer* subSizer = 0;
 		if ((window = item->GetWindow())) {
-			wxStaticText* text = wxDynamicCast(window, wxStaticText);
+			wxStaticText* text = dynamic_cast<wxStaticText*>(window);
 			if (text) {
 #ifdef __WXMAC__
 				const int offset = 3;
@@ -429,7 +431,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 				continue;
 			}
 
-			wxNotebook* book = wxDynamicCast(window, wxNotebook);
+			wxNotebook* book = dynamic_cast<wxNotebook*>(window);
 			if (book) {
 				int maxPageWidth = 0;
 				for (unsigned int j = 0; j < book->GetPageCount(); ++j) {
@@ -453,8 +455,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 				continue;
 			}
 
-			if (wxDynamicCast(window, wxCheckBox) || wxDynamicCast(window, wxRadioButton) || wxDynamicCast(window, wxChoice))
-			{
+			if (dynamic_cast<wxCheckBox*>(window) || dynamic_cast<wxRadioButton*>(window) || dynamic_cast<wxChoice*>(window)) {
 #if WRAPDEBUG >= 3
 				plvl printf("Leave: WrapRecursive on unshrinkable control failed: %s\n",
 					static_cast<char const*>(wxString(window->GetClassInfo()->GetClassName())));
@@ -472,8 +473,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 
 			// Add border of static box sizer
 			wxStaticBoxSizer* sboxSizer;
-			if ((sboxSizer = wxDynamicCast(subSizer, wxStaticBoxSizer)))
-			{
+			if ((sboxSizer = dynamic_cast<wxStaticBoxSizer*>(subSizer))) {
 				int top, other;
 				sboxSizer->GetStaticBox()->GetBordersForSizer(&top, &other);
 				subBorder += other * 2;
@@ -497,8 +497,8 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 		}
 	}
 
-	wxStaticBoxSizer* sboxSizer = wxDynamicCast(sizer, wxStaticBoxSizer);
-	if( sboxSizer ) {
+	wxStaticBoxSizer* sboxSizer = dynamic_cast<wxStaticBoxSizer*>(sizer);
+	if (sboxSizer) {
 #ifdef __WXGTK3__
 		gtk_widget_set_size_request(sboxSizer->GetStaticBox()->GetHandle(), -1, -1);
 		GtkRequisition req;
@@ -783,22 +783,18 @@ bool CWrapEngine::UnwrapRecursive(wxWindow* wnd, wxSizer* sizer)
 
 		wxWindow* window;
 		wxSizer* subSizer;
-		if ((window = item->GetWindow()))
-		{
-			wxStaticText* text = wxDynamicCast(window, wxStaticText);
-			if (text)
-			{
+		if ((window = item->GetWindow())) {
+			wxStaticText* text = dynamic_cast<wxStaticText*>(window);
+			if (text) {
 				wxString unwrapped = UnwrapText(text->GetLabel());
 				text->SetLabel(unwrapped);
 
 				continue;
 			}
 
-			wxNotebook* book = wxDynamicCast(window, wxNotebook);
-			if (book)
-			{
-				for (unsigned int j = 0; j < book->GetPageCount(); ++j)
-				{
+			wxNotebook* book = dynamic_cast<wxNotebook*>(window);
+			if (book) {
+				for (unsigned int j = 0; j < book->GetPageCount(); ++j) {
 					wxNotebookPage* page = book->GetPage(j);
 					UnwrapRecursive(wnd, page->GetSizer());
 				}
@@ -827,7 +823,7 @@ int CWrapEngine::GetWidthFromCache(const char* name)
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
 	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
-	auto root = xml.Load();
+	auto root = xml.Load(true);
 	auto  element = root.child("Layout");
 	if (!element) {
 		return 0;
@@ -854,18 +850,20 @@ int CWrapEngine::GetWidthFromCache(const char* name)
 
 void CWrapEngine::SetWidthToCache(const char* name, int width)
 {
-	if (!m_use_cache)
+	if (!m_use_cache) {
 		return;
+	}
 
-	if (!name || !*name)
+	if (!name || !*name) {
 		return;
+	}
 
 	// We have to synchronize access to layout.xml so that multiple processes don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
 	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
-	auto root = xml.Load();
+	auto root = xml.Load(true);
 	auto element = root.child("Layout");
 	if (!element) {
 		return;
@@ -895,31 +893,35 @@ CWrapEngine::CWrapEngine()
 	CheckLanguage();
 }
 
-static wxString GetLocaleFile(const wxString& localesDir, wxString name)
+static wxString GetLocaleFile(wxString const& localesDir, wxString name)
 {
-	if (wxFileName::FileExists(localesDir + name + _T("/filezilla.mo")))
+	if (wxFileName::FileExists(localesDir + name + _T("/filezilla.mo"))) {
 		return name;
-	if (wxFileName::FileExists(localesDir + name + _T("/LC_MESSAGES/filezilla.mo")))
+	}
+	if (wxFileName::FileExists(localesDir + name + _T("/LC_MESSAGES/filezilla.mo"))) {
 		return name + _T("/LC_MESSAGES");
+	}
 
 	size_t pos = name.Find('@');
-	if (pos > 0)
-	{
+	if (pos > 0) {
 		name = name.Left(pos);
-		if (wxFileName::FileExists(localesDir + name + _T("/filezilla.mo")))
+		if (wxFileName::FileExists(localesDir + name + _T("/filezilla.mo"))) {
 			return name;
-		if (wxFileName::FileExists(localesDir + name + _T("/LC_MESSAGES/filezilla.mo")))
+		}
+		if (wxFileName::FileExists(localesDir + name + _T("/LC_MESSAGES/filezilla.mo"))) {
 			return name + _T("/LC_MESSAGES");
+		}
 	}
 
 	pos = name.Find('_');
-	if (pos > 0)
-	{
+	if (pos > 0) {
 		name = name.Left(pos);
-		if (wxFileName::FileExists(localesDir + name + _T("/filezilla.mo")))
+		if (wxFileName::FileExists(localesDir + name + _T("/filezilla.mo"))) {
 			return name;
-		if (wxFileName::FileExists(localesDir + name + _T("/LC_MESSAGES/filezilla.mo")))
+		}
+		if (wxFileName::FileExists(localesDir + name + _T("/LC_MESSAGES/filezilla.mo"))) {
 			return name + _T("/LC_MESSAGES");
+		}
 	}
 
 	return wxString();
@@ -932,7 +934,7 @@ bool CWrapEngine::LoadCache()
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
 	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
-	auto document = xml.Load();
+	auto document = xml.Load(true);
 
 	if (!document) {
 		m_use_cache = false;
@@ -944,16 +946,17 @@ bool CWrapEngine::LoadCache()
 	bool cacheValid = true;
 
 	auto layoutElement = document.child("Layout");
-	if (!layoutElement)
+	if (!layoutElement) {
 		layoutElement = document.append_child("Layout");
+	}
 
-	const wxString buildDate = CBuildInfo::GetBuildDateString();
+	std::wstring const buildDate = CBuildInfo::GetBuildDateString();
 	if (GetTextAttribute(layoutElement, "Builddate") != buildDate) {
 		cacheValid = false;
 		SetTextAttribute(layoutElement, "Builddate", buildDate);
 	}
 
-	const wxString buildTime = CBuildInfo::GetBuildTimeString();
+	std::wstring const buildTime = CBuildInfo::GetBuildTimeString();
 	if (GetTextAttribute(layoutElement, "Buildtime") != buildTime) {
 		cacheValid = false;
 		SetTextAttribute(layoutElement, "Buildtime", buildTime);
@@ -963,8 +966,9 @@ bool CWrapEngine::LoadCache()
 	// -----------------------------
 
 	auto resources = layoutElement.child("Resources");
-	if (!resources)
+	if (!resources) {
 		resources = layoutElement.append_child("Resources");
+	}
 
 	CLocalPath resourceDir = wxGetApp().GetResourceDir();
 	resourceDir.AddSegment(_T("xrc"));
@@ -974,21 +978,22 @@ bool CWrapEngine::LoadCache()
 
 	wxString xrc;
 	for (bool found = dir.GetFirst(&xrc, _T("*.xrc")); found; found = dir.GetNext(&xrc)) {
-		if (!wxFileName::FileExists(resourceDir.GetPath() + xrc))
+		if (!wxFileName::FileExists(resourceDir.GetPath() + xrc)) {
 			continue;
+		}
 
 		fz::datetime const date = fz::local_filesys::get_modification_time(fz::to_native(resourceDir.GetPath() + xrc));
-		wxString const ticks = std::to_wstring(date.get_time_t());
+		std::wstring const ticks = std::to_wstring(date.get_time_t());
 
 		auto resourceElement = FindElementWithAttribute(resources, "xrc", "file", xrc.mb_str());
 		if (!resourceElement) {
 			resourceElement = resources.append_child("xrc");
-			SetTextAttribute(resourceElement, "file", xrc);
+			SetTextAttribute(resourceElement, "file", xrc.ToStdWstring());
 			SetTextAttribute(resourceElement, "date", ticks);
 			cacheValid = false;
 		}
 		else {
-			wxString xrcNodeDate = GetTextAttribute(resourceElement, "date");
+			std::wstring xrcNodeDate = GetTextAttribute(resourceElement, "date");
 			if (xrcNodeDate.empty() || xrcNodeDate != ticks) {
 				cacheValid = false;
 				SetTextAttribute(resourceElement, "date", ticks);
@@ -1006,14 +1011,15 @@ bool CWrapEngine::LoadCache()
 	}
 
 	// Get current language
-	wxString language = wxGetApp().GetCurrentLanguageCode();
-	if (language.empty())
-		language = _T("default");
+	std::wstring language = wxGetApp().GetCurrentLanguageCode().ToStdWstring();
+	if (language.empty()) {
+		language = L"default";
+	}
 
-	auto languageElement = FindElementWithAttribute(layoutElement, "Language", "id", language.mb_str());
+	auto languageElement = FindElementWithAttribute(layoutElement, "Language", "id", fz::to_utf8(language).c_str());
 	if (!languageElement) {
 		languageElement = layoutElement.append_child("Language");
-		SetTextAttribute(languageElement, "id", language.mb_str());
+		SetTextAttribute(languageElement, "id", language);
 	}
 
 	// Get static text font and measure sample text
@@ -1022,7 +1028,7 @@ bool CWrapEngine::LoadCache()
 	wxStaticText* pText = new wxStaticText(pFrame, -1, _T("foo"));
 
 	wxFont font = pText->GetFont();
-	wxString fontDesc = font.GetNativeFontInfoDesc();
+	std::wstring fontDesc = font.GetNativeFontInfoDesc().ToStdWstring();
 
 	auto fontElement = languageElement.child("Font");
 	if (!fontElement)
@@ -1052,9 +1058,9 @@ bool CWrapEngine::LoadCache()
 
 	if (!name.empty()) {
 		fz::datetime const date = fz::local_filesys::get_modification_time(fz::to_native(localesDir.GetPath() + name + _T("/filezilla.mo")));
-		wxString const ticks = std::to_wstring(date.get_time_t());
+		std::wstring const ticks = std::to_wstring(date.get_time_t());
 
-		wxString languageNodeDate = GetTextAttribute(languageElement, "date");
+		std::wstring languageNodeDate = GetTextAttribute(languageElement, "date");
 		if (languageNodeDate.empty() || languageNodeDate != ticks) {
 			SetTextAttribute(languageElement, "date", ticks);
 			cacheValid = false;
@@ -1073,7 +1079,7 @@ bool CWrapEngine::LoadCache()
 		return true;
 	}
 
-	if (!xml.Save(true)) {
+	if (!xml.Save(false)) {
 		m_use_cache = false;
 	}
 	return true;
@@ -1084,9 +1090,10 @@ void CWrapEngine::ClearCache()
 	// We have to synchronize access to layout.xml so that multiple processes don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
-	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("layout.xml"));
-	if (file.FileExists())
-		wxRemoveFile(file.GetFullPath());
+	std::wstring const path = COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR);
+	if (!path.empty()) {
+		fz::remove_file(fz::to_native(path + L"layout.xml"));
+	}
 }
 
 void CWrapEngine::CheckLanguage()
