@@ -24,6 +24,9 @@ wxObject *wxTextCtrlXmlHandlerEx::DoCreateResource()
 	if (HasParam(wxT("maxlength")))
 		text->SetMaxLength(GetLong(wxT("maxlength")));
 
+	if (HasParam(wxT("hint")))
+		text->SetHint(GetText(wxS("hint")));
+
 	return text;
 }
 
