@@ -1133,12 +1133,11 @@ int sftp_cmd_ls(struct sftp_command *cmd)
 {
     struct fxp_handle *dirh;
     struct fxp_names *names;
-    struct fxp_name **ournames;
-    int nnames, namesize;
     const char *dir;
     char *cdir, *unwcdir, *wildcard;
     struct sftp_packet *pktin;
     struct sftp_request *req;
+    struct sftp_request *reqs[4];
     int i;
     int ret;
 
@@ -1195,14 +1194,16 @@ int sftp_cmd_ls(struct sftp_command *cmd)
 	fzprintf(sftpError, "Unable to open %s: %s", dir, fxp_error());
 	ret = 1;
     } else {
-	nnames = namesize = 0;
-	ournames = NULL;
-
+	reqs[0] = fxp_readdir_send(dirh);
+	reqs[1] = fxp_readdir_send(dirh);
+	reqs[2] = fxp_readdir_send(dirh);
+	reqs[3] = fxp_readdir_send(dirh);
+	int ri = 0;
 	while (1) {
 
-	    req = fxp_readdir_send(dirh);
-            pktin = sftp_wait_for_reply(req);
-	    names = fxp_readdir_recv(pktin, req);
+            pktin = sftp_wait_for_reply(reqs[ri]);
+	    names = fxp_readdir_recv(pktin, reqs[ri]);
+	    reqs[ri] = NULL;
 
 	    if (names == NULL) {
 		if (fxp_error_type() == SSH_FX_EOF)
@@ -1215,42 +1216,35 @@ int sftp_cmd_ls(struct sftp_command *cmd)
 		break;
 	    }
 
-	    if (nnames + names->nnames >= namesize) {
-		namesize += names->nnames + 128;
-		ournames = sresize(ournames, namesize, struct fxp_name *);
+	    for (i = 0; i < names->nnames; i++) {
+		if (!wildcard || wc_match(wildcard, names->names[i].filename)) {
+		    unsigned long mtime = 0;
+		    if (names->names[i].attrs.flags & SSH_FILEXFER_ATTR_ACMODTIME) {
+			mtime = names->names[i].attrs.mtime;
+		    }
+		    fzprintf_raw_untrusted(sftpListentry, "%s", names->names[i].longname);
+		    fzprintf_raw_untrusted(sftpUnknown, "%lu", mtime);
+		    fzprintf_raw_untrusted(sftpUnknown, "%s", names->names[i].filename);
+		}
 	    }
 
-	    for (i = 0; i < names->nnames; i++)
-		if (!wildcard || wc_match(wildcard, names->names[i].filename))
-		    ournames[nnames++] = fxp_dup_name(&names->names[i]);
-
 	    fxp_free_names(names);
+	    reqs[ri++] = fxp_readdir_send(dirh);
+	    ri %= 4;
+	}
+	for (i = 0; i < 4; ++i) {
+		if (reqs[ri]) {
+	            pktin = sftp_wait_for_reply(reqs[ri]);
+		    sfree(reqs[ri]);
+		    sfree(pktin);
+		}
+		++ri;
+		ri %= 4;
 	}
 	req = fxp_close_send(dirh);
         pktin = sftp_wait_for_reply(req);
 	fxp_close_recv(pktin, req);
 
-	/*
-	 * Now we have our filenames. Sort them by actual file
-	 * name, and then output the longname parts.
-	 */
-        /*if (nnames > 0)
-            qsort(ournames, nnames, sizeof(*ournames), sftp_name_compare);*/
-
-	/*
-	 * And print them.
-	 */
-	for (i = 0; i < nnames; ++i) {
-	    unsigned long mtime = 0;
-	    if (ournames[i]->attrs.flags & SSH_FILEXFER_ATTR_ACMODTIME) {
-		mtime = ournames[i]->attrs.mtime;
-	    }
-	    fzprintf_raw_untrusted(sftpListentry, "%s", ournames[i]->longname);
-	    fzprintf_raw_untrusted(sftpUnknown, "%lu", mtime);
-	    fzprintf_raw_untrusted(sftpUnknown, "%s", ournames[i]->filename);
-	    fxp_free_name(ournames[i]);
-	}
-	sfree(ournames);
 	ret = 0;
     }
 
@@ -1581,7 +1575,7 @@ int sftp_cmd_mkdir(struct sftp_command *cmd)
     for (i = 1; i < cmd->nwords; i++) {
 	dir = canonify(cmd->words[i], 0);
 	if (!dir) {
-	    fzprintf(sftpError, "%s: canonify: %s", dir, fxp_error());
+	    fzprintf(sftpError, "%s: canonify: %s", cmd->words[i], fxp_error());
 	    return 0;
 	}
 
@@ -3013,6 +3007,8 @@ static void usage(void)
     printf("  -hostkey aa:bb:cc:...\n");
     printf("            manually specify a host key (may be repeated)\n");
     printf("  -batch    disable all interactive prompts\n");
+    printf("  -proxycmd command\n");
+    printf("            use 'command' as local proxy\n");
     printf("  -sshlog file\n");
     printf("  -sshrawlog file\n");
     printf("            log protocol details to a file\n");
@@ -3021,8 +3017,10 @@ static void usage(void)
 
 static void version(void)
 {
-  printf("psftp: %s\n", ver);
-  cleanup_exit(1);
+  char *buildinfo_text = buildinfo("\n");
+  printf("psftp: %s\n%s\n", ver, buildinfo_text);
+  sfree(buildinfo_text);
+  exit(0);
 }
 
 /*
@@ -3203,6 +3201,11 @@ static int psftp_connect(char *userhost, char *user, int portnumber)
 
     back = &ssh_backend;
 
+    logctx = log_init(NULL, conf);
+    console_provide_logctx(logctx);
+
+    platform_psftp_pre_conn_setup();
+
     err = back->init(NULL, &backhandle, conf,
 		     conf_get_str(conf, CONF_host),
 		     conf_get_int(conf, CONF_port),
@@ -3212,9 +3215,7 @@ static int psftp_connect(char *userhost, char *user, int portnumber)
 	fzprintf(sftpError, "ssh_init: %s", err);
 	return 1;
     }
-    logctx = log_init(NULL, conf);
     back->provide_logctx(backhandle, logctx);
-    console_provide_logctx(logctx);
     while (!back->sendok(backhandle)) {
 	if (back->exitcode(backhandle) >= 0)
 	    return 1;
@@ -3447,8 +3448,6 @@ int psftp_main(int argc, char *argv[])
     argv += i;
     back = NULL;
 
-    platform_psftp_post_option_setup();
-
     /*
      * If the loaded session provides a hostname, and a hostname has not
      * otherwise been specified, pop it in `userhost' so that
